/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2006 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package applications.exerciseapplet;

import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JScrollBar;

import matrix.animation.Animator;
import matrix.animation.VisualAnimator;

public class ExerciseVisualAnimator extends VisualAnimator {

    public ExerciseVisualAnimator(Animator a) {
        super(a);    
    }
    
    protected void initComponents() {
    	Insets i = new Insets(3, 6, 3, 6);
        begin = getJButton("pics/Rewind16.gif", "Begin");
        begin.setToolTipText("Begin");
        begin.setMargin(i);
        begin.setBackground(getBackground());
        undo = getJButton("pics/StepBack16.gif", "Backward");
        undo.setToolTipText("Backward");
        undo.setMargin(i);
        undo.setBackground(getBackground());
        redo = getJButton("pics/StepForward16.gif", "Forward");
        redo.setToolTipText("Forward");
        redo.setMargin(i);
        redo.setBackground(getBackground());
        end = getJButton("pics/FastForward16.gif", "End");
        end.setToolTipText("End");
        end.setMargin(i);
        end.setBackground(getBackground());
        undo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                undo();
                fireActionEvent(e);
            }
        });

        redo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                redo();
                fireActionEvent(e);
            }
        });

        begin.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                rewind();
                fireActionEvent(e);
            }
        });

        end.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                end();
                fireActionEvent(e);
            }
        });

        sb = new JScrollBar(JScrollBar.HORIZONTAL);
    }

    protected void layoutComponents() {
        setLayout(new FlowLayout(FlowLayout.CENTER, 3, 0));
        add(begin);
        add(undo);
        add(redo);
        add(end);
        sb = null;
        validate();
    }

    /**
     * Returns a JButton that has the picture from the file fileName. If the
     * file can't be found, then the given text buttonText is used. The file
     * is searched from location "/imgLoc".
     */
    public static JButton getJButton(String imgLoc, String buttonText) {
        JButton jb = new JButton();
        URL imUrl = ExerciseVisualAnimator.class.getResource("/" + imgLoc);
        if (imUrl != null)
            jb.setIcon(new ImageIcon(imUrl, buttonText));
        else
            jb.setText(buttonText);
        return jb;
    }

}
