package applications.lib;

import matrix.animation.Animator;
import matrix.structures.FDT.*;



/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class FDT2Text {


    public static String fdt2Text(FDT fdt) {
	if (fdt instanceof Tree) {
	    return fdt2Text((Tree)fdt);
	} 

	else if (fdt instanceof Array) {
	    return fdt2Text((Array)fdt);
	} 

	else if (fdt instanceof LinkedList) {
	    return fdt2Text((LinkedList)fdt);
	} 
	
	// this is for matrix.structures.CDT.probe.BinSearchTree, that does not implement
	// Tree interface, but holds the tree as it's root element!!
	else if (fdt instanceof matrix.structures.CDT.probe.BinSearchTree 
		 && fdt.getElement() != null && fdt.getElement() instanceof Tree) {
	    return fdt2Text((FDT) fdt.getElement());
	} else {
	    matrix.util.Note.err(null, "Textual representation of FDT class " + fdt.getClass() + " not supported.");
	    return "";
	}
    }
    
    /**
     * Returns level order traversal of the tree.
     * 
     * Eg.   
     * <pre>
     *
     *       M
     *      /\
     *     A  T
     *    / \  \
     *       I  X
     *
     * </pre>
     *
     * is:  'M' 'A' 'T' '' 'I' 'X'
     *
     * 
     */
    public static String fdt2Text(Tree tree) {
	if (tree == null || tree.getElement() == null) return "''";
	
	StringBuffer buf = new StringBuffer();
	Queue queue = new Queue();
	queue.insert(tree);
	String sep = "";
	do {
	    buf.append(sep);
	    Object next = queue.remove();
	    if (next instanceof Tree && ((Tree)next).getElement() != null) {
		tree = (Tree)next;
		buf.append("'");
		buf.append(tree.getElement().toString());
		buf.append("'");
		Tree[] children = tree.getSubTrees();
		for (int i = 0; i < children.length; i++) {
		    if (children[i] == null) {
			queue.insert(new Object());
		    }
		    else {
			queue.insert(children[i]);
		    }
		}
	    }
	    
	    // there is empty leaf node
	    else {
		buf.append("''");
	    }
	    sep = " ";
	} while (!queue.isEmpty());
	return buf.toString();
    }



    /**
     * Returns a string representation of the array.
     *
     * Eg. 'A' 'B' 'C' 'D'
     */
    public static String fdt2Text(Array array) {
	StringBuffer buf = new StringBuffer();
	String sep = "";
	for (int i = array.getFirst(); i < array.getLast()+1; i++, sep=" ") {
	    buf.append(sep);
	    Object obj = array.getObject(i);
	    if (obj == null) {
		buf.append("''");
	    } else {
		buf.append("'");
		buf.append(obj.toString());
		buf.append("'");
	    }
	}
	return buf.toString();
    }



    /**
     * Returns a string representation of the list.
     *
     * Eg. 'A' 'B' 'C' 'D'
     */
    public static String fdt2Text(LinkedList list) {
	StringBuffer buf = new StringBuffer();
	String sep = "";
	while (list != null) {
	    buf.append(sep);
	    if (list.getElement() == null) {
		buf.append("''");
	    } else {
		buf.append("'");
		buf.append(list.getElement().toString());
		buf.append("'");
	    }
	    list = list.getNext();
	    sep = " ";
	}
	return buf.toString();
    }


    
    public static String fdt2Text(FDT[] ss) {
	StringBuffer buf = new StringBuffer();
	String sep = "";
	for (int i = 0; i < ss.length; i++,sep = ", ") {
	    buf.append(sep);
	    buf.append(fdt2Text(ss[i]));
	}
	return buf.toString();
    }


    public static String sequence(Animator a, FDT[] ss) {

	Animator original = Animator.getActiveAnimator();
	Animator.setActiveAnimator(a);
	int origIndex = 0;
	while (a.hasPreviousOperation()) {
	    a.undo();
	    origIndex++;
	}

	StringBuffer buf = new StringBuffer();
	buf.append("[");
	buf.append(fdt2Text(ss));
	buf.append("]");

	while(a.hasNextOperation()) {
	    a.redo();	 
	    buf.append("[");
	    buf.append(fdt2Text(ss));
	    buf.append("]");

	}

	// set animator and structures to original state
	a.rewind();
	for (int i = 0; i < origIndex; i++) {
	    a.redo();
	}

	// set back original animator (what ever it was)
	Animator.setActiveAnimator(original);

	return buf.toString();
	
    }
    
    

    static class Queue {
	private java.util.LinkedList list = new java.util.LinkedList();

	void insert(Object o) {
	    list.add(o);
	}
	
	Object remove() {
	    return list.removeFirst();
	}

	boolean isEmpty() {
	    return list.isEmpty();
	}

    }
}
