package applications.trakla2.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;

import applications.trakla2.datalogging.*;

public class ExerciseSessionImpl extends UnicastRemoteObject implements ExerciseSession {

    private static final long serialVersionUID = 7060496096942031710L;

    long seed = 0;

    String studentid;

    String courseCode;

    int round;

    int exerciseNumber;

    ExerciseServerImpl server;

    protected ExerciseSessionImpl(ExerciseServerImpl server) throws RemoteException {
        this.server = server;
        seed = server.nextSeed();
    }


    public long getSeed() throws RemoteException {
        return seed;
    }


    public String getStudentid()  throws RemoteException {
        return studentid;
    }


    public void setStudentid(String v) throws RemoteException {
        studentid = v;
    }


    public String getCourseCode() throws RemoteException {
        return courseCode;
    }


    public void setCourseCode(String v) throws RemoteException {
        courseCode = v;
    }


    public int getRound() throws RemoteException {
        return round;
    }


    public void setRound(int v) throws RemoteException {
        round = v;
    }


    public int getExerciseNumber() throws RemoteException {
        return exerciseNumber;
    }


    public void setExerciseNumber(int v) throws RemoteException {
        exerciseNumber = v;
    }


    String exerciseClassName;

    /**
     * Get the value of exerciseClassName.
     * @return value of exerciseClassName.
     */
    public String getExerciseClassName() {
        return exerciseClassName;
    }

    /**
     * Set the value of exerciseClassName.
     * @param v  Value to assign to exerciseClassName.
     */
    public void setExerciseClassName(String  v) {
        this.exerciseClassName = v;
    }



    public void init(InitData data) throws RemoteException {
        setStudentid(data.getStudentid());
        setCourseCode(data.getCourseCode());
        setRound(data.getRound());
        setExerciseNumber(data.getExerciseNumber());
        setExerciseClassName(data.getExerciseClassName());
        data.setTimeStamp(new java.util.Date());
        data.setSeed(seed);

        // save log message
        log(data);

    }



    public void reset(ResetData data) throws RemoteException {
        seed = server.nextSeed();
        if (data == null)
            data = new ResetData();
        fill(data);
        log(data);
    }



    public void startIdle() throws RemoteException {
        IdleStartData data = new IdleStartData();
        fill(data);
        log(data);
    }



    public void endIdle() throws RemoteException {
        IdleEndData data = new IdleEndData();
        fill(data);
        log(data);
    }



    public void modelAnswerOpened(ModelAnswerOpenedData data) throws RemoteException {
        if (data == null) data = new ModelAnswerOpenedData();
        fill(data);
        log(data);
    }


    public void modelAnswerClosed(ModelAnswerData data) throws RemoteException {
        if (data == null) data = new ModelAnswerData();
        fill(data);
        // save serialized object
        String msg = server.save(data);
        data.setFileName(msg);
        // save log message
        log(data);
    }



    public void grade(GradeData data) throws RemoteException {
        fill(data);
        // save serialized object
        String msg = server.save(data);

        data.setFileName(msg);
        // save log message
        log(data);
    }



    public Map submit(SubmissionData data) throws RemoteException {
        fill(data);
        try {
            // save serialized object
            String msg = server.save(data);

            // save log message
            data.setFileName(msg);
            log(data);

	    // check the fingerprint 
	    //         - 29.10.2004 fingerprint should be checked
	    //                   at SubmissionHandlers -vkaravir
	    /* String fingerprint = getFingerprint(data);
	    if (!data.getFingerprint().equals(fingerprint)) {
		Hashtable ret = new Hashtable();
		ret.put("operationOk", new Boolean(false));
		ret.put("message", "ERROR: Invalid fingerprint");
		server.error("Student " + data.getStudentid() + " trying to submit with invalid fingerprint: " + fingerprint);
		return ret;
		}*/


            // send data to submission handlers
	    Map ret = server.submit.submitExercise(data);
	    if (!ret.containsKey("message"))
		ret.put("message", data.studentMessage());
	    return ret;
        } catch (Exception e) {
            ExerciseServerImpl.error(e.getMessage() + "\n"+data.toString());
            e.printStackTrace(ExerciseServerImpl.errorLog);
            Hashtable ret = new Hashtable();
            ret.put("operationOk", new Boolean(false));
            ret.put("message", "ERROR: " + e.getMessage());
            return ret;
        }
    }




    void log(ExerciseData data) {
        ExerciseServerImpl.log(data.toString());
    }



    private void fill(ExerciseData data) {
        data.setTimeStamp(new java.util.Date());
        data.setStudentid(studentid);
        data.setCourseCode(courseCode);
        data.setRound(round);
        data.setExerciseNumber(exerciseNumber);
        data.setExerciseClassName(exerciseClassName);
        data.setSeed(seed);
    }

    // The following method might be used through relection or after deserialization.
    private String getFingerprint(SubmissionData data) {
        if (data == null) {
            ExerciseServerImpl.error("No data submitted");
            ExerciseServerImpl.log("No data submitted");
            return "no fingerprint";
        }
        String fp = "" + data.getCourseCode() + data.getRound() + 
        data.getExerciseNumber() + data.getMaxPoints() + 
        System.getProperty("trakla2.server.FINGERPRINT") + 
        data.getExerciseClassName() + data.getStudentid().toUpperCase();
        try {
            fp = SubmissionHandler.calculateMD5(fp.trim());
        } catch (Exception e) {
            //XXX What to do?
            return null;
        }
        return fp;
    }
}
