package applications.trakla2.server;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import applications.trakla2.datalogging.SubmissionData;

public class XmlRpcResultClient extends SubmissionHandler {
    private static Object rpcClient;
    private static Method execMethod;
    public static final String DEFAULT_PORT = "9876";
    public static final String DEFAULT_HOST = "localhost";
    // Error code when failing to create XML-RPC Client
    private static final String CREATION_FAILED = "E1";
    // Error code when failing to execute the remote method 
    private static final String EXECUTION_FAILED = "E2";

    private static boolean createRpcClient() {
        try {
            int port = Integer.parseInt(System.getProperty("xmlrpc.server.port", DEFAULT_PORT));
            String host = System.getProperty("xmlrpc.server.host", DEFAULT_HOST);
            Class c = Class.forName("org.apache.xmlrpc.XmlRpcClient");
            Constructor cons = c.getConstructor(new Class[] {String.class, Integer.TYPE});        
            rpcClient = cons.newInstance(new Object[] {host, new Integer(port)});
            return true;
         } catch (Exception e) {
            e.printStackTrace();
         }
        return false;
    }

    public Map submitExercise(SubmissionData d) throws Exception {
        Map ret = new HashMap();
        if (rpcClient == null)
            if (!createRpcClient()) {
                ret.put("operationOk", new Boolean(false));
                ret.put("message", "ERROR: Failed to submit (" + CREATION_FAILED + ")");
                return ret;
            }
        Vector params = new Vector();
        params.add(d.getStudentid());
        params.add(d.getCourseCode());
        params.add(new Integer(d.getRound()));
        params.add(new Integer(d.getExerciseNumber()));
        params.add(new Integer(d.getGrade()));
        params.add(d.getExerciseClassName());
        params.add(new Integer(d.getMaxPoints()));
        // start experimental code
        String fileName = d.getFileName();
        String dataDir = System.getProperty("trakla2.server.DATADIR");
        if(fileName.startsWith(dataDir)) {
            fileName = fileName.substring(dataDir.length());
        }
        params.add(fileName);
        // end experimental code
        params.add(d.getFingerprint());
        Map result = new HashMap();
        try {
            if (execMethod == null)
                execMethod = rpcClient.getClass().getMethod("execute", 
                        new Class[] {String.class, params.getClass()});
            result = (Map)execMethod.invoke(rpcClient, new Object[] {"result.addResult", params});
        } catch (Exception e) {
            e.printStackTrace();
            result.put("operationOk", new Boolean(false));
            result.put("message", "ERROR: Submission failed (" + EXECUTION_FAILED + ")");
            return result;
        }
        return result;
    }
}
