/**
 * Linearizer class for creating String representations of FDTs
 * Implemented using the visitor pattern.
 * 
 * Typically used by calling the <b>linearize</b> method with
 * the FDT to be linearized.
 * 
 * @author Otto Seppl
 * @version 1.0
 * Created on Apr 28, 2005
 */

package applications.trakla2.util;

import matrix.structures.FDT.Array;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.Graph;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.Tree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.indexed.StaticLinkedList;
import matrix.structures.FDT.substructures.Vertex;

import matrix.structures.util.FDTVisitAssistant;
import matrix.structures.util.FDTVisitor;

import java.util.*;

public class Linearizer implements FDTVisitor{
    
    /**
     * This method is the one that should be called to linearize the gived FDT structure.
     * The rest of the methods in this class are called by the FDTVisitAssistant
     *  
     * @param fdt The FDT to be linearized to an array of String objects
     * @return the result of the linearization
     */
    
      public static String linearize(FDT fdt){
         Linearizer linearizer = new Linearizer();
         return FDTVisitAssistant.accept(fdt, linearizer).toString();
      }
    
      //********** VISITOR METHODS ***********//
      
      public Object visitNonFDT(Object o){
        ArrayList list = new ArrayList();
        list.add(o.toString());
      	return list;
      }
      
      public Object visit(FDT fdt){
        ArrayList list = new ArrayList();
        list.add(fdt.getElement().toString());
      	return list;
      }
      
      public Object visit(Array a) {

        ArrayList resultBuffer = new ArrayList();
        
        int aFirst = a.getFirst();
        int aLast  = a.getLast();
       
        Object aObj;

        for(int i = aFirst; i <= aLast; i++) {
        	aObj = a.getObject(i);
            resultBuffer.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        }
        
        return resultBuffer;
      }
      
      public Object visit(Struct s) {
                    
          ArrayList resultBuffer = new ArrayList();
          Object[] fields = s.getFields();
          
          for(int i = 0; i <fields.length; i++) {
              resultBuffer.addAll((ArrayList)FDTVisitAssistant.accept(fields[i], this));
          }
          
          return resultBuffer;
      }
   
      
      public Object visit(LinkedList a) {
        ArrayList returnValue = new ArrayList();
        
         Object aObj = a.getElement();
         if(aObj == null)
            returnValue.add(null);
         else
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
         
         while(a.hasNext()){
            a    = a.getNext();
            aObj = a.getElement();
            
            if(aObj == null)
                returnValue.add(null);
             else
                returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
         }
         
         return returnValue;
      }


        public Object visit(StaticLinkedList a) {
        	a.setFirst();
        	Object aObj;
        	ArrayList returnValue = new ArrayList();
            
            while(a.isInList()) {
            	aObj = a.currValue();
                if(aObj == null)
                    returnValue.add(null);
                 else
                    returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
                a.next();
            }
            return returnValue;
        }

      public Object visit(Tree a) {
      	ArrayList returnValue = new ArrayList();
        
        if (a==null)
            return returnValue;

        Object aObj = a.getElement();
        if(aObj == null)
            returnValue.add(null);
         else
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        
        
        Tree[] aTree = a.getSubTrees();
        for(int i = 0; i < aTree.length; i++)
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aTree[i], this));

        return returnValue;
      }
        
      public Object visit(Graph a) {
      	ArrayList returnValue = new ArrayList();
        
        if(a == null)
            return returnValue;
        
        Vertex[] aVertices = a.getVertices();
        Object ao;

        for(int i = 0; i < aVertices.length; i++) {
          ao = aVertices[i].getElement();
          if(ao == null)
            returnValue.add(null);
          else
          	returnValue.addAll((ArrayList)FDTVisitAssistant.accept(ao, this));

        }
        return returnValue;
      }

}
