package applications.trakla2.viewer;

import java.applet.Applet;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

import matrix.util.configuration.MatrixConfiguration;

public class TestApplet extends Applet implements ActionListener {
    
    private InetAddress address;
    private Label display;
    private Socket socket;
    
    private int port = 1100;

    private String md5          = "";
    private String host         = "";
    private String fileLocation = "";
    
    public void init() {
	MatrixConfiguration.setCodebaseURL(getCodeBase().toString());
	MatrixConfiguration.getInstance();

        md5          = this.getParameter("md5");
        fileLocation = this.getParameter("file");
                
        host = getCodeBase().getHost();
        
        try {
            address = InetAddress.getByName(host);
        } catch (UnknownHostException e) {
            System.out.println("Sanity check failed - no such internet address");
            return;
        }
                
        //Set up the UI.
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridBag);
        
        Label l1 = new Label("Socket created, internet address is "+address+", MD5 is "+md5+ "file is " + fileLocation, Label.CENTER);
        c.anchor = GridBagConstraints.SOUTH;
        c.gridwidth = GridBagConstraints.REMAINDER;
        gridBag.setConstraints(l1, c); 
        add(l1);
        
        display = new Label("(no data received yet)", Label.CENTER);
        c.anchor = GridBagConstraints.NORTH;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(display, c); 
        add(display);
        
        c.anchor = GridBagConstraints.SOUTH;
        c.gridwidth = 1;
        c.weightx = 0.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.NONE;
                
        Button button = new Button("Get data");
        gridBag.setConstraints(button, c); 
        add(button);
        
        button.addActionListener(this);
    }
    
    public Insets getInsets() {
        return new Insets(4,4,5,5);
    }
    
    public void paint(Graphics g) {
        Dimension d = getSize();
        Color bg = getBackground();
        
        g.setColor(bg);
        g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
        g.draw3DRect(3, 3, d.width - 7, d.height - 7, false);
    }
    
    void doIt() {

        Object result = null;
        
        BufferedWriter os=null;
        ObjectInputStream is=null;
     
        
        try {
            socket = new Socket(host, port);
        } catch (IOException e) {
            System.out.println("Couldn't connect to host");
            return;
        }
        
        try{
            os = new BufferedWriter( new OutputStreamWriter(socket.getOutputStream()));
            is = new ObjectInputStream(socket.getInputStream());
        }catch(IOException e){
            System.err.println("Couldn't open the streams");
            e.printStackTrace();
        }
        
        try { 
            //Send in the md5 sum            
            os.write(md5);
            os.newLine();
            
            //Send in the file location            
            os.write(fileLocation);
            os.newLine();            
            
            os.flush();
                        
            result = is.readObject();
	    applications.trakla2.util.SubmissionViewer.openTrakla2Answer(null, (applications.trakla2.datalogging.ExerciseData)result);

        } catch (IOException e) {
            System.out.println("Something went wrong");
            e.printStackTrace();
            return;
        } catch (ClassNotFoundException e) {
            System.out.println("Class not found");
            e.printStackTrace();
            return;
            
        }
        
        display.setText("Got an object of class : "+result.getClass().getName());
    }
    
    public void actionPerformed(ActionEvent event) {        
        doIt();
    }
}

