package applications.trakla2.viewer;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;

import applications.trakla2.datalogging.SubmissionData;

public class TestServer {

    public static void main(String[] args) {
        ServerSocket ss=null;
        Socket s=null;
        
        String dataDirectory = "";
        if(args != null && args.length >= 1)
            dataDirectory = args[0];
        
        try{
            ss = new ServerSocket(1100);
            System.out.println("Starting exercise test server at port " + ss.getLocalPort() + ". Data directory is " + dataDirectory);
        }catch(IOException e){
            System.err.println("Socket creation failed "+e);
            return;
        }
        
        while(true){
            try{
                s = ss.accept();
                
                ObjectOutputStream os=null;
                BufferedReader is=null;
                
                os = new ObjectOutputStream (s.getOutputStream());
                is = new BufferedReader(new InputStreamReader(s.getInputStream()));
                
                //Read in the md5 sum                
                String md5 = is.readLine();

                //Read in the file location                
                String fileLocation = is.readLine();
                fileLocation = dataDirectory + fileLocation;
                
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(fileLocation));
                Object o = ois.readObject();
                
                // Calculate and compare here the md5 sum to the one received from the applet
                String fingerprint = "";
		if(o instanceof SubmissionData) {
		    fingerprint = getFingerPrint((SubmissionData)o);
		}
                
                if (fingerprint.equals(md5)){
                                
                  os.writeObject(o);
                  os.flush();

                }
                else{
		    //log the bad things
		    System.out.println("Invalid fingerprint");
                }
                  
                //close the connection
                os.close();
                is.close();
                s.close();
                
            }catch(IOException e){
                System.err.println("User session failed "+e);                    
            }
            catch(ClassNotFoundException e){
                System.err.println("Unknown local class type "+e);                                    
            }
        }
        
    }

    public static String getFingerPrint(SubmissionData data) {
	String print = "";
	try {
	    print = FingerprintHelper.calculateMD5(
						   FingerprintHelper.getFingerprint(data.getCourseCode(),
										    data.getRound(), 
										    data.getExerciseNumber(),
										    data.getStudentid(),
										    data.getMaxPoints(), data.getExerciseClassName()));
	    
	} catch(NoSuchAlgorithmException e) {
	    e.printStackTrace();
	}
	return print;
    }
}
