/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */





package content.exercises;

import java.util.Random;

import content.interfaces.ButtonExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseSelfAssessment;
import content.exercises.structures.ExerAVL;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.Note;

public class AVL_Insert_Rotate extends Tree_Rotations implements SimulationExerciseSelfAssessment, ButtonExercise, ModelAnswerNames, ConfigureVisualType {

    private String last;
    public static final boolean DEBUG = false;

    public AVL_Insert_Rotate() {
    }

    public int[] judge() {
	ExerAVL res = (ExerAVL)(getInitialStructures()[1]);
	int i=singleRot;
       	res.realInsert(t.getObject(i));
	int points = 0;
	if (res.equals(ex))
	    points = 1;
	int[] results = new int[1];
	results[0]=points;
	return results;
    }

    public FDT[] init() {
	int i;
        boolean singleNeeded; 
	boolean rootChanged;
        Random R = new Random(seed);
        do {
	    singleRot = 0;
	    rootChanged = false;
	    singleNeeded = false;
	    ex=new ExerAVL();
            // Can contain duplicates.
            S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,16);
	    
            t = new Table(S);
     
            
            for(i = 0; i < S.length(); i++) {
		ex.realInsert(t.getObject(i));
		if (ex.singleWasNeeded() && !singleNeeded && i > MIN_KEYS)
		    singleNeeded=true;
		if (singleNeeded && singleRot <  MIN_KEYS) {
		    singleRot = i;
		    rootChanged = ex.rootWasChanged();
		}
	    }
	    

            if (DEBUG)
                System.out.println("iteration ["+S+"] single:"+singleRot+" root:"+rootChanged+" double:"+ex.doubleWasNeeded());
        } while(singleNeeded == false || singleRot < MIN_KEYS || rootChanged || ex.doubleWasNeeded());
	
        last = S.substring(singleRot, singleRot+1);
        ex = new ExerAVL();
	for (i = 0; i < singleRot; i++)
                ex.realInsert(t.getObject(i));
        FDT[] s = {new Table(last), ex};

        return s;
    }



    public String[] getStructureNames() {
        String[] s =  { "Key", "AVL Tree" };
        return s;
    }

    public FDT[] solve() {
        Note.out(this, "Solve");

        Animator a = Animator.getActiveAnimator();

	ExerAVL res = (ExerAVL)(getInitialStructures()[1]);
	int i=singleRot;

	a.startOperation();
	res.realInsert(t.getObject(i));
	a.endOperation();

        Note.out(this, "Solved");
        return new FDT[] {res };
    }

    public FDT[] getInitialStructures() {
	ExerAVL res = new ExerAVL();
        
	for (int i = 0; i < singleRot; i++)
	    res.realInsert(t.getObject(i));
        return new FDT[] { new Table(last), res };
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        VisualTypeConf tre = super.conf()[0];
        return new VisualTypeConf[] { tabl, tre};
    }

    static final long serialVersionUID = -384577600060725254L;
}
