/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package content.exercises;

import content.interfaces.SimulationExerciseModel;
import matrix.structures.FDT.FDT;

/**
 * This is a abstract class for all exercises that feature some sort
 * of data structure manipulation, algorithm animation or both --
 * exercises that implement SimulationExercise interface.
 *
 * @author Petri Tenhunen
 * @version $Revision: 1.5 $
 */

public abstract class AbstractSimulationExercise implements SimulationExerciseModel {

    /** The data structures used in this exercise are initialized
     * in this method. The creation of the input of the exercise
     * can be handled here, if desired. The input could also,
     * for example, be given in the class constructor. The FDTs
     * returned by this method must be in the state they should be
     * before the user starts solving the exercise.
     *
     * @return an FDT array containing all FDTs used in this exercise.
     */
    public abstract FDT[] init();

    protected long seed = 1;
    public void setSeed(long seed) {this.seed = seed;}
    public long getSeed() {return this.seed;}

    public abstract String[] getStructureNames();
    public abstract FDT[] getAnswer();
    public abstract FDT[] getInitialStructures();

    public FDT[] makeModelAnswer() {
	return solve();
    }

    public abstract FDT[] solve();

}
