/*
 * MatrixPro -- algorithm simulation system
 * Copyright (C) 2003-2004 See the AUTHORS file and the 
 * author tag below for more information on the copyright owners.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises;

import java.util.Random;

public class Analysis_Selection_Asymptotic extends Analysis_Selection {

    private String[][] codeFragments = {
            { " f(n) = O(g(n)) -> g(n) = O(f(n)) ",
                    " f(n) = O(g(n)) -> g(n) = omega(f(n)) " },
            { " f(n) = O(g(n)) -> g(n) = theta(f(n))",
                    " f(n) = O(g(n)) and g(n) = O(f(n)) -> g(n) = theta(f(n)) " },
            { " f(n) + g(n) = theta(min{f(n), g(n)}) ",
                    " f(n) + g(n) = theta(max{f(n), g(n)}) " },
            { " f(n)*g(n) = O(f(n)*g(n)) ", " f(n)*g(n) = O(max{f(n),g(n)}) " },
            { " (n^2)/2 in O(n^3) ",
                    " (n^2)/2 in theta(n^3) " },
            { " ln(n) = O(log(n)) ", " lg(n) = O(ln(n)) " },
            { " (lgn)^k = O(n^e) ", " (lgn)^k = omega(n^e) " },
            { " n^k = O(c^n) ", " c^n = O(n^k) " },
            { " log(n!) = O(log(n^n)) ", " log(n^n) = O(log(n!)) " },
            { " n^lgm = O(m^lgn) ", " m^logn = O(n^logm) " } };

    private String[][] correctAnswers = { { " false ", " true " },
            { " false ", " true " }, { " false ", " true " },
            { " true ", " false " }, { " true ", " false " },
            { " true ", " true " }, { " true ", " false " },
            { " true ", " true " }, { " true ", " true " },
            { " true ", " true " } };
    
    public Analysis_Selection_Asymptotic() {
        super();
    }

    protected int getNumberOfVariations() {
        return codeFragments.length;
    }

    protected String[] getSelectionOptions() {
        return new String[] { " true ", " false " };
    }

    protected int getNumberOfItems() {
        return 3;
    }

    protected String getItem(int i) {
        return getItem(i, 0);
    }
    
    protected String getItem(int i, int variation) {
        return codeFragments[i][variation];
    }

    public String getDescription() {
        Random rand = new Random(getSeed());
        int option;
        String descr = "";
        int[] positions = randomPositions();
        for (int i = 0; i < getNumberOfItems(); i++) {
            option = rand.nextInt(1);
            descr += i + ")\n" + getItem(positions[i], option);
            if (i != getNumberOfItems() - 1)
                descr += "\n\n";
            correctSelections.setObject(correctAnswers[positions[i]][option], i);
        }
        return descr;
    }

    static final long serialVersionUID = -2645553732583065386L;
}
