/*
 * Matrix -- algorithm simulation system Copyright (C) 2000-2004 See the AUTHORS
 * file and the author tag below for more information on the copyright owners.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package content.exercises;

public class Analysis_Selection_IterativeCode extends Analysis_Selection {

    private String[] codeFragments = {
	"sum = 0\nfor (i=1..n) do\n   for (j=1..n) do\n      for (k=1..n) do\n         sum = sum + 1", 
	"sum = 0\nfor (i=1..n) do\n   for (j=1..i*i) do\n      for (k=1..j*j*j) do\n         sum = sum + 1", 
	"sum = 0\nfor (i=1..n) do\n   for (j=1..i/2) do\n      for (k=1..j/2) do\n         sum = sum + 1", 
	"sum = 0\nfor (i=1..n) do\n   for (j=1..log(n)) do\n      for (k=1..log(i)) do\n         sum = sum + 1",
	"sum = 0\nfor (i=1..n) do\n   for (j=1..i/2) do\n      for (k=1..log(j)) do\n         sum = sum + 1",
	"sum = 0\nfor (i=1..n) do\n   for (j=1..n) do\n      if (i < j) then\n         sum = sum + j",
	"sum = 0\nfor (i=1..n) do\n   if (odd(i)) then\n      for (j=1..n) do\n            sum = sum + 1", 
	"sum = 0\nfor (i=1..n) do\n   for (j=1..n) do\n      if (i+j < n) then\n         for (k=1..i+j) do\n            sum = sum + k",
	"sum = 0\nfor (i=n..n^2) do\n   for (j=1..i) do\n      if (odd(i)) then\n         for (k=1..j) do\n            sum = sum + 1", 
	"sum = 0\nfor (i=0..100) do\n   for (j=0..i*i) do\n         for (k=0..j*j*j) do\n            sum = sum + 1",
	"Queue.put(t)  // link t is a pointer to binary tree\nrepeat\n   t = Queue.get(); // O(1) operation\n   if (t != null) then\n      visit(t) // O(1) operation\n      Queue.put(t->left) // O(1) operation\n      Queue.put(t->right)\nuntil (Queue.isEmpty());",
	"for (i=0..n-2) do\n   min = i\n   for (j=i+1..n-1) do\n      if (a[j] < a[min])\n         min = j\n   tmp = a[min]\n   a[min] = a[i]\n   a[i] = tmp",
	"a[0] = INT_MIN\nfor (i=2..n)\n   tmp = a[i]\n   j=i\n   while (a[j-1] > tmp) do\n      a[j] = a[j-1]\n      j = j - 1\n   a[j] = tmp",
	"left = 0\nright = n-1\nrepeat\n   probe = (left + right)/2\n   if (key < a[probe])\n      right = probe - 1\n   else\n      left = probe + 1\nuntil (key == a[probe] or (left > right))\nif (key == a[probe]) then\n   return probe\nelse\n   return -1" };	

    private String[] correctAnswers = { " O(n^3) ", " O(n^9) ", " O(n^3) ", " O(n*(log(n))^2) ", " O((n^2)*log(n)) ", " O(n^2) ", " O(n^2) ", " O(n^3) ", " O(n^6) ", " O(1) ", " O(n) ", " O(n^2) ", " O(n^2) ", " O(log(n)) " };

    public Analysis_Selection_IterativeCode() {
        super();
    }

    protected int getNumberOfVariations() {
        return codeFragments.length;
    }
    
    protected String[] getSelectionOptions() {
        return new String[] { " O(1) ", " O(log(n)) ", " O(n) ", " O(n*log(n)) ", " O(n*(log(n))^2) ", " O(n^2) ", " O((n^2)*log(n)) ", " O(n^3) ", " O(n^4) ", " O(n^6) ", " O(n^9) "  };
    }

    protected int getNumberOfItems() {
        return 3    ;
    }

    protected String getItem(int i) {
        return codeFragments[i];
    }

   
    public String getDescription() {
        String descr = "";
        int[] positions = randomPositions();
        for (int i=0; i < getNumberOfItems(); i++) {
            descr += i + ")\n" + getItem(positions[i]);
            if (i != getNumberOfItems()-1)
                descr += "\n\n";
            correctSelections.setObject(correctAnswers[positions[i]], i);
        }
        return descr;
    }

    static final long serialVersionUID = 8668078064348172238L;
}
