/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.*;

import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.util.*;

public class BST_TraverseLevel implements SimulationExerciseModel, StyledExercise, ModelAnswerNames, ConfigureVisualType, LayoutExercise {

  BinSearchTree bst = null;
  LinkedList l1;
  private String keys;
  public static final boolean DEBUG = false;

  private PseudoCode vCode = null;

  public static final String[] code = {
	  "_1 PROCEDURE TraverseLevel(TreeNode tnode)",
	  "    Queue queue = NEW Queue_/1",
	  "",
	  "_2 queue.put(tnode) _/2",
	  "_3 WHILE NOT queue.empty() _/3",
	  "_4     tnode := queue.get() _/4",
	  "_5     visit(tnode) _/5",
	  "_6     IF tnode.getLeft() != NULL _/6",
	  "_7         queue.put(tnode.getLeft()) _/7",
	  "_8     IF tnode.getRight() != NULL _/8",
	  "_9         queue.put(tnode.getRight()) _/9",
	  "",
	  "_10 RETURN _/10"
  };
  
  public BST_TraverseLevel() {
  }


    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public FDT[] init() {
        bst = new BinSearchTree();
        keys = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed),10);
            Table t = new Table(keys);
        bst.insert(t);
        l1 = new QueueImpl();
        FDT[] tmp = { l1, bst };
        return tmp;
    }

    public String[] getStructureVisualisations() {
        return new String[] { "list", "layered tree" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "list", "layered tree", "list", "pseudocode"  };
    }


  public String[] getStructureNames() {
    String[] tmp = { "levelorder", "Tree" };
    return tmp;
  }

  public String getDescription() {
    return "";
  }

  public FDT[] solve() {
    LinkedList sl1, last1;
    FDT[] fd = getInitialStructures();
    sl1 = (LinkedList)fd[0];//new LinkedListImpl();
    last1 = sl1;
    //    Table tbl = new Table(keys);
    BinaryTree bt = (BinaryTree)((BinSearchTree)fd[1]).getElement();//(BinaryTree) bst.getElement();
    StringTokenizer pre = new StringTokenizer(Traverse.traverseLevelOrder(bt));
    String tmp;
    Animator a = Animator.getActiveAnimator();
    while(pre.hasMoreTokens()) {
      a.startOperation();
      try {
        tmp = pre.nextToken();
        last1.setNext(last1.getNewNode(tmp));
        last1 = last1.getNext();
      }catch(NoSuchElementException e) {
        Note.err(this, "Error during solve, one StringTokenizer shorter than pre");
      }
      a.endOperation();
    }
    FDT[] ans = { sl1.getNext() };
    return ans;
  }

  private void setCodeLine(int line, Animator a) {
	  a.startOperation();
	  vCode.setLocation(line);
	  a.endOperation();
  }

    public FDT[] makeModelAnswer() {
    	FDT[] ini = getInitialStructures();
    	Object node;
    	QueueImpl visited, aux, vAux; // vAux is the visual aux
    	BinaryTree bt;
    	
    	bt = (BinaryTree) ((BinSearchTree) ini[1]).getElement();
    	visited = (QueueImpl) ini[0];
    	vAux = new QueueImpl();
    	aux = new QueueImpl();
    	if (vCode == null) {
    		vCode = new PseudoCode();
    		vCode.setCodeLines(code);
    		vCode.setCodeName("Level order traversal");
    	}
    	vCode.setLocation(1);
    	
    	// Simulation starts here
    	Animator a = Animator.getActiveAnimator();
    	setCodeLine(2, a);
    	
    	a.startOperation();
    	vAux.ADTEnqueue(bt.getElement().toString());
    	vCode.setLocation(3);
    	a.endOperation();
    	aux.ADTEnqueue(bt);
    	
    	while (!aux.ADTIsEmpty()) {
    		setCodeLine(4, a);
    		
    		a.startOperation();
    		node = vAux.ADTDequeue();
    		vCode.setLocation(5);
    		a.endOperation();
    		bt = (BinaryTree) aux.ADTDequeue();
    		
    		a.startOperation();
    		visited.insert(node.toString());
    		vCode.setLocation(6);
    		a.endOperation();
    		
    		if (bt.getLeft() != null) {
    			setCodeLine(7, a);
    			
    			a.startOperation();
    			vAux.ADTEnqueue(bt.getLeft().getElement().toString());
    			vCode.setLocation(8);
    			a.endOperation();
    			aux.ADTEnqueue(bt.getLeft());
    		} else {
    			setCodeLine(8, a);
    		}
    		
    		if (bt.getRight() != null) {
    			setCodeLine(9, a);
    			
    			a.startOperation();
    			vAux.ADTEnqueue(bt.getRight().getElement().toString());
    			vCode.setLocation(3);
    			a.endOperation();
    			aux.ADTEnqueue(bt.getRight());   			
    		} else {
    			setCodeLine(3, a);
    		}
    	}
    	setCodeLine(10, a);  	
    	// Simulation ends here
    	
    	return new FDT[] { visited, ini[1], vAux, vCode };
    }


  public FDT[] getInitialStructures() {
    Table tbl = new Table(keys);
    BinSearchTree tr = new BinSearchTree();
    tr.insert(tbl);
    return new FDT[] { new QueueImpl(), tr };
  }

  public FDT[] getSimulatedStructures() {
    return new FDT[] { l1, bst };
  }

  public FDT[] getAnswer() {
    FDT[] tmp = { l1 };
    return tmp;
  }

  public String[] getModelAnswerNames() {
    String[] s = { "List of Visited Nodes", "Binary Search Tree", "Queued nodes", "Pseudo Code" };
    return s;
  }

  public SimulationExerciseModel getModelAnswer() {
    return null;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tre = new VisualTypeConf();
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    VisualTypeConf dummy = new VisualTypeConf();
    return new VisualTypeConf[] { lis, tre, dummy, dummy };
  }

  static final long serialVersionUID = -7118807974129115178L;

  public GridBagConstraints[] getConstraints() {
	Insets i = new Insets(10, 10, 10, 10);
	GridBagConstraints r[] = new GridBagConstraints[2];
	GridBagConstraints c;
	// List
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = GridBagConstraints.REMAINDER;
	r[0] = c;
	// Tree
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	r[1] = c;
	
	return r;
  }

  public GridBagConstraints[] getModelAnswerConstraints() {
	Insets i = new Insets(10, 10, 10, 10);
	GridBagConstraints r[] = new GridBagConstraints[4];
	GridBagConstraints c;
	
	// List
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	r[0] = c;
	// Tree
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 0;
	c.gridy = 0;
	c.gridheight = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.PAGE_START;
	r[1] = c;
	// Queue
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = GridBagConstraints.REMAINDER;
	r[2] = c;
	// Code
	c = new GridBagConstraints();
	c.insets = i;
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = GridBagConstraints.REMAINDER;
	r[2] = c;

	return r;
  }

}
