/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.*;

import content.exercises.structures.ActivationRecord;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.CDT.probe.Stack;
import matrix.structures.FDT.*;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.util.*;

public class BST_TraversePost implements SimulationExerciseModel, StyledExercise,
				ModelAnswerNames, ConfigureVisualType, LayoutExercise {

  BinSearchTree bst = null;
  LinkedList l1;
  private String keys;
  public static final boolean DEBUG = false;
 
  private PseudoCode vCode = null;
  private Stack callStack;

  public static final int      frameCols = 1;
  public static final String[] frameLabels = { "tnode" };
  public static final String[] code = {
		"_1 1 PROCEDURE TraversePostorder(TreeNode tnode) _/1",
		"",
		"_2 2 IF tnode != NULL _/2",
		"_3 3     TraversePostorder(tnode.getLeft()) _/3",
		"_4 4     TraversePostorder(tnode.getRight()) _/4",
		"_5 5     visit(tnode) _/5",
		"",
		"_6 6 RETURN _/6"
  };
  
  public BST_TraversePost() {
  }


    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {
        bst = new BinSearchTree();
        keys = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed), 10);
        Table t = new Table(keys);
        bst.insert(t);
        l1 = new QueueImpl();
        FDT[] tmp = { l1, bst };
        return tmp;
    }

    public String[] getStructureVisualisations() {
        return new String[] { "list", "layered tree" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "list", "layered tree", "array", "pseudocode"  };
    }


  public String[] getStructureNames() {
    String[] tmp = { "postorder", "Tree" };
    return tmp;
  }

  public String getDescription() {
    return "";
  }

  public FDT[] solve() {
    LinkedList sl1, last1;
    FDT[] fd = getInitialStructures();
    sl1 = (LinkedList)fd[0];//new LinkedListImpl();
    last1 = sl1;
    //    Table tbl = new Table(keys);
    BinaryTree bt = (BinaryTree)((BinSearchTree)fd[1]).getElement();//(BinaryTree) bst.getElement();
    StringTokenizer pre = new StringTokenizer(Traverse.traversePostOrder(bt));
    String tmp;
    Animator a = Animator.getActiveAnimator();
    while(pre.hasMoreTokens()) {
      a.startOperation();
      try {
        tmp = pre.nextToken();
        last1.setNext(last1.getNewNode(tmp));
        last1 = last1.getNext();
      }catch(NoSuchElementException e) {
        Note.err(this, "Error during solve, one StringTokenizer shorter than pre");
      }
      a.endOperation();
    }
    FDT[] ans = { sl1.getNext() };
    return ans;
  }


    public FDT[] makeModelAnswer() {
		FDT[] ini = getInitialStructures();
		QueueImpl lst;
		BinaryTree bt;

		bt = (BinaryTree) ((BinSearchTree) ini[1]).getElement();
		lst = (QueueImpl) ini[0];
		callStack = new Stack(8);
		if (vCode == null) {
			vCode = new PseudoCode();
			vCode.setCodeLines(code);
			vCode.setCodeName("Postorder Traversal");
		}
		vCode.setLocation(0);
		prettyTraversal(bt, lst, -1);
		Animator.getActiveAnimator().startOperation();
		callStack.pop(null);
		vCode.setLocation(0);
		Animator.getActiveAnimator().endOperation();
		return new FDT[] { lst, bt, callStack, vCode };
    }



  public FDT[] getInitialStructures() {
    Table tbl = new Table(keys);
    BinSearchTree tr = new BinSearchTree();
    tr.insert(tbl);
    return new FDT[] { new QueueImpl(), tr };
  }

  public FDT[] getSimulatedStructures() {
    return new FDT[] { l1, bst };
  }

  public FDT[] getAnswer() {
    FDT[] tmp = { l1 };
    return tmp;
  }

  public String[] getModelAnswerNames() {
	String[] s = { "List of Visited Nodes", "Binary Search Tree", "Call Stack", "Pseudo Code" };
	return s;
  }

  private void setCodeLine(int line, Animator a) {
	a.startOperation();
	vCode.setLocation(line);
	a.endOperation();
  }

  private void pushStackFrame(BinaryTree node, int rline) {
	String value = "null";
	if (node != null && node.getElement() != null)
		value = node.getElement().toString();
	String[] values = { value };
	callStack.insert(new ActivationRecord(frameLabels, values, rline,
			frameCols));
  }

  private void prettyTraversal(BinaryTree t, QueueImpl l, int rline) {
	Animator a = Animator.getActiveAnimator();

	a.startOperation();
	pushStackFrame(t, rline);
	vCode.setLocation(1);
	a.endOperation();

	setCodeLine(2, a);
	if (t != null) {
		setCodeLine(3, a);
		prettyTraversal(t.getLeft(), l, 4);
		a.startOperation();
		callStack.pop(null);
		vCode.setLocation(4);
		a.endOperation();

		prettyTraversal(t.getRight(), l, 5);
		a.startOperation();
		callStack.pop(null);
		vCode.setLocation(5);
		a.endOperation();

		a.startOperation();
		l.insert(t.getElement().toString());
		vCode.setLocation(6);
		a.endOperation();
	}
	setCodeLine(6, a);
  }

  public SimulationExerciseModel getModelAnswer() {
    return null;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tre = new VisualTypeConf();
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    VisualTypeConf sta = new VisualTypeConf();
    sta.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.ROTATED, "true");
    VisualTypeConf cod = new VisualTypeConf();
    return new VisualTypeConf[] { lis, tre, sta, cod };
  }

  public GridBagConstraints[] getModelAnswerConstraints() {
	Insets i = new Insets(10, 10, 10, 10);
	GridBagConstraints r[] = new GridBagConstraints[4];
	GridBagConstraints c;

	// List
	c = new GridBagConstraints();
	c.gridx = 1;
	c.gridy = 0;
	c.insets = i;
	r[0] = c;
	// Tree
	c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.insets = i;
	c.gridheight = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.PAGE_START;
	r[1] = c;
	// Stack
	c = new GridBagConstraints();
	c.gridx = 2;
	c.gridy = 0;
	c.insets = i;
	c.gridheight = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.PAGE_START;
	r[2] = c;
	// Code
	c = new GridBagConstraints();
	c.gridx = 1;
	c.gridy = 1;
	c.insets = i;
	r[3] = c;

	return r;
  }

  public GridBagConstraints[] getConstraints() {
		Insets i = new Insets(10, 10, 10, 10);
		GridBagConstraints r[] = new GridBagConstraints[2];
		GridBagConstraints c;
		
		// List
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.gridy = 1;
		c.insets = i;
		r[0] = c;
		// Tree
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = i;
		r[1] = c;

		return r;
  }

  static final long serialVersionUID = -6673336618844185500L;
}
