/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.exercises.structures.ExerStackList;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.BinaryTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.util.RandomKey;
import matrix.util.Note;

public class CommonTreePreorderWithStack implements SimulationExerciseModel, StyledExercise, ModelAnswerNames, ConfigureVisualType {
    private CommonTreeImpl tree; 
    private matrix.structures.CDT.probe.Stack st;
    private ExerStackList l1;
    
    private String keys;
    
    long seed = 1;
    
    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    private CommonTreeImpl turnBST2CT(BinaryTree bt) {
	CommonTreeImpl ct = new CommonTreeImpl(bt.getElement());
	if (bt.getLeft() != null)
	    ct.setFirstChild(turnBST2CT(bt.getLeft()));
	if (bt.getRight() != null)
	    ct.setNextSibling(turnBST2CT(bt.getRight()));
	return ct;
    }

    private CommonTreeImpl createNewTree(String keys) {
        BinSearchTree bst = new BinSearchTree();
	Table t = new Table(keys);
        bst.insert(t);
	
	CommonTreeImpl root = new CommonTreeImpl(new Key("Z"));
	root.setFirstChild(turnBST2CT((BinaryTree)bst.getElement()));
	return root;
    }

    public FDT[] init() {
        tree = new CommonTreeImpl();
        keys = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed), 10);
        tree = createNewTree(keys);
	st = new matrix.structures.CDT.probe.Stack();
        l1 = new ExerStackList(st);
        FDT[] tmp = { tree, st, l1 };

        return tmp;
    }

    public String[] getStructureVisualisations() {
        return new String[] { "layered tree", "array", "list" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "array"  };
    }

    public String[] getStructureNames() {
	String[] tmp = { "Common Tree", "Stack", "Traversing Order" };
	return tmp;
    }

    public String[] getModelAnswerNames() {
	String[] s = { "Common Tree" };
	return s;
    }

    public String getDescription() {
	return "";
    }

    public FDT[] solve() {
	Note.out(this, "solve");
	// pseudo stack for the exercise (we simulate teh same stack as the learner does)
	matrix.structures.CDT.probe.Stack resStack = new matrix.structures.CDT.probe.Stack();
	// real stack in order to do the traversal
	java.util.Stack helpStack = new java.util.Stack();

	FDT[] fd = getInitialStructures();
    	CommonTreeImpl t = (CommonTreeImpl)fd[0];
	CommonTreeImpl next, firstChild, nextSibling;

	Note.out(this, "t="+t);

	Animator a = Animator.getActiveAnimator();
	
	a.startOperation();	// put the root on the stack
	helpStack.push(t);
	resStack.insert(""+t.getElement());
	a.endOperation();

	while(!helpStack.empty()) {

	    Note.out(this, "stack not empty");
	
	    a.startOperation();	// pop
	    resStack.pop(null);
	    next = (CommonTreeImpl)helpStack.pop();
	    a.endOperation();

	    if (next == null) break;

	    if (next.getNextSibling() != null){

		a.startOperation();	// push
		nextSibling = next.getNextSibling();
		helpStack.push(nextSibling);
		resStack.insert(""+nextSibling.getElement());
		a.endOperation();

	    }

	    if (next.getFirstChild() != null) {

		a.startOperation(); // push
		firstChild=next.getFirstChild();
		helpStack.push(firstChild);
		resStack.insert(""+firstChild.getElement());
		a.endOperation();

	    }

	}
	FDT[] ans = { resStack }; // simulated stack
	return ans;
    }

    public FDT[] makeModelAnswer() {
	return solve();
    }

    public FDT[] getInitialStructures() {
	CommonTreeImpl t = new CommonTreeImpl();
	t = createNewTree(keys);
	return new FDT[] { t, new QueueImpl() };
    }

    public FDT[] getSimulatedStructures() {
	return new FDT[] { tree, st, l1 };
    }

    public FDT[] getAnswer() {
	FDT[] tmp = { st };
	return tmp;
    }

    /*
    public SimulationExerciseModel getModelAnswer() {
	return null;
    }
    */

    public VisualTypeConf[] conf() {
	VisualTypeConf tre = new VisualTypeConf();
	tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
	tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
	VisualTypeConf sta = new VisualTypeConf();
	sta.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
	sta.enable("matrix.visual.VisualArray", VisualTypeConf.HIGHLIGHT_OPERATION);
	sta.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
	sta.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
	VisualTypeConf lis = new VisualTypeConf();
	lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
	lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
	return new VisualTypeConf[] { tre, sta, lis};
    }

    static final long serialVersionUID = -8150132687124584889L;
}
