/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2006 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises;

import java.util.Arrays;
import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import content.exercises.structures.AuxiliaryTableCounting;
import content.exercises.structures.TrueFalseKey;
import content.interfaces.*;

public class DynamicProgramming_SelectEvents implements
        SimulationExerciseModel, StyledExercise, ModelAnswerNames,
        ConfigureVisualType, JudgeBlocks {

    private long seed = 1;

    private static int size = 5;

    private AuxiliaryTableCounting resultTable;

    private Table beginTable, endTable, profitTable;

    private Table selectTable;

    private int[] begin;

    private int[] end;

    private int[] profit;

    public DynamicProgramming_SelectEvents() {
        super();
    }

    public FDT[] solveThis() {

        FDT[] structures = getInitialStructures();
        Table alku = (Table) structures[0];
        Table loppu = (Table) structures[1];
        AuxiliaryTableCounting tuotto = (AuxiliaryTableCounting) structures[2];
        AuxiliaryTableCounting tulos = (AuxiliaryTableCounting) structures[3];
        Table varaus = (Table) structures[4];

        Animator an = Animator.getActiveAnimator();

        an.startOperation();
        tulos.setObject(tuotto.getObject(size - 1), size - 1);
        varaus.setObject(new TrueFalseKey(true), size - 1); // should be set to true
        an.endOperation();

        for (int i = size - 2; i >= 0; i--) {
            // j := aikasin tapahtuma, jolle Alku[j] >= Loppu[i] tai j := n+1
            int j = event(alku, loppu, i, size);
            if (peekTable(tuotto, i) + peekTable(tulos, j) >= peekTable(tulos, i + 1)) {
                an.startOperation();
                varaus.setObject(new TrueFalseKey(true), i);
                tulos.setObject(tuotto.getObject(i), i);
                tulos.setObject(tulos.getObject(j), i);
                an.endOperation();
            } else {
                an.startOperation();
                varaus.setObject(new TrueFalseKey(false), i);
                tulos.setObject(tulos.getObject(i + 1), i);
                an.endOperation();
            }
        }
        clear(alku, loppu, varaus);
        return new FDT[] { tulos, varaus };
    }

    private void clear(Table alku, Table loppu, Table varaus) {
        Animator animator = Animator.getActiveAnimator();
        for (int i = 0; i < size; i++) {
            if (((TrueFalseKey) varaus.getObject(i)).isSelected()) {
                for (int j = i + 1; j < size; j++) {
                    if (overlap(i, j, alku, loppu)) {
                        if (((TrueFalseKey) varaus.getObject(j)).isSelected()) {
                            animator.startOperation();
                            varaus.setObject(new TrueFalseKey(false), j);
                            animator.endOperation();
                        }
                    } else 
                        break;
                }
            }
        }
        
    }

    private boolean overlap(int i, int j, Table alku, Table loppu) {
        return peekTable(loppu, i) > peekTable(alku, j);
    }

    private int event(Table alku, Table loppu, int i, int size) {
        for (int j = i; j < size; j++) {
            if (peekTable(alku, j) >= peekTable(loppu, i))
                return j;
        }
        return size;
    }

    private int add(Object add1, Object add2) {
        return ((Integer) add1).intValue() + ((Integer) add2).intValue();
    }

    private int peekTable(Table t, int index) {
        return Integer.parseInt(t.getObject(index).toString());
    }
    
    public FDT[] solve() {
        FDT[] structures = solveThis();
        return new FDT[] { structures[0], structures[1] };
    }

    public FDT[] makeModelAnswer() {

        return solveThis();

    }

    public String[] getStructureVisualisations() {

        return new String[] { "array", "array", "array", "array", "array" };

    }

    public String[] getModelAnswerVisualisations() {

        return new String[] { "array", "array" };

    }

    public String[] getModelAnswerNames() {

        return new String[] { "Result", "Select" };

    }

    public VisualTypeConf[] conf() {

        VisualTypeConf nothingConf = new VisualTypeConf();

        VisualTypeConf profitConf = new VisualTypeConf();
        profitConf.enable("matrix.visual.VisualKey",
                          VisualTypeConf.DRAG_OPERATION);
        profitConf.enable("matrix.visual.VisualKey",
                          VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf resultConf = new VisualTypeConf();
        resultConf.enable("matrix.visual.VisualArrayComponent",
                          VisualTypeConf.DROP_OPERATION);
        resultConf.enable("matrix.visual.VisualArrayComponent",
                          VisualTypeConf.HIGHLIGHT_OPERATION);
        resultConf.enable("matrix.visual.VisualKey",
                          VisualTypeConf.DRAG_OPERATION);
        resultConf.enable("matrix.visual.VisualKey",
                          VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf selectConf = new VisualTypeConf();
        selectConf.enable("matrix.visual.VisualKey",
                          VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { nothingConf, nothingConf, profitConf,
                resultConf, selectConf };

    }

    public FDT[] init() {
        Random rand = new Random(getSeed());
        begin = matrix.util.RandomKey.createIntKeys(rand, 8, 18, size);
        Arrays.sort(begin);
        end = matrix.util.RandomKey.createIntKeys(rand, 1, 6, size);
        for (int i = 0; i < begin.length; i++) {
            end[i] += begin[i];
        }
        profit = matrix.util.RandomKey.createIntKeys(rand, 1, 99, size);

        beginTable = new Table(begin);
        endTable = new Table(end);
        profitTable = new AuxiliaryTableCounting(profit);
        selectTable = new Table(makeTrueFalseKeys(size));
        resultTable = new AuxiliaryTableCounting(new int[size + 1]);
        return new FDT[] { beginTable, endTable, profitTable, 
                resultTable, selectTable };

    }

    private TrueFalseKey[] makeTrueFalseKeys(int num) {
        TrueFalseKey[] vKeys = new TrueFalseKey[num];
        for (int i = 0; i < num; i++) {
            vKeys[i] = new TrueFalseKey(false);
        } 
        return vKeys;
    }

    public void setSeed(long v) {

        this.seed = v;
    }

    public long getSeed() {

        return seed;
    }

    public String[] getStructureNames() {

        return new String[] { "Begin", "End", "Profit", "Result", "Select" };

    }

    public FDT[] getAnswer() {

        return new FDT[] {resultTable, selectTable};
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(begin), new Table(end), new AuxiliaryTableCounting(profit),
                new AuxiliaryTableCounting(new int[size + 1]),
                new Table(makeTrueFalseKeys(size)) };
    }

    public String getDescription() {

        return "";
    }

}
