package content.exercises;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.util.RandomKey;
import content.exercises.structures.ExerHeap;
import content.exercises.structures.HeapSortHeap;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

public class HeapSort implements SimulationExerciseModel, ConfigureVisualType, StyledExercise, ModelAnswerNames, SwapBehaviour {

	private HeapSortHeap heap, modelHeap;
	private long seed;
	private static int length = 16;
	private static int max = 80;
	private int[] keys;
	
	public FDT[] solve() {
		modelHeap = new HeapSortHeap(length);
		for(int i = 0; i < keys.length; i++) {
			insertKey(keys[i], modelHeap);
		}
		Animator a = Animator.getActiveAnimator();
		for(int i = 0; i < keys.length-1; i++) {
			a.startOperation();
			Object o = modelHeap.getObject(0);
			modelHeap.deleteMin();
			modelHeap.setObject(o, keys.length-(i+1));
			a.endOperation();
		}
		return new FDT[] {modelHeap};
	}

	public FDT[] makeModelAnswer() {
		FDT[] f = solve();
		return new FDT[] {f[0], f[0]};
	}

	/** 
	 * Correct type of key created in order to make this look like max-heap.
	 * @param i
	 * @param h
	 */
	private void insertKey(int i, ExerHeap h) {
		int comp = max-i;
		String cs;
		if(comp < 10) {
			cs = "" + 0 + "" + comp;
		} else {
			cs = "" + comp;
		}
		Key k = new Key(cs);
		k.setVisualizationString(""+i);
		h.insert(k);		
	}
	
	public FDT[] init() {
		keys = RandomKey.createIntKeys(new java.util.Random(seed), 1, max, length);
		heap = new HeapSortHeap(length);
		for(int i = 0; i < keys.length;i++) {
			insertKey(keys[i], heap);
		}
		return new FDT[] {heap, heap };
	}

	public void setSeed(long seed) {
		this.seed = seed;
	}

	public long getSeed() {
		return seed;
	}

	public String[] getStructureNames() {
		return new String[] {"array view", "tree view" };
	}

	public FDT[] getAnswer() {
		return new FDT[] {heap};
	}

	public FDT[] getInitialStructures() {
		ExerHeap h = new HeapSortHeap(length);
		return new FDT[] {h,h};
	}

	public String getDescription() {
		return "";
	}

	public VisualTypeConf[] conf() {
		VisualTypeConf array = new VisualTypeConf();
        array.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        array.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        array.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf tree = new VisualTypeConf();
        tree.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tree.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tree.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        return new VisualTypeConf[] { array, tree };
	}

	public String[] getStructureVisualisations() {
		return new String[] {"array", "layered tree"};
	}

	public String[] getModelAnswerVisualisations() {
		return getStructureVisualisations();
	}

	public String[] getModelAnswerNames() {
		return getStructureNames();
	}

	public boolean getSwapBehaviour() {
		return true;
	}

}
