/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Heap_BuildHeap.java,v 1.41 2007/02/06 13:23:24 oseppala Exp $ */

package content.exercises;

import java.util.Random;

import content.interfaces.KnownMisconceptions;

import content.interfaces.ConfigureVisualType;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;
import content.interfaces.StateLegality;

import content.exercises.structures.*;

import matrix.animation.AnimatedStructures;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.util.MatrixComparable;
import matrix.util.*;

public class Heap_BuildHeap implements  SimulationExerciseModel, ModelAnswerNames,  
                                        StyledExercise, ConfigureVisualType,
                                        KnownMisconceptions, SwapBehaviour,
StateLegality, JudgeBlocks{
    ExerHeap bh,bh2;
    //String dataString;
    int[] dataString;
    String marker = "Heap_BuildHeap";
    Object [] objects;
    public static final boolean DEBUG = false;

    public Heap_BuildHeap() {
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    
    //--------EXERCISE INSTANCE VERIFICATION CODE BEGINS--------//
    private transient int recursiveSwaps;
    private transient int partlyRecursiveSwaps;
    private transient int leftSwaps;
    private transient int rightSwaps;
    private transient int swaps;    
        
    public boolean heapify(Key[] heap, int i){
        int smaller = i*2;
        if (heap[(i*2+1)-1].leq(heap[(i*2)-1]))
            smaller++;
        
        if (heap[i-1].gt(heap[smaller-1])){
            Key temp      = heap[i-1];
            heap[i-1]       = heap[smaller-1];
            heap[smaller-1] = temp;
            
            if (smaller == i*2)
                leftSwaps++;                //Swap was made with the left child
            else
                rightSwaps++;               //Swap was made with the right child
            swaps++;
            
            if (smaller <= heap.length/2){
               if (heapify(heap, smaller))   
                   recursiveSwaps++;       //recursive swap
               else
                   partlyRecursiveSwaps++; //swap ends before reaching bottom of the heap
            }
            return true; //swapped
        }
        return false; // Did not swap
    }
    
    //this method verifies that certain algorithm branches must be taken for
    //a certain input to be accepted as an exercise input
    
    public boolean accept(Object[] keys){
        recursiveSwaps  = 0;
        partlyRecursiveSwaps = 0;
        leftSwaps       = 0;
        rightSwaps      = 0;
        swaps           = 0;
        
        Key[] copies = new Key[keys.length];
        
        for (int i=0; i<keys.length; i++){ // Copy to preserve originals
            copies[i] = (Key)keys[i];
        }
                
        for (int i=keys.length/2; i>0; i--){ //Solve and find out acceptability
            heapify(copies, i);
        }
                
        return recursiveSwaps > 0 && leftSwaps>0 && rightSwaps>0 && swaps > 3 && partlyRecursiveSwaps > 0;
    }
    //--------EXERCISE INSTANCE VERIFICATION CODE ENDS--------//

    
    
    public FDT[] init() {

        Random r = new java.util.Random(seed);
        do{
          //dataString = RandomKey.createNoDuplicateUppercaseRandomKey(r,15);
          dataString = RandomKey.createIntKeys(r, 10, 99, 15); 
          objects = new Key[dataString.length];
          for (int i = 0; i < dataString.length; i++)
            objects[i] = new Key(""+dataString[i]);
            // objects[i] = new Key(""+dataString.charAt(i));

        }while (!accept(objects));
          
        bh = new ExerHeap(objects);
        FDT[]  s = { bh, bh };
        return s;
    }


    // styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "array" , "leaf tree"};
        return s;
    }


    public String[] getModelAnswerVisualisations() {
        String[] s = { "leaf tree" };
        return s;
    }


    public FDT[] init(Object o) {
        return init();
    }


    public String[] getStructureNames() {
        String[] s = { "Array Representation of Binary Heap", "Binary Heap" };
        return s;
    }

    public String getDescription() {
        return "";
    }

    public FDT[] solve() {

        bh2 = (ExerHeap)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=objects.length/2; i>=0; i--) {
            a.startOperation();
            bh2.minHeapify(i);
            //bh2.insert(t2.getObject(i));
            a.endOperation();
            }
        return new FDT[] { bh2 };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }



    public FDT[] getInitialStructures() {
        ExerHeap tmp = new ExerHeap(objects);
        return new FDT[] { tmp };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { bh };
    }

    public FDT[] getAnswer() {
        FDT[] s = { bh };
        return s;
    }

    public String[] getModelAnswerNames() {
        String[] s = { "Binary Heap" };
        return s;
    }


    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        return new VisualTypeConf[] { tabl, tre };
    }


    //-- Helper method which checks if the heap given is
    //   a legal heap in the end of the sequence. Returns
    //   the animator into its original state.
    
    public boolean legalEndState(Animator a, FDT[] f){
       Animator oldAnim = Animator.getActiveAnimator();
       Animator.setActiveAnimator(a);
       
       int forwardStates = 0;
       
       while (a.hasNextOperation()){
           a.redo();
           forwardStates++;
       }
       
       boolean result = false;
       if (f[0] instanceof ExerHeap) // Backwards compatible - Old exerheap does not have this method
           result = ((ExerHeap)f[0]).isMinHeap();
       
       for (;forwardStates > 0;forwardStates--)
           a.undo();
       
       Animator.setActiveAnimator(oldAnim);
       return result;
    }
    
    public boolean legalState(FDT[] structures){
        boolean result = false;
        if (structures[0] instanceof ExerHeap)
            result = ((ExerHeap)structures[0]).isMinHeap();
        
        return result;
    }
    
    
    /* KnownMisconceptions interface */
    
    public matrix.animation.AnimatedStructures[] createMisconceptionSequences(){
    	AnimatedStructures[] sequences = new AnimatedStructures[]{
            
                //Error combinations
                // Recursion omitted, Two Swaps LR or RL, Two Swaps but only RL, Levelwise fixing of recursion 
            
                makeMisconception(true,  false, false, false), //Norec
                makeMisconception(false, true,  false, false), //Swap both
                makeMisconception(true,  true,  false, false), //Norec+swapBoth
                makeMisconception(false, true,  true , false), //BothRL
                makeMisconception(true,  true,  true , false), //BothRLNorec

                makeMisconception(true,  false, false, true),  //NorecFixLvl
                makeMisconception(true,  true,  true , true),  //RLNorecFixLvl

                // Mirror variant
                
                leftToRight(),
                
                // Instantly swapping keys up 
                
                percolateUp(true),
                
                // Iterative fix of recursion
                
                percolateUp(false),

                // Starting the iteration from top
                
                topDown(),
                
                //Max-heap instead of min-heap
                maxHeap(),

                // Skipping a swap
                
                skip(1),
                skip(2),
                skip(3),
                skip(4),
                skip(5),
                skip(6),
                skip(7),
                skip(8),
                skip(9),
                skip(10),
                skip(11)
        };
        
        return sequences;
      }

      public String[] getMisconceptionNicknames(){
        return new String[]{
                "NoRec",        // percolateDown does not continue recursion
                "Swaps Both",   // Swapping is performed with the bigger element first and then the smaller
                "NoRec+Both",   // Both the two symptoms above
                "BothRL",       // Two swaps occur if the right child is smaller than its father
                "BothRLNoRec",  // the symptom above and no recursion
                
                "NoRecFixLvl",
                "RLNrecFixLvl",
                
                "LToRight",
                "PercUp",
                "IterFix",
                "TopDown",
                "MaxHeap",
                
                "SKIP 1",
                "SKIP 2",
                "SKIP 3",
                "SKIP 4",
                "SKIP 5",
                "SKIP 6",
                "SKIP 7",
                "SKIP 8",
                "SKIP 9",
                "SKIP 10",
                "SKIP 11"

                
        };      
      }
      
      private static final int[][] baseCases = {
          {-1},
          {-1},
          {0,1}, //norec + both based on norec and both
          {-1},
          {0,3}, //BothRLNorec based on bothrl and norec
          {0},
          {0,3,4,5}, //RLNORecFixLvl based on NoRec, BothRL, NorecBothRL and NoRecFixLEvel
          {-1}, // lefttoright
          {-1}, // percup
          {0},  // iterfix based on norec
          {-1}, // topdown
          {-1}, // maxheap
        {-1},{-1},{-1},{-1},{-1},{-1},{-1},{-1},{-1},{-1},{-1}
      };
      
      public int[][] getBaseCases(){
          return baseCases;
      }
      
      public String[] getGuidanceStrings(){
        return new String[]{
                ("<b>Rekursio puuttuu</b><br> Aina kun <em>oikeassa algoritmissa</em> vaihdetaan kaksi alkiota keskenn (swap), alaspin siirtynyt alkio voi tarvita rekursiivista siirtmist "+
                "yh alemmas. Ratkaisustasi nm rekursiiviset askeleet puuttuvat kokonaan. </font><hr><font face=\"Arial, Helvetica\">"+
                "<b>Recursion missing</b><br> A swap made in the heap might cause a heap property violation further down the tree. "+
                "Your solution sequence is missing one or more (recursive) swaps that correct the property violation."),

                ("<b>Ylimrisi vaihtoja</b><br> <em>Oikeassa algoritmissa</em> alkio tulee tarvittaessa vaihtaa pienemmn lapsensa kanssa. "+
                "Kun molemmat lapset ovat pienempi kuin niiden is, <em>sinun ratkaisussasi</em> is vaihdetaan joskus molempien lasten kanssa.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Extra swaps</b><br> <em>In the correct algorithm</em> the contents of a node should be swapped with the smaller of the two children if necessary. "+
                "When both of the children are smaller than their father, <em>in your solution</em> the father is swapped with both children."),

                ("<b>Ylimrisi vaihtoja</b> ja <b>Rekursio puuttuu</b><br> <em>Oikeassa algoritmissa</em> alkio tulee tarvittaessa vaihtaa pienemmn lapsensa kanssa. "+
                "Alaspin siirtynyt alkio voi mys tarvita rekursiivista siirtmist yh alemmas. "+
                "Kun molemmat lapset ovat pienempi kuin niiden is, <em>sinun ratkaisussasi</em> is vaihdetaan joskus molempien lasten kanssa. Lisksi "+
                "ratkaisustasi puuttuvat edellmainitut rekursiiviset askeleet. </font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Extra swaps</b> and <b>Recursion missing</b><br> You now swap first the bigger element with the father node and then do this again with the smaller child node. "+
                "You should however only swap the smaller element with the father if it is necessary. You also don't perform any recursive steps."),

                ("<b>Ylimrisi vaihtoja</b><br> <em>Oikeassa algoritmissa</em> alkio tulee tarvittaessa vaihtaa <b>pienemmn</b> lapsensa kanssa. "+
                "<em>Sinun ratkaisussasi</em> vertaat ensin vasenta lasta isn ja vaihdat tarvittaessa, ja sitten teet saman oikealle lapselle. Tllin is vaihdetaan joskus molempien lasten kanssa.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Extra swaps</b><br> You first compare the right child with the father node and swap if necessary. "+
                "Then you compare the left child and swap if necessary. What you should do however is to perform the swap only with the "+
                "smaller of the two children if it is necessary."),

                ("<b>Ylimrisi siirtoja</b> ja <b>Rekursio puuttuu</b><br> <em>Oikeassa algoritmissa</em> alkio tulee tarvittaessa vaihtaa <b>pienemmn</b> lapsensa kanssa ja tarvittaessa jatkaa siirtmist rekursiivsesti alaspin "+
                "<em>Sinun ratkaisussasi</em> vertaat ensin vasenta lasta isn ja vaihdat tarvittaessa, ja sitten teet saman oikealle lapselle. Tllin is vaihdetaan joskus molempien lasten kanssa. Lisksi ratkaisustasi puuttuvat rekursiiviset askeleet.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Extra swaps</b> and <b>Recursion missing</b><br> You first compare the right child with the father node and swap if necessary. "+
                "Then you compare the left child and swap if necessary. What you should do however is to perform the swap only with the "+
                "smaller of children if it was necessary. You should also repeat this if the swap creates a heap property violation futher down the tree."),

                ("<b>\"Rekursiota\" taso kerrallaan</b><br> Aina kun <em>oikeassa algoritmissa</em> vaihdetaan kaksi alkiota keskenn (swap), alaspin siirtynyt alkio voi tarvita rekursiivista siirtmist "+
                "yh alemmas. Ratkaisussasi nm askeleet tehdn jlkikteen - vasta kun koko keon \"taso\" on kyty lpi.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>\"Recursion\" one level at a time</b><br> A swap made in the heap should be dealt with instantly when it happens. You currently perform the "+
                "correcting steps later, probably going to the end of a level in the tree, until starting to perform the "+
                "correcting steps."),

                ("<b>\"Rekursiota\" taso kerrallaan</b> ja <b>Ylimrisi vaihtoja</b><br> Aina kun <em>oikeassa algoritmissa</em> vaihdetaan kaksi alkiota keskenn (swap), alaspin siirtynyt alkio voi tarvita rekursiivista siirtmist "+
                "yh alemmas. Ratkaisussasi nm askeleet tehdn jlkikteen - vasta kun koko keon \"taso\" on kyty lpi. Lisksi <em>Oikeassa algoritmissa</em> solmu vaihdetaan vain pienemmn lapsensa kanssa. <em>Sinun ratkaisussasi</em> "+
                "saatat ensin vaihtaa oikeanpuoleisen lapsen issolmun kanssa ja sen jlkeen viel isn vasemmanpuoleisen kanssa.</font><hr><font face=\"Arial, Helvetica\">"+               
                "<b>\"Recursion\" one level at a time</b> and <b>Extra swaps</b><br>You now compare each node to its father and swap if necessary until you reach the end of that level in the tree. "+
                "The possible heap violations are the corrected by going through the lower tree level again."),

                ("<b>Peilikuva</b><br> <em>Oikeassa algoritmissa</em> kekoa kydn lpi oikealta vasemmalle. <em>Sinun ratkaisussasi</em> kekoa "+
                "ksitelln vasemmalta oikealle.</font><hr><font face=\"Arial, Helvetica\">"+
/* LR */        "<b>Mirror Image</b><br> You seem to work on the structure from left to right. The correct algorithm works from right to left."),

                ("<b>\"Suoraan huipulle\"</b><br> <em>Sinun ratkaisussasi</em> etsitn ensin keon pienin solmu ja nostetaan se suoraan keon huipulle. Seuraavaksi pienin tulee tmn lapseksi jne. <b>Tm on todella kaukana oikeasta algoritmista</b> - lue tehtvn ohjeita ja kurssimateriaalia.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Straight to the top</b><br>You start from the top, swapping the smallest element to the top, and continue swapping the smallest elements to "+
                "the root of their respective subtree. This is very far from the actual algorithm."),


                ("<b>Korjaus seuraavalla kierroksella</b><br> Aina kun <em>oikeassa algoritmissa</em> vaihdetaan kaksi alkiota keskenn (swap), alaspin siirtynyt alkio voi tarvita vlitnt rekursiivista siirtmist "+
                "yh alemmas. Ratkaisussasi nm rekursiiviset askeleet tehdn jlkikteen - vasta kun koko keko on kyty lpi.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>I'll fix it on the next round</b><br>A swap made in the heap should be dealt with instantly when it happens. You currently perform the "+
                "correcting steps later, probably processing the whole tree, until starting to perform the "+
                "correcting steps."),

                ("<b>Aloitus huipulta</b><br> <em>Oikeassa algoritmissa</em> suuret alkiot painuvat rekursiivisesti puussa alaspin. Oikea algoritmi kuitenkin aloittaa keon pohjalta. <em>Sinun ratkaisusi</em> "+
                "siirt alkioita oikealla tavalla, mutta aloittaa tyn keon huipulta.</font><hr><font face=\"Arial, Helvetica\">"+                
                "<b>Str<ting from the top</b><br>You start from the top, percolating down the elements until the heap property is restored for that key. "+
                "The real Build-heap starts from the bottom of the heap."),

                ("<b>Max-Heap</b><br> Ratkaisusi on oikeaan suuntaan, mutta rakennat maksimikekoa (isoin alkio huipulle), vaikka tehtvss pyydettiin minimikekoa (pienin alkio huipulle).</font><hr><font face=\"Arial, Helvetica\">"+                
/* maxheap */   "<b>Max-Heap</b><br> You are working to create a Max-Heap, in this exercise however you were asked to make a Min-Heap."),

                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it",
//                "You skipped a swap in the correct algorithm",
                "<b>Yksittinen puuttuva askel</b><br> Ratkaisustasi puuttuu yksi alkioiden vaihto ja sen seurauksena ehk mys rekursiivisia vaihtoja.</font><hr><font face=\"Arial, Helvetica\"><b>A missing swap</b><br>You skipped a swap in the correct algorithm - and possibly some other ones because of it"
//                "You skipped a swap in the correct algorithm"
        };     
      }

    public AnimatedStructures makeMisconception(
            boolean noRecursion, 
            boolean swapBoth, 
            boolean rightLeft,
            boolean fixLevelWise) {

        
        //BrokenHeap is a subclass of ExerHeap that has a broken Heapify method.
        //Errors in the heapify method are controlled via the constructor
        class BrokenHeap extends ExerHeap{
           
			private static final long serialVersionUID = -8017203642482300789L;
			boolean recurseError;
            boolean swapBoth;
            boolean rightLeft;
            
            public BrokenHeap(
                    Object[] objs, 
                    boolean recurseError, 
                    boolean swapBoth, 
                    boolean rightLeft) {
                super(objs);
                this.recurseError = recurseError;
                this.swapBoth = swapBoth;
                this.rightLeft = rightLeft;
            }
            
            public void minHeapify(int i) {
                if(i >= getSize())
                    return;
                MatrixComparable p = (MatrixComparable)getObject(i);
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));
                if(lc == null)
                    return;
                if(rc == null || rc.equals(EMPTY)) {
                    if(p.gt(lc) && !lc.equals(EMPTY)) {
                        swap(i, leftChild(i));
                    }
                    return;
                }
                
                
                if(lc.gt(rc)) {
                    if (swapBoth && p.gt(lc) && !lc.equals(EMPTY) && !rightLeft) {
                        swap(i, leftChild(i));
                        if (!recurseError)
                            minHeapify(leftChild(i));
                        
                        p=(MatrixComparable)getObject(i);
                    }
                    
                    
                    if(p.gt(rc) && !rc.equals(EMPTY)) {
                        swap(i, rightChild(i));
                        if (!recurseError)
                            minHeapify(rightChild(i));
                    }
                }else{
                    if (swapBoth && p.gt(rc) && !rc.equals(EMPTY)) {
                        swap(i, rightChild(i));
                        if (!recurseError)
                            minHeapify(rightChild(i));
                        
                        p=(MatrixComparable)getObject(i);
                    }
                    
                    if(p.gt(lc) && !lc.equals(EMPTY)) {
                        swap(i, leftChild(i));
                        if (!recurseError)
                            minHeapify(leftChild(i));
                    }
                }
            }
        }
        
        if (noRecursion || swapBoth || rightLeft)
        	bh2 = new BrokenHeap(objects, noRecursion, swapBoth, rightLeft);            
        else
            bh2 = new ExerHeap(objects);
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);
        int restartLevel = objects.length/4;
        
        int fixRounds = 3;

        for (int i=objects.length/2-1; i>=0; i--) {
            if ((i == restartLevel-1 || (i == 0 && fixRounds>0)) && fixLevelWise){

                if (restartLevel > 0) //XXX Checkme
                    restartLevel /= 2;
                else{
                    restartLevel = objects.length/4;
                    fixRounds--;
                
                    a.startOperation();
                    bh2.minHeapify(0);
                    a.endOperation();                 

                }

                i = (objects.length/2);
                continue;
            }
            
            a.startOperation();
            bh2.minHeapify(i);
            a.endOperation();                 
        }
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh2 }, a);
    }

    
    
    public AnimatedStructures leftToRight() {
        
        bh2 = new ExerHeap(objects);
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);

        int stopLevel = (objects.length/2);
        do{
            for (int i=stopLevel/2; i<stopLevel; i++) {
                a.startOperation();
                bh2.minHeapify(i);
                a.endOperation();                 
            }
            stopLevel/=2;
        }while(stopLevel != 0);
        
        
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh2 }, a);
    }

    public AnimatedStructures skip(final int which) {
        
        bh2 = new ExerHeap(objects){
            
			private static final long serialVersionUID = 3920196442304583671L;

			int counter = 0;
            boolean noHeapify = false;
            
            public void swap(int a, int b){
                counter++;
                if (which == counter){
                    noHeapify = true;
                    return;
                }
                super.swap(a,b);
            }
            
            public void minHeapify(int i) {
                if (noHeapify){
                    noHeapify = false;
                    return;
                }
                super.minHeapify(i);
            }
        
        };
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);

        for (int i=objects.length/2; i>=0; i--) {
            a.startOperation();
            bh2.minHeapify(i);
            a.endOperation();
        }
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh2 }, a);
    }

    
    public AnimatedStructures maxHeap() {
        
        bh2 = new ExerHeap(objects){
            
			private static final long serialVersionUID = 7993058818957090288L;

			public void minHeapify(int i) {
                if(i >= getSize())
                    return;
                MatrixComparable p = (MatrixComparable)getObject(i);
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));
                
                if (p==null || p.equals(EMPTY)){ //EXTRA CODE FOR INCONSISTENT STATES
                    if ((lc == null || lc.equals(EMPTY)) &&
                        (rc == null || rc.equals(EMPTY)))
                        return;
                    if (lc == null || lc.equals(EMPTY)){
                        swap(i, rightChild(i));
                        minHeapify(rightChild(i));
                        return;
                    }

                    if (rc == null || rc.equals(EMPTY)){
                        swap(i, leftChild(i));
                        minHeapify(leftChild(i));
                        return;
                    }

                    if(lc.gt(rc)) {                
                        swap(i, rightChild(i));
                        minHeapify(rightChild(i));
                        return;
                    }
                    else{                
                        swap(i, leftChild(i));
                        minHeapify(leftChild(i));
                        return;
                    }
                } // EXTRA ENDS

                if(lc == null || lc.equals(EMPTY))
                    return;
                
                if(rc == null || rc.equals(EMPTY)) { //ADDED rc.equals empty OS
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        swap(i, leftChild(i));
                        minHeapify(leftChild(i));
                    }
                    return;
                }
                if(lc.lt(rc)) {
                    if(p.lt(rc) && (rc!=null && !rc.equals(EMPTY))) {
                        swap(i, rightChild(i));
                        minHeapify(rightChild(i));
                    }
                } else {
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        swap(i, leftChild(i));
                        minHeapify(leftChild(i));
                    }
                }
            }
        
        };
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);

        for (int i=objects.length/2; i>=0; i--) {
            a.startOperation();
            bh2.minHeapify(i);
            a.endOperation();                 
        }        
        
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh2 }, a);
    }
    
    public AnimatedStructures topDown() {
        
        bh2 = new ExerHeap(objects);
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);

        for (int i=0; i<objects.length/2; i++) {
            a.startOperation();
            bh2.minHeapify(i);
            a.endOperation();                 
        }        
        
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh2 }, a);
    }

    
    public AnimatedStructures percolateUp(boolean percolateMode) {
        
        class UpHeap extends ExerHeap{
                   
			private static final long serialVersionUID = 3854577269203071995L;

			public UpHeap(Object[] objects){
                super(objects);
            }
            
            public void setUp(int index){
                swap(index, findSmallest(index));
            }
            
            public int findSmallest(int index){
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(index));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(index));
                
                if (
                        (lc == null || lc.equals(EMPTY)) &&
                        (rc == null || rc.equals(EMPTY)))
                        return index;               
                                
                MatrixComparable here = (MatrixComparable)getObject(index);
                                 
                if(lc == null || lc.equals(EMPTY)){
                    int smallRight = findSmallest(rightChild(index));                
                    
                    if (here.leq((MatrixComparable)getObject(smallRight)))
                        return index;
                    else
                        return smallRight;
                }
                else if(rc == null || rc.equals(EMPTY)) {
                    int smallLeft = findSmallest(leftChild(index));                
                    
                    if (here.leq((MatrixComparable)getObject(smallLeft)))
                        return index;
                    else
                        return smallLeft;
                }
            
                int smallRight = findSmallest(rightChild(index));                
                int smallLeft  = findSmallest(leftChild(index));                
            
                int smallerChild = 
                    ((MatrixComparable)getObject(smallRight)).leq((MatrixComparable)getObject(smallLeft))?
                            smallRight:smallLeft;
                
                return here.leq((MatrixComparable)getObject(smallerChild))?index:smallerChild;
            }

            public void heapifySingle(int i) {
                if(i >= getSize())
                    return;
                MatrixComparable p = (MatrixComparable)getObject(i);
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));
                if((lc == null || lc.equals(EMPTY)) &&
                   (rc == null || rc.equals(EMPTY)))
                    return;

                if(rc == null || rc.equals(EMPTY)) {
                    if(p.gt(lc) && !lc.equals(EMPTY)) {
                        swap(i, leftChild(i));
                    }
                    return;
                }
                
                if(lc == null || lc.equals(EMPTY)) {
                    if(p.gt(rc) && !rc.equals(EMPTY)) {
                        swap(i, rightChild(i));
                    }
                    return;
                }
                
                if(lc.lt(rc)) {
                    if (!lc.equals(EMPTY) && p.gt(lc)) {
                        swap(i, leftChild(i));
                    }
                }
                else                    
                    if(!rc.equals(EMPTY) && p.gt(rc)) {
                        swap(i, rightChild(i));
                    }
            }
        }        
        
        UpHeap bh3 = new UpHeap(objects);
        
        Animator oldAnim = Animator.getActiveAnimator();

        Animator a=new Animator();
        Animator.setActiveAnimator(a);

        int stopLevel = (objects.length/2);
       
        if (percolateMode){
            for (int i=0; i<stopLevel; i++) {
                a.startOperation();
                bh3.setUp(i);
                a.endOperation();                 
            }
        }
        else{
            for (int j=0; j<8; j++) //supports sizes up to 256 entries, replace with log2 of objects.length
            for (int i=stopLevel; i>=0; i--) {        
                a.startOperation();
                bh3.heapifySingle(i);
                a.endOperation();                 
            }
        }
        
        Animator.setActiveAnimator(oldAnim);
        return new AnimatedStructures(new FDT[]{bh3 }, a);
    }

    static final long serialVersionUID = 9123896676872755713L;
}
