/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Heap_BuildMaxHeap.java,v 1.2 2007/03/08 11:01:55 archie Exp $ */

package content.exercises;

import java.util.Random;

import content.interfaces.ConfigureVisualType;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;
import content.interfaces.StateLegality;

import content.exercises.structures.*;

// import matrix.animation.AnimatedStructures;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.util.MatrixComparable;
// import matrix.structures.util.MatrixComparable;
import matrix.util.*;

public class Heap_BuildMaxHeap implements SimulationExerciseModel,
        ModelAnswerNames, StyledExercise, ConfigureVisualType, SwapBehaviour,
        StateLegality, JudgeBlocks {

    private static final long serialVersionUID = 88374372806040467L;

    ExerMaxHeap bh, bh2;

    // String dataString;
    int[] dataString;

    String marker = "Heap_BuildHeap";

    Object[] objects;

    public static final boolean DEBUG = false;

    public static final String[] code = new String[] {
            "_0,1 PROCEDURE BUILD-MAX-HEAP(Heap A)",
            "",
            "_2 2 FOR (int i = heap-size(A)/2-1; i >= 0; i--)_/2",
            "_3 3   MAX-HEAPIFY(A[i])_/3,/1",
            "",
            "_4,5 PROCEDURE MAX-HEAPIFY(Node q)_/5",
            "",
            "_6 2   heap order property between q and its children is checked_/6",
            "_7 3   IF the heap order property is violated",
            "3.1    swap the nodes p and q in which p is the largest children_/7",
            "3.2    continue from 2 until",
            "    _8 q reaches a position where the heap order property",
            "    is satisfied or reaches a leaf node_/4,/8,/0" };

    public Heap_BuildMaxHeap() {
    }
    
    public void c(Animator a, PseudoCode code, int location){
        a.startOperation();
        code.setLocation(location);
        a.endOperation();            
    }
    /**
     * In this exercise a drag operation performs a swap
     * 
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * 
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * 
     * @param v
     *            Value to assign to seed.
     */
    public void setSeed(long v) {
        this.seed = v;
    }

    // --------EXERCISE INSTANCE VERIFICATION CODE BEGINS--------//
    private transient int recursiveSwaps;

    private transient int partlyRecursiveSwaps;

    private transient int leftSwaps;

    private transient int rightSwaps;

    private transient int swaps;

    public boolean heapify(Key[] heap, int i) {
        int smaller = i * 2;
        if (heap[(i * 2 + 1) - 1].geq(heap[(i * 2) - 1]))
            smaller++;

        if (heap[i - 1].lt(heap[smaller - 1])) {
            Key temp = heap[i - 1];
            heap[i - 1] = heap[smaller - 1];
            heap[smaller - 1] = temp;

            if (smaller == i * 2)
                leftSwaps++; // Swap was made with the left child
            else
                rightSwaps++; // Swap was made with the right child
            swaps++;

            if (smaller <= heap.length / 2) {
                if (heapify(heap, smaller))
                    recursiveSwaps++; // recursive swap
                else
                    partlyRecursiveSwaps++; // swap ends before reaching bottom
                                            // of the heap
            }
            return true; // swapped
        }
        return false; // Did not swap
    }

    // this method verifies that certain algorithm branches must be taken for
    // a certain input to be accepted as an exercise input

    public boolean accept(Object[] keys) {
        recursiveSwaps = 0;
        partlyRecursiveSwaps = 0;
        leftSwaps = 0;
        rightSwaps = 0;
        swaps = 0;

        Key[] copies = new Key[keys.length];

        for (int i = 0; i < keys.length; i++) { // Copy to preserve originals
            copies[i] = (Key) keys[i];
        }

        for (int i = keys.length / 2; i > 0; i--) { // Solve and find out
                                                    // acceptability
            heapify(copies, i);
        }

        return recursiveSwaps > 0 && leftSwaps > 0 && rightSwaps > 0
                && swaps > 8 && partlyRecursiveSwaps > 0;
    }

    // --------EXERCISE INSTANCE VERIFICATION CODE ENDS--------//

    public FDT[] init() {

        Random r = new java.util.Random(seed);
        do {
            // dataString = RandomKey.createNoDuplicateUppercaseRandomKey(r,15);
            dataString = RandomKey.createIntKeys(r, 10, 99, 15);
            objects = new Key[dataString.length];
            for (int i = 0; i < dataString.length; i++)
                objects[i] = new Key("" + dataString[i]);
            // objects[i] = new Key(""+dataString.charAt(i));

        } while (!accept(objects));

        bh = new ExerMaxHeap(objects);
        FDT[] s = { bh, bh };
        return s;
    }

    // styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "array", "leaf tree" };
        return s;
    }

    public String[] getModelAnswerVisualisations() {
        String[] s = { "leaf tree", "pseudo" };
        return s;
    }

    public FDT[] init(Object o) {
        return init();
    }

    public String[] getStructureNames() {
        String[] s = { "Array Representation of Binary Heap", "Binary Heap" };
        return s;
    }

    public String getDescription() {
        return "";
    }

    public FDT[] solve() {
        return new FDT[] { solveModel(false)[0] };
    }

    public FDT[] solveModel(final boolean model) {

        final PseudoCode pseudoCode = new PseudoCode();
        pseudoCode.setCodeLines(code);
        pseudoCode.setCodeName("Build MaxHeap");
        pseudoCode.setLocation(1);
        final Animator a = Animator.getActiveAnimator();
        // start
        bh2 = new ExerMaxHeap(objects) {

            private static final long serialVersionUID = 70546980560196976L;

            public void maxHeapify(int i) {

                if (!model)
                    super.maxHeapify(i);
                

                // if(i >= keys)
                if (i >= getSize()) {
                    c(a, pseudoCode, 8);
                    return;
                }
                MatrixComparable p = (MatrixComparable) getObject(i);
                MatrixComparable lc = (MatrixComparable) getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable) getObject(rightChild(i));

                c(a, pseudoCode, 6);

                if (lc == null || lc.equals(EMPTY)) {
                    if (model)c(a, pseudoCode, 8);
                    return;
                }
                if (rc == null || rc.equals(EMPTY)) {
                    if (p.lt(lc) && (lc != null && !lc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    } else {
                        c(a, pseudoCode, 8);
                    }
                    return;
                }
                if (lc.lt(rc)) {
                    if (p.lt(rc) && (rc != null && !rc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, rightChild(i));
                        a.endOperation();
                        maxHeapify(rightChild(i));
                    } else {
                        c(a, pseudoCode, 8);
                    }
                } else {
                    if (p.lt(lc) && (lc != null && !lc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    } else {
                        c(a, pseudoCode, 8);
                    }
                }
            }
        };
        
        for (int i = objects.length/2 - 1 ; i >= 0; i--) {
            if (model){
                c(a, pseudoCode, 2);
                c(a, pseudoCode, 3);
            }
            if (!model)a.startOperation();
            bh2.maxHeapify(i);
            if (!model)a.endOperation();
            // bh2.insert(t2.getObject(i));
        }
        if (model){
            //last for condition check
            c(a, pseudoCode, 2);
            //End procedure
            c(a, pseudoCode, 0);
        }
        return new FDT[] { bh2, pseudoCode };
    }

    public FDT[] makeModelAnswer() {
        return solveModel(true);
    }

    public FDT[] getInitialStructures() {
        ExerMaxHeap tmp = new ExerMaxHeap(objects);
        return new FDT[] { tmp };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { bh };
    }

    public FDT[] getAnswer() {
        FDT[] s = { bh };
        return s;
    }

    public String[] getModelAnswerNames() {
        String[] s = { "Binary Heap", "Pseudocode" };
        return s;
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey",
                VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey",
                VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf code = new VisualTypeConf();
        return new VisualTypeConf[] { tabl, tre, code };
    }

    // -- Helper method which checks if the heap given is
    // a legal heap in the end of the sequence. Returns
    // the animator into its original state.

    public boolean legalEndState(Animator a, FDT[] f) {
        Animator oldAnim = Animator.getActiveAnimator();
        Animator.setActiveAnimator(a);

        int forwardStates = 0;

        while (a.hasNextOperation()) {
            a.redo();
            forwardStates++;
        }

        boolean result = false;
        if (f[0] instanceof ExerMaxHeap) // Backwards compatible - Old
                                            // exerheap does not have this
                                            // method
            result = ((ExerMaxHeap) f[0]).isMaxHeap();

        for (; forwardStates > 0; forwardStates--)
            a.undo();

        Animator.setActiveAnimator(oldAnim);
        return result;
    }

    public boolean legalState(FDT[] structures) {
        boolean result = false;
        if (structures[0] instanceof ExerMaxHeap)
            result = ((ExerMaxHeap) structures[0]).isMaxHeap();

        return result;
    }

}
