/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: Heap_DeleteMax.java,v 1.2 2007/03/08 11:01:55 archie Exp $*/

package content.exercises;

import java.util.Random;

import content.interfaces.ButtonExercise;
import content.interfaces.StyledExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerMaxHeap;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.util.MatrixComparable;
//import matrix.util.Note;
import matrix.util.RandomKey;

public class Heap_DeleteMax extends AbstractSimulationExercise implements SimulationExerciseModel, ModelAnswerNames, ConfigureVisualType, ButtonExercise, SwapBehaviour, StyledExercise {
	
    private static final long serialVersionUID = -5525899008029646282L;
    
    Table t;
	ExerMaxHeap bt, bh;
	
	private int[] S = null;
	public static final boolean DEBUG = false;
	
	public static final String[] code = new String[]{
		"_0,1 PROCEDURE DeleteMax(Node q)",
		"",
		"_2 Swap the root node with the last node on the lowest level",
        "   and decrease the heap-size by one (here, the last node is deleted). _/2",
		"_3 Restore the heap order property by calling MAX-HEAPIFY on the root node_/3,/1",
		"",
		"_4,5 PROCEDURE MAX-HEAPIFY(Node q)_/5",
		"",
		"_6 2   heap order property between q and its children is checked_/6",
		"_7 3   IF the heap order property is violated",
		"3.1    swap the nodes p and q in which p is the largest children_/7",
		"3.2    continue from 2 until",
		"    _8 q reaches a position where the heap order property",
		"    is satisfied or reaches a leaf node_/4,/8,/0"
	};
	
	public void c(Animator a, PseudoCode code, int location){
		a.startOperation();
		code.setLocation(location);
		a.endOperation();            
	}
	
	
	/**
	 * In this exercise a drag operation performs a swap
	 * @return
	 */
	public boolean getSwapBehaviour() {
		return true;
	}
	
//   styled exercise

    public String[] getStructureVisualisations() {
        String[] s = { "array" , "layered tree" };
        return s;
    }


    public String[] getModelAnswerVisualisations() {
        String[] s = { "array", "layered tree", "pseudo" };
        return s;
    }

	public FDT[] init() {
        final int keys = 13; // the number of keys in the initial heap
        Random r = new java.util.Random(seed);
        do {
            S = RandomKey.createIntKeys(r, 10, 99, keys); // between 10 and 99, thus can be compared lexicographically
        } while (!isAcceptable());
        t = new Table(S);
		bt = new ExerMaxHeap(15);
        for(int i = 0; i < t.size(); i++)
            bt.insert(bt.new DeletableKey(t.getObject(i)));
		FDT[] s = { bt, bt };
		return s;
	}
	
	/*
     * @return true if the current instance of input (see init()) is acceptable
	 */
    private boolean isAcceptable() {
        ExerMaxHeap test = new ExerMaxHeap(15);
        Table t = new Table(S);
        int recursions = 0;
        for (int i=0; i<t.size(); i++)
            test.insert(t.getObject(i));
        for (int i=0; i<3; i++)
            test.deleteMax();
        recursions=test.getNumberOfRecursions();
        //matrix.util.Note.out(this, "recursions="+recursions);
        if ((recursions < 9) || (recursions > 10)) // almost the same amount of work in all instances
            return false;
        if (!test.hasInterrupted())
            return false;
        if (test.getMaxNumberOfRecursionsDuringSingleCall() < 4)
            return false;
        if (!test.hasEqualChildren())
            return false;
        return true;
    }
    
	public FDT[] init(Object o) {
		return init();
	}
	
	
	public String[] getStructureNames() {
		String[] s = { "Array Representation of Heap", "Binary Heap" };
		return s;
	}
	
	public String getDescription() {
		return "";
	}
	
	public FDT[] makeModelAnswer() {
		return solveModel(true);
	}
	
	public FDT[] solve(){
		return new FDT[]{solveModel(false)[0]};
	}
	
	public FDT[] solveModel(final boolean model) {
		
		final PseudoCode pseudoCode = new PseudoCode();
		pseudoCode.setCodeLines(code);
		pseudoCode.setCodeName("Heap DeleteMin");
		pseudoCode.setLocation(1);
		
		final Animator a = Animator.getActiveAnimator();
		
		bh = new ExerMaxHeap(15) {
			
            private static final long serialVersionUID = 1443847687021077469L;

            public void deleteMax() {
                if (model)a.startOperation();
                MatrixComparable tmp = (MatrixComparable)getObject(keys.eval() - 1);
                
                setObject(tmp, 0);
                setObject(new Key(EMPTY), keys.eval() - 1);
                if (model)a.endOperation();
                keys.dec();
                c(a, pseudoCode, 3);
                maxHeapify(0);
            }
            
            public void maxHeapify(int i) {
                
                if (!model)
                    super.maxHeapify(i);
                
                // if(i >= keys)
                if(i >= getSize()) {
                    c(a, pseudoCode, 8);
                    return;
                }
                MatrixComparable p = (MatrixComparable)getObject(i);
                MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));

                c(a, pseudoCode, 6);
                          
                if(lc == null || lc.equals(EMPTY)) {
                    c(a, pseudoCode, 8);
                    return;
                }
                if(rc == null || rc.equals(EMPTY)) { 
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    } else { c(a, pseudoCode, 8); }
                    return;
                }
                if(lc.lt(rc)) {
                    if(p.lt(rc) && (rc!=null && !rc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, rightChild(i));
                        a.endOperation();
                        maxHeapify(rightChild(i));
                    } else { c(a, pseudoCode, 8); }
                } else {
                    if(p.lt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                        c(a, pseudoCode, 7);
                        a.startOperation();
                        swap(i, leftChild(i));
                        a.endOperation();
                        maxHeapify(leftChild(i));
                    } else { c(a, pseudoCode, 8); }
                }
            }
		};
		
		for(int i = 0; i < t.size(); i++) {
			bh.insert(t.getObject(i));
		}
		for(int i = 0; i < 3; i++) {
			if (!model)a.startOperation();
            if (model)c(a, pseudoCode, 2);
			bh.deleteMax();
			if (!model)a.endOperation();
		}
        //End procedure
        if (model)c(a, pseudoCode, 0);
        FDT[] s = { bh, bh, pseudoCode };
        
		return s;
	}
	
	public FDT[] getInitialStructures() {
        ExerMaxHeap eh = new ExerMaxHeap(15);
		return new FDT[] { eh, eh };
	}
	
	public FDT[] getSimulatedStructures() {
		return new FDT[] { bt, bt };
	}
	
	public String[] getModelAnswerNames() {
		String[] s = { "Array Representation of Heap", "Binary Heap", "Pseudo code" };
		return s;
	}
	
	
	public FDT[] getAnswer() {
		return new FDT[] { bt };
	}
	
	public SimulationExerciseModel getModelAnswer() {
		return this;
	}
	
	public String[] buttonNames() {
		String[] s = { "Decrement heapsize" };
		return s;
	}
	
    public void decrementHeapsize() {
       // matrix.util.Note.out(this, "decrementheapsize");
        Animator.getActiveAnimator().startOperation();
        bt.decrementHeapsize();
        Animator.getActiveAnimator().endOperation();
    }
    
	public String[] buttonCommands() {
		String[] s = { "/decrementHeapsize" };
		return s;
	}
	
	public VisualTypeConf[] conf() {
		VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
		VisualTypeConf tre = new VisualTypeConf();
		tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
		tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
		tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
		tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
		tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        VisualTypeConf code  = new VisualTypeConf();
		return new VisualTypeConf[] { tabl, tre, code };
	}
	
	
}
