/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: Heap_Insert_Delete.java,v 1.31 2007/02/09 12:38:35 archie Exp $*/

package content.exercises;

import content.interfaces.ButtonExercise;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerHeap;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.util.MatrixComparable;
import matrix.util.RandomKey;

public class Heap_Insert_Delete extends AbstractSimulationExercise implements SimulationExerciseModel, ModelAnswerNames, ConfigureVisualType, ButtonExercise, SwapBehaviour {

    Table t;
    ExerHeap bt, bh;

    private int[] S = null;
    public static final boolean DEBUG = false;

    public static final String[] code = new String[]{
        "_1 PROCEDURE Insert(Node q)",
        "",
        "_2 Insert new node in the leftmost free position on the lowest level._/2",
        "_3 Restore heap property by calling SIFT-UP on the new node_/3,/1",
        "",
        "_4,5 PROCEDURE Sift-Up(Node q)_/5",
        "",
        "_6 2   heap order property between q and its parent node p is checked_/6",
        "_7 3   IF the heap order property is violated",
        "3.1    swap the nodes p and q_/7",
        "3.2    continue from 2 until",
        "    _8 q reaches a position where the heap order property",
        "    is satisfied or reaches the root node_/4,/8",
        "",
        "_9 Procedure DeleteMin()",
        "   Code omitted_/9"
    };

    public void c(Animator a, PseudoCode code, int location){
        a.startOperation();
        code.setLocation(location);
        a.endOperation();            
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    public FDT[] init() {
        //t = new Table("KVEFGOQXNP");
        S = RandomKey.createIntKeys(new java.util.Random(seed), 10, 99, 13); // between 10 and 99, thus can be compared lexicographically
        t = new Table(S);
        bt = new ExerHeap(15);
        FDT[] s = { t, bt };
        return s;
    }


    public FDT[] init(Object o) {
        return init();
    }


    public String[] getStructureNames() {
        String[] s = { "Stream of keys", "Heap" };
        return s;
    }

    public String getDescription() {
        return "";
    }

    public FDT[] makeModelAnswer() {
        return solveModel(true);
    }

    public FDT[] solve(){
        return new FDT[]{solveModel(false)[0]};
    }

    public FDT[] solveModel(final boolean model) {

        final PseudoCode pseudoCode = new PseudoCode();
        pseudoCode.setCodeLines(code);
        pseudoCode.setCodeName("Heap Insert & Delete");
        pseudoCode.setLocation(1);

        final Animator a = Animator.getActiveAnimator();

        bh = new ExerHeap(15){

            private static final long serialVersionUID = 4093740609312959944L;

            public int insert(Object o) {
                int swaps = 0;
                a.startOperation();
                pseudoCode.setLocation(2);
                setObject(o, keys.eval());
                keys.inc();
                a.endOperation();

                c(a, pseudoCode, 3);
                swaps = bubble(keys.eval()-1);
                c(a, pseudoCode, 8);

                return swaps;
            }

            protected int bubble(int n) {
                MatrixComparable p = (MatrixComparable)getObject(parent(n));
                MatrixComparable t = (MatrixComparable)getObject(n);
                c(a, pseudoCode, 6);

                if (p == null || p.gt(t)) {
                    c(a, pseudoCode, 7);
                    if (model)a.startOperation();
                    swap(n,parent(n));
                    if (model)a.endOperation();
                    c(a, pseudoCode, 6);
                    return bubble(parent(n)) + 1;
                } return 0;
            }

            public void deleteMin() {
                c(a, pseudoCode, 9);
                if (model)a.startOperation();
                MatrixComparable tmp = (MatrixComparable)getObject(keys.eval() - 1);
                setObject(tmp, 0);
                setObject(new Key(EMPTY), keys.eval() - 1);
                if (model)a.endOperation();
                keys.dec();
                minHeapify(0);
            }

            public void minHeapify(int i) {

                if (!model)
                    super.minHeapify(i);

                // if(i >= keys)
                    if(i >= getSize())
                        return;
                    MatrixComparable p = (MatrixComparable)getObject(i);
                    MatrixComparable lc = (MatrixComparable)getObject(leftChild(i));
                    MatrixComparable rc = (MatrixComparable)getObject(rightChild(i));


                    if(lc == null || lc.equals(EMPTY))
                        return;
                    if(rc == null || rc.equals(EMPTY)) { 
                        if(p.gt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                            a.startOperation();
                            swap(i, leftChild(i));
                            a.endOperation();
                            minHeapify(leftChild(i));
                        }
                        return;
                    }
                    if(lc.gt(rc)) {
                        if(p.gt(rc) && (rc!=null && !rc.equals(EMPTY))) {
                            a.startOperation();
                            swap(i, rightChild(i));
                            a.endOperation();
                            minHeapify(rightChild(i));
                        }
                    } else {
                        if(p.gt(lc) && (lc!=null && !lc.equals(EMPTY))) {
                            a.startOperation();
                            swap(i, leftChild(i));
                            a.endOperation();
                            minHeapify(leftChild(i));
                        }
                    }
            }


        };
        Table tbl = (Table)getInitialStructures()[0];
        for(int i = 0; i < tbl.size(); i++) {
            if (!model)a.startOperation();
            bh.insert(tbl.getObject(i));
            if (!model)a.endOperation();
        }
        for(int i = 0; i < 3; i++) {
            if (!model)a.startOperation();
            bh.deleteMin();
            if (!model)a.endOperation();
        }
        FDT[] s = { bh, pseudoCode };
        return s;
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerHeap(15) };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, bt };
    }

    public String[] getModelAnswerNames() {
        String[] s = { "Binary Heap", "Pseudo code" };
        return s;
    }


    public FDT[] getAnswer() {
        return new FDT[] { bt };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public String[] buttonNames() {
        String[] s = {"Delete"};
        return s;
    }


    public String[] buttonCommands() {
        String[] s = {"reflectSelectedVisualType(exerciseReflectionCommand(deleteRoot))"};
        return s;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        return new VisualTypeConf[] { tabl, tre };
    }

    static final long serialVersionUID = -7671756502477641250L;
}
