/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package content.exercises;
import content.interfaces.*;
import matrix.animation.Animator;
import matrix.decoration.StyleSheet;
import matrix.decoration.StyleSheetAdapter;
import matrix.decoration.Styled;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.memory.VirtualBoolean;
import matrix.util.RandomKey;
import java.awt.*;
import java.util.Random;
import matrix.util.*;



//Enable the swap parameter in the applet parameters.



public class InsertionSort implements SimulationExerciseModel,  StyledExercise, ModelAnswerNames, ConfigureVisualType, JudgeBlocks, SwapBehaviour {


    //ATRIBUTES

    private Table table;
    private Table tempTable; //Table which size is one. It is used as a temp variable when doing the exercise.
    private long seed = 1;
    private String[] s;
    private String tempString;
    private String initialValueOfTmp;
    private Application app;


    //METHODS

    public boolean getSwapBehaviour() {
        return false;
    }

    public FDT[] solve(){

        FDT[] structures = getInitialStructures();
        Table tab = (Table) structures[0];
        Table temp = (Table) structures[1];
        Animator an = Animator.getActiveAnimator();

        for (int i = 1; i < tab.size(); i++) { 

            an.startOperation();
            temp.setObject(tab.getObject(i), 0);
            an.endOperation();
            int tempKey= Integer.parseInt(((Key)tab.getObject(i)).toString());

            int j= i;

            while ((j > 0) && (Integer.parseInt(((Key)tab.getObject(j-1)).toString()) > tempKey)) {
                an.startOperation();
                tab.setObject((Key)tab.getObject(j-1),j);
                an.endOperation();
                j= j-1;
            } 

            an.startOperation();
            tab.setObject(temp.getObject(0),j);
            an.endOperation();
        }
        return new FDT[]{tab, temp};
    }


    public FDT[] makeModelAnswer(){

        return solve();

    }
    public String[] getStructureVisualisations(){

        return new String[]  {"array", "array"};

    }
    public String[] getModelAnswerVisualisations(){

        return new String[] {"array", "array"};

    }
    public String[] getModelAnswerNames(){

        return new String[] {"Input", "tmp"};

    }
    public VisualTypeConf[] conf(){

        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);

        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);

        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tabl2.setVisualConfValue("matrix.visual.VisualArray", VisualTypeConf.INDEXED, "false");
        return new VisualTypeConf[] { tabl1, tabl2};

    }

    public FDT[] init(){

        s=new String[16];
        tempString = " ";
        Random random = new Random(seed);

        int d[];
        d=matrix.util.RandomKey.createIntKeys(random,0,30,16);

        for (int i=0;i<s.length;i++){

            s[i] = d[i] + "";

        }
        initialValueOfTmp = RandomKey.createIntKey(random, -9, 0) + "";

        //Put the largest key (30,35) in the one of the positions 0,1,2,3,4
        s[RandomKey.createIntKey(random, 0, 5)] = RandomKey.createIntKey(random, 30,36) + ""; 

        table = new ClickTable(s);
        tempTable = new Table(tempString);
        tempTable.setObject(initialValueOfTmp, 0);   		
        return new FDT[] { table, tempTable };


    }
    public void setSeed(long v){

        this.seed = v;
    }


    public long getSeed(){

        return seed;
    }


    public String[] getStructureNames(){

        return new String[]  {"Input", "tmp"};

    }
    public FDT[] getAnswer(){

        return new FDT[]{table, tempTable};
    }
    public FDT[] getInitialStructures(){
        ClickTable ct = new ClickTable(s);
        Table t = new Table(tempString);

        t.setObject(initialValueOfTmp, 0); 
        return new FDT[] { ct,t };

    }
    public String getDescription(){

        return "";
    }


    static final class ClickTable extends Table{


        public ClickTable(String s){
            super(s);  

            for (int i=0; i<s.length(); i++)
                setObject(new ClickKey(s.substring(i,i+1)), i);    
        }

        public ClickTable(String[] ss) {
            super(ss);
            for(int i=0; i < ss.length; i++)
                setObject(new ClickKey(ss[i]), i);
        }

        static final long serialVersionUID = 8668291687539197721L;   
    }

    static final class ClickKey extends Key implements Styled{

        final VirtualBoolean selected = new VirtualBoolean(false, this, "Has this key been selected");

        public boolean equals(Object o){
            if (o instanceof ClickKey){
                return equals((ClickKey)o);
                //return ((ClickKey)o).equals(this);
            }
            else return super.equals(o);
        }

        public StyleSheet getStyleSheet(){
            return new StyleSheetAdapter(){

                public Color getDefaultBackgroundColor(){
                    if (ClickKey.this.selected.eval() == false)
                        return super.getDefaultBackgroundColor();
                    else
                        return Color.magenta;
                }

                public Color getDefaultPenColor(){
                    if (ClickKey.this.selected.eval() == false)
                        return super.getDefaultPenColor();
                    else
                        return Color.white;
                }

                public Color getVisitedBackgroundColor(){
                    if (ClickKey.this.selected.eval() == false)
                        return super.getDefaultBackgroundColor();
                    else
                        return Color.magenta;
                }

                public Color getVisitedPenColor(){
                    if (ClickKey.this.selected.eval() == false)
                        return super.getDefaultPenColor();
                    else
                        return Color.white;
                }

                static final long serialVersionUID = 7029031304710091814L;  
            };
        }

        public boolean isVisited(){
            return selected.eval();
        }

        public void setVisited(boolean b){
            selected.assign(b);
        }

        public void select(){
            //selected.assign(true);
            selected.assign(!selected.eval());

        }

        public void drag_source(){
            selected.assign(true);
            System.err.println("Here");
        }

        public void drag_destination(){
            selected.assign(true);
            System.err.println("and here");

        }

        public ClickKey(String keyText){
            super(keyText);
        }

        static final long serialVersionUID = 5356225316236393677L;    
    }
    static final long serialVersionUID = 5324781321629168537L;
}
