/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: RBT_coloring.java,v 1.11 2007/09/06 08:37:59 oseppala Exp $ */

package content.exercises;

import java.awt.Color;

import java.util.Random;

import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseSelfAssessment;
import content.interfaces.SwapBehaviour;
import content.exercises.structures.ExerColoringRBTree;

import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.RBTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;

public class RBT_coloring implements SimulationExerciseSelfAssessment, ModelAnswerNames, ConfigureVisualType, SwapBehaviour {
    ExerColoringRBTree user_RBT;
    RBTree model_RBT;
    String S = null;
    String feedback;
    public static final boolean DEBUG = false;

    public RBT_coloring() {
    }

    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }


    public FDT[] init() {

        // 9-13 keys
        Random R = new Random(seed);
        int n = Math.abs(R.nextInt()%5);
        // Can contain duplicates
        String head;
        if (n >= 2) 
            head = "ABCDEFG";
        else
            head = "ZYXWVUT";
        S = head + matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,15)+
        matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);

        user_RBT = new ExerColoringRBTree();
        Table tbl = (Table)getInitialStructures()[0];
        for (int i=0; i<tbl.size(); i++) {
            user_RBT = (ExerColoringRBTree)user_RBT.superInsert(tbl.getObject(i));
        }
        // Let us get rid of colors
        user_RBT.color(Color.blue);
        FDT[]  s = { user_RBT };
        return s;
    }



    public String[] getStructureNames() {
        String[] s = { "Red Black Tree" };
        return s;
    }


    public String getDescription() {
        return "";
    }


    public FDT[] solve() {
        model_RBT = new RBTree();
        Table tbl = (Table)getInitialStructures()[0];

        for (int i=0; i<tbl.size(); i++) {
            model_RBT = (RBTree)model_RBT.insert(tbl.getObject(i));
        }

        return new FDT[] { model_RBT };
    }


    public String[] getModelAnswerNames() {
        String[] s = { "Red-Black Tree" };
        return s;
    }

    public FDT[] makeModelAnswer() {
        return solve();
    }


    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerColoringRBTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { user_RBT };
    }

    public FDT[] getAnswer() {
        FDT[] s = { user_RBT };
        return s;
    }

    // ** Implementation of interface SimulationExerciseSelfAssessment

    public int[] judge() {
        /*
         * validity[0] tells if root is black
         * validity[1] tells if there are equal number of black nodes on all paths
         * validity[2] tells if no red node has red children
         */
        boolean[] validity = user_RBT.isValidRBT(); 
        if (validity[0] && validity[1] && validity[2]) {
            feedback = "Exercise correct.";
            return new int[] {1};
        } else {
            feedback = "The following condition(s) were not met:\n";
            if(!validity[0]) {
                feedback += "The root node should be black.\n";
            }
            if(!validity[1]) {
                feedback += "All paths to leaf nodes should have equal number of black nodes.\n";
            }
            if(!validity[2]) {
                feedback += "Red nodes should have only black children.\n";
            }
            return new int[] {0};
        }
    }

    public int getMaxPoints() {
        return 1;
    }

    public String getFeedbackString() {
        return feedback;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tre};
    }


    static final long serialVersionUID = -7576022670921264541L;
}
