/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: RST_Insert.java,v 1.26 2006/10/26 08:48:51 vkaravir Exp $ */

package content.exercises;

import java.util.Random;

import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.SwapBehaviour;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.RadixSearchTrie;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;

public class RST_Insert implements SimulationExerciseModel, ModelAnswerNames,
                                   ConfigureVisualType, SwapBehaviour {

    Table t;
    BinTree bt;
    String marker = "DST_Insert";
    String S;
    public static final boolean DEBUG = false;

    public RST_Insert() {
    }

    public boolean getSwapBehaviour(){
        return true;
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }




    public FDT[] init() {
        // 15-20 keys
        Random R = new Random(seed);
        int n = Math.abs(R.nextInt()%3);
        // Cannot contain duplicates
        S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,5+n);
        t = new Table(S);
        //t.setObject(null, 10);
        bt = new BinTree();
        return new FDT[] { t, bt };
    }


    public String[] getStructureNames() {
        String[] tmp = { "Stream of Keys", "Radix Search Trie" };
        return tmp;
    }

    public String getDescription() {
        return "";


    }

    public FDT[] solve() {
        RadixSearchTrie dst = new RadixSearchTrie();
        Animator a = Animator.getActiveAnimator();
        Table tbl = (Table)getInitialStructures()[0];
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            dst.insert(tbl.getObject(i));
            a.endOperation();
        }
        return new FDT[] { (FDT)dst.getElement() };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new BinTree() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, bt };
    }

    public FDT[] getAnswer() {
        FDT[] tmp = { bt };
        return tmp;
    }


    public String[] getModelAnswerNames() {
        String[] s = { "Radix Search Trie" };
        return s;
   }


    public SimulationExerciseModel getModelAnswer() {
        return this;
    }


    public String[] buttonNames() {
        String[] s = {"Swap on", "Swap off"};
        return s;
    }


    public String[] buttonCommands() {
        String[] s = {"swapOn", "swapOff"};
        return s;
    }


    public int[] buttonGroups() {
        int[] n = new int[1];
        n[0] = 0;
        return n;
    }


    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();

        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);

        tre.enable("matrix.visual.VisualLayeredTreeComponent",
                   VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }


    static final long serialVersionUID = -5830078575432800959L;
}
