/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import content.interfaces.*;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import content.exercises.structures.*;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.structures.CDT.probe.Stack;
import java.awt.*;
import java.awt.event.*;
import javax.swing.JButton;
import java.util.Random;
import matrix.util.*;


public class RecursiveMergeSort implements SimulationExerciseModel, AWTComponentUtilizer, 
        StyledExercise, ModelAnswerNames, ConfigureVisualType, JudgeBlocks,
        HasRotatedContainers, LayoutExercise {   

    //ATRIBUTES
	private static final int stackFrame_cols = 3;
    private static final String[] stackFrame_labels = {
    	"left", "mid", "right"
    };
    
    private Table mergeStudentTable;
    
    private TableMergeSort aidStudentTable;

    private Stack callStack;

    private PseudoCode vCode = null;
    
    private long seed = 1;
    
    private String s, h;
    
    private Application app;
    
    private Table mergeTable, aidTable;

    public static final String[] code = new String[] {
        "_1 1 PROCEDURE MergeSort(Array a, Integer left, Integer right) _/1",
        "",
        "_2 2 IF left < right _/2",
        "_3 3     mid := (left + right) / 2 _/3",
        "_4 4     MergeSort(a, left, mid) _/4",
        "_5 5     MergeSort(a, mid + 1, right) _/5",
        "_6 6     Merge(a, left, mid, right) _/6",
        "",
        "_7 7 RETURN _/7"
    };

    
    //METHODS
    public void setApplication(Application a) {
        app = a;
    }
    
    public Application getApplication() {
        return app;
    }
    
    private void mergesort(int left, int right) {
        
        if (left < right) {
            
            int mid = (left + right) / 2;
            mergesort(left, mid);
            mergesort(mid + 1, right);

            merge(left, mid, right);
            
        }
    }
    
    private void pushStackFrame(int left, int right, int mid, int ret) {
    	String values[] = {
    			String.valueOf(left),
    			(mid < 0 ? "??" : String.valueOf(mid)),
    			String.valueOf(right),
    	};
    	callStack.insert((FDT) new ActivationRecord(stackFrame_labels, values, ret, stackFrame_cols));
    }

    private void prettyMergesort (int left, int right, int ret) {

        Animator a  = Animator.getActiveAnimator();

        a.startOperation();
        pushStackFrame(left, right, -1, ret);
        vCode.setLocation(1);
        a.endOperation();

        a.startOperation();
        vCode.setLocation(2);
        a.endOperation();
        if (left < right) {

            a.startOperation();
            vCode.setLocation(3);
            a.endOperation();
            int mid = (left + right) / 2;

            a.startOperation();
            callStack.pop(null);
            pushStackFrame(left, right, mid, ret);
            vCode.setLocation(4);
            a.endOperation();
            prettyMergesort(left, mid, 5);

            a.startOperation();
            callStack.pop(null);
            vCode.setLocation(5);
            a.endOperation();
            prettyMergesort(mid + 1, right, 6);

            a.startOperation();
            callStack.pop(null);
            vCode.setLocation(6);
            a.endOperation();
            merge(left, mid, right);
        }

        a.startOperation();
        vCode.setLocation(7);
        a.endOperation();
    }

    
    void merge(int left, int mid, int right) {
        
        int i, j, k;
        int cmpI = 0, cmpJ = 0;
        Animator an = Animator.getActiveAnimator();
        
        i = left;
        j = mid + 1;
        k = left;
        while (i <= mid && j <= right) {
            
            cmpI = Integer.parseInt(((Key) mergeTable.getObject(i)).toString());
            cmpJ = Integer.parseInt(((Key) mergeTable.getObject(j)).toString());
            
            if (cmpI <= cmpJ) {
                an.startOperation();
                aidTable.setObject(mergeTable.getObject(i), k);
                an.endOperation();
                k++;
                i++;
                
            } else {
                an.startOperation();
                aidTable.setObject(mergeTable.getObject(j), k);
                an.endOperation();
                k++;
                j++;
                
            }
        }
        
        if (i <= mid) {
            while (i <= mid) {
                an.startOperation();
                aidTable.setObject(mergeTable.getObject(i), k);
                an.endOperation();
                k++;
                i++;
            }            
        }

        if (j <= right) {
            while (j <= right) {
                an.startOperation();
                aidTable.setObject(mergeTable.getObject(j), k);
                an.endOperation();
                k++;
                j++;
            }            
        }

        an.startOperation();
        for (i = left; i <= right; i++) {
            mergeTable.setObject(aidTable.getObject(i), i);
            aidTable.setObject(new Key(""), i);
        }
        an.endOperation();

        
    }
    
    public FDT[] solve() {
        
        mergeTable = (Table) getInitialStructures()[0];
        aidTable = (Table) getInitialStructures()[1];
        mergesort(0, mergeTable.size() - 1);
        
        return new FDT[] { mergeTable, aidTable };
    }
    
    public FDT[] makeModelAnswer() {
        
        mergeTable = (Table) getInitialStructures()[0];
        aidTable   = (Table) getInitialStructures()[1];
        callStack  = new Stack(6);
        if (vCode == null) {
            vCode = new PseudoCode();
            vCode.setCodeLines(code);
            vCode.setCodeName("Mergesort code");
        }
        vCode.setLocation(0);
        prettyMergesort(0, mergeTable.size() - 1, -1);
        Animator.getActiveAnimator().startOperation();
        callStack.pop(null);
        Animator.getActiveAnimator().endOperation();

        return new FDT[] { mergeTable, aidTable, callStack, vCode };
    }

    public boolean[] getContainerRotations() {
        return new boolean[] { false, false };
    }

    public boolean[] getModelAnswerRotations() {
        return new boolean[] { false, false, true, false };
    }
    
    public String[] getStructureVisualisations() {      
        return new String[] { "array", "array", "array" };
    }
    
    public String[] getModelAnswerVisualisations() {
        return new String[] { "array", "array", "array", "pseudocode" };
    }
    
    public String[] getModelAnswerNames() {     
        return new String[] { "Input", "Auxiliary Table", "Call Stack", "Pseudo Code" };
    }
    
    public VisualTypeConf[] conf() {
        
        VisualTypeConf tabl1 = new VisualTypeConf();
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl1.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl1.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        tabl1.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        
        VisualTypeConf tabl2 = new VisualTypeConf();
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl2.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        tabl2.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        VisualTypeConf dummy = new VisualTypeConf();
        return new VisualTypeConf[] { tabl1, tabl2, dummy, dummy };
        
    }
    
    public FDT[] init() {
        
        s = "";
        
        int d[];
        d = matrix.util.RandomKey.createIntKeys(new Random(seed), 0, 10, 11);
        
        for (int i = 0; i < d.length; i++) {
            s = s + d[i];
        }
        
        h = "           ";
        mergeStudentTable = new Table(s);
        aidStudentTable = new TableMergeSort(h);
        
        return new FDT[] { mergeStudentTable, aidStudentTable };
        
    }
    
    public void setSeed(long v) {
        
        this.seed = v;
    }
    
    public long getSeed() {
        
        return seed;
    }
    
    public String[] getStructureNames() {
        
        return new String[] { "Input", "Auxiliary Table" };
        
    }
    
    public FDT[] getAnswer() {
        
        return new FDT[] { mergeStudentTable, aidStudentTable };
    }
    
    // Buttons
    
    public Component[] getAWTComponents() {
        JButton push = new JButton("Move");
        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });
        
        return new Component[] { push };
    }

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }
    
    public FDT[] getInitialStructures() {
        
        return new FDT[] { new Table(s), new Table(h) };
        
    }
    
    public String getDescription() {
        
        return "";
    }
    
    public void pushButton() {
        
        int index = 0;
        Key k;
        Animator.getActiveAnimator().startOperation();
        while (index < aidStudentTable.size()) {
            
            k = (Key) aidStudentTable.next();
            
            if (k != null && k.toString().trim().equals("") == false) {
                
                mergeStudentTable.setObject(aidStudentTable.getObject(index), index);
                aidStudentTable.setObject(new Key(""), index);
                
            }
            
            index++;
        }
        Animator.getActiveAnimator().endOperation();
        aidStudentTable.reset();
        mergeStudentTable.reset();
        app.validate();
        app.validateAnimator();
        
    }

    public GridBagConstraints[] getConstraints () {
        Insets i = new Insets(10, 10, 10, 10);
        GridBagConstraints c;

        c = new GridBagConstraints();
        c.insets    = i;
        c.gridwidth =  GridBagConstraints.REMAINDER;
        return new GridBagConstraints[] { c, c };
    }

    public GridBagConstraints[] getModelAnswerConstraints() {
        Insets i = new Insets(10, 10, 10, 10);
        GridBagConstraints[] r = new GridBagConstraints[4];
        GridBagConstraints   c;

        // Array
        c = new GridBagConstraints();
        c.insets = i;
        c.gridx  = 0;
        c.gridy  = 0;
        r[0]     = c;
        // Auxiliary array
        c = new GridBagConstraints();
        c.insets = i;
        c.gridx = 0;
        c.gridy = 1;
        r[1] = c;
        // Call stack
        c = new GridBagConstraints();
        c.insets = i;
        c.gridx = 1;
        c.gridy = 0;
        c.gridheight = GridBagConstraints.REMAINDER;
        c.anchor = GridBagConstraints.PAGE_START;
        r[2] = c;
        // Pseudocode
        c = new GridBagConstraints();
        c.insets = i;
        c.gridx = 0;
        c.gridy = 2;
        c.anchor = GridBagConstraints.PAGE_START;
        r[3] = c;

        return r;
    }

    static final long serialVersionUID = 2953048071727138376L;
}
