/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import javax.swing.JButton;

import content.exercises.structures.ColorableKey;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ConfigureVisualType;
import content.interfaces.HasRotatedContainers;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;
import content.interfaces.SwapBehaviour;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.Stack;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.util.*;
import matrix.visual.VisualType;


public class RecursiveQuicksort implements SimulationExerciseModel,
StyledExercise, ModelAnswerNames,
ConfigureVisualType, AWTComponentUtilizer,
HasRotatedContainers,/* UsesSplit,*/ JudgeBlocks,
SwapBehaviour{
    
    Table userTable;
    Stack userStack;
    
    private Application app;
    
    public static final int LEFT_INDEX  = 0;
    public static final int RIGHT_INDEX = 1;
    
    public void setApplication(Application a) {
        app=a;
    }
    
    public Application getApplication() {
        return app;
    }
    
    public static final int ARRAY_SIZE = 15;
    
    
    protected String[] keyArray = null;
    public static final boolean DEBUG = false;
    
    
    /**
     * In this exercise a drag operation performs a swap
     * @return
     */
    public boolean getSwapBehaviour() {
        return true;
    }
    
    long seed = 1;
    
    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }
    
    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }
    
    public FDT[] init() {
        java.util.Random r =  new java.util.Random(seed);	       
        keyArray = createKeyArray(r, 'A', 'Z', ARRAY_SIZE);	
        
        FDT[] structures = getInitialStructures();
        
        userTable = (ColorIgnoreTable)structures[0];
        userStack = (Stack)structures[1];
        
        return structures;
    }
    
    public FDT[] getInitialStructures(){
        
        
        ColorIgnoreTable userTable = new ColorIgnoreTable(makeKeys(keyArray));
        userTable.selectRange(0, ARRAY_SIZE - 1);
        
        Stack userStack = new Stack();
        pushParameters(userStack, userTable, new int[]{0,ARRAY_SIZE-1});
        pushHighlights(userTable, new int[]{0,ARRAY_SIZE-1});
        
        FDT[]  s = { userTable, userStack };
        return s;
    }
    
    public FDT[] getSolveStructures(){
        
        Table nTable = new ColorIgnoreTable(makeKeys(keyArray));
        nTable.selectRange(0, ARRAY_SIZE - 1);
        
        Stack nStack = new Stack();
        pushParameters(nStack, nTable, new int[]{0,ARRAY_SIZE-1});
        pushHighlights(nTable, new int[]{0,ARRAY_SIZE-1});
        
        FDT[]  s = { nTable, nStack };
        return s;
    }
    
    
    public String[] getStructureNames() {
        String[] s = { "Table of Keys to be sorted", "Call Stack"};
        return s;
    }
    
    public String[] getStructureVisualisations() {
        String[] s = { "array", "array"};
        return s;
    }
    
    public String[] getModelAnswerVisualisations() {
        return getStructureVisualisations();
    }
    
    public String[] getModelAnswerNames() {
        return getStructureNames();
    }
    
    public SimulationExerciseModel getModelAnswer() {
        return this;
    }
    
    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        
        VisualTypeConf stack = new VisualTypeConf();
        return new VisualTypeConf[] { tabl, stack };
    }
    
    
    public FDT[] getAnswer() {
        FDT[] s = { userTable, userStack };
        return s;
    }
    
    
 
    
    
    public FDT[] solve() {
        return solveHighlight(false);
    }
    
    public FDT[] makeModelAnswer() {
        return solveHighlight(false);
    }
    
    public FDT[] solveHighlight(boolean hlight){
        FDT[] structures = getSolveStructures();
        Table mTable     = (Table)structures[0];
        Stack mStack     = (Stack)structures[1];
        
        Animator a = Animator.getActiveAnimator();
        quickSort(a, mTable, mStack, hlight);
        
        a.startOperation();
        popParameters(mStack);	
        for (int i=0; i<mTable.size(); i++)
            highlightKey(mTable, i, Color.lightGray);	    
        a.endOperation();	
        
        return new FDT[] {mTable, mStack};
    }
    
    
    // --------RotatedContainers begin
    public boolean[] getContainerRotations(){
        return new boolean[]{false, true};
    }
    public boolean[] getModelAnswerRotations() {
        return getContainerRotations();
    }
    // --------RotatedContainers end
    
    
    
    public String getDescription() {
        return "";
    }
    
    //========================================================
    
    
    
    /**
     Sorts the given table using recursive quicksort.
     
     @param a animator for animating the algorithm
     @param table the table to be sorted
     @param size the size of the table
     @param stack the recursion stack
     @param highlightOn a flag indicating whether to highlight keys in the table when
     the algorithm proceeds. Highlighting is in generating the model solution.
     */
    static void quickSort(Animator a, Table table, Stack stack, boolean highlightOn) {
        
        int cutoff = 3;
        //PEEK the top of the stack (get parameters)
        
        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            top = ((Key)top).getObject();
        
        Table paramsTable = (Table)top;	
        int[] params = getParameters(paramsTable);
        
        int left  = params[LEFT_INDEX];
        int right = params[RIGHT_INDEX];	
        
        try{
            if ((right - left) < cutoff){
                
                //STUPID SORT FOR 1-3 elements 
                if (right == left)
                    return;
                
                a.startOperation();			
                if (getParameter(table,right) < getParameter(table,left))
                    swap(a, table, left, right, highlightOn);
                
                if (getParameter(table,(left + right) / 2) < getParameter(table,left))
                    swap(a, table, left, (left + right) / 2, highlightOn);
                
                if (getParameter(table,(left + right) / 2) > getParameter(table,right))
                    swap(a, table, right, (left + right) / 2, highlightOn);
                a.endOperation();	
                
                return;
            }
            
            int leftCursor = left;
            int rightCursor = right - 1;
            
            int pivot = getParameter(table, right);
            
            do {
                while (getParameter(table, leftCursor) < pivot) {
                    leftCursor++;
                }
                
                while (rightCursor >= 0 && getParameter(table,rightCursor) >= pivot) {
                    rightCursor--;
                }
                
                if (leftCursor < rightCursor){
                    swap(a, table, leftCursor, rightCursor, highlightOn);		
                }
            }while (leftCursor < rightCursor);
            
            if (leftCursor != right)
                swap(a, table, leftCursor, right, highlightOn);		
            
            int[] leftPartition  = new int[2];
            int[] rightPartition = new int[2];
            
            leftPartition[LEFT_INDEX]  = left;
            leftPartition[RIGHT_INDEX] = leftCursor-1;
            
            rightPartition[LEFT_INDEX]  = leftCursor+1;
            rightPartition[RIGHT_INDEX] = right;
            
            if (leftCursor -1 > left){
                if(highlightOn){
                    a.startOperation();	
                    table.selectRange(left, leftCursor -1);
                    a.endOperation();	
                }
                
                pushParametersAnimated(stack, table, leftPartition, a);
                quickSort(a, table, stack, highlightOn);
                popParametersAnimated(stack, table, a);
            }
            
            if (leftCursor+1 < right){
                if(highlightOn){
                    a.startOperation();	
                    table.selectRange(leftCursor+1, right);
                    a.endOperation();	
                }
                
                pushParametersAnimated(stack, table, rightPartition, a);
                quickSort(a, table, stack, highlightOn);
                popParametersAnimated(stack, table, a);		    		 
            }
            
        }catch(NumberFormatException e){
            Note.show(null, "Illegal value in table.");	    
        }
        
    }	        
    
    static void swap(Animator a, Table table, int hi, int lo, boolean highlightOn){
        
        a.startOperation();
        //clearHighlight(table,hi);
        //clearHighlight(table,lo);
        
        table.swap(hi,lo);
        a.endOperation();	
        
    }
    
    
    
    
    
    static void highlightTable(Table t, int lo, int hi, Color color) {
        for (int i = lo; i <= hi; i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                highlightKey(((ColorableKey)o), color);
            }
        }
    }
    
    
    static void highlightKey(Object k, Color c) {
        if (k instanceof ColorableKey)
            ((ColorableKey)k).getStyleSheet().setDefaultBackgroundColor(c);
    }
    
    
    static void highlightKey(Table t, int index, Color c) {
        //Note.out(null, "index : "+index+" ("+t.getObject(index).getClass().getName()+")");
        ((ColorableKey)t.getObject(index)).getStyleSheet().setDefaultBackgroundColor(c);	
    }
    
    static void clearHighlight(Table t, int index) {		
        Object o = t.getObject(index);
        if (o instanceof ColorableKey) {
            ((ColorableKey)o).setDefaultBackgroundColor(Color.white);
        }		
    }
    
    static void clearHighlight(Table t) {
        
        for (int i = 0; i < t.size(); i++) {
            Object o = t.getObject(i);
            if (o instanceof ColorableKey) {
                ((ColorableKey)o).setDefaultBackgroundColor(Color.white);
            }
        }
        
    }
    
    
    static void pushParametersAnimated(Stack stack, Table table, int[] params, Animator a) {
        a.startOperation();
        
        table.selectRange(params[0], params[1]);
        
        pushParameters(stack, table, params);	
        pushHighlights(table, params);
        a.endOperation();
        
        
    }
    
    static void pushParameters(Stack stack, Table table, int[] params) {
        Table t = new Table(params.length);
        setParameters(t, params);
        stack.insert((FDT)t);	
    }
    
    static void pushHighlights(Table table, int[] params){
        //	clearHighlight(table);
        //for (int i=0; i<params[0]; i++)
        //    highlightKey(table, i, Color.lightGray);
        
        if (params[1] - params[0] < 3) 
            for (int i=params[0]; i<=params[1]; i++)
                highlightKey(table, i, Color.yellow);
        else
            highlightKey(table, params[1], Color.orange);
    }
    
    
    static int[] popParametersAnimated(Stack stack, Table table, Animator a) {
        a.startOperation();
        int [] arr=popParameters(stack);	
        popHighlights(stack, table, arr);
        a.endOperation();	
        
        
        return arr;
    }
    
    static void popHighlights(Stack stack, Table table, int[] arr){
        int [] newTop = peekParameters(stack);
        table.selectRange(newTop[0], newTop[1]);	
        
        //clearHighlight(table);
        for (int i=0; i<=arr[1]; i++)
            highlightKey(table, i, Color.lightGray);
        
        if (arr[0] == newTop[0]){ // Is this the left subarea
            if (arr[1]+1 <= table.getLast()){
                highlightKey(table, arr[1]+1, Color.orange);
            }
        }else{
            if (arr[0]-1 >= table.getFirst()){
                highlightKey(table, arr[0]-1, Color.orange);
            }
        }
    }
    
    static int[] popParameters(Stack stack) {
        int[] params = peekParameters(stack);
        stack.pop(null);
        return params;
    }
    
    static int[] peekParameters(Stack stack) {
        Table params;
        
        Object top = stack.getTopOfStack();
        if (top instanceof Key)
            params = (Table)((Key)top).getObject();
        else
            params = (Table)top;
        
        if (params==null)
            return null;	
        
        return getParameters(params);
    }
    
    
    
    //---conversion methods for int tables----------------
    static void setParameters(Table t, int[] params) {
        for (int i = 0; i < params.length; i++) {
            t.setObject(params[i]+"", i);
        }
    }
    
    static int[] getParameters(Table params) {
        
        
        int[] p = new int[params.size()];
        for (int i = 0; i < p.length; i++) {
            try {
                String s=params.getObject(i).toString();
                p[i] = Integer.parseInt(s);
            } catch (NumberFormatException e) {		
                p[i] = -1;
            } catch (NullPointerException e) {
                p[i] = -1;
            }
        }
        return p;
    }
    
    static int getParameter(Table params, int index){
        String s = null;
        try{
            s=params.getObject(index).toString();
        }catch(NullPointerException idiot){
            Note.out(null, ""+index);
            throw idiot;
        }
        
        
        if (s == null)
            Note.show(null,"NULL!");
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {		
            return s.charAt(0); //Array Elements are sorted by first character
        } catch (NullPointerException e) {
            return -1;
        }
        
    }
    //---------------------------------------------------------
    
    
    /**
     @param r random number generator
     @param size number of characters of the key to create
     */
    /*    public String[] createKeyArray(Random r, int min, int max, int size) {
     String[] str = new String[size];
     
     for (int i=0 ; i < size; i++) {
     // Ascii codes of letters A-Z are from 1 to 26.
      int rand = ((int)(r.nextDouble() * 26));
      str[i] = "" + rand;//(char)('A' + rand);
      }
      return str;
      }
      */
    
    public String[] createKeyArray(Random r, char min, char max, int size) {
        String[] str = new String[size];
        
        boolean duplicatePivot = true;
        
        while (duplicatePivot){
            for (int i=0 ; i < size; i++) {
                // Ascii codes of letters A-Z are from 1 to 26.
                int rand = ((int)(min + r.nextDouble() * (max-min)));
                str[i] = "" + (char)rand;
            }
            
            java.util.Arrays.sort(str, String.CASE_INSENSITIVE_ORDER);
            
            //First best case pivot
            if (!unique(str, str[size / 2]))
                continue;
            
            //Second best case pivot (left side)
            if (!unique(str, str[(size-1)/4]))
                continue;
            
            //Worst case 1
            if (!unique(str, str[size-1]))
                continue;
            
            //Worst case 2
            if (!unique(str, str[size-2]))
                continue;
            
            //Worst case 3
            if (!unique(str, str[size-3]))
                continue;
            
            //Additional case 4 (because of pivot coloring problem)
            if (!unique(str, str[size-4]))
                continue;
            
            
            duplicatePivot = false;	    
        }
        
        //for (int i=0; i<str.length; i++)
        //    Note.out(null, str[i]);
        
        //place first pivot and worst case pivot
        _swap(str, size-1, size/2);
        
        //place first left pivot
        _swap(str, (size -1) / 4, ( size/2)-1);
        
        //Mix everything else
        _mix(str, 0, (size/2)-2, r); //below best case pivot
        _mix(str, (size/2)+1, size-4, r); //between the worst case pivots
        
        
        // inverse sort begins
        
        int left  = (size/2) -1;
        int right = (size/2);
        
        while (left >= 0 && right <size-1){
            if (r.nextBoolean()){
                _swap(str, left, right);
                left--;
                right++;
            }
            else{
                if (r.nextBoolean()) left--;
                if (r.nextBoolean()) right++;		
            }
        }
        
        return str;
    }
    
    private void _swap(String[] tab, int a, int b){
        String temp = tab[a];
        tab[a] = tab[b];
        tab[b] = temp;
    }
    
    private void _mix(String[] tab, int lo, int hi, Random r){
        int times = r.nextInt((hi-lo));
        
        for (int i=0;i <times; i++){
            _swap(tab, lo + r.nextInt(hi - lo + 1), lo + r.nextInt(hi - lo + 1));
        }		
    }
    
    private boolean unique(String[] tab, String item){
        int count = 0;
        for (int i=0; i< tab.length; i++)
            if (tab[i].equals(item))
                count++;
        return count == 1;
    }
    
    private int _index(String[] tab, String item){
        for (int i=0; i< tab.length; i++)
            if (tab[i].equals(item))
                return i;
        return -1;
    }
    
    
    
    // Buttons
    
    public Component[] getAWTComponents() {
        JButton push = new JButton("Call");
        push.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                pushButton();
            }
        });
        
        JButton pop = new JButton("Return");
        pop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                popButton();
            }
        });
        
        return new Component[]{push, pop};
    }
    
    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }
    
    
    boolean containsStructure(VisualType vt, FDT s) {
        if (vt == null)
            return false;
        else if (vt.getStructure() == null)
            return false;
        else if (vt.getStructure() == s) 
            return true;
        else if (!(vt.getParent() instanceof VisualType))
            return false;
        else
            return containsStructure((VisualType)vt.getParent(), s);
    }
    
    
    
    
    void pushButton() {
        int[] params =new int[2]; 
        
        params[0] = userTable.selectionStart();
        params[1] = userTable.selectionEnd();
        
        getApplication().getAnimator().startOperation();
        pushParameters(userStack, userTable, params);	
        pushHighlights(userTable, params);
        getApplication().getAnimator().endOperation();
        
        
        validateApplication();
    }
    
    void popButton() {
        
        //in this exercise one shouldn't pop the final element off the stack
        if (userStack.getStackSize()==0)
            return;
        
        getApplication().getAnimator().startOperation();
        int[] parameters = popParameters(userStack);
        
        //>ADDED
        /* *	Object top = userStack.getTopOfStack();
         if (top instanceof Key)
         top = ((Key)top).getObject();
         
         Table paramsTable = (Table)top;	
         parameters = getParameters(paramsTable);
         
         * */
        //<ADDED
        if (userStack.getStackSize() == 0){
            for (int i=0; i<userTable.size(); i++)
                highlightKey(userTable, i, Color.lightGray);	    
        }
        else{
            popHighlights(userStack, userTable, parameters);
        }
        getApplication().getAnimator().endOperation();
        
        //*userTable.selectRange(parameters[0], parameters[1]);
        
        validateApplication();
    }
    
    
    void clearTable(Table t) {
        for (int i = 0; i < t.size(); i++) {
            t.setObject(" ", i);
        }
    }
    
    
    void validateApplication() {
        if (app != null) {
            app.validate();
            app.validateAnimator();
        }
    }
    
    
    
    private ColorableKey[] makeKeys(String[] keys) {
        ColorableKey[] vKeys = new ColorableKey[keys.length];
        for (int i = 0; i < keys.length; i++) {
            vKeys[i] = new ColorableKey(keys[i]);
        } 
        return vKeys;
    }
    
    static class ColorIgnoreTable extends Table{
        
        public ColorIgnoreTable(Key[] keys){
            super(keys);
        }
        
        public boolean equals(Object other){
            //Note.out(null, "called!");
            
            Array a = this;
            Array b = (Array)other;
            
            if(a == b) return true;
            if(a == null || b == null) return false;
            
            int aFirst = a.getFirst();
            int aLast = a.getLast();
            
            if(aFirst != b.getFirst() || aLast != b.getLast())
                return false;
            
            Object aObj;
            Object bObj;
            
            for(int i = aFirst; i < aLast; i++) {
                aObj = a.getObject(i);
                bObj = b.getObject(i);      
                if(aObj == bObj) continue;
                if(aObj != null && !(((Key)aObj).getComparisonString().equals(((Key)bObj).getComparisonString())))
                    return false;
                if(aObj == null || bObj == null)
                    return false;
            }
            if (aFirst == aLast) {
                aObj = a.getObject(aFirst);
                bObj = b.getObject(aFirst);
                if(aObj == bObj) return true;
                if(aObj != null && !(((Key)aObj).getComparisonString().equals(((Key)bObj).getComparisonString())))
                    return false;
                if(aObj == null || bObj == null)
                    return false;
            }
            return true;
            
            
        }
        
        static final long serialVersionUID = -5606843093473877921L;	
    }
    static final long serialVersionUID = -2884066769614640771L;
}
