/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import matrix.animation.Animator;
import content.exercises.structures.PriorityExerVertex;

public class SSSP_Dijkstra extends MST_Prim {

    public SSSP_Dijkstra(String label) {
        super(label);
    }

    public SSSP_Dijkstra(PriorityExerVertex[] g, PriorityExerVertex r) {
        super(g, r);
    }

    public PriorityExerVertex run() {
        Animator a = Animator.getActiveAnimator();
        PriorityQueue Q = new PriorityQueue();
        //Note.out(this, "Running SSSP_Dijkstra...");
        for (int u=0; u<g.length; u++) {
            g[u].setPriority(Integer.MAX_VALUE);
            g[u].setVisited(false);
            g[u].setFather(null);
        }
        a.startOperation();
        //Note.out(this, "Init complete");
        r.setPriority(0);
        //Note.out(this, "Inserting root into Q...");
        Q.insert(r);
        int count = 0;
        while (!Q.isEmpty()) {
            PriorityExerVertex u;
            u = Q.deleteMin();
            if (u.getVisited())
                continue;
            //Note.out(this, "->u="+u);
            u.setVisited(true);
            u.setLabel(++count+"");
            // add edge into the spanning tree
            //Note.out(this, "add edge " + u.getFather() + "->" + u);
            PriorityExerVertex f = u.getFather();
            if (f != null) {
                f.referenceSelectedTo(u);
                a.endOperation();
                a.startOperation();
            }
            PriorityExerVertex[] pev = u.getAdjacentVertices();
            for (int i=0; i<pev.length; i++) {
                PriorityExerVertex v = pev[i];
                int w = u.getWeight(v); // W(e=(u,v))
                if ((!v.getVisited()) && (u.getPriority() + w < v.getPriority())) {
                    //Note.out(this, "Q.insert " + v);
                    v.setFather(u);
                    if (v.getPriority() < Integer.MAX_VALUE) {
                        v.setPriority(u.getPriority() + w);
                        Q.update(v);
                    } else {
                        v.setPriority(u.getPriority() + w);
                        Q.insert(v);
                    }
                }
            }
        }
        a.endOperation();
        return r;
    }
    static final long serialVersionUID = -7456705255350583678L;
}

