package content.exercises;

import java.awt.Color;

import matrix.animation.Animator;
import matrix.structures.FDT.probe.Key;
import content.exercises.structures.ExerLabeledGraph_DH;
import content.exercises.structures.ExerQueue_DH;
import content.exercises.structures.ExerLabeledHeap_DH;
import content.exercises.structures.PriorityExerVertex;
import content.exercises.structures.PriorityExerVertex_DH;

/**
 * This class solves the Dijkstra_Heap exercise by using the three structures
 * shown in the exercise (that is, graph, heap, and queue).
 */
public class SSSP_Dijkstra_Heap {

    private ExerLabeledGraph_DH graph;
    private ExerLabeledHeap_DH heap;
    private ExerQueue_DH queue;
    private PriorityExerVertex_DH startNode;
    private PriorityExerVertex_DH[] graphVertices;
    
    /**
     * Constructor which gets all the structures the algorithm needs.
     * @param g graph
     * @param h heap
     * @param q queu
     * @param node graph node where the algorithm should start
     * @param nodes all the graph nodes
     */
    public SSSP_Dijkstra_Heap(ExerLabeledGraph_DH g, ExerLabeledHeap_DH h, ExerQueue_DH q, PriorityExerVertex_DH node, PriorityExerVertex_DH[] nodes) {
        graph = g;
        heap = h;
        queue = q;
        startNode = node;
        graphVertices = nodes;
    }
    
    /**
     * This method initializes the nodes used in the model answer so that everytime 
     * the model answer is shown, the graph nodes are initialized. 
     *
     */
    public void initializeNodes() {
        for (int i = 0; i < graphVertices.length; i++) {
            graphVertices[i].setColor(Color.white);
            graphVertices[i].setDistanceOfNode(Integer.MAX_VALUE);
            graphVertices[i].setFrom(null);

            PriorityExerVertex[] adjs = graphVertices[i].getAdjacentVertices();
            for (int j = 0; j < adjs.length; j++) {
                graphVertices[i].setReferenceColor(j, Color.blue);
            }
        }
    }

    /**
     * This method runs the Dijkstra's algorithm for the given structures.
     *
     */
    public void run() {
        //first initialize the nodes of the graph
        initializeNodes();
        Animator a = Animator.getActiveAnimator();

        //first insert the key of the first node to heap
        a.startOperation();
        heap.insert(startNode.getElement());
        a.endOperation();

        //help variables
        Key tempKey = null;
        PriorityExerVertex_DH tempNode = null;
        PriorityExerVertex[] succs = null;
        
        while (heap.getMinimumKey() != null) {
            a.startOperation();
            //insert the key from the top of heap to the queue
            tempKey = heap.getMinimumKey();
            queue.insert(tempKey); //queue.insert removes the key from the heap automatically
            a.endOperation();
            
            tempNode = null;
            //find the node from the graph that corresponds to the key that was inserted in the queue
            for (int i = 0; i < graphVertices.length; i++) {
                if (graphVertices[i].getElement().equals(tempKey)) {
                    tempNode = graphVertices[i];
                    break;
                }
            }
            //get successors of the chosen node
            succs = tempNode.getAdjacentVertices();
            
            for (int i = 0; i < succs.length; i++) {
                if (succs[i].getColor().equals(Color.black)) {
                    //node is already in the queue, so it is not added to the heap
                    continue;
                } else if (succs[i].getColor().equals(Color.gray)) {
                    /* Node is already in the heap, so it is not added again to the heap.
                     * Instead of, check the priorities and if the priority value of the node to be
                     * added is smaller than the value that is already in the heap, update the 
                     * priority value in the heap 
                     */
                    int newDistance = graph.getDistance((Key)succs[i].getElement());
                    if (newDistance < heap.getPriorityOfKey((Key)succs[i].getElement())) {
                        a.startOperation();
                        heap.updatePriorityOfKey((Key)succs[i].getElement(), newDistance);
                        a.endOperation();
                    }
                } else {
                    //node is totally new so add it to the heap
                    a.startOperation();
                    heap.insert(succs[i].getElement());
                    a.endOperation();
                }
            }
            
        }
    }
    
    
}
