/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import content.interfaces.*;
import content.exercises.structures.ExerStackList;
import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.BinaryTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.*;
import matrix.structures.code.probe.PseudoCode;
import matrix.util.RandomKey;

public class Stack_PreOrder implements ComparableExercise, StyledExercise, ModelAnswerNames,
				ConfigureVisualType, LayoutExercise, HasRotatedContainers {

    private BinSearchTree bst = null;
    private ExerStackList l1;
    // private LinkedListImpl l1;
    private matrix.structures.CDT.probe.Stack st;
    
    public static final boolean DEBUG = false;
    private String keys;

    private PseudoCode vCode;
    
    public static final String[] code = {
    	"_1 PROCEDURE StackPreorder(TreeNode tnode)",
    	"    Stack stack = NEW Stack _/1",
    	"",
    	"_2 stack.push(tnode) _/2",
    	"_3 WHILE NOT stack.empty() _/3",
    	"_4     tnode = stack.pop() _/4",
    	"_5     visit(tnode) _/5",
    	"_6     IF tnode.getRight() != NULL _/6",
    	"_7         stack.push(tnode.getRight()) _/7",
    	"_8     IF tnode.getLeft() != NULL _/8",
    	"_9         stack.push(tnode.getLeft()) _/9",
    	"",
    	"_10 RETURN _/10"
    };

    
    long seed = 1;
    
    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }
    
    public FDT[] init() {
        bst = new BinSearchTree();
        keys = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed), 10);
        Table t = new Table(keys);
        bst.insert(t);
	st = new matrix.structures.CDT.probe.Stack();
        l1 = new ExerStackList(st);
        FDT[] tmp = {bst, st, l1 };

        return tmp;
    }

    public String[] getStructureVisualisations() {
        return new String[] { "layered tree", "visual array", "list" };
    }

    public String[] getModelAnswerVisualisations() {
        return new String[] { "tree", "list", "list", "pseudocode" };
    }


  public String[] getStructureNames() {
    String[] tmp = { "Binary Tree", "Stack", "Traversing Order" };
    return tmp;
  }

  public String getDescription() {
    return "";
  }


   


  public FDT[] solve() {
    matrix.structures.CDT.probe.Stack resStack = new matrix.structures.CDT.probe.Stack();
    FDT[] fd = getInitialStructures();
    
    //    Table tbl = new Table(keys);
    BinaryTree bt = (BinaryTree)((BinSearchTree)fd[0]).getElement();//(BinaryTree) bst.getElement();
    
    

    /*
  1     while (S.notEmpty()) do
  2         next = S.pop()
  3         visit(next)
  4         if (next->right != NULL) S.push(next->right)
  5         if (next->left != NULL) S.push(next->left)
  

    */
    String order = "";
    java.util.Stack helpStack = new java.util.Stack();
    BinaryTree next,right,left;
    Animator a = Animator.getActiveAnimator();
    a.startOperation();	
    // Push
    helpStack.push(bt);
    resStack.insert(""+bt.getElement());
    if (DEBUG)
        System.out.println(resStack+":"+bt.getElement());
    a.endOperation();
    while(!helpStack.empty()) {
	a.startOperation();	
	// Pop
	order=order+helpStack;
    if (DEBUG)
        System.out.print(resStack+":");

	//resStack.delete(null);
	resStack.pop(null);

	next = (BinaryTree)helpStack.pop();
    if (DEBUG)
        System.out.println(resStack+":");
	a.endOperation();
	if (next == null) break;
	if (next.getRight() != null){
	    a.startOperation();	
	    right = next.getRight();
	    helpStack.push(right);
	    resStack.insert(""+right.getElement());
	    a.endOperation();
	}
	if (next.getLeft() != null) {
	    a.startOperation();
	    left=next.getLeft();
	    helpStack.push(left);
	    resStack.insert(""+left.getElement());
	    a.endOperation();
	}

    }
    if (DEBUG)
        System.out.println(order);

    FDT[] ans = {resStack };
    return ans;
  }

  private void setCodeLine(int line, Animator a) {
	  a.startOperation();
	  vCode.setLocation(line);
	  a.endOperation();
  }

    public FDT[] makeModelAnswer() {
    	FDT[] ini = getInitialStructures();
    	Object node;
    	QueueImpl visited;
    	StackImpl vAux; // vAux is the visual aux
    	java.util.Stack aux;
    	BinaryTree bt;
    	
    	bt = (BinaryTree) ((BinSearchTree) ini[0]).getElement();
    	visited = (QueueImpl) ini[1];
    	vAux = new StackImpl();
    	aux = new java.util.Stack();
    	if (vCode == null) {
    		vCode = new PseudoCode();
    		vCode.setCodeLines(code);
    		vCode.setCodeName("Level order traversal");
    	}
    	vCode.setLocation(1);
    	
    	// Simulation starts here
    	Animator a = Animator.getActiveAnimator();
    	setCodeLine(2, a);
    	
    	a.startOperation();
    	vAux.ADTPush(bt.getElement().toString());
    	vCode.setLocation(3);
    	a.endOperation();
    	aux.push(bt);
    	
    	while (!aux.empty()) {
    		setCodeLine(4, a);
    		
    		a.startOperation();
    		node = vAux.ADTPop();
    		vCode.setLocation(5);
    		a.endOperation();
    		bt = (BinaryTree) aux.pop();
    		
    		a.startOperation();
    		visited.insert(node.toString());
    		vCode.setLocation(6);
    		a.endOperation();
    		
    		if (bt.getRight() != null) {
    			setCodeLine(7, a);
    			
    			a.startOperation();
    			vAux.ADTPush(bt.getRight().getElement().toString());
    			vCode.setLocation(8);
    			a.endOperation();
    			aux.push(bt.getRight());
    		} else {
    			setCodeLine(8, a);
    		}
    		
    		if (bt.getLeft() != null) {
    			setCodeLine(9, a);
    			
    			a.startOperation();
    			vAux.ADTPush(bt.getLeft().getElement().toString());
    			vCode.setLocation(3);
    			a.endOperation();
    			aux.push(bt.getLeft());   			
    		} else {
    			setCodeLine(3, a);
    		}
    	}
    	setCodeLine(10, a);  	
    	// Simulation ends here
    	
    	return new FDT[] { ini[0], vAux, visited, vCode };
    }

  public String[] getModelAnswerNames() {
    String[] s = { "Binary Search Tree", "Stacked Nodes", "List of Visited Nodes", "Pseudo code" };
    return s;
  }

  public FDT[] getInitialStructures() {
    Table tbl = new Table(keys);
    BinSearchTree tr = new BinSearchTree();
    tr.insert(tbl);
    return new FDT[] { tr, new QueueImpl() };
  }

  public FDT[] getSimulatedStructures() {
    return new FDT[] { bst, st, l1 };
  }


  public FDT[] getAnswer() {
    FDT[] tmp = { st };
    return tmp;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tre = new VisualTypeConf();
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf sta = new VisualTypeConf();
    sta.enable("matrix.visual.VisualArray", VisualTypeConf.DROP_OPERATION);
    sta.enable("matrix.visual.VisualArray", VisualTypeConf.HIGHLIGHT_OPERATION);
    sta.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    sta.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    VisualTypeConf dummy = new VisualTypeConf();
    return new VisualTypeConf[] { tre, sta, lis, dummy };
  }

  public GridBagConstraints[] getConstraints() {
	  GridBagConstraints c = new GridBagConstraints();
	  c.insets = new Insets(10, 10, 10, 10);
	  c.gridwidth = GridBagConstraints.REMAINDER;
	  return new GridBagConstraints[] { c, c, c };
  }

  public GridBagConstraints[] getModelAnswerConstraints() {
	  Insets i = new Insets(10, 10, 10, 10);
	  GridBagConstraints r[] = new GridBagConstraints[4];
	  GridBagConstraints c;
	  
	  // Tree and vertical stack
	  c = new GridBagConstraints();
	  c.insets = i;
	  c.gridheight = GridBagConstraints.REMAINDER;
	  c.anchor = GridBagConstraints.PAGE_START;
	  r[0] = c;
	  r[1] = c;
	  // Visited node list and pseudocode
	  c = new GridBagConstraints();
	  c.insets = i;
	  c.gridwidth = GridBagConstraints.REMAINDER;
	  r[2] = c;
	  r[3] = c;
	  
	  return r;
  }

  public boolean[] getContainerRotations() {
	  return new boolean[] { false, false, false };
  }

  public boolean[] getModelAnswerRotations() {
	  return new boolean[] { false, true, false, false };
  }

  public int[] getCompareIndices() {
	  return new int[] {1};
  }
  
  public boolean canRecover() {
      return true;
  }

  static final long serialVersionUID = -2641400449334399953L;

public int[] getGradeIndices() {
	  return new int[] {1};
}
  
}
