package content.exercises.sda;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
//import java.util.Iterator;
import java.util.Random;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import matrix.animation.Animator;
//import matrix.decoration.StyleSheet;
//import matrix.decoration.StyleSheetAdapter;
//import matrix.decoration.Styled;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.CDT;
import matrix.structures.CDT.probe.QueueImpl;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.substructures.Vertex;
import matrix.structures.memory.VirtualObject;
import matrix.structures.simulationextensions.DeleteEnabled;
import matrix.structures.simulationextensions.Selectable;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.spatial.FDT.probe.Circle;
import matrix.structures.spatial.FDT.probe.PolyLine;
import matrix.structures.spatial.FDT.probe.SpatialElement;
//import matrix.structures.spatial.FDT.probe.SpatialElement;
import matrix.structures.spatial.FDT.probe.SpatialElementTable;
import matrix.util.Application;
import matrix.util.Note;

import content.exercises.sda.structures.ExerciseGeometricGraph;
import content.exercises.sda.structures.ExerciseGeometricGraphVertex;
import content.exercises.sda.structures.PointGenerator;
import content.exercises.sda.structures.PointGenerator.IllegalConstraintsException;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ConfigureVisualType;
import content.interfaces.JudgeBlocks;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

public class ExpandingWave implements SimulationExerciseModel, StyledExercise,  ConfigureVisualType, AWTComponentUtilizer,
ActionListener, JudgeBlocks, ModelAnswerNames {

	/**
	 * 
	 */
	private static final long serialVersionUID = 9166763936662194331L;
	
	private WaveGraph graph, modelGraph;
	private WaveList list;
	private QueueImpl modelList;
	private static final String dummy = " ";
	private long seed;
	private static final int MINX = 10;
	private static final int MINY = 10;
	private static final int MAXX = 300;
	private static final int MAXY = 200;
	private static final int NRO_NODES = 6;
	private static final int MIN_DIST = 50; 
	private Point[] points;
	private VirtualObject current, neighbour;
	// transient keyword required for serialization. An instance of Application does not serialize.
	private transient Application app;
	private boolean connect;
	
	private static final Color AREA_COLOR = Color.GRAY;
	private static final Color AREA_LINE_COLOR = Color.GRAY;
	private static final Color LINE_COLOR = Color.BLUE;
	private static final Color CIRCLE_COLOR = Color.DARK_GRAY;
	private static final Color CIRCLE_FILL_COLOR = Color.GRAY;
	private static final Color POINT_COLOR = Color.BLUE;
	private static final Color CURRENT_COLOR = Color.RED;
	private static final Color NEIGHBOUR_COLOR = Color.YELLOW;
	private static final Color NEIGHBOUR_CANDIDATE_COLOR = Color.GREEN;
	
	private static final PaintingStyleDecorator areaDecorator;
	private static final PaintingStyleDecorator pointDecorator;	
	private static final PaintingStyleDecorator currentDecorator;
	private static final PaintingStyleDecorator neighbourDecorator;
	private static final PaintingStyleDecorator neighbourCandidateDecorator;
	private static final PaintingStyleDecorator lineDecorator;
	private static final PaintingStyleDecorator circleDecorator;

	static {
        float[] areaDashPattern = { 4, 2 };
		areaDecorator = new PaintingStyleDecorator();
		areaDecorator.setLineColor(AREA_LINE_COLOR);
		areaDecorator.setFillColor(AREA_COLOR);
		areaDecorator.setTransparency(0.10);
		areaDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL, 5, areaDashPattern, 0);
		pointDecorator = new PaintingStyleDecorator();
		pointDecorator.setLineColor(POINT_COLOR);
		pointDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
		currentDecorator = new PaintingStyleDecorator();
		currentDecorator.setLineColor(CURRENT_COLOR);
		currentDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
		neighbourDecorator = new PaintingStyleDecorator();
		neighbourDecorator.setLineColor(NEIGHBOUR_COLOR);
		neighbourDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
		neighbourCandidateDecorator = new PaintingStyleDecorator();
		neighbourCandidateDecorator.setLineColor(NEIGHBOUR_CANDIDATE_COLOR);
		neighbourCandidateDecorator.setStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
		lineDecorator = new PaintingStyleDecorator();
        lineDecorator.setLineColor(LINE_COLOR);
        lineDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
		circleDecorator = new PaintingStyleDecorator();
        circleDecorator.setLineColor(CIRCLE_COLOR);
        circleDecorator.setFillColor(CIRCLE_FILL_COLOR);
        circleDecorator.setStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);
        circleDecorator.setTransparency(0.35);
	}
	
	public FDT[] solve() {
		if(points == null) {
			Note.err(this, "Points not initialized, cannot solve exercise!");
			return null;
		}
		modelList = new ModelWaveList();
		modelGraph = new WaveGraph();
		int index = 0;
		Vertex[] border = new Vertex[6];
		border[0] = modelGraph.addVertex(MINX, MINY, ""+index++);
		border[1] = modelGraph.addVertex(MINX, MAXY, ""+index++);
		border[2] = modelGraph.addVertex((MINX + (MAXX-MINX)/2), MAXY, ""+index++);
		border[3] = modelGraph.addVertex(MAXX, MAXY, ""+index++);
		border[4] = modelGraph.addVertex(MAXX, MINY, ""+index++);
		border[5] = modelGraph.addVertex((MINX+(MAXX-MINX)/2), MINY, ""+index++);
		for(int i = 0; i <= border.length;i++) {
			int first, second;
			first = (i % border.length);
			second = ((i+1) % border.length);
			makeConnection(border[first], border[second]);
			((WaveVertex)border[first]).setEdge(true);
		}
		// border created
		for(int i = 0; i < points.length; i++) {
			int x, y;
			x = (int)points[i].getX();
			y = (int)points[i].getY();
			Vertex v = modelGraph.addVertex(x,y, ""+index++);
		}
		Animator a = Animator.getActiveAnimator();
		modelList.insert(border[0]);
		WaveVertex current, neighbour, next;
		neighbour = null;
		current = null;
		while(!modelList.isEmpty()) {
			a.startOperation();
			if(neighbour != null) {
				WavePoint p = neighbour.getPoint();
				p.setPaintingStyleDecorator(pointDecorator);
			}
			if(current != null) {
				WavePoint p = current.getPoint();
				p.setPaintingStyleDecorator(pointDecorator);
			}
			current = (WaveVertex)modelList.ADTDequeue();
			WavePoint p = current.getPoint();
			p.setPaintingStyleDecorator(currentDecorator);
			neighbour = getKnownNeighbour(current);
			p = neighbour.getPoint();
			p.setPaintingStyleDecorator(neighbourDecorator);
			a.endOperation();
			if(!neighbour.isQueued()) {
				a.startOperation();
				modelList.ADTEnqueue(neighbour);
				neighbour.setQueued(true);
				a.endOperation();
			}
			while(hasMoreNeighbours(current, neighbour)) {
				next = nextNeighbour(current,neighbour,true);
				a.startOperation();
				makeConnection(current, next);
				makeConnection(neighbour, next);
				p = neighbour.getPoint();
				p.setPaintingStyleDecorator(pointDecorator);
				neighbour = next;
				p = neighbour.getPoint();
				p.setPaintingStyleDecorator(neighbourDecorator);
				modelGraph.setCircle(null);
				a.endOperation();
				if(!neighbour.isQueued()) {
					a.startOperation();
					modelList.ADTEnqueue(neighbour);
					neighbour.setQueued(true);
					a.endOperation();
				}
			}
		}
		return new FDT[] {modelGraph, modelList };
	}

	/**
	 * Calculates the known neighbour of the given vertex.
	 * @param v
	 * @return
	 */
	private WaveVertex getKnownNeighbour(WaveVertex v) {
		Vertex[] vert = v.getSuccessors();
		// handle special border vertex cases.
		if(v.isEdge() &&  (v.getY() == MAXY || (v.getX() == MINX && v.getY() == MINY))) { 			
			WaveVertex n1 = null;
			WaveVertex n2 = null;
			for(int i = 0; i < vert.length; i++) {
				if(vert[i] != null && ((WaveVertex)vert[i]).isEdge()) {
					// this works since each edge vertex has two neighbours that are also edge vertices
					if(n1 == null) {
						n1 = (WaveVertex)vert[i];
					} else {
						n2 = (WaveVertex)vert[i];
					}
				}				
			}
			if(v.getY() == MAXY) {
				return n2;
			} else {
				return n1;
			}
		}
		
		TreeMap map = new TreeMap();
		for(int i = 0; i < vert.length; i++) {
			double d = getAngleToPositiveX(v, (WaveVertex)vert[i]);
			if(d == -1) {
				Note.out(this, "Vertex " + vert[i] + " returned by getSuccessors not successor of " + v);
			} else {
				map.put(new Double(d), vert[i]);
			}
		}
		//Iterator it = map.values().iterator();
		if(map.isEmpty()) {
			Note.err(this, "Base vertex " + v + " has no successors, cannot solve exercise!");
			return null;
		}
		WaveVertex current = (WaveVertex)map.remove(map.lastKey());
		WaveVertex next = null;
		while(!map.isEmpty()) {
			next = (WaveVertex)map.remove(map.lastKey());
			if(!(current.hasSuccessor(next) && next.hasSuccessor(current))) {
				if(current.isEdge() && next.isEdge()) {
					double a = getAngleToPositiveX(v, current);
					double b = getAngleToPositiveX(v, next);
					if(Math.abs(b-a) < (Math.PI/2)) {
						return next;
					}
				}  else {
					return current;
				}
				
			}
			current = next;
		}
		return current;
	}
	
	private boolean hasMoreNeighbours(WaveVertex current, WaveVertex neighbour) {
		return (nextNeighbour(current,neighbour, false) != null);
	}
	
	private WaveVertex nextNeighbour(WaveVertex base, WaveVertex neighbour, boolean animated) {
		int time = 1;
		double x1, x2, y1, y2;
		x1 = base.getX();
		y1 = base.getY();
		x2 = neighbour.getX();
		y2 = neighbour.getY();
		// replace neighbour's coordinates with the coordinates of the line segment halfpoint
		x2 = (x1+x2)/2;
		y2 = (y1+y2)/2;
		// lengths of the edges of the right triangle composed of current point, point stored in x2,y2 and circle
		// center. c is also the radius of the circle.
		double radius = time*Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
		Animator anim = Animator.getActiveAnimator();
		while(time*radius < 200) {
			if(animated) {
				anim.startOperation();
			}
			WaveVertex[] vertices = getNeighbourCandidates(modelGraph, base, neighbour, time);
			if(animated) {
				anim.endOperation();
			}
			if(vertices != null) {
				double maxAngle = 0;
				WaveVertex max = null;
				for(int i=0; i < vertices.length; i++) {
					if(toLeft(base, neighbour, vertices[i])) {
						continue;
					}
					double angle = calculateAngle(base,neighbour,vertices[i]);
					if(max == null) {
						max = vertices[i];
						maxAngle = angle;
					} else if(Math.abs(maxAngle-angle) < 0.0001) {
						WaveVertex tmp = new WaveVertex(base.getX()+1, base.getY(), dummy);
						double a = calculateAngle(base,tmp,vertices[i]);
						double b = calculateAngle(base,tmp,max);
						if(a > b) {
							maxAngle = angle;
							max = vertices[i];
						}
					} else if(angle > maxAngle) {
						maxAngle = angle;
						max = vertices[i];
					}
				}
				if(max != null && !(base.hasSuccessor(max) && neighbour.hasSuccessor(max))) {
					return max;	
				}
			}
			time++;			
		}
		return null;
	}

	/**
	 * Calculates whether the parameter point is to the left side of the vector created by
	 * base and neighbour
	 * @param base
	 * @param neighbour
	 * @param point
	 * @return
	 */
	private boolean toLeft(WaveVertex base, WaveVertex neighbour, WaveVertex point) {
		//return true;
		double a = getAngleToX(base, neighbour);
		double b = getAngleToX(base, point);
		//double tmp = Math.sin(b-a);
		return Math.sin(b-a) > 0;
	}
		
	/**
	 * Calculates the angle created by the parameter vertices. The vertex point is the point of the angle,
	 * and the vertices base and neighbour create the sides.
	 * @param base
	 * @param neighbour
	 * @param point
	 * @return the angle in radians.
	 */
	private double calculateAngle(WaveVertex base, WaveVertex neighbour, WaveVertex point) {
		double i1, i2, j1, j2;
		i1 = base.getX() - point.getX();
		i2 = neighbour.getX() - point.getX();
		j1 = base.getY() - point.getY();
		j2 = neighbour.getY() - point.getY();
		return Math.acos((i1*i2 + j1*j2)/(Math.sqrt(i1*i1+j1*j1)*Math.sqrt(i2*i2+j2*j2)));

	}
	
/**
 * Returns the current neighbour candidates for a given base and neighbour vertices. The parameter
 * integer tells the radius of the search circle as multiples of the half distance between base and
 * neighbour 
 * @param g the graph to search the vertices from
 * @param base base vertex
 * @param neighbour base's know neighbour
 * @param round int larger than 0; the search round
 * @return Array containing neighbour candidates, or null if none found.
 */
	private WaveVertex[] getNeighbourCandidates(WaveGraph g, WaveVertex base, WaveVertex neighbour, int round) {
		if(round < 1) {
			return null;
		}
		double x1, x2, y1, y2, x, y;
		// get the coordinates of the base, and the known neighbour
		x1 = base.getX();
		y1 = base.getY();
		x2 = neighbour.getX();
		y2 = neighbour.getY();
		// replace neighbour's coordinates with the coordinates of the line segment halfpoint
		x2 = (x1+x2)/2;
		y2 = (y1+y2)/2;
		// lengths of the edges of the right triangle composed of current point, point stored in x2,y2 and circle
		// center. c is also the radius of the circle.
		double a, b, c;
		a = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
		// parts of the eigenvector
		double i, j;
		i = (x2-x1)/a;
		// The positive direction of the y-axis is down on the monitor, so this is the correct 
		// calculation!
		j = (y2-y1)/a;
		// coordinates of the circle center
		if(round > 1) {
			c = a*round;
			b = c*Math.cos(Math.asin(a/c));
			// create orthogonal vector for the eigenvector presented by i and j
			double i2, j2;
			i2 = j2 = 0;
			if(i <= 0 && j < 0) {
				i2 = j;
				j2 = -i;
			} else if(i > 0 && j <= 0) {
				i2 = j;
				j2 = -i;
			} else if(i >= 0 && j > 0) {
				i2 = j;
				j2 = -i;
			} else if(i < 0 && j >= 0) {
				i2 = j;
				j2 = -i;
			}
			// calculate circle center
			x = x2 + b*i2;
			y = y2 + b*j2;
		} else {
			x = x2;
			y = y2;
			c = a;
		}
		Circle cir = new Circle(x,y,c,"");
		cir.setPaintingStyleDecorator(circleDecorator);
		g.setCircle(cir);
		// center of the circle has been calculated, now we can start comparing points to it.
		Vertex[] vertices = g.getVertices();
		Vector vec = new Vector();
		for(int k =0; k < vertices.length; k++) {
			WaveVertex v = (WaveVertex)vertices[k];
			// skip special cases
			if(v == base || v == neighbour) {
				continue;
			}
			// if the distance between the current vertex and circle center is smaller than
			// circle radius, it is in the circle.
			if(Math.sqrt((x-v.getX())*(x-v.getX()) + (y-v.getY())*(y-v.getY())) < c) {
				vec.add(v);
			}
		}
		if(vec.size() == 0) {
			return null;
		}
		WaveVertex[] ret = new WaveVertex[vec.size()];
		for(int k = 0; k < ret.length; k++) {
			ret[k] = (WaveVertex)vec.get(k);
		}
		return ret;
	}
	
	/**
	 * Returns the angle the base-neighbour vector has to the positive X axis. The 
	 * angle is calculated counterclockwise, and therefore vectors that have negative
	 * y-component have angles over pi. The angle is returned in radians and is
	 * always a positive value
	 *  
	 * If the parameter vertices aren't neighbours -1 is returned.
	 * 
	 * @param base base vertex, source of the vector
	 * @param neighbour, goal of the vector, neighbour to base
	 * @return the angle, or -1 if vertices aren't neighbours
	 */
	private double getAngleToPositiveX(WaveVertex base, WaveVertex neighbour) {
		if(!base.hasSuccessor(neighbour)) {
			return -1;
		}
		double tmp = getAngleToX(base, neighbour);
		if(!neighbour.isEdge() && tmp == 0) {
			return 2*Math.PI;
		}
		return tmp;
	}

	/**
	 * Calculates the angle between positive x axis and the vector created by the parameter
	 * points. Angle is counterclockwise.
	 * @param base
	 * @param neighbour
	 * @return angle between 0 and 2*Pi
	 */
	private double getAngleToX(WaveVertex base, WaveVertex neighbour) {
		double i1, i2, j1, j2, angle;
		i1 = 1;
		j1 = 0;
		i2 = neighbour.getX() - base.getX();
		j2 = neighbour.getY() - base.getY();
		angle = Math.acos((i1*i2 + j1*j2)/(Math.sqrt(i1*i1+j1*j1)*Math.sqrt(i2*i2+j2*j2)));
		// the above calculate will give a value between 0 and PI, therefore, it must be
		// corrected if the actual counterclockwise angle is greater than PI
		if(j2 < 0) {
			angle = 2*Math.PI - angle;
		}
		return angle;		
	}
	
	public FDT[] makeModelAnswer() {
		return solve();
	}

	public FDT[] init() {
		list = new WaveList();
		graph = new WaveGraph();
		current = new VirtualObject(null, null, "ExpandingWave : The node currently being rotated around");
		neighbour = new VirtualObject(null, null, "ExpandingWave : The cursor in the current rotation");
		Random rnd = new Random();
		int index = 0;
		connect = true;
		// create border points
		Vertex[] border = new Vertex[6];
		border[0] = graph.addVertex(MINX, MINY, ""+index++);
		border[1] = graph.addVertex(MINX, MAXY, ""+index++);
		border[2] = graph.addVertex((MINX + (MAXX-MINX)/2), MAXY, ""+index++);
		border[3] = graph.addVertex(MAXX, MAXY, ""+index++);
		border[4] = graph.addVertex(MAXX, MINY, ""+index++);
		border[5] = graph.addVertex((MINX+(MAXX-MINX)/2), MINY, ""+index++);
		for(int i = 0; i <= border.length;i++) {
			int first, second;
			first = (i % border.length);
			second = ((i+1) % border.length);
			makeConnection(border[first], border[second]);
			((WaveVertex)border[first]).setEdge(true);
		}
		// border created
		Point2D[] points2D = null;
		try {
			points2D = PointGenerator.generatePoints(MINX+(MIN_DIST/2),MINY+(MIN_DIST/2),MAXX-(MIN_DIST/2),MAXY-(MIN_DIST/2),NRO_NODES,MIN_DIST,rnd);
		}catch(IllegalConstraintsException e) {
			Note.err(this, "Cannot create points!");
			return null;
		}
		points = new Point[points2D.length];
		for(int i = 0; i < points.length; i++) {
			int x, y;
			points[i] = new Point((int)points2D[i].getX(),(int)points2D[i].getY());
			x = (int)points[i].getX();
			y = (int)points[i].getY();
			graph.addVertex(x,y, ""+index++);
		}
		list.insert(border[0]);
		FDT[] f = new FDT[] {graph, list};
		return f;
	}

	private void makeConnection(Vertex v1, Vertex v2) {
		v1.addSuccessor(v2);
		v2.addSuccessor(v1);
		if(v1 instanceof WaveVertex && v2 instanceof WaveVertex) {
			WaveVertex v11 = (WaveVertex) v1;
			WaveVertex v22 = (WaveVertex)v2;
			WaveGraph g = v11.getGraph();
			PolyLine p = new PolyLine(new matrix.structures.spatial.FDT.probe.Point[] {v11.getPoint(), v22.getPoint()}, "");
			g.insert(p);
		}
	}
	
	public void setSeed(long seed) {
		this.seed = seed;
	}

	public long getSeed() {
		return seed;
	}

	public String[] getStructureNames() {
		return new String[] { "Points", "queue" };
	}

	public FDT[] getAnswer() {
		return new FDT[] { graph, list };
	}

	public FDT[] getInitialStructures() {
		return new FDT[] { new WaveGraph(), new WaveList() };
	}

	public String getDescription() {
		return "Make a delaunay triangulation for the given set of points using expanding wave from lower left corner";
	}

	public String[] getStructureVisualisations() {
		return new String[] { "area", "list" };
	}

	public String[] getModelAnswerVisualisations() {
		return getStructureVisualisations();
	}

	public VisualTypeConf[] conf() {
		VisualTypeConf gr = new VisualTypeConf();
		VisualTypeConf ls = new VisualTypeConf();
        gr.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.HIGHLIGHT_OPERATION);
        gr.enable("matrix.visual.VisualArea$AreaVisualization", VisualTypeConf.DRAG_OPERATION);
		ls.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
		ls.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
		ls.enable("matrix.visual.VisualListComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
		//ls.enable("matrix.visual.VisualListComponent", VisualTypeConf.DROP_OPERATION);
		ls.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
		ls.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
		return new VisualTypeConf[] { gr, ls };
	}

	private static String connectString = "Connect";
	private static String calculateString = "Calculate Angle";
	private static String circleString = "Get Candidates";
	private static String labelString = "Angle: ";
	private transient JLabel label;
	private int nro;
	//private double x, y;
	
	public Component[] getAWTComponents() {
		nro = 1;
		// create the components
		JRadioButton connect = new JRadioButton(connectString);
		connect.setActionCommand(connectString);
		connect.setSelected(true);
		connect.addActionListener(this);
		JRadioButton calculate = new JRadioButton(calculateString);
		calculate.setActionCommand(calculateString);
		calculate.addActionListener(this);
		ButtonGroup b = new ButtonGroup();
		b.add(connect);
		b.add(calculate);
		JButton circle = new JButton(circleString);
		circle.setActionCommand(circleString);
		circle.addActionListener(this);
		label = new JLabel(labelString+"N/A");
		// create a panel and add the components to it
		GridBagLayout gbl = new GridBagLayout();
		JPanel panel = new JPanel(gbl);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.anchor = GridBagConstraints.WEST;
		panel.add(connect, gbc);
		gbc.insets = new Insets(0,0,20,0);
		panel.add(calculate, gbc);
		gbc.insets = new Insets(0,0,0,0);
		panel.add(label, gbc);
		panel.add(circle, gbc);
		//return new Component[] {connect, calculate, circle, label };
		return new Component[] { panel };
	}

	public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }

    public void setApplication(Application a) {
		app = a;
	}
	
	public Application getApplication() {
		return app;
	}

	public void actionPerformed(ActionEvent e) {
		if(connectString.equals(e.getActionCommand())) {
			connect = true;
			Vertex[] vertices = graph.getVertices();
			getApplication().getAnimator().startOperation();
			for(int k =0; k < vertices.length; k++) {
				WaveVertex v = (WaveVertex)vertices[k];
				// skip special cases
				if(v == getCurrent() || v == getNeighbour()) {
					continue;
				}
				//WavePoint p = v.getPoint();
				//p.setPaintingStyleDecorator(pointDecorator);
				//v.setSelectionTime(Selectable.NOT_SELECTED);
			}
			//graph.setCircle(null);
			getApplication().getAnimator().endOperation();
			nro = 1;
		} else if(calculateString.equals(e.getActionCommand())) {
			connect = false;			
		} else if(circleString.equals(e.getActionCommand())) {
			if(getNeighbour() == null) {
				label.setText(labelString + "N/A");
			} else {
				Vertex[] vertices = graph.getVertices();
				for(int l = 0; l < vertices.length; l++) {
					if(getCurrent() == vertices[l] || getNeighbour() == vertices[l]) {
						continue;
					}
					WavePoint p = ((WaveVertex)vertices[l]).getPoint();
					p.setPaintingStyleDecorator(pointDecorator);
					//((WaveVertex)vertices[l]).setSelectionTime(Selectable.NOT_SELECTED);
				}
				WaveVertex[] vert = getNeighbourCandidates(graph, getCurrent(), getNeighbour(), nro);
				if(vert != null) {
					getApplication().getAnimator().startOperation();
					for(int k = 0; k < vert.length; k++) {
						WavePoint p = vert[k].getPoint();
						p.setPaintingStyleDecorator(neighbourCandidateDecorator);
						//vert[k].setSelectionTime(1);
					}
					getApplication().getAnimator().endOperation();
				}
				nro++;
				double x1, x2, y1, y2;
				x1 = (getCurrent()).getX();
				y1 = (getCurrent()).getY();
				x2 = (getNeighbour()).getX();
				y2 = (getNeighbour()).getY();
				// replace neighbour's coordinates with the coordinates of the line segment halfpoint
				x2 = (x1+x2)/2;
				y2 = (y1+y2)/2;
				// lengths of the edges of the right triangle composed of current point, point stored in x2,y2 and circle
				// center. c is also the radius of the circle.
				if(nro*Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1)) > 200) {
					nro = 1;
				}
				//getApplication().validate();
			}
		}
		getApplication().validate();

	}
	
	public void setCurrent(WaveVertex v) {
		WaveVertex old = getCurrent();
		if(old != null) {
			WavePoint oldp = old.getPoint();
			oldp.setPaintingStyleDecorator(pointDecorator);
		}
		current.setObject(v); 
		if(v != null) {
			WavePoint p = v.getPoint();
			p.setPaintingStyleDecorator(currentDecorator);
		}
	}
	
	public WaveVertex getCurrent() {
		return (WaveVertex)current.getObject();
	}

	public void setNeighbour(WaveVertex v) {
		WaveVertex old = getNeighbour();
		if(old != null && old != getCurrent()) {
			WavePoint oldp = old.getPoint();
			oldp.setPaintingStyleDecorator(pointDecorator);
		}
		neighbour.setObject(v);
		if(v != null) {
			WavePoint p = v.getPoint();
			p.setPaintingStyleDecorator(neighbourDecorator);
		}
	}
	
	public WaveVertex getNeighbour() {
		return (WaveVertex)neighbour.getObject();
	}
	
	class WaveGraph extends ExerciseGeometricGraph implements Area {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -7980760375643245375L;
		private VirtualObject circle;
		private Rectangle bounds;
		private SpatialElementTable elements;
		
		public WaveGraph() {
			super();
			elements = new SpatialElementTable(0,MAXX+10, MAXY+10);
			// the constructor will create an empty key into index 0
			elements.setObject(null,0);
			circle = new VirtualObject(null, null, "(Visualization) circle used for selecting next neighbor");
			bounds = new Rectangle(0,0,MAXX+10,MAXY+10);
		}

		public void setCircle(Circle c) {
			circle.setObject(c);
		}
		
		public Circle getCircle() {
			return (Circle)circle.getObject();
		}
		
		public Vertex getNewVertex(int x, int y, Object key) {
			return new WaveVertex(x,y,key);
		}

		public Vertex addVertex(int x, int y, Object key) {
			Vertex v = super.addVertex(x,y,key);
			if(v instanceof WaveVertex) {
				((WaveVertex)v).setGraph(this);
			}
			return v;
		}
	
		public Area[] getSubAreas() {
			return new Area[0];
		}

		public SpatialElement[] getSpatialElements() {
			Vertex[] v = getVertices();
			int l = v.length;
			if(getCircle() != null) {
				l++;
			}
			l += elements.size();
			int i = 0;
            SpatialElement[] ret = new SpatialElement[l];
			for(;i<v.length;i++) {
				ret[i] = ((WaveVertex)v[i]).getPoint();
			}
			for(;(i-v.length) < elements.size();i++ ) {
				ret[i] = (SpatialElement)elements.getObject((i-v.length));
			}
			if(getCircle() != null) {
				ret[i] = getCircle();
			}			
			return ret;
		}

		public void insert(SpatialComparable e) {
			elements.insert(e);
		}

		public void delete(SpatialComparable e) {
			elements.delete(e);
		}

		public void addSubArea(Area a) {
			// Dummy implementation is sufficient
		}

		public void removeSubArea(Area a) {
			// Dummy implementation is sufficient
		}

		public Rectangle getBoundingPolygon() {
			return bounds;
		}

		public boolean hasPaintingStyleDecorator() {
			return true;
		}

		public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
			// empty function
		}

		public PaintingStyleDecorator getPaintingStyleDecorator() {
			return areaDecorator;
		}
	}
	
	class WaveVertex extends ExerciseGeometricGraphVertex  {

		/**
		 * 
		 */
		private static final long serialVersionUID = -6535524695810206023L;
		long time;
		private boolean queued;
		private boolean edge;
		private WavePoint p;
		private WaveGraph g;
		
		
		public WaveVertex(int x, int y, Object key) {
			super(x,y,key);
			time = 0;
			queued = false;
			String s = "1";
			if(key != null) {
				s = key.toString();
			}
			p = new WavePoint(x,y,s);
			p.setVertex(this);
		}
		
		public WaveVertex(int x, int y, Object key, WavePoint point) {
			this(x,y,key);
			p = point;
		}

		public void setGraph(WaveGraph gg) {
			g = gg;
		}

		public WaveGraph getGraph() {
			return g;
		}
		
		public boolean isQueued() {
			return queued;
		}
		
		public void setQueued(boolean b) {
			queued = b;
		}

		public boolean isEdge() {
			return edge;
		}
		
		public void setEdge(boolean b) {
			edge = b;
		}

		public void setPoint(WavePoint point) {
			p = point;
		}

		public WavePoint getPoint() {
			return p;
		}
		
		public void setSelect(long globalClockTimestamp) {
			Note.out(this, "selected");
			nro = 1;
			// mode where entities will be connected into graph
			if(connect) {
				if(getCurrent() == null) {
					return;
				}
				getApplication().getAnimator().startOperation();
				if(getNeighbour() == null) {
					setNeighbour(this);
					g.setCircle(null);
				} else {
					makeConnection(this, getCurrent());
					makeConnection(this, getNeighbour());
					setNeighbour(this);
					g.setCircle(null);
				}
				WaveGraph g = getGraph();
				Vertex[] v = g.getVertices();
				for(int i = 0; i < v.length; i++) {
					if(getCurrent() == v[i] || getNeighbour() == v[i]) {
						continue;
					}
					WavePoint p = ((WaveVertex)v[i]).getPoint();
					p.setPaintingStyleDecorator(pointDecorator);
				}
				getApplication().getAnimator().endOperation();
			} else { // mode where angles are calculated
				// The angle is calculated using vector calculus. Given two vectors a and b, the following
				// holds: cos(a,b) = (a.b)/(|a|*|b|) where a.b is the scalar product of a and b.
				//therefore a,b = acos((a.b)/(|a|*|b|))
				double i1, i2, j1, j2;
				WaveVertex cur = getCurrent();
				WaveVertex nei = getNeighbour();
				if(cur == null || nei == null) {
					label.setText(labelString + "N/A");
				} else {
					i1 = cur.getX() - getX();
					i2 = nei.getX() - getX();
					j1 = cur.getY() - getY();
					j2 = nei.getY() - getY();
					double angle = Math.acos((i1*i2 + j1*j2)/(Math.sqrt(i1*i1+j1*j1)*Math.sqrt(i2*i2+j2*j2)));
					String aString = "" + Math.toDegrees(angle);
					if(aString.length() < 6) {
						label.setText(labelString + aString);
					} else {
						label.setText(labelString + aString.substring(0, 6));
					}
				}
			}
		}

		public long getSelectionTime() {
			return time;
		}

		public void addSuccessor(Vertex v) {
			if(v instanceof ExerciseGeometricGraphVertex) {
				super.addSuccessor(v);
			}
		}

		
		public void setSelectionTime(long l) {
			time = l;
		}
	}

	class ModelWaveList extends QueueImpl {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -7779934437448440741L;

		public CDT insert(Object key) {
			if(key instanceof WaveVertex) {
				return super.insert(new WaveKey((WaveVertex)key));
			} else {
				Note.out(this, "key "+ key + " not wavevertex!");
				return this;
			}			
		}
	}
	
	class WaveList extends QueueImpl implements DeleteEnabled {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -8099187922378329484L;


		public CDT insert(Object key) {
			if(key instanceof WavePoint) {
				return insert(((WavePoint)key).getVertex());
			}
			if(key instanceof WaveVertex) {
				return super.insert(new WaveKey((WaveVertex)key));
			} else {
				Note.out(this, "key "+ key + " not wavevertex!");
				return this;
			}
		}

		public CDT delete(Object o) {
			if(o instanceof WaveKey) {
				WaveVertex v = (WaveVertex)((WaveKey)o).getObject();
				setCurrent(v);
				setNeighbour(null);
				return super.delete(o);
			}
			return this;
		}
		
		
		public boolean shouldDelete() {
			return true;
		}
	}	
	
	class WavePoint extends matrix.structures.spatial.FDT.probe.Point implements Selectable {

		/**
		 * 
		 */
		private static final long serialVersionUID = 7609514819713572702L;
		private WaveVertex vertex;
		
		public WavePoint(int x, int y, String s) {
			super(x,y,s);
			setDraggable(true);
			this.setPaintingStyleDecorator(pointDecorator);
		}

		public void setVertex(WaveVertex v) {
			vertex = v;
		}
		
		public WaveVertex getVertex() {
			return vertex;
		}
		
		public void setSelect(long globalClockTimestamp) {
			vertex.setSelect(globalClockTimestamp);
		}

		public long getSelectionTime() {
			return vertex.getSelectionTime();
		}
		
	}
	/*
	class NonRandom {
		
		private int[] nros = {50, 190, 60, 60, 110, 255, 150, 270, 190, 75, 200, 225, 215, 130, 270, 225, 295, 65, 330, 250 };
		private int i;
		
		NonRandom() {
			i = 0;
		}
		
		public int nextInt(int foo) {
			int ret = nros[i];
			i = (i+1)%(NRO_NODES*2);
			return ret;
		}
	}
	*/
	public static String makeVisualizationString(WaveVertex v) {
		return "("+v.getX()+","+v.getY()+")";
	}

	class WaveKey extends Key {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 2304433957459360631L;

		WaveKey(WaveVertex v) {
			super(v);
			setVisualizationString(v.getElement().toString());
			setComparisonString(v.getElement().toString());
			//setVisualizationString(makeVisualizationString(v));
			//setComparisonString(getVisualizationString());
		}
		
	}

    public String[] getModelAnswerNames() {
        
        return new String[]{"Triangulation", "Queue"};
    }
	
}