package content.exercises.sda;


import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.substructures.Vertex;
import matrix.util.Application;
import matrix.util.Note;
import content.exercises.sda.structures.DoublyConnectedEdgeList;
import content.exercises.sda.structures.ExerciseGeometricGraph;
import content.exercises.sda.structures.ExerciseGeometricGraphVertex;
import content.exercises.sda.structures.PolygonGenerator;
import content.interfaces.AWTComponentUtilizer;
import content.interfaces.ConfigureVisualType;
import content.interfaces.ModelAnswerNames;
import content.interfaces.SimulationExerciseModel;
import content.interfaces.StyledExercise;

/**
 * Polygon traversal algorithm exercise. The algorithm used is described in
 * Berg et. al, Simple traversal of a subdivision without extra storage, Int. J. Geographical Information Systems, 
 * 1997, Vol. 11, No. 4 :366
 *  
 * @author jtn
 *
 */
public class PolygonTraversal implements SimulationExerciseModel, StyledExercise,  ConfigureVisualType, AWTComponentUtilizer,
	ActionListener, ModelAnswerNames {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8211036894188861714L;
	//private PolygonTraversalGeometricGraph graph;
	private ExerciseGeometricGraph graph;
	DoublyConnectedEdgeList list;
	ExerciseGeometricGraph modelGraph;
	long seed;
	
	// Solve method for this exercise. Currently creates a duplicate of the static
	// test input. The actual solving of the exercise is done by using the
	// DoublyConnectedEdgeList -class.
	public FDT[] solve() {
		modelGraph = new ExerciseGeometricGraph();
		// start stupid copypasted graph initialization
		modelGraph.addVertex(40, 40, "");
		modelGraph.addVertex(60, 140, "");
		modelGraph.addVertex(80, 200, "");
		modelGraph.addVertex(140, 220, "");
		modelGraph.addVertex(200, 240, "");
		modelGraph.addVertex(260, 220, "");
		modelGraph.addVertex(240, 160, "");
		modelGraph.addVertex(270, 80, "");
		modelGraph.addVertex(200, 60, "");
		modelGraph.addVertex(160, 20, "");
		modelGraph.addVertex(100, 40, "");
		modelGraph.addVertex(140, 80, "");
		modelGraph.addVertex(120, 140, "");
		modelGraph.addVertex(160, 180, "");
		modelGraph.addVertex(220, 180, "");
		modelGraph.addVertex(200, 120, "");
		modelGraph.addVertex(180, 80, "");
		modelGraph.addVertex(180, 160, "");
		modelGraph.addVertex(60,60, "");
		Vertex[] v = modelGraph.getVertices();
		setSuc(v,0,1);
		setSuc(v,1,2);
		setSuc(v,2,3);
		setSuc(v,3,4);
		setSuc(v,4,5);
		setSuc(v,5,6);
		setSuc(v,6,7);
		setSuc(v,7,8);
		setSuc(v,8,9);
		setSuc(v,9,10);
		setSuc(v,10,0);
		setSuc(v,10,11);
		setSuc(v,12,11);
		setSuc(v,16,11);
		setSuc(v,16,8);
		setSuc(v,15,16);
		setSuc(v,15,17);
		setSuc(v,17,14);
		setSuc(v,17,12);
		setSuc(v,12,1);
		setSuc(v,12,13);
		setSuc(v,3,13);
		setSuc(v,4,14);
		setSuc(v,6,14);
		// end stupid copypasted graph initialization
		// start stupid DCEL initialization
		DoublyConnectedEdgeList dcel = new DoublyConnectedEdgeList();
		dcel.setGraph(modelGraph);
		modelGraph.visit(40,40,100,40);
		addPolygon(modelGraph, dcel, new Vertex[] {v[0],v[10],v[11],v[12],v[1]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[1],v[12],v[13],v[3],v[2]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[3],v[13],v[12],v[17],v[14],v[4]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[4],v[14],v[6],v[5]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[6],v[14],v[17],v[15],v[16],v[8], v[7]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[11],v[16],v[15],v[17],v[12]});
		addPolygon(modelGraph,dcel, new Vertex[] {v[8],v[16],v[11],v[10],v[9]});
		dcel.polygonTraverse(modelGraph.getX(v[18]), modelGraph.getY(v[18]),modelGraph.getX(v[10]), modelGraph.getY(v[10]),
				modelGraph.getX(v[11]), modelGraph.getY(v[11]));
		return new FDT[] { modelGraph };
	}
	
	/**
	 * Adds a new polygon to the given DCEL-structure. The vertices of the polygon must be in the parameter
	 * ExerciseGeometricGraph. The structures do not check whether the vertices really create a valid
	 * polygon. Possible checks must be done elsewhere. The vertices in the array must be given
	 * in a counterclockwise order, starting from any vertex on the border of the cell.
	 * 
	 * @param gr ExerciseGeometricGraph that contains all vertices in the parameter array
	 * @param dcel DoublyConnectedEdgeList where the new polygon is to be added
	 * @param vertices Array containing the vertices of this polygon.
	 */
	private void addPolygon(ExerciseGeometricGraph gr, DoublyConnectedEdgeList dcel, Vertex[] vertices) {
		int[] x, y;
		x = new int[vertices.length];
		y = new int[vertices.length];
		for(int i = 0; i < x.length; i++) {
			x[i]=gr.getX(vertices[i]);
			y[i]=gr.getY(vertices[i]);
		}
		dcel.addCell(x,y);
	}

	public FDT[] makeModelAnswer() {
		return solve();
	}

	public FDT[] init() {
		checkEntry = false;
		// start stupid, static graph initialization
		/*
		graph = new PolygonTraversalGeometricGraph();
		graph.addVertex(40, 40, "");
		graph.addVertex(60, 140, "");
		graph.addVertex(80, 200, "");
		graph.addVertex(140, 220, "");
		graph.addVertex(200, 240, "");
		graph.addVertex(260, 220, "");
		graph.addVertex(240, 160, "");
		graph.addVertex(270, 80, "");
		graph.addVertex(200, 60, "");
		graph.addVertex(160, 20, "");
		graph.addVertex(100, 40, "");
		graph.addVertex(140, 80, "");
		graph.addVertex(120, 140, "");
		graph.addVertex(160, 180, "");
		graph.addVertex(220, 180, "");
		graph.addVertex(200, 120, "");
		graph.addVertex(180, 80, "");
		graph.addVertex(180, 160, "");
		graph.addVertex(60,60,"");
		Vertex[] v = graph.getVertices();
		setSuc(v,0,1);
		setSuc(v,1,2);
		setSuc(v,2,3);
		setSuc(v,3,4);
		setSuc(v,4,5);
		setSuc(v,5,6);
		setSuc(v,6,7);
		setSuc(v,7,8);
		setSuc(v,8,9);
		setSuc(v,9,10);
		setSuc(v,10,0);
		setSuc(v,10,11);
		setSuc(v,12,11);
		setSuc(v,16,11);
		setSuc(v,16,8);
		setSuc(v,15,16);
		setSuc(v,15,17);
		setSuc(v,17,14);
		setSuc(v,17,12);
		setSuc(v,12,1);
		setSuc(v,12,13);
		setSuc(v,3,13);
		setSuc(v,4,14);
		setSuc(v,6,14);
*/
		// end stupid, static graph initialization
		// begin stupid, static DCEL initialization
/*
		list = new DoublyConnectedEdgeList();
		list.setGraph(graph);
		graph.setDecl(list);
		graph.visit(40,40, 100, 40);
		addPolygon(graph, list, new Vertex[] {v[0],v[10],v[11],v[12],v[1]});
		addPolygon(graph, list, new Vertex[] {v[1],v[12],v[13],v[3],v[2]});
		addPolygon(graph, list, new Vertex[] {v[3],v[13],v[12],v[17],v[14],v[4]});
		addPolygon(graph, list, new Vertex[] {v[4],v[14],v[6],v[5]});
		addPolygon(graph, list, new Vertex[] {v[6],v[14],v[17],v[15],v[16],v[8], v[7]});
		addPolygon(graph, list, new Vertex[] {v[11],v[16],v[15],v[17],v[12]});
		addPolygon(graph, list, new Vertex[] {v[8],v[16],v[11],v[10],v[9]});
		list.setX(60);
		list.setY(60);
		// end stupid, static DCEL initialization
		 */
		graph = (ExerciseGeometricGraph)PolygonGenerator.generatePolygonMapAsGraph(500,500,250,250,50);
		return new FDT[] { graph };
	}

	private void setSuc(Vertex[] v, int i, int j) {
		v[i].addSuccessor(v[j]);
		v[j].addSuccessor(v[i]);
	}
	
	public void setSeed(long seed) {
		this.seed = seed;
	}

	public long getSeed() {
		return seed;
	}

	public String[] getStructureNames() {
		return new String[] { "Polygon graph" };
	}

	public FDT[] getAnswer() {
		// TODO Auto-generated method stub
		return new FDT[] {graph};
	}

	public FDT[] getInitialStructures() {
		return new FDT[] { new PolygonTraversalGeometricGraph() };
	}

	public String getDescription() {
		return "";
	}

	public String[] getStructureVisualisations() {
		return new String[] { "geometric graph" };
	}

	public String[] getModelAnswerVisualisations() {
		return getStructureVisualisations();
	}

	public VisualTypeConf[] conf() {
		VisualTypeConf gr = new VisualTypeConf();
		//gr.setVisualConfValue("matrix.visual.VisualGeometricGraph", matrix.visual.VisualGraph.DIRECTED, "false");
		gr.setVisualConfValue("matrix.visual.VisualGeometricGraph", VisualTypeConf.FLIPPEDY, "true");
		gr.enable("matrix.visual.VisualReference", VisualTypeConf.HIGHLIGHT_OPERATION);
		gr.setVisualConfValue("matrix.visual.VisualGeometricGraph", matrix.visual.VisualGraph.DIRECTED, "true");
		return new VisualTypeConf[] { gr };
	}
	
	private String traverseString = "Traverse";
	private String checkString = "Find Entry";
	private transient JLabel label;
	
	/**
	 * Creates the AWT components required by this exercise: two radiobuttons and a JLabel for
	 * showing text. The buttons control whether selecting an edge will mark the edge as visited
	 * or gives information whether the edge is the entry half-edge to the cell (polygon) it borders.
	 * 
	 * Radiobutton Traverse tells, when active, to mark a clicked edge as visited.
	 * Radiobutton Find Entry tells, when active, to show whether the clicked half-edge is the entry edge
	 * to the cell it borders.
	 * JLabel tells the state of the exercise. If traversing mode is on, it reads "traversing". If entry
	 * finding mode is on and the latest selected half-edge is entry half-edge, it reads "entry". If the
	 * latest half-edge is not entry it reads "not entry".
	 */
	public Component[] getAWTComponents() {
		Component[] components = new Component[3];
		JRadioButton traverse = new JRadioButton(traverseString);
		traverse.setActionCommand(traverseString);		
		traverse.setSelected(true);
		JRadioButton check = new JRadioButton(checkString);
		check.setActionCommand(checkString);
		ButtonGroup b = new ButtonGroup();
		b.add(traverse);
		b.add(check);
		traverse.addActionListener(this);
		check.addActionListener(this);
		components[0] = traverse;
		components[1] = check;

		label = new JLabel("traversing");
		components[2] = label;
		
		return components;
	}

    public GridBagConstraints[] getAWTComponentConstraints() {
        return null;
    }

	/**
	 * This method is required by the AWTComponentUtilizer method, but is
	 * not needed in this exercise.
	 */
	public void setApplication(Application a) {}

	public void actionPerformed(ActionEvent e) {
		if(traverseString.equals(e.getActionCommand())) {
			checkEntry = false;
			label.setText("traversing");
		}
		if(checkString.equals(e.getActionCommand())) {
			checkEntry = true;
			label.setText("Not Entry");
		}
	}

	private boolean checkEntry;

	/**
	 * Geometric graph used in this exercise. 
	 * @author jtn
	 *
	 */
	public class PolygonTraversalGeometricGraph extends ExerciseGeometricGraph {

		private DoublyConnectedEdgeList dcel;
		
		public PolygonTraversalGeometricGraph() {
			super();
			checkEntry = false;
		}

		public void setDecl(DoublyConnectedEdgeList dcel) {
			this.dcel = dcel;
		}
		

		public Vertex getNewVertex(int x, int y, Object key) {
			return new PPVertex(x,y,key,this);
		}
		
	}

	
	/**
	 * Geometric graph vertex used in this exercise.
	 * @author jtn
	 *
	 */
	class PPVertex extends ExerciseGeometricGraphVertex {

		PolygonTraversalGeometricGraph gr;
		
		public PPVertex(int x, int y, Object o, PolygonTraversalGeometricGraph g) {
			super(x, y, o);
			gr = g;
		}

		/**
		 * Selects an edge and, depending on the selected mode either checks whether the
		 * selected edge is entry half-edge or visits it.
		 */
		public void referenceSelectedTo(FDT to) {
			if(checkEntry) {
				if(gr.dcel == null) {
					Note.err(this, "Error: Cannot solve exercise, DCEL not set");
					return;
				} 
				if(!(to instanceof ExerciseGeometricGraphVertex)) {
					Note.err(this, "Error: reference selected to non-Vertex: " + to);
					return;
				}
				ExerciseGeometricGraphVertex v = (ExerciseGeometricGraphVertex)to;
				int myx, myy, tox, toy;
				myx = getX();
				myy = getY();
				tox = v.getX();
				toy = v.getY();
				if(gr.dcel.isEntry(myx, myy, tox, toy)) {
					label.setText("Entry");
				} else {
					label.setText("Not Entry");
				}
			} else {
				super.referenceSelectedTo(to);
			}
		}
		
	}


	public String[] getModelAnswerNames() {
		return new String[] {"Polygon map"};
	}


	
}
