package content.exercises.sda.structures;



import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;


import matrix.animation.Animator;

/**
 * Incomplete implementation of DCEL that also contains implementation of the polygon traversing algorithm.
 * Requires and instance of ExerciseGeometricGraph that represents the same polygon map to exist in order
 * to work correctly. This class should not be used outside exercises, since it contains several 
 * techniques that should not be used in common Matrix classes.
 * 
 * @author jtn
 *
 */
public class DoublyConnectedEdgeList implements Serializable {

	private HashMap vertices;
	private HashMap cells;
	private HashMap halfEdges;
	private int vid, hid, cid;
	private ExerciseGeometricGraph graph;
	int x, y;
	
	public DoublyConnectedEdgeList() {
		vertices = new HashMap();
		cells = new HashMap();
		halfEdges = new HashMap();
		vid = hid = cid = 0;
	}

	public void setGraph(ExerciseGeometricGraph g) {
		graph = g;
	}
	
	public void setX(int i) {
		x = i;
	}

	public void setY(int i) {
		y = i;
	}
	
	/**
	 *  Traverses the given polygon using the Traversal algorithm described in: '
	 *  
	 *  Berg et. al, Simple traversal of a subdivision without extra storage, Int. J. Geographical Information Systems, 
	 *  1997, Vol. 11, No. 4 :366
	 *  
	 * @param x X-coordinate of the reference point
	 * @param y Y-coordinate of the reference point
	 * @param xstart1 X-coordinate of the origin vertex of the start edge
	 * @param ystart1 Y-coordinate of the origin vertex of the start edge
	 * @param xstart2 X-coordinate of the goal vertex of the start edge
	 * @param ystart2 Y-coordinate of the goal vertex of the start edge
	 */
	public void polygonTraverse(int x, int y, int xstart1, int ystart1, int xstart2, int ystart2) {
		this.x = x;
		this.y = y;
		Vertex start = getRealVertex(xstart1, ystart1);
		Vertex next = getRealVertex(xstart2, ystart2);
		HalfEdge startEdge = getRealHalfEdge(start, next);
		HalfEdge curr = startEdge;
		Animator a = Animator.getActiveAnimator();
		do {
			a.startOperation();
			graph.visit(curr.origin.x, curr.origin.y, curr.next.origin.x, curr.next.origin.y);
			a.endOperation();
			if(curr == entry(curr.incidentFace)) {
				curr = curr.twin.next;
			} else if (curr.twin == entry(curr.twin.incidentFace)){
				curr = curr.twin.next;
			} else {
				curr = curr.next;
			}
		}while (curr != startEdge);
	}

	private HalfEdge entry(Cell c) {
        System.out.println("entry called for " + c);
		if(c == null) {
			return null;
		}
		HalfEdge start = c.incidentEdge;
		HalfEdge curr = start;
		double min = Double.MAX_VALUE;
		double tmp;
		Vector candidates = new Vector();
		do {
			tmp = distance(curr);
			if(Math.abs(tmp - min) < 0.01 && curr.twin.incidentFace != null) {
				candidates.add(curr);
			} else if (tmp < min && curr.twin.incidentFace != null) {
				min = tmp;
				candidates.clear();
				candidates.add(curr);
			}
			curr = curr.next;
		} while(start != curr);
		if(candidates.isEmpty()) {
            return null;
			//throw new RuntimeException("No candidates edge found");
		}
		HalfEdge cand = (HalfEdge)candidates.get(candidates.size()-1);
		if(candidates.size() == 1) {
		    System.out.println("returning "+cand);
            return cand;
		} else {
			double[] d = closest(cand);
			if(Math.abs(cand.origin.x - d[0]) < 0.001 && Math.abs(cand.origin.y - d[0]) < 0.001 && cand.prev.twin.incidentFace != null) {
			    System.out.println("returning tiebreak "+cand.prev);
                return cand.prev;
            } else {
                System.out.println("returning tiebreak"+ cand);
                return cand;
			}
		}
	}

	/**
	 *  Tells whether the halfedge defined by the start and end coordinates is the entry halfedge
	 *  to the cell it borders.
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @return
	 */
	public boolean isEntry(int x1, int y1, int x2, int y2) {
		Vertex v1 = getRealVertex(x1,y1);
		Vertex v2 = getRealVertex(x2,y2);
		HalfEdge he = getRealHalfEdge(v1,v2);
		return he == entry(he.incidentFace);
	}
	
	private double distance(HalfEdge e) {
		double[] d = closest(e);
		return Math.sqrt((x-d[0])*(x-d[0]) + (y-d[1])*(y-d[1])); 
	}
	
/**
 * Calculates and returns the closest point between the given point (x,y) and the
 * parameter halfedge
 * @param e
 * @return
 */
	private double[] closest(HalfEdge e) {

		// The closest point is either a point in the interior of the halfedge, or one of its
		// endpoints. Possible interior point is found by calculating the point where the line
		// defined by the halfedge intersects the orthogonal line defined by (x,y)
		int x1, x2, y1, y2, t1, t2;
		x1 = e.origin.x;
		y1 = e.origin.y;
		x2 = e.next.origin.x;
		y2 = e.next.origin.y;
		// construct vector components	
		t1 = x2-x1;
		t2 = y2-y1;
		// construct components for orthogonal vector 
		int x0 = t2;
		int y0 = -t1;
		// turn vector components into points that, along with (x,y)
		// define a line orthogonal to the line defined by the
		// halfedge e
		x0 = x + x0;
		y0 = y + y0;
		// calculate intersection point using method from http://mathworld.wolfram.com/Line-LineIntersection.html
		
		// calculate denominator
		double den = determinant(x-x0, y-y0, x1-x2, y1-y2); 

		double tmp1 = determinant(x, y, x0, y0);
		double tmp2 = determinant(x1,y1,x2,y2);

		// calculate x and y coordinates of the intersection point
		double xcross = determinant(tmp1, x-x0, tmp2, x1-x2)/den;
		double ycross = determinant(tmp1, y-y0, tmp2, y1-y2)/den;

		// if intersection point is inside line segment defined by hafledge e, the
		// distance is between the intersection point and (x,y)
		if(((xcross < x1 && xcross > x2) || (xcross < x2 && xcross > x1)) &&
			((ycross < y1 && ycross > y2) || (ycross < y2 && xcross > y1))) {
			return new double[] {xcross,ycross};
		} else if(Math.sqrt((x-x1)*(x-x1) + (y-y1)*(y-y1)) <
				Math.sqrt((x-x2)*(x-x2) + (y-y2)*(y-y2))) {
			// else return the halfedge endpoint which is closer to (x,y)
			return new double[] {x1,y1};
		} else {
			return new double[] {x2,y2};
		}
		
	}

	private double determinant(double a, double b, double c, double d) {
		return a*d - b*c;
	}

	
	/**
	 * Method for adding new cells. It is assumed that the vertices are given in counterclockwise order.
	 * @param x
	 * @param y
	 */
	public void addCell(int[] x, int[] y) {
		Cell c = new Cell(cid++);
		cells.put(""+c.id, c);		
		Vertex prev, curr;
		prev = getVertex(x[x.length-1],y[x.length-1]);
		HalfEdge he, pair, prevEdge;
		prevEdge = null;
		for(int i = 0; i < x.length; i++) {
			curr = getVertex(x[i], y[i]);
			he = getHalfEdge(prev, curr);
			pair = getHalfEdge(curr, prev);
			he.twin = pair;
			he.origin = prev;
			he.incidentFace = c;
			he.prev = prevEdge;
			if(prevEdge != null) {
				prevEdge.next = he;
			}
			prevEdge = he;
			prev = curr;
		}
		curr = getVertex(x[0],y[0]);
		he = getHalfEdge(prev,curr);
		prevEdge.next = he;
		he.prev = prevEdge;
		c.incidentEdge = he;
	}

	private Vertex getRealVertex(int x, int y) {
		if(vertices.containsKey(makeVertexKey(x,y))) {
			return (Vertex)vertices.get(makeVertexKey(x, y));
		} else {
			throw new RuntimeException("vertex (" + x + "," + y + ") not found");
		}
	}
	
	private Vertex getVertex(int x, int y) {
		if(vertices.containsKey(makeVertexKey(x,y))) {
			return (Vertex)vertices.get(makeVertexKey(x, y));
		} else {
			Vertex v = new Vertex(vid++, x, y);
			vertices.put(makeVertexKey(x,y), v);
			return v;
		}
	}

	private HalfEdge getRealHalfEdge(Vertex from, Vertex to) {
		if(halfEdges.containsKey(makeHalfEdgeKey(from,to))) {
			return (HalfEdge)halfEdges.get(makeHalfEdgeKey(from,to));
		} else {
			throw new RuntimeException("Halfedge between " + makeVertexKey(from.x,from.y) + " and " + makeVertexKey(to.x,to.y) + " not found");
		}		
	}
	
	private HalfEdge getHalfEdge(Vertex from, Vertex to) {
		if(halfEdges.containsKey(makeHalfEdgeKey(from,to))) {
			return (HalfEdge)halfEdges.get(makeHalfEdgeKey(from,to));
		} else {
			HalfEdge pair = new HalfEdge(hid++);
			halfEdges.put(makeHalfEdgeKey(from, to), pair);
			pair.origin = from;
			return pair;
		}
		
	}

	private String makeCellKey(int id) {
		return "" + id;
	}
	
	private String makeVertexKey(int i, int j) {
		return "(" + i + "," + j + ")";
	}

	private String makeHalfEdgeKey(Vertex from, Vertex to) {
		return "" + makeVertexKey(from.x,from.y) + makeVertexKey(to.x, to.y);
	}


	class Vertex implements Serializable {
		
		private int x, y, id;
		private HalfEdge incidentEdge;

		Vertex(int id, int x, int y) {
			this(id, x,y,null);
		}
		
		Vertex(int id, int x, int y, HalfEdge e) {
			this.id = id;
			this.x = x;
			this.y = y;
			incidentEdge = e;
		}
        
        public String toString() {
            return makeCellKey(id);
        }
		
	}
	
	class HalfEdge implements Serializable {

		int id;
		HalfEdge twin, next, prev;
		Vertex origin;
		Cell incidentFace;
		
		HalfEdge(int id) {
			this.id = id;
		}

	}
	
	class Cell implements Serializable {

		int id;
		Cell outerComponent;
		Vector innerComponents;
		HalfEdge incidentEdge;

		Cell(int id) {
			this(id, null);
			
		}
		
		Cell(int id, Cell outer) {
			this.id = id;
			outerComponent = outer;
			innerComponents = new Vector();
		}
		
	}
}
