package content.exercises.sda.structures;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

import matrix.structures.FDT.FDT;
import matrix.structures.FDT.substructures.Source;
import matrix.structures.FDT.substructures.Vertex;
import matrix.structures.memory.VirtualArray;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.memory.VirtualObject;
import matrix.util.Note;

public class ExerciseGeometricGraphVertex implements Vertex, Source, Serializable {
		
		/**
	 * 
	 */
	private static final long serialVersionUID = -6195872329760815564L;
		protected VirtualInteger x, y;
		protected VirtualArray successors;
		protected HashMap selectedSuccessors;
		protected VirtualObject key;
		protected VirtualBoolean selected;

		
		public ExerciseGeometricGraphVertex(int x, int y, Object o) {
			this.x = new VirtualInteger(x, this, "x-coordinate of the node");
			this.y= new VirtualInteger(y, this, "y-coordinate of the node");
			key = new VirtualObject(o, this, "data of the node");
			selectedSuccessors = new HashMap();
			successors = new VirtualArray(this, "successors of the node");
		}
		
		public String toString() {
			return "geometric vertex "+getElement()+": ("+getX()+","+getY()+")";
		}
		
		public int getX() {
			return x.eval();
		}
		
		public int getY() {
			return y.eval();
		}

		public void setCoordinates(int x, int y) {
			this.x.assign(x);
			this.y.assign(y);
		}


		public Vertex[] getSuccessors() {
			Vertex[] v = new Vertex[successors.size()];
			for(int i = 0; i < successors.size();i++) {
				v[i] = (Vertex)successors.getObject(i);
			}
			return v;
		}


		public Object getElement() {
			return key.getObject();
		}


		public void setElement(Object o) {
			key.setObject(o);
		}


		public void setSuccessors(Vertex[] v) {
			Note.out(this, "addSuccessors not implemented");
		}

	    private void addSuccessor(Vertex v, int pos) {
	    	if(v instanceof ExerciseGeometricGraphVertex) {
	    		int i = 1;
	    		// HS: Fixed bug. Invariant Pos was Last+1 when exiting Loop
	    		for(; i <= pos && successors.getObject(i) != null; i++) {
	    			if(v == successors.getObject(i))
	    				return;
	    			/*
	    			 if(successors.getObject(i) == null)
	    			 break;
	    			 */
	    		}
	    		while(successors.getObject(i) != null) {
	    			i++;
	    		}
	    		successors.setObject(v, i);
	    	}
	    }

		public void addSuccessor(Vertex v) {
			if(v instanceof ExerciseGeometricGraphVertex) {
				addSuccessor(v,successors.size());
				selectedSuccessors.put(v, new VirtualBoolean(false, this, "Is the outgoing edge selected"));
			}
		}

		/**
		 * Tests whether the parameter vertex is a neighbour of this vertex
		 * @param v
		 * @return
		 */
		public boolean hasSuccessor(Vertex v) {
			return selectedSuccessors.containsKey(v);
		}

		public void removeSuccessor(Vertex v) {
			Note.out(this, "removeSuccessor not implemented");
		}


		public void referenceSelectedTo(FDT to) {
			toggleSuccessor(to);
		}

		public void toggleSuccessor(Object o) {
			if(selectedSuccessors.containsKey(o)) {
				VirtualBoolean b = (VirtualBoolean)selectedSuccessors.get(o);
				b.assign(!b.eval());
			} else {
				Note.out(this, "toggleSuccessor called for object "+ o + " which is not a successor");
			}			
		}
		
		public void selectSuccessor(Object o) {
			if(selectedSuccessors.containsKey(o)) {
				VirtualBoolean b = (VirtualBoolean)selectedSuccessors.get(o);
				b.assign(true);
			} else {
				Note.out(this, "selectSuccessor called for object "+ o + " which is not a successor");
			}
		}

		public void unselectSuccessor(Object o) {
			if(selectedSuccessors.containsKey(o)) {
				VirtualBoolean b = (VirtualBoolean)selectedSuccessors.get(o);
				b.assign(false);
			} else {
				Note.out(this, "unselectSuccessor called for object "+ o + " which is not a successor");
			}			
		}

		protected HashMap getSelectedSuccessors() {
			return selectedSuccessors;
		}
		
		public boolean isVisitedReferenceTo(FDT to) {
			if(!selectedSuccessors.containsKey(to)) {
				return false;
			} else {
				VirtualBoolean b = (VirtualBoolean)selectedSuccessors.get(to);
				return b.eval();
			}
		}
		
		protected boolean shallowEquals(ExerciseGeometricGraphVertex v) {
			if(getX() == v.getX() && getY() == v.getY()) {
				if(getElement() == null) {
					return v.getElement() == null;
				} else {
					return getElement().equals(v.getElement());
				}
			}
			return false;
		}
		
		public boolean equals(Object o) {
			if(o instanceof ExerciseGeometricGraphVertex) {
				ExerciseGeometricGraphVertex e = (ExerciseGeometricGraphVertex)o;
				if(!shallowEquals(e)) {
					return false;
				}
				Vertex[] mysucc = getSuccessors();
				Vertex[] osucc = e.getSuccessors();
				if(mysucc.length != osucc.length) {
					return false;
				}
				for(int i = 0; i < mysucc.length; i++) {
					ExerciseGeometricGraphVertex mv = (ExerciseGeometricGraphVertex)mysucc[i];
					ExerciseGeometricGraphVertex ov = (ExerciseGeometricGraphVertex)osucc[i];
					if(mv == ov) {
						continue;
					}
					if(mv == null && ov != null) {
						return false;
					}
					if(!(mv.shallowEquals(ov))) {
						return false;
					}
				}
				HashMap hsucc = getSelectedSuccessors();
				HashMap vsucc = e.getSelectedSuccessors();
				if(hsucc.size() > vsucc.size()) {
					HashMap tmp = hsucc;
					hsucc = vsucc;
					vsucc = tmp;
				}
				Iterator i = hsucc.keySet().iterator();
				outloop:	
					// the two hashmaps are considered similar if all keys in the map with less
					// keys also are in the map with more keys
					while(i.hasNext()) {
					ExerciseGeometricGraphVertex vert = (ExerciseGeometricGraphVertex)i.next();
					Iterator vi = vsucc.keySet().iterator();
					while(vi.hasNext()) {
						ExerciseGeometricGraphVertex overt = (ExerciseGeometricGraphVertex)vi.next();
						if(vert.shallowEquals(overt)) {
							VirtualBoolean vb1 = (VirtualBoolean)hsucc.get(vert);
							VirtualBoolean vb2 = (VirtualBoolean)vsucc.get(overt);
							if(vb1.eval() == vb2.eval()) {
								continue outloop;	
							} else {
								return false;
							}
						}
					}
					return false;
				}
				return true;
			}
			return false;
		}
}
