package content.exercises.sda.structures;

import java.io.Serializable;
import java.util.Iterator;

import matrix.structures.FDT.LinkedList;
import matrix.structures.memory.VirtualInteger;
import matrix.structures.memory.VirtualObject;

// TODO comment
// not synchronized
// do not allow null keys
// allows duplicates -> deleteall
// visualized in matrix through the LinkedList interface
public class SinglyLinkedList implements LinkedList, Serializable {

    protected VirtualObject headNode;
    protected VirtualInteger size;
    
    public SinglyLinkedList() {
        this(null);
    }
    
    public SinglyLinkedList(Object key) {
        if (key == null) {
            this.headNode = new VirtualObject(null, null, "headNode");
            this.size = new VirtualInteger(0, null, "size");
        } else {
            this.headNode = new VirtualObject(new Node(key), null, "headNode");
            this.size = new VirtualInteger(1, null, "size");
        }
    }

    public boolean isEmpty() {
        return (this.headNode.getObject() == null);
    }
    
    public Object getHead() {
        if (!this.isEmpty()) {
            return this.getHeadNode().getKey();
        } else {
            return null;
        }
    }
    
    private Object getFirst() {
        return this.getHead();
    }
    
    public Object removeFirst() {
        return this.removeHead();
    }
    
    public int size() {
        return this.size.eval();
    }
    
    public void insert(Object key) {
        if (key != null) {
            this.setHeadNode(new Node(key, this.getHeadNode()));
            this.size.inc();
        }
    }

    public Iterator iterator() {
        return new Iterator() {
            private Node previousNode = null;
            private Node currentNode = null;
            private Node nextNode = SinglyLinkedList.this.getHeadNode();
            
            public boolean hasNext() {
                return (this.nextNode != null);
            }
            
            public Object next() {
                this.previousNode = (this.currentNode == null) ? this.previousNode : this.currentNode;
                this.currentNode = this.nextNode;
                this.nextNode = this.currentNode.getNextNode();
                return this.currentNode.getKey();
            }
            
            public void remove() {
                if (this.currentNode != null) {
                    if (this.currentNode == SinglyLinkedList.this.getHeadNode()) {
                        this.currentNode = null;
                        SinglyLinkedList.this.removeHead();
                    } else {
                        this.currentNode = null;
                        SinglyLinkedList.this.removeNode(this.currentNode, this.previousNode);
                    }
                }
            }
        };
    }

    protected Node getHeadNode() {
        return (Node)this.headNode.getObject();
    }
    
    protected void setHeadNode(Node newAnchorNode) {
        this.headNode.setObject(newAnchorNode);
    }
    
    protected void removeNode(Node node, Node predecessor) {
        predecessor.setNext(node.getNextNode());
        node.clear();
        this.size.dec();
    }
    
    
    // delete one/first occurence
    public boolean delete(Object key) {
        if (!this.isEmpty()) {
            if (this.getHeadNode().getKey().equals(key)) {
                this.removeHead();
                return true;
            } else {
                Node previousNode = this.getHeadNode();
                Node currentNode;
                while (previousNode.hasNext()) {
                    currentNode = previousNode.getNextNode();
                    if (currentNode.getKey().equals(key)) {
                        this.removeNode(currentNode, previousNode);
                        return true;
                    }
                    previousNode = currentNode;
                }
            }
        }
        return false;
    }
    
    // delete all occurences
    public void deleteAll(Object key) {
        while ((!this.isEmpty()) && (this.getHeadNode().getKey().equals(key))) {
            this.removeHead();
        }
        if (!this.isEmpty()) {
            Node previousNode = this.getHeadNode();
            Node currentNode;
            while (previousNode.hasNext()) {
                currentNode = previousNode.getNextNode();
                if (currentNode.getKey().equals(key)) {
                    this.removeNode(currentNode, previousNode);
                }
                previousNode = currentNode; 
            }
        }
    }

    public boolean contains(Object key) {
        if (!this.isEmpty()) {
            Node currentNode = this.getHeadNode();
            do {
                if (currentNode.getKey().equals(key)) {
                    return true;
                }
                currentNode = currentNode.getNextNode(); 
            } while (currentNode != null);
        }
        return false;
    }
    
    public Object removeHead() {
        if (!this.isEmpty()) {
            Object key = this.getHeadNode().getKey();
            Node head = this.getHeadNode();
            this.setHeadNode(head.getNextNode());
            head.clear();
            this.size.dec();
            return key;
        } else {
            return null;
        }
    }
    
    public void clear() {
        while (this.size() > 0) {
            this.removeHead();
        }
    }
    
    public Object[] toArray() {
        Object[] keys = new Object[this.size()];
        Node currentNode = this.getHeadNode();
        for (int i = 0; i < keys.length; i++) {
            keys[i] = currentNode.getKey();
            currentNode = currentNode.getNextNode(); 
        }
        return keys;
    }
    
    public void toArray(Object[] keys) {
        Node currentNode = this.getHeadNode();
        for (int i = 0; i < keys.length; i++) {
            keys[i] = currentNode.getKey();
            currentNode = currentNode.getNextNode(); 
        }
    }

    public boolean equals(Object obj) {
        if (obj instanceof SinglyLinkedList) {
            SinglyLinkedList list = (SinglyLinkedList)obj;
            if (list == this) {
                return true;
            } else {
                if (this.size() == list.size()) {
                    Node currentNode = this.getHeadNode();
                    Node listNode = list.getHeadNode();
                    while (currentNode != null) {
                        if (!currentNode.getKey().equals(listNode.getKey())) {
                            return false;
                        }
                        currentNode = currentNode.getNextNode(); 
                        listNode = listNode.getNextNode();
                    }
                    return true;
                } else {
                    return false;
                }
            }
        } else {
            return false;
        }
    }
    
    public int hashCode() {
        // TODO
        //return Arrays.deepHashCode(this.toArray());
        return 0;
    }
    
    
    
    // ** Implementation of interface LinkedList (visualization interface) **
    
    public Object getElement() {
        if (this.isEmpty()) {
            return null;
        } else {
            return this.getHeadNode().getElement();
        }
    }
    
    public LinkedList getNext() {
        if (this.isEmpty()) {
            return null;
        } else {
            return this.getHeadNode().getNext();
        }
    }
    
    public boolean hasNext() {
        if (this.isEmpty()) {
            return false;
        } else {
            return this.getHeadNode().hasNext();
        }
    }
    
    public void setElement(Object o) {
    }
    
    public void setNext(LinkedList l) {
    }

    public LinkedList getNewNode(Object key) {
        return null;
    }
    
    
    protected class Node implements LinkedList, Serializable {
        private VirtualObject nextNode;
        private final Object key;
        
        public Node(Object key) {
            this(key, null);
        }
        
        public Node(Object key, Node nextNode) {
            this.key = key;
            this.nextNode = new VirtualObject(nextNode, null, "nextNode");
        }
        
        public void setNext(Node nextNode) {
            this.nextNode.setObject(nextNode);
        }
        
        public Node getNextNode() {
            return (Node)this.nextNode.getObject();
        }
        
        public Object getKey() {
            return this.key;
        }
        
        public void clear() {
            this.nextNode.setObject(null);
        }
        
        public boolean equals(Object obj) {
            return (this == obj);
        }

        // ** Implementation iof interface LinkedList (visualization interface). **
        public Object getElement() {
            return this.getKey();
        }
        public boolean hasNext() {
            return (this.getNextNode() != null);
        }
        public LinkedList getNext() {
            return this.getNextNode();
        }
        public LinkedList getNewNode(Object key) {
            return null;
        }
        public void setElement(Object o) {
        }
        public void setNext(LinkedList l) {
        }
        
        
        private static final long serialVersionUID = 5332415848542320764L;
    }

    private static final long serialVersionUID = 899635709574001183L;
}
