package content.exercises.structures;

import java.awt.Color;

import matrix.decoration.StyleSheet;
import matrix.decoration.StyleSheetAdapter;
import matrix.decoration.Styled;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.probe.Table;
import matrix.uitools.StructurePanel;

/**
 * A general implementation of ActivationRecord. It works mostly as a Struct
 * but this class automatically organises the fileds and the values according
 * to the desired number of columns.
 * 
 * @author Isaac Jurado
 */
public class ActivationRecord implements Struct, Styled {

	private static final String[] rec_labels = { null, "", "" };
	private Object[]   rec_values;
	private StyleSheet style;
	
	private static class MiniStruct implements Struct {
		private String[] stLabels;
		private Object[] stValues;
		
		public MiniStruct(String[] lbls, Object[] vls) {
			stLabels   = lbls;
			stValues   = vls;
		}
		
		public String   getFieldName (int i) { return stLabels[i]; }
		public Object   getField     (int i) { return stValues[i]; }
		public String[] getFieldNames()      { return stLabels; }
		public Object[] getFields    ()      { return stValues; }				
		// Ignored
		public void   setField  (Object v, int i) {}
		public void   setElement(Object o)        {}
		public Object getElement()                { return null; }

		private static final long serialVersionUID = -5735715387504518781L;
	}
	
	private static final class AddressField extends MiniStruct implements Styled {
		private StyleSheet mini_style;
		
		public AddressField(String r) {
			super(new String[] { null, "Return to" }, new Object[] { "", r });
			Color bg   = StructurePanel.getBgColor();
			mini_style = new StyleSheetAdapter();
			mini_style.setDefaultBackgroundColor(bg);
			mini_style.setVisitedBackgroundColor(bg);
			mini_style.setDefaultBorderColor(bg);
			mini_style.setVisitedBorderColor(bg);
		}
		
		public StyleSheet getStyleSheet() { return mini_style; }

		private static final long serialVersionUID = -8425429132977297743L;
	}
	
	public ActivationRecord(String[] labels, Object[] values, String ret, int columns) {
		Table        topRow;
		AddressField bottomRow;
		int          rows, large_cols, c;
		
		topRow     = new Table(columns);
		bottomRow  = new AddressField(ret);
		rec_values = new Object[3];
		rows       = labels.length / columns;
		large_cols = labels.length % columns;
		c = 0;
		if (large_cols > 0) {
			rows++;
			for (; c < large_cols; c++)
				topRow.setObject(createColumn(c, columns, rows, labels, values), c);
			rows--;
		}
		// Now we do the same but with smaller columns.
		for (; c < columns; c++)
			topRow.setObject(createColumn(c, columns, rows, labels, values), c);
		
		rec_values[0] = "";
		rec_values[1] = topRow;	
		rec_values[2] = bottomRow;
		
		// Setup the stylesheet
		Color bg = StructurePanel.getBgColor();
		style    = new StyleSheetAdapter();
		style.setDefaultBackgroundColor(bg);
		style.setVisitedBackgroundColor(bg);
		style.setDefaultBorderColor(bg);
		style.setVisitedBorderColor(bg);
	}
	
	public ActivationRecord(String[] labels, Object[] values, int rline, int columns) {
		this(labels, values, (rline < 0 ? "caller function" : "line "+rline), columns);
	}

	public ActivationRecord(String[] labels, Object[] values, int columns, String ret) {
		this(labels, values, ret, columns);
	}
	
	public ActivationRecord(String[] labels, Object[] values, int columns) {
		this(labels, values, -1, columns);
	}
	
	private static Struct createColumn(int c, int cols, int rows, String[] labels, Object[] values) {
		String[] col_labels = new String[rows + 1];
		Object[] col_values = new Object[rows + 1];
		
		col_labels[0] = null;
		col_values[0] = "";
		for (int i = 1; i <= rows; i++) {
			col_labels[i] = labels[c];
			col_values[i] = values[c];
			c += cols;
		}
		return new MiniStruct(col_labels, col_values);
	}

	// Implemented interfaces
	public StyleSheet getStyleSheet()      { return style; }
	public Object     getField     (int i) { return rec_values[i]; }
	public String     getFieldName (int i) { return rec_labels[i]; }
	public Object[]   getFields    ()      { return rec_values; }
	public String[]   getFieldNames()      { return rec_labels; }
	
	// Ignored
	public Object getElement()                { return null; }
	public void   setElement(Object o)        {}
	public void   setField  (Object o, int i) {}

	private static final long serialVersionUID = 6658354994838742768L;
}
