/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises.structures;

import matrix.decoration.*;
import matrix.structures.CDT.probe.AVLTree;
import matrix.structures.FDT.*;
import matrix.structures.FDT.probe.*;
import matrix.structures.other.Rotations;
import matrix.structures.util.MatrixComparable;
import matrix.structures.CDT.CDT;
/**
 * AVL Tree with insert disabled
 *
 * @author Harri Salonen
 */

public class ExerAVL extends AVLTree {
    private boolean needDouble = false;
    private boolean needSingle = false;
    public static final boolean DEBUG = false;
    private boolean rotate = true;
    private boolean rootChanged = false;
    
    protected class EDTAVLNode extends AVLNode {
        
        public EDTAVLNode(MatrixComparable key) {
            super(key);
            
        }
        
        
        public StyleSheet getStyleSheet() {
            if (myStyle != null)
                return myStyle;
            else {
                myStyle = new StyleSheetAdapter() {
                    //public boolean isInfoEnabled() { return false; }
                    //public boolean isLabelEnabled() { return !empty(); }
                    //public String getLabel() { return ""; }
                    static final long serialVersionUID = 406750981751124L;
                };
                return myStyle;
            }
        }
        
        public void SingleRotationRight() {
            Rotations.rotateWithLeft_((BinaryTree)this);
        }
        
        
        public void SingleRotationLeft() {
            Rotations.rotateWithRight_((BinaryTree)this);
        }
        
        
        public void DoubleRotationLeft() {
            Rotations.rotateDoubleWithLeft_((BinaryTree)this);
        }
        
        
        public void DoubleRotationRight() {
            Rotations.rotateDoubleWithRight_((BinaryTree)this);
        }
        
        
        public Tree getNewNode(Object key) {
            return new EDTAVLNode((MatrixComparable)key);
        }
        
        static final long serialVersionUID = 6663702569780046607L;
    }
    
    
    // Methods for interface CDT
    
    /**
     * New insert method, which keeps track of rotations
     */
    public FDT realInsert(Object o) {
        //Note.out(this, "INSERT " + o);
        if (DEBUG)
            System.out.println(o);
        Object rootObj = this.getElement();
        needDouble = false;
        needSingle = false;
        rootChanged = false; 
        
        if (o == null) {
            return this;
        } else if (o instanceof MatrixComparable) {
            if (empty()) {
                
                setElement((MatrixComparable)o);
            } else
                setElement(AVLInsert((AVLNode)getElement(), (MatrixComparable)o));
            /*
             } else if (o instanceof Table) {
             Table t = (Table)o;
             for (int i=0; i < t.size(); i++)
             insert(t.getObject(i));
             } else {
             return insert(new Key(o));
             */
        }	
        rootChanged = !(rootObj == this.getElement());
        return this;
    }
    
    /**
     * New insert method, which does not rotate
     */
    public FDT dumbInsert(Object o) {
        //Note.out(this, "INSERT" + o);
        if (DEBUG)
            System.out.println(o);
        if (o == null) {
            return this;
        } else if (o instanceof MatrixComparable) {
            if (empty()) {
                setElement((MatrixComparable)o);
            } else
                setElement(AVLInsertDumb((AVLNode)getElement(), (MatrixComparable)o));
            
        } else if (o instanceof Table) {
            Table t = (Table)o;
            for (int i=0; i < t.size(); i++)
                insert(t.getObject(i));
        } else {
            return insert(new Key(o));
        }
        return this;
    }
    
    /**
     * New insert method, which keeps track of rotations
     */
    protected  AVLNode AVLInsert(AVLNode t, MatrixComparable e) {
        if (t == null || t.getElement() == null) {
            t = new AVLNode();
            t.setHeight(0);
            t.setElement(e);
        } else if (((MatrixComparable)t.getElement()).gt(e)) {
            t.setLeft(AVLInsert((AVLNode)t.getLeft(), e));
            //evalHeight(t);
            if (getHeight((AVLNode)t.getLeft()) - getHeight((AVLNode)t.getRight()) == 2) {
                if (((MatrixComparable)t.getLeft().getElement()).gt(e)) {
                    t = (AVLNode)Rotations.withLeftChild(t);
                    needSingle = true;
                } else {
                    t = (AVLNode)Rotations.doubleWithLeftChild(t);
                    needDouble = true;
                }
            }
            
        } else {
            t.setRight(AVLInsert((AVLNode)t.getRight(), e));
            //evalHeight(t);
            if (getHeight((AVLNode)t.getRight()) - getHeight((AVLNode)t.getLeft()) == 2) {
                if (!((MatrixComparable)t.getRight().getElement()).gt(e)) {
                    t = (AVLNode)Rotations.withRightChild(t);
                    needSingle = true;
                } else {
                    t =(AVLNode) Rotations.doubleWithRightChild(t);
                    needDouble = true;
                }
            }
        }
        
        updateHeight((AVLNode)t.getRight());
        updateHeight((AVLNode)t.getLeft());        
        updateHeight(t);
        return t;
    }
    
    
    /**
     * New insert method, which does not rotate
     */
    protected  AVLNode AVLInsertDumb(AVLNode t, MatrixComparable e) {
        if (t == null || t.getElement() == null) {
            t = new AVLNode();
            t.setHeight(0);
            t.setElement(e);
        } else if (((MatrixComparable)t.getElement()).gt(e)) {
            t.setLeft(AVLInsertDumb((AVLNode)t.getLeft(), e));
        } else {
            t.setRight(AVLInsertDumb((AVLNode)t.getRight(), e));
        }
        
        updateHeight((AVLNode)t.getRight());
        updateHeight((AVLNode)t.getLeft());        
        updateHeight(t);
        return t;
    }
    
    
    /**
     * Root was changed during last insert
     * @return truth value of above
     */
    public boolean rootWasChanged() {
        return rootChanged;
    }
    
    
    /**
     * Single rotation was needed during last insert
     * @return truth value of above
     */
    public boolean singleWasNeeded() {
        return needSingle;
    }
    
    /**
     * Double rotation was needed during last insert
     * @return truth value of above
     */
    public boolean doubleWasNeeded() {
        return needDouble;
    }
    
    public Tree getNewNode(Object key) {
        return new EDTAVLNode((MatrixComparable)key);
    }
    
    /**
     * Dummy insert method
     */
    public CDT insert(Object o) {
        return this;
    }
    /*
     public boolean equals(Object o) {
     
     if (o instace of ExerAVL) {
     Object oRoot = this.getElement();
     
     if (oRoot.equals(((ExerAVL)o).getElement()) &&
     oRoot.getLeft().equals((ExerAVL)o).getLeft)
     } else return false;
     }
     */
    static final long serialVersionUID = 406750981751124L;
}
