package content.exercises.structures;

import matrix.structures.CDT.probe.BinSearchTree;
import matrix.structures.FDT.BinaryTree;
import matrix.structures.FDT.probe.Key;
import matrix.structures.util.MatrixComparable;

/**
 * BinarySearchTree which inserts duplicates to the right subtree.
 * 
 * @author kstalnac
 *
 */
public class ExerBinSearchTree_DuplicatesRight extends BinSearchTree {

    public ExerBinSearchTree_DuplicatesRight() {
        super();
    }
    
    public ExerBinSearchTree_DuplicatesRight(Object o) {
        super(o);
    }
    
    /**
     * Help method for insertion which puts the duplicates to the right
     * subtree.
     */
    protected BinaryTree insertHelp(MatrixComparable k, BinaryTree tree) {
        if(tree == null)
            return (BinaryTree) getNewNode(k);
        MatrixComparable key = (MatrixComparable)tree.getElement();
        if(key == null) {
            tree.setElement(k);
            return tree;
        }
        if(k.lt(key))
            tree.setLeft(insertHelp(k, tree.getLeft()));
        else
            tree.setRight(insertHelp(k, tree.getRight()));
        return tree;
    }

    /**
     * Help method for deletion which takes into account that
     * possible duplicates are in the right subtree. That is, 
     * in the case of that tree has two children the smallest key
     * is searched from the right subtree for substition with the 
     * keyt to be deleted.
     */
    protected BinaryTree deleteHelp(Key k, BinaryTree tree) {
        Key key = tree != null ? (Key)tree.getElement() : null;
        if (key == null) 
            return null;
        if(k.equals(key)) {  // found the node to be deleted
            BinaryTree left = tree.getLeft();
            BinaryTree right = tree.getRight();
            
            if(left == null)
                tree = tree.getRight();
            else if(right == null)
                tree = tree.getLeft();
            else {
                Key tmp = getSmallest(right);
                tree.setElement(tmp);
                tree.setRight(deleteSmallest(right));
            }
        } 
        else if(k.lt(key))
            tree.setLeft(deleteHelp(k, tree.getLeft()));
        else
            tree.setRight(deleteHelp(k, tree.getRight()));
        return tree;
    }

    /**
     * Returns the smallest key of the tree
     */
    protected Key getSmallest(BinaryTree tree) {
        if(tree.getLeft() == null)
            return (Key)tree.getElement();
        else
            return getSmallest(tree.getLeft());
    }

    /**
     * Deletes the smallest key from the tree and returns the tree after
     * the deletion.
     */
    protected BinaryTree deleteSmallest(BinaryTree tree) {
        if(tree.getLeft() == null) {
            return tree.getRight();
        } 
        else {
            tree.setLeft(deleteSmallest(tree.getLeft()));
            return tree;
        }
    }

    private static final long serialVersionUID = -5738220117339607576L;
}
