package content.exercises.structures;

import matrix.structures.FDT.probe.Key;
import matrix.structures.util.MatrixComparable;

/**
 * This is sub class of the Key class that is used in the exercises (in the heap of 
 * the Dijkstra exercise.) This contains a separate priority element which is the priority
 * of this key.
 * 
 * @author kstalnac
 */
public class ExerKeyWithPriority_DH extends Key {

    /** Priority of this key. */
    private ExerPriorityElement_DH priority;
    
    /**
     * Constructor
     * @param o object of the new key
     * @param s string for the new key
     */
    public ExerKeyWithPriority_DH(Object o, String s) {
        super(o, s);
        priority = new ExerPriorityElement_DH();
    }
    
    /**
     * Constructor
     * @param o object of the new key
     * @param s string representation of the new key
     * @param priority priority of the new key
     */
    public ExerKeyWithPriority_DH(Object o, String s, int priority) {
        super(o, s);
        this.priority = new ExerPriorityElement_DH(priority);
    }
    
    /**
     * Sets the priority element of the key.
     * @param e new priority element
     */
    public void setPriorityElement(MatrixComparable e){
        if (e instanceof ExerPriorityElement_DH) {
            priority = (ExerPriorityElement_DH)e;
        }
    }

    /**
     * Returns the priority element of this key
     * @return priority element
     */
    public MatrixComparable getPriorityElement(){
        return priority;
    }

    public boolean equals(Object o) {
        if (o instanceof ExerKeyWithPriority_DH) {
            ExerKeyWithPriority_DH tempKey = (ExerKeyWithPriority_DH)o;
            if (getComparisonString().equals(tempKey.getComparisonString()) &&
                ((ExerPriorityElement_DH)getPriorityElement()).getPriority() ==
                    ((ExerPriorityElement_DH)tempKey.getPriorityElement()).getPriority()) {
                /* Comparison string and the priority value of this object and the object given as
                 * a parameter equals.
                 */
                return true;
            } else {
                return false;
            }
        } else {
            //otherwise use the super classes (that is Key's) equals method
            return super.equals(o);
        }
    }

    private static final long serialVersionUID = 81054321193930774L;
}
