/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package content.exercises.structures;

import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.simulationextensions.Minimized;
import matrix.structures.util.MatrixComparable;
import matrix.structures.memory.VirtualBoolean;
import matrix.animation.Animator;

public class ExerMinimizedRBTree extends ExerRBTree {
    
    public class ExerMinimizedRBNode extends ExerRBNode implements Minimized {
        private static final long serialVersionUID = -5564416457544339892L;
        private VirtualBoolean isMinimized = new VirtualBoolean(true, this, "Is the node minimized");
        
        public ExerMinimizedRBNode() {
            super();
        }
        
        public ExerMinimizedRBNode(MatrixComparable key) {
            super(key);
        }
        
        public Tree getNewNode(Object key) {
            //Note.out(this, "create new node " + key);
            ExerMinimizedRBNode tmp =  new ExerMinimizedRBNode((MatrixComparable)key);
            tmp.setRed(true);
            return tmp;
        }
        
        public void setSelect(long timeStamp) {
            isMinimized.toggle();
        }
        
        public boolean isMinimized() {
            return isMinimized.eval();
        }
        
        public void setMinimized(boolean value) {
            isMinimized.assign(value);
        }
        
        public boolean equals(ExerMinimizedRBNode e) {
            boolean flag = (isMinimized() == e.isMinimized());
            /*
             if (!flag)
             Note.out(this, "Minimized = " + isMinimized() + " for " + this + ", but not for " + e + "(" + e.isMinimized() + ")");
             */
            if (!flag)
                return false;
            if ((getLeft() == null) && (e.getLeft() != null))
                return false;
            if ((getLeft() != null) && (e.getLeft() == null))
                return false;
            if ((getRight() == null) && (e.getRight() != null))
                return false;
            if ((getRight() != null) && (e.getRight() == null))
                return false;
            if ((getLeft() == null) && (getRight() == null))
                return true;
            if (getLeft() == null)
                return getRight().equals(e.getRight());
            if (getRight() == null)
                return getLeft().equals(e.getLeft());
            return getLeft().equals(e.getLeft()) && getRight().equals(e.getRight());
        }
        
        public boolean equals(Object e) {
            boolean flag = super.equals(e);
            if (flag && (e instanceof ExerMinimizedRBNode))
                return equals((ExerMinimizedRBNode)e);
            else
                return false;
        }
        
    }
    
    public Tree getNewNode(Object key) {
        //Note.out(this, "create new ExerMinimizedRBNode " + key);
        return new ExerMinimizedRBNode((MatrixComparable)key);
    }
    
    public Object search(Object o) {
        //Note.out(this, "Search " + o);
        RedBlackNode root = (RedBlackNode)getElement();
        while (notFound((ExerMinimizedRBNode)root, o)) {
            //Note.out(this, "root.getElement() = " + root.getElement().getClass());
            //Note.out(this, "o = " + o.getClass());
            
            if (((MatrixComparable)root.getElement()).gt((MatrixComparable)o)) {
                if (root.getLeft() == null)
                    break;
                root = (RedBlackNode)root.getLeft();
            } else { // root > o
                if (root.getRight() == null)
                    break;
                root = (RedBlackNode)root.getRight();
            }
        }
        return this;
    }
    
    private boolean notFound(ExerMinimizedRBNode node, Object o) {
        Animator a = Animator.getActiveAnimator();
        a.startOperation();
        node.setMinimized(false);
        a.endOperation();
        return !node.getElement().equals(o);
    }
    
    static final long serialVersionUID = -8279204555769956022L;
}
