package content.exercises.structures;

import matrix.structures.CDT.CDT;
import matrix.structures.CDT.probe.QueueImpl;
import matrix.structures.FDT.probe.Key;

/**
 * This class is used in the Dijsktra_Heap exercise.
 * When a key is inserted to this queue, this will automatically delete the same key from the
 * heap and inform the graph about the chose.
 * 
 * @author kstalnac
 *
 */
public class ExerQueue_DH extends QueueImpl {

    /** graph */
    private ExerLabeledGraph_DH graph;
    /** heap */
    private ExerLabeledHeap_DH heap;
    
    /**
     * Constructor.
     * @param graph graph used in the Dijkstra_Heap exercise
     * @param heap heap used in the Dijkstra_Heap exercise
     */
    public ExerQueue_DH(ExerLabeledGraph_DH graph, ExerLabeledHeap_DH heap) {
        super();
        this.graph = graph;
        this.heap = heap;
    }
    
    /**
     * Inserts the key to this queue.
     * Deletes the given key from the heap.
     * Informs the graph about the selected key.
     */
    public CDT insert(Object key) {
        
        Key minimumKey = heap.getMinimumKey();
        if (minimumKey != null && minimumKey.equals(key)) {
            //delete the key from the heap
            heap.delete(((Key)key).getOrig());
            //inform the graph about the chosen key/node and its final distance from the start node
            graph.setLastChosen((Key)key, ((ExerPriorityElement_DH)((ExerKeyWithPriority_DH)minimumKey).getPriorityElement()).getPriority());
            return super.insert(key);
            
        } else { 
            //key is not from the top of the heap so do nothing
            return this;
        }
        
    }
    
    private static final long serialVersionUID = -6438713080089772199L;
}
