/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2003-2006 Matrix Group
 * (See the AUTHORS file and the author tag below for more information 
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: ExerVertexVisitableKey.java,v 1.3 2007/08/20 13:27:28 oseppala Exp $*/

package content.exercises.structures;

import java.awt.Color;

import matrix.animation.Animator;
import matrix.structures.FDT.probe.Key;
import matrix.structures.memory.VirtualInteger;

/**
 * This implementation of Key is intended to be used with the ExerVertex.
 * First click on this key visits the ExerVertex and the second marks
 * it finished. Note, that this key should not be used in exercises
 * where the keys can be drag&dropped to other nodes (or at least the
 * structure must then take care of updating the related ExerVertex).
 * 
 * @see content.exercises.structures.ExerVertex
 * @see content.exercises.BFS
 * @see content.exercises.DFS 
 * @author vkaravir@cs.hut.fi
 */
public class ExerVertexVisitableKey extends Key{
    protected ExerVertex ev;
    private   VirtualInteger clicks;
    
    /**
     * Contructs a new ExerVertexVisitableKey with the given
     * String as the key and related to the given ExerVertex.
     * @param key The String of this key.
     * @param ev The ExerVertex this key is used in.
     */
    public ExerVertexVisitableKey(String key, ExerVertex ev) {
        super(key);
        this.ev = ev;
        clicks = new VirtualInteger(0, this, "Number of times the key has been clicked");
    }
    
    /**
     * This method is invoked each time this key is selected. 
     * The first time it is selected the related ExerVertex
     * is marked visited. On the second time the vertex is marked
     * finished. Note, that going backwards in the animation changes
     * this and is able to undo the clicks.
     */
    public void select() {
        if (ev == null)
            return;
        if (clicks.eval() == 0) {
            Animator.getActiveAnimator().startOperation();
            clicks.inc();
            ev.markVisited();
            Animator.getActiveAnimator().endOperation();
        } else if (clicks.eval() == 1) {
            Animator.getActiveAnimator().startOperation();
            clicks.inc();
            ev.markFinished();
            Animator.getActiveAnimator().endOperation();
        }
    }
    
    public String getLabel() {
        return super.toString();
    }
    
    public String toString() {
    	Color  c  = ev.getColor();
    	String me = super.toString();
    	
    	return '(' + me + ", " + c + ')';    	
    }
    
    private static final long serialVersionUID = -5805640088601801231L;
}