/*
 * Matrix -- algorithm simulation framework
 * Copyright (C) 2003-2005 See the AUTHORS file and the 
 * author tag below for more information on the copyright owners.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises.structures;

import matrix.structures.FDT.probe.Key;
import matrix.structures.memory.VirtualBoolean;

/**
 * Table where only one of the keys is visible at the time.
 * 
 * @see InvisibleKey
 *
 */
public class InvisibleKey extends Key {
    
    VirtualBoolean isSelected = new VirtualBoolean(false, this, "Has this key been selected");
    
    private static final String DEFAULT_INVISIBLE_KEY = "?";
    
    private String invisibleKey;
    
    private InvisibleKeyTable parent;
    
    public InvisibleKey(String value) {
        this(value, DEFAULT_INVISIBLE_KEY);
    }
    
    public InvisibleKey(String value, String invisibleKey) {
        super(value);
        this.invisibleKey = invisibleKey;
    }
    
    public InvisibleKey(String value, InvisibleKeyTable parent) {
        this(value);
        setParent(parent);
    }
    
    public boolean isSelected() {
        return isSelected.eval();
    }
    
    public void set() {
        isSelected.assign(true);
    }
    
    public void clear() {
        isSelected.assign(false);
    }
    
    public void select() {
        parent.setSelectedKey(this);
    }
    
    public String getVisualizationString() {
        if (!isSelected())
            return invisibleKey;
        else
            return super.getVisualizationString();
    }
    
    public void setParent(InvisibleKeyTable table) {
        this.parent = table;
    }
    
    public boolean equals(Object o) {
        if (o instanceof InvisibleKey)
            return getIdentityString().equals(((InvisibleKey)o).getIdentityString());
        return false;
    }
    static final long serialVersionUID = -7164432746970935260L;
}
