/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2006 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package content.exercises.structures;

import matrix.structures.FDT.probe.Key;
import matrix.structures.memory.VirtualBoolean;

/**
 * A key that can be toggled between true and false.
 */
public class TrueFalseKey extends Key {

    private VirtualBoolean isSelected = new VirtualBoolean(false, this, "Value of this key (true/false)");

    public TrueFalseKey(boolean b) {
        super(b + "");
        isSelected.assign(b);
    }
    
    public boolean isSelected() {
        return isSelected.eval();
    }
    
    public String getVisualizationString() {
        return isSelected() + "";
    }

    public void select() {
        isSelected.toggle();

    }

    public boolean equals(Object o) {
        if (o instanceof Key) {
            return ((Key)o).getVisualizationString().equals(this.getVisualizationString());
        } else if (o instanceof Boolean) {
            return ((Boolean)o).booleanValue() == isSelected();
        }
        return false;
    }
    
    private static final long serialVersionUID = -7912335546906405370L;
}
