package content.interfaces;
import java.awt.Component;
import java.awt.GridBagConstraints;

import matrix.util.Application;

/**
 * Interface for specifying a set of AWT components
 * that are to be added in an <code>ExercisePanel</code>.
 *
 * For example, can be used to add buttons and checkboxes 
 * to an exercise for additional simulation purposes.
 * 
 * @see ExercisePanel
 */
public interface AWTComponentUtilizer {

    /**
     * @return  The components that are to be added in the panel.
     */
    public Component[] getAWTComponents();
    
    
    /**
     * This is used to provide the application reference to the 
     * exercise implementing the interface, where it might be 
     * needed for validation. The implementation can be blank 
     * if the reference isn't needed.
     * 
     * @param a The application reference.
     */
    public void setApplication(Application a);
    
    /**
     * Specify <code>GridBagConstraints</code> to be used with the 
     * components. The constraints are applied in order to the 
     * corresponding components and therefore there must be as many 
     * constraint objects as there are components.
     * 
     * The method should return null if no constraints are specfied.
     * 
     * @return  The constraints to be used when adding the components
     *          given with <code>getAWTComponents()</code> in the panel
     *          or null if none specified.
     */
    public GridBagConstraints[] getAWTComponentConstraints();

}
