package content.interfaces;
import matrix.animation.AnimatedStructures;

/**
 * @author Otto Seppl
 *
 * This interface defines the additional methods an exercise class can implement to
 * create simulations from different algorithm variants. These simulation sequences can then be used
 * to assess whether the student solution is consistent with any of them.
 */
public interface AlgorithmVariants{

    /**
     * This method is the equivalent of the solve method in SimulationExerciseModel
     * interface. It creates and returns FDTs for different algoritm variants.
     * 
     * @return an array of solutions for the different algorithm variants
     */
    public AnimatedStructures[] createVariantSequences();    
    
    /**
     * Each variant is described by a nickname that can describes it in a short form.
     * This method returns a nickname for each sequence returned by 
     * createVariantSequences.
     *
     * @return nicknames for the variants
     */
    public String[] getVariantNames();
        
}
