/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: ComparableExercise.java,v 1.3 2007/02/27 11:24:55 etanol Exp $ */

package content.interfaces;

/**
 * A comparable exercise is an exercise that allows a {@link ComparisonPanel} to
 * be constructed from it. From here it is possible to enable or disable the
 * recovery mode. Be careful when requesting recovery mode and make sure the
 * {@link DistanceGraph} class supports the {@link FDT}s returned by the
 * exercise.
 * 
 * @author Isaac Jurado
 */
public interface ComparableExercise extends SimulationExerciseModel {

	/**
	 * Get the indices of the structures to be compared. Each element of this
	 * array will be used to index the array returned by
	 * <code>getModelAnswerNames()</code>,
	 * <code>getModelAnswerVisualizations()</code> and similar.
	 * 
	 * @return an array of indices.
	 */
	public int[] getCompareIndices();

	/**
	 * Get the indices of the structures to be graded. Each element of this
	 * array will be used to index the array returned by <code>solve()</code>
	 * and <code>getAnswer()</code>
	 * 
	 * @return an array of indices.
	 */
	public int[] getGradeIndices();

	/**
	 * Enable recovery mode. This method tells if the exercises supports
	 * recovery mode or not.
	 * 
	 * @return <code>true</code> if recovery mode should be enabled,
	 *         <code>false</code> otherwise.
	 */
	public boolean canRecover();

}
