package content.interfaces;
import matrix.animation.AnimatedStructures;

/**
 * @author Otto Seppl
 *
 * This interface defines the additinal methods an exercise class can implement to create
 * simulations of with known misconceptions. These simulation sequences can then be used to
 * detect corresponding misconceptions from the student solution.
 */
public interface KnownMisconceptions{

    /**
     * This method is the equivalent of the solve method in SimulationExerciseModel
     * interface. It creates and returns FDTs for different misconceptions about
     * the algorithm being checked. The first index defines the misconception and
     * the second the FDT.
     * 
     * @return an array of misconcepted solutions
     */
    public AnimatedStructures[] createMisconceptionSequences();    
    
    /**
     * Each misconception is described by a nickname that can describes it in a short form.
     * This method returns a nickname for each sequence returned by 
     * createMisconceptionSequences.
     *
     * @return nicknames for the misconceptions
     */
    public String[] getMisconceptionNicknames();
    
    /**
     * If the learner can be given some guiding information about a specific
     * misconception it should be returned with this method.
     * 
     * @return guidance strings for the misconceptions
     */
    
    public String[] getGuidanceStrings();
    
    /**
     * If a misconception is known to be a special case of a more general misconception
     * this table will point out the general case. The base case (no general case) is
     * marked as -1
     */
    
    public int[][] getBaseCases();
}
