/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: startOperation.java,v 1.13 2004/05/26 08:05:48 vkaravir Exp $ */

package matrix.animation;

import matrix.visual.VisualType;

/**
 * "Opening bracket" operation to encapsulate
 * atomic operations together in <code>Animator</code>.
 *
 * @see endOperation
 * @see Animator
 */
public class startOperation implements AnimatedOperation {

    static final long serialVersionUID = 707117233658238456L;

    AnimatedOperation prefix = null;
    String annotation = null;

    /** Create a new beginning for an animation block */
    public startOperation() {
        // prefix = null;
    }

    public startOperation(String annotation) {
        this.annotation = annotation;
    }


    /** Constructor for beginning of an animation block where
        parameter AnimatedOperation refers to some additional
        animated operation for this block. */
    public startOperation(AnimatedOperation ao) {
        prefix = ao;
    }

    public void undo() {
        //Note.out(this, "undoing w. prefix " + prefix);
        if (prefix != null) prefix.undo();
        // end of undo
    }

    public void redo() {
        // start of redo
        VisualType.touch(null); // Visual hint: clear previously touched objcts
        if (prefix != null) prefix.redo();
    }


    public void setAnnotation(String s) {
        annotation = s;
    }


    public String getAnnotation() {
        return annotation;
    }


    public String toString() { return "("; }


}
