/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2005 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Queue.java,v 1.4 2005/12/20 08:57:16 kstalnac Exp $ */

package matrix.structures.ADT.probe;

import java.util.HashMap;
import java.util.Map;

import matrix.structures.CDT.probe.QueueImpl;
import matrix.structures.CDT.CDT;
import matrix.structures.ADT.ADT;

/**
 * Queue is an abstract data type of queue. This can be used as an ordinary 
 * queue when implementing, for example, exercises.
 * This inherits QueueImpl CDT structure so this can also be directly 
 * visualized in the Matrix.
 *  
 */
public class Queue implements ADT, matrix.structures.ADT.Queue{

    
    //--ADT Factory---------------------------------------------//
    private matrix.structures.ADT.Queue implementation;
    private static CDT currentFactory  = new QueueImpl();    
    private static Map implementations = new HashMap();
    
    static{
        implementations.put("Queue",       new QueueImpl());
    }
    
    /**
     * This method returns the underlying CDT structure which can be used
     * to visualize this ADT structure.
     */
    public CDT getCDT(){
        return (CDT)implementation;
    }
    
    public Queue getNewInstance(String implementingClassName){
        CDT factory = (CDT)implementations.get(implementingClassName);
        CDT newInstance = factory.getNewInstance();
        return new Queue(newInstance);
    }    

    public Queue(CDT implementation){
        this.implementation = (matrix.structures.ADT.Queue)implementation;
    }
    
    public Queue(){
        implementation = (matrix.structures.ADT.Queue)currentFactory.getNewInstance();
    }
	/**
	 * Returns true if the queue is empty.
	 * @return true if the queue is empty
	 */
	public boolean ADTIsEmpty() {		
		return implementation.ADTIsEmpty();
	}	
    
    public Object ADTDequeue(){
        return implementation.ADTDequeue();
    }
    
    public void ADTEnqueue(Object object){
        implementation.ADTEnqueue(object);
    }
    
    public Object ADTPeek(){
        return implementation.ADTPeek();
    }
    
    static final long serialVersionUID = -238548867434245629L;
}
