/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Stack.java,v 1.13 2006/12/14 12:14:59 oseppala Exp $ */

package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;

public class Stack extends Table implements CDT {
        
    //Stack stores its data in the virtualarray in VanillaTable.
    private VirtualInteger stackSize;
    
    /**
     * Constructor
     *
     */
    public Stack() {
        super("    ");
        stackSize     = new VirtualInteger(0, this, "stack size");
    }
    
    public CDT getNewInstance(){
        return new Stack();
    }
    
    
    public Stack(int i) {
        super(" ");
        stackSize     = new VirtualInteger(0, this, "stack size");
        
        if(i > 0) {
            for(int j = 0; j < i; j++) {
                setObject(new Key(" "), j);
            }
        }
    }
    
    public Object getElement() {	
        Table va = new Table();

        for(int i=0; i <= getLast(); i++) {
            va.setObject(getObject(i), i);
        }
        return va;
    }
    
    
    /**
     * Inserts key to the top of the stack.
     *
     * @param o key to be inserted
     * @return stack after insertion
     */
    public CDT insert(Object o) {
        int index = stackSize.eval();

        if (index >= getLast()) {
            if(o instanceof FDT)
                setObject(o, index);
            else
                setObject(new Key(o), index);
            setLast(getLast()*2 + 1);
        }
        else {
            if(o instanceof FDT)
                setObject(o, index);
            else 
                setObject(new Key(o), index);
        }
        index++;
        stackSize.assign(index);
        return this;
    }
    
    
    /**
     * Deletes the given key and all the keys before it from the stack.
     *
     * @param o key to be deleted
     * @param stack after the deletion
     */
    public CDT delete(Object o) {
        boolean isInStack = false;
        boolean hasOriginalInStack = false;

        for(int i = 0; i < stackSize.eval(); i++) {
            if(o instanceof Key){
                if(getObject(i) == ((Key)o).getOrig()) {
                    isInStack = true;
                    hasOriginalInStack = true;
                }
                else {
                    if(getObject(i) == o) {
                        isInStack = true;
                    }
                }
            } else {
                if(getObject(i) == o)
                    isInStack = true;
            }
        }

        if(isInStack) {
            if((o instanceof Key) && hasOriginalInStack) {
                while(!(getObject(stackSize.eval() - 1) == ((Key)o).getOrig())){
                    pop(o);
                }
                pop(o);
                return this;
            }
            else {
                while(!(getObject(stackSize.eval() - 1 ) == o)){
                    pop(o);
                }
                pop(o);
                return this;
            }
        }
        return this;
    }
    
    /**
     * Removes the top key from the stack.
     *
     * @param o can be anything
     * @return stack after deletion
     */
    public FDT pop(Object o) {
        int index = stackSize.eval();
        if (index > 0) {
            index--;
            stackSize.assign(index);
            setObject(null, index);
        }
        else {
            setObject(null, 0);
        }
        //FIXME : this should probably return this?
        return getArray();
    }	
    
    public Object search(Object o) {
        //FIXME : this should probably return this?
        return getArray();
    }
    
    /**
     * Returns the top key of the stack.
     *
     * @return key from the top of the stack
     */
    public Object getTopOfStack() {
        int index = stackSize.eval();
        if(index > 0)
            return getObject(index - 1);
        else
            return getObject(0);
    }
    
    public int getStackSize(){
        return stackSize.eval();
    }
    
    public boolean isEmpty(){
        return stackSize.eval() == 0;
    }
    
    static final long serialVersionUID = 7919275024495283856L;
}
