/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: DoublyLinkedList.java,v 1.4 2004/05/25 13:28:11 kstalnac Exp $ */

package matrix.structures.FDT;

/**
 * Interface for dynamic doubly linked lists. This interface extends
 * LinkedList interface by adding methods for handling the previous node
 * in the list.
 */
public interface DoublyLinkedList extends LinkedList {

    /**
     * Returns the previous node in the list.
     * @return the previous node
     */
    public DoublyLinkedList getPrev();

    /**
     * Sets the previous node in the list.
     * @param l the new previous node
     */
    public void setPrev(DoublyLinkedList l);

    /**
     * Tells if this node has a previous node.
     * @return true if there is a previous node, false otherwise
     */
    public boolean hasPrev();

    static final long serialVersionUID = -4251199726833267588L;
}
