/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.indexed.StaticLinkedList;
import matrix.structures.other.Comparer;
import matrix.util.Note;

/**
 * This is an array that can be visualised as a list
 */

//XXX Check whether we could support iterable (vanillatable -> table)
public class ArrayList extends VanillaTable implements StaticLinkedList {

    private int current;
    private boolean end;

    public ArrayList() {
        this(10);
        current = 0;
        end = false;
    }


    public ArrayList(int i) {
        super();
        setObject(null, 0);
        setObject(null, i);
        //Note.out(this, "First: " + getFirst());
        //Note.out(this, "Last: " + getLast());
        current = getFirst();
        end = true;
   }

  public boolean equals(Object o) {
    if(o instanceof StaticLinkedList)
      return Comparer.compareList(this, (StaticLinkedList)o);
    else
      return super.equals(o);
  }

    public void append(Object item) {
        setObject(item, getLast() + 1);
    }

    public Object currValue() {
        return getObject(current);
    }

    public void insert(Object item) {
        int tmp = current;
        Object tmp1 = item;
        Object tmp2 = getObject(tmp);
        while(tmp1 != null) {
            setObject(tmp1, tmp);
            tmp1 = tmp2;
            tmp2 = getObject(++tmp);
        }
    }

    public boolean isEmpty() {
        setFirst();
        return end;
    }

    public boolean isInList() {
        return !end;
    }

    public void next() {
        if(current == getLast()) {
            Note.out(this, "End of list");
            current++;
            end = true;
            return;
        }
        //int tmp = current;
        current++;
        while(getObject(current) == null && current < getLast())
            current++;
        //Note.out(this, "First: " + current);
        if(getObject(current) != null)
            return;
        if(current == getLast()) {
          //Note.out(this, "Last reached");
            current = getFirst();
            end = true;
        }
    }

    public void setFirst() {
        end = false;
        current = getFirst();
        while(getObject(current) == null && current < getLast())
            current++;
        //Note.out(this, "First: " + current);
        if(getObject(current) != null)
            return;
        if(current == getLast()) {
          //Note.out(this, "Last reached");
            current = getFirst();
            end = true;
        }

    }

    public void setPos(int pos) {
        setFirst();
        while(pos > 0) {
            next();
            pos--;
        }
    }


    static final long serialVersionUID = 2120974964218588444L;
}
