/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: BinTree.java,v 1.26 2006/12/12 14:09:10 oseppala Exp $ */

package matrix.structures.FDT.probe;

// Binary Tree implementation

//import java.awt.Container;
import java.util.Vector;
import java.util.ArrayList;

import matrix.structures.FDT.*;
import matrix.structures.FDT.substructures.Vertex;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;
import matrix.structures.simulationextensions.Selectable;
import matrix.structures.util.Iterator;
import matrix.structures.util.PolyMorph;
import matrix.util.Note;

/**
 * This is an implementation of a binary tree. A binary tree is a
 * tree in which each node has two children (either of which can be null).
 */
public class BinTree implements BinaryTree, Tree, Array, Vertex, Iterator, PolyMorph, Selectable {
    
    private VirtualObject data;
    private VirtualObject left;
    private VirtualObject right;
    

    public BinTree getParent(Object o) {
        return null;
    }

    /** construct a new empty <code>BinaryTree</code> */
    public BinTree() {
        data = new VirtualObject(this, "node data");
        left = new VirtualObject(this, "left child");
        right= new VirtualObject(this, "right child");
        
    }

    /**
     * Constructs a new <code>BinaryTree</code>.
     * @param obj the contents of the root node
     */
    public BinTree(Object obj) {
        data = new VirtualObject(this, "node data");
        left = new VirtualObject(this, "left child");
        right= new VirtualObject(this, "right child");

        data.setObject(obj);
    }

    public boolean equals(Object o) {
        if(o instanceof Tree)
            return Comparer.compareIdentical(this, (Tree)o);
        return false;
    }

    // FDT interface Methods
    public FDT getNewStructure() { return new BinTree(); }

    // CDT (not actually defined as an interface)

    // Return and set the element value
    public Object getElement() {
        return data.getObject();
    }

    public void setElement(Object v) {
        //Note.out(this, "setElement " + v);
        data.setObject(v);
    }

    // Return and set the left child
    public BinaryTree getLeft() {
        return (BinaryTree)left.getObject();
    }

    public void setLeft(BinaryTree p) {
        left.setObject(p);
    }

    // Return and set the right child
    public BinaryTree getRight() {
        return (BinaryTree)right.getObject();
    }

    public void setRight(BinaryTree p) {
        right.setObject(p);
    }

    // Return true if this is a leaf node
    public boolean isLeaf() {
        return ((left.getObject() == null) && (right.getObject() == null));
    }


    public Object getObject(int index) {
        // Do we really need this?

        switch (index) {

        case 0 :
            return data.getObject();
        case 1 :
            return left.getObject();
        case 2 :
            return right.getObject();
        default :
            throw new IllegalArgumentException("Index out of range in "+this);
        }
    }

    public void setObject(Object o, int index) {
        if (index == 0)
            setElement(o);
        else
            Note.show(this, "Illegal operation");
    }

    public int getFirst() { return 0; }

    public int getLast() { return 2; }

    public void setFirst(int i) {
        // do nothing 
    }

    public void setLast(int i) {
        // do nothing
    }

    // Methods for interface Tree
    public int getSubTreeCount() {
        if (getLeft() != null || getRight() != null || getElement() != null)
          return 2;
        else
            return 0;
        //if ((getLeft() == null) && (getRight() == null)) return 0;
        //if ((getLeft() != null) && (getRight() != null)) return 2;
        //return 1;
    }

    public Tree[] getSubTrees() {
        Tree[] t = new Tree[2];
        t[0] = getLeft();
        t[1] = getRight();
        return t;
    }

    // Methods for interface Dynamic Tree

    public void setSubTree(Tree t, int child) {
        if ((t == null) || (t instanceof BinaryTree)) {
            BinaryTree bt = (BinaryTree)t;
            if (child == 0) setLeft(bt);
            else setRight(bt);
        } else Note.warning(this, "Parameter " + t + " not a binary tree");
    }

    public Tree getNewNode(Object key) {
        return new BinTree(key);
    }

  //public void insert(FDT s) { setElement(s); }

    public String toString() {
        if (getElement() instanceof Key)
            return getElement().toString();
        else {
            String s = "null";
            if (getElement() != null) s = getElement().getClass().toString();
            return super.toString() + "(" + s + ")";
        }
    }

    // Method(s) for interface Vertex
    public Vertex[] getSuccessors() {
        Tree[] t = getSubTrees();
        return new Vertex[] { (Vertex)t[0],(Vertex)t[1] };
    }

    // Method(s) for interface Graph
    /*
    public Vertex[] getVertices() {
        Vertex[] vLeft = null, vRight = null;
        if (getLeft() instanceof Graph)
            vLeft = ((Graph)getLeft()).getVertices();
        if (getRight() instanceof Graph)
            vRight = ((Graph)getRight()).getVertices();
        int size = 1;
        if (vLeft != null) size += vLeft.length;
        if (vRight != null) size += vRight.length;
        Vertex[] v = new Vertex[size];
        int cnt = 0;
        v[cnt++] = this;
        if (vLeft != null)
            for (int i=0; i<vLeft.length; i++)
                v[cnt++] = vLeft[i];
        if (vRight != null)
            for (int i=0; i<vRight.length; i++)
                v[cnt++] = vRight[i];
        return v;
    }
    */
    public void addSuccessor(Vertex v) {}

    public void removeSuccessor(Vertex v) {}
    
    public void setSuccessors(Vertex[] v) {
        // TODO Auto-generated method stub        
    }


    // --- methods for interface iterator ---

    private Vector queue = null;

    public void reset() {
        queue = new Vector(10, 1);
        queue.addElement(this);
    }

    public boolean hasNext() {
        return (queue == null || queue.size() != 0);
    }

    public Object next() {
        if(queue == null) {
            queue = new Vector(10, 1);
            queue.addElement(this);
        }
        Tree tmp = (Tree)queue.elementAt(0);
        queue.removeElementAt(0);
        Tree[] c = tmp.getSubTrees();
        for(int i=0; i < c.length;i++)
            if(c[i] != null)
                queue.addElement(c[i]);
        return tmp.getElement();
    }

    // implementation for interface PolyMorph

    /*These ArrayLists are used only when creating new BinTree
     *through methods of PolyMorph interface.  These ArrayLists
     *contain same nodes at the beginning. When two childs have been
     *added to a parent, that parent is removed from nodesForPolyMorph
     *so that no more childs can be added to it.  However, same parent
     *still exists in childNodes ArrayList so that it is still
     *possible to add that to other parent's child.*/
    transient protected ArrayList nodesForPolyMorph = new ArrayList(); //contains parents
    transient protected ArrayList childNodes = new ArrayList(); //contains childs


    /**
     * Creates a new node (a new BinTree object) that is then returned.
     *
     * @param fdt FDT that is to be inserted in the new node
     * @return a created new node that have parameter fdt in it
     */
    public FDT createNewNode(FDT fdt) {
	//        System.out.println("creating new node: " + fdt);
	FDT fdttemp = null;
	if (getElement() == null) {
	    setElement(fdt);
	    nodesForPolyMorph.add(this);
	    childNodes.add(this);
	    return this;
	}
	else {
	    fdttemp = getNewNode(fdt);
	    nodesForPolyMorph.add(fdttemp);
	    childNodes.add(fdttemp);
	    return fdttemp;
	}
    }

    /**
     * Sets the successors of a father node. Parameters are same nodes
     * that were created and returned in createNewNode method.
     *
     * @param father a node whose successor is to be set
     * @param child a node that is to be set as a successor of a
     * father node
     */
    public void setSuccessor(FDT father, FDT child) {
	//	System.out.println("setting " + child + " as a successor for " + father);

	if (nodesForPolyMorph.contains(father) && childNodes.contains(child)) {

	    //nodesForPolyMorph.contains(child)) {
	    BinTree f = (BinTree)father;
	    BinTree c = (BinTree)child;

	    if (f.getLeft() == null)
		f.setLeft(c);
	    else if (f.getRight() == null) {
		f.setRight(c);

		/*Next two if-clauses handle the null keys that are
		 *inserted in the structure. Finally, they are removed
		 *from the structure to prevent their visualization to
		 *be shown. They are used to be able to insert single
		 *key as a right child. (First, null key is inserted
		 *as a left child, then key is inserted as a right
		 *child, and, finally, null key from left child is
		 *removed.)
		 */
		if(f.getLeft().getElement() instanceof Key)
		    if(((Key)f.getLeft().getElement()).getComparisonString().equals("null"))
			f.setLeft(null);

		if(f.getRight().getElement() instanceof Key)
		    if(((Key)f.getRight().getElement()).getComparisonString().equals("null"))
			f.setRight(null);
		//Father removed from the list of father nodes to
		//prevent additional nodes to be inserted (two nodes
		//have been inserted so far)
		nodesForPolyMorph.remove(nodesForPolyMorph.indexOf(father));
	    }
	}

    }

    /** This method is invoked if the user selects this node.
     * Subclasses should override this method in order to give a semantic for this simulation operation.
     * Known use: RedBlackNode that changes the color of the node.
     */
    public void setSelect(long timeStamp) {
	//Note.out(this, "This method in intentionally left unimplemented.");
    }

    /**
     * Subclasses should override this method in order to give a semantic for this simulation operation.
     */
    public long getSelectionTime(){
    	return 0; // The entity is never selected
    }
    
    static final long serialVersionUID = 449457555191564025L;

}
