/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.FDT.probe;

import matrix.structures.util.MatrixComparable;
import matrix.util.Note;

/**
 * This is a singleton <code>Class</code> representing the
 * lexicographically infinity. It is greater than any other
 * Element.
 *
 * @author Jussi Nikander
 */
public class LexInf extends Key {

    private static MatrixComparable inf = null;

    private LexInf() {
        super("\uFFFF\uFFFF\uFFFF ...");
    }

    public void setElement(Object o) {
        Note.err(this,
                 "Element: Cannot perform setElement("+o+") for primitive");
    }

    public Object getElement() {
        return this;
    }

    
    public static MatrixComparable getInstance() {
        if(inf == null)
            inf = new LexInf();
        return inf;
    }

    public boolean eq(MatrixComparable k) {
        return false;
    }

    public boolean lt(MatrixComparable k) {
        return false;
    }

    public boolean gt(MatrixComparable k) {
        return true;
    }

    public boolean geq(MatrixComparable k) {
        return true;
    }

    public boolean leq(MatrixComparable k) {
        return false;
    }

    public boolean equals(Object k) {
        if(k != this)
            return false;
        return true;
    }

    public String getComparisonString() {
        return "\uFFFF\uFFFF\uFFFF ...";
    }
    
    public String getVisualizationString(){
        return "infinity";
    }

    static final long serialVersionUID = 7773956070359944730L;
}
