/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.structures.FDT.probe;

import java.io.Serializable;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;

/**
 * Implementation of a linked list. A linked list is a collection of
 * nodes that contain keys, and a pointer to the next node
 * in the list.
 *
 * @author Jussi Nikander, Archie
 * @version 2.0
 */
public class LinkedListImpl implements LinkedList, Serializable  {

    private VirtualObject data;
    private VirtualObject successor;

    public LinkedListImpl() {

        data     = new VirtualObject(this, "data of the node");
        successor= new VirtualObject(this, "successor of the node");
    }

    public LinkedListImpl(Object key) {
        this();
        setElement(key);
    }

    public boolean equals(Object o) {
        if(o instanceof LinkedList)
            return Comparer.compareList(this, (LinkedList)o);
        else
            return false;
    }

    //--- interface FDT ----

    public Object getElement() {
        return data.getObject();
    }

    public void setElement(Object o) {
        if(o instanceof FDT || o == null)
            data.setObject(o);
        else
            data.setObject(new Key(o));
    }

    // --- interface LinkedList ---

    public LinkedList getNext() {
        return (LinkedList) successor.getObject();
    }

    public void setNext(LinkedList l) {
        successor.setObject(l);
    }

    public boolean hasNext() {
        return getNext() != null;
    }

    public LinkedList getNewNode(Object key) {
        return new LinkedListImpl(key);
    }

    public String toString() {
        return "LinkedListImpl: key is: " + (String)((data.getObject()==null) ? "NULL":((Key)data.getObject()).getIdentityString());
    }


    static final long serialVersionUID = -537868976419949417L;
}
