/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.substructures.Vertex;
import matrix.util.Note;

/**
 * An implementation of Vertex for undirected graphs. All edges added to
 * a graph consisting of UndirVertices are made two-way.
 *
 * @author Jussi Nikander
 */

public class UndirVertex extends VertexImpl implements Vertex {

    public UndirVertex() {
        super();
    }

    public UndirVertex(Object o) {
        super(o);
    }

    public VertexImpl getNewVertex(Object o) {
        return new UndirVertex(o);
    }

    public void addSuccessor(Vertex v) {
        if(v instanceof UndirVertex) {
            add(v);
            ((UndirVertex)v).add(this);
        }
    }

    private void add(Vertex v) {
        super.addSuccessor(v);
    }

    public void setSuccessors(Vertex[] v) {
        Note.out(this, "Not implemented for undirected graphs.");
    }

    public void removeSuccessor(Vertex v) {
        if(v instanceof UndirVertex) {
            remove(v);
            ((UndirVertex)v).remove(this);
        }
    }


    private void remove(Vertex v) {
        super.removeSuccessor(v);
    }

    static final long serialVersionUID = -3596929801825800036L;
}
