/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Vertex.java,v 1.3 2006/06/28 12:33:53 oseppala Exp $ */
/* 24/05/2006 moved from package matrix.structures.FDT (vkaravir) */ 

package matrix.structures.FDT.substructures;

import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Graph;

//import matrix.structures.memory.*;

/** Common interface for all Vertices.
 * This interface defines the methods necessary to iterate through
 * a set of vertices.
 * @see Graph
 *
 * @author Ari Korhonen, Riku Saikkonen
 * @version $Revision: 1.3 $
 */
public interface Vertex extends FDT {

    /**
     * Retuns the successor nodes of this vertex.
     *@return the successor nodes of this vertex.
     */
    public Vertex[] getSuccessors();

    /**
     * Returns the key of this vertex.
     * @return the key
     */
    public Object getElement();

    /**
     * Sets the key of this vertex.
     * @param o the new key
     */
    public void setElement(Object o);

    /**
     * Sets the successors of this node.
     * @param v the new list of successors
     */
    public void setSuccessors(Vertex[] v);

    /**
     * Adds a successor to the list of successor vertices
     * @param v the new successor
     */
    public void addSuccessor(Vertex v);

    /**
     * Removes a successor from the list of successor vertices
     * @param v the successor to be removed
     */
    public void removeSuccessor(Vertex v);

    static final long serialVersionUID = 2663296277736260922L;
}
