/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/** Quick and dirty... */

package matrix.structures.code;

//import matrix.visual.stylesheet.*;
import java.awt.Color;

import matrix.decoration.*;
import matrix.structures.FDT.probe.Key;

/** Abstraction for single code line in Code frame. Implements Styled
 * in order to visualize code lines colored properly, thus the coloring
 * of the final version should be informative.
 * Under construction: usability testing required.
 */
public class StyledCodeLine extends Key implements CodeLine, Styled {
    Code code;
    CodeLine codeline;

    public StyledCodeLine(Code code, CodeLine codeline) {
        super(codeline.getCodeLine());
        this.code = code;
        this.codeline = codeline;
    }

    /* --- interface Styled --- */
    public StyleSheet getStyleSheet() {
        return new StyleSheetAdapter() {
                public Color getDefaultBorderColor() {
                    if (code.getTime()-codeline.getStamp() < 0)
                        return Color.blue;
                    Color c = Color.red;
                    switch (code.getTime() - codeline.getStamp()) {
                    case 0: return c.darker().darker().darker();
                    case 1: return c.darker().darker();
                    case 2: return c.darker();
                    default: return c;
                    }
                }
                public Color getVisitedBorderColor() {
                    return Color.blue;
                }
                public Color getDefaultBackgroundColor() {
                    if (code.getTime()-codeline.getStamp() < 0)
                        return Color.red;
                    Color c = Color.white;
                    switch (code.getTime() - codeline.getStamp()) {
                    case 0: return c.darker().darker().darker();
                    case 1: return c.darker().darker();
                    case 2: return c.darker();
                    default: return c;
                    }
                }
                public Color getVisitedBackgroundColor() {
                    return getDefaultBackgroundColor();
                }
                public Color getDefaultPenColor() {
                    if (code.getTime()-codeline.getStamp() <= 0)
                        return Color.white;
                    else
                        return Color.blue;
                }
                public boolean isInfoEnabled() {
                    return codeline.getDescription() != null;
                }
                public String getInfo() { return codeline.getDescription(); }
                public int getWaitTime() { return 100; }
		static final long serialVersionUID = 173293856720183616L;
            };
    }

    /* --- interface CodeLine --- */
    public String getCodeLine() { return codeline.getCodeLine(); }
    public void stamp(int time) { codeline.stamp(time); }
    public int getStamp() { return codeline.getStamp(); }
    public String getDescription() { return codeline.getDescription(); }

    static final long serialVersionUID = 173293856720183616L;
}
