/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2006 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.code.probe;

import matrix.structures.code.AnnotatedPseudocode;
import matrix.structures.memory.VirtualBoolean;
import matrix.structures.memory.VirtualInteger;

/**
 * This class holds both pseudocode and
 * the current location within it.
 *
 * The class for visualizing the datais matrix.visual.VisualPseudocode.
 *
 * @author Otto Seppl
 * @version $Revision: 1.3 $
 * 
 * @see matrix.visual.VisualPseudocode
 */


public class PseudoCode implements AnnotatedPseudocode {

    private VirtualInteger location = new VirtualInteger(0, this, "currently active code portion"); 
    private VirtualBoolean errorState = new VirtualBoolean(false, this, "is the current state erroneous");
    
    private String[] annotatedCodelines;
    private String codeName;    
    
    public String getCodeName() {
        return codeName;
    }
    
    public void setCodeName(String codeName){
        this.codeName = codeName;
    }

    public void setCodeLines(String[] codeLines){
        annotatedCodelines = codeLines;
    }
    
    public String[] getCodeLines() {
        return annotatedCodelines;
    }

    public int getLocation() {
        return location.eval();
    }

    public void setLocation(int location) {
        if (location != this.location.eval())
            errorState.assign(false);
        
        this.location.assign(location);
    }

    // Required by the FDT-interface, but not applicable
    
    public Object getElement() {
        // XXX Not applicable ?
        return null;
    }

    public void setElement(Object o) {
        // XXX Not applicable ?
    }

    public void markError(){
        errorState.assign(true);
    }

    public boolean getErrorState(){
        return errorState.eval();
    }
}
