/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: MemoryStructure.java,v 1.12 2006/05/18 12:01:00 vkaravir Exp $ */

package matrix.structures.memory;

import java.io.Serializable;

import matrix.structures.FDT.FDT;

/**
 * The characteristics of the memory structure are that it can store and retrieve
 * elements of any type by simply addressing them with an integer.
 *
 */
public interface MemoryStructure extends Serializable {
    //public void swap(int i, int j);
    //public void setObject(Object obj, int index);
    //public Object getObject(int index);
    //public int size();

    /**
     * Returns Object from desired index
     * @return the Object at memoryblock index.
     *
     */
    public Object getMem(int index);

    /**
     * Set Object at memory block index.
     *
     * @param index index of memoryblock to change
     * @param obj   new value to be set in memoryblock
     */
    public void setMem(int index, Object obj);

    /**
     * set MemoryStructure to be animated or not
     *
     * @param val true set MemoryStructure to be animated with matrix animator
     */
    //public void setAnimated(boolean val);

    /**
     * Returns information if MemoryStructure is animated or not.
     *
     * @return true if memoryStructure is animated
     */
    //public boolean isAnimated();

    /** Get the name of this MemoryStructure. */ 
    public String getName();
    /** Set the name of this MemoryStructure. The name should describe
     * the usage of this object. */ 
    public void setName(String name);
    /** Get the FDT that this MemoryStructure is part of. */
    public FDT getFDT();
    /** Set the FDT that this MemoryStructure is part of. */
    public void setFDT(FDT fdt);
    
    static final long serialVersionUID = 9124025352451572490L;
}
