/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VirtualObject.java,v 1.11 2007/08/22 10:04:47 oseppala Exp $ */

package matrix.structures.memory;

/*
    Data Structure library - 4/96 PNL
*/
import matrix.animation.Animator;
import matrix.structures.FDT.FDT;

/**
 * VirtualObject is a Matrix <code>MemoryStructure</code> that holds a single object in it.
 * For Matrix animation features to be available, the instance variables of the
 * class representing the memory structure to be animated must be held in a class that
 * implements <code>MemoryStructure</code>.
 *
 * @author Jussi Nikander
 */
public final class VirtualObject implements MemoryStructure {

    /** Contents of this VirtualObject */
    protected Object memory;
    protected String name;
    protected FDT fdt;

    /** Constructs a new <code>VirtualObject</code>. */
    protected VirtualObject() {
    }

    /**
     * Constructs a new <code>VirtualObject</code>.
     * @param o the contents of this Object
     */
    protected VirtualObject(Object o) {
        this();
        memory = o;
    }
    
    public VirtualObject(FDT fdt, String name) {
        this();
        setFDT(fdt);
        setName(name);
    }
    
    public VirtualObject(Object o, FDT fdt, String name) {
        this(o);
        setFDT(fdt);
        setName(name);
    }

    /**
     * Sets the contents of this <code>VirtualObject</code> to a new value.
     * @param o the new contents of this node
     */
    public void setObject(Object o) {
        Animator.getActiveAnimator().putAssignment(this, o, 0);
        memory = o;
    }

    /**
     * Retrieves the contents of this node.
     * @return the contents
     */
    public Object getObject() {
        return memory;
    }

    public Object getMem(int index) {
        return memory;
    }

    public void setMem(int index, Object obj) {
        memory = obj;
    }

    public String toString() {
        if(memory == null)
            return null;
        return memory.toString();
    }


    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public FDT getFDT() {
        return fdt;
    }

    public void setFDT(FDT fdt) {
        this.fdt = fdt;
    }

    static final long serialVersionUID = 8381486804453980737L;
}
