/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.other;

import matrix.structures.FDT.indexed.StaticLinkedList;
import matrix.structures.memory.*;
import matrix.util.Note;
import matrix.structures.FDT.probe.Key;
import matrix.structures.FDT.probe.VanillaTable;

/**
 * This is a Deque to be used in judging Deque exercises in Matrix. It is not
 * meant to be manipulated through the GUI.
 *
 * @author Jussi Nikander
 */

public class Deque extends VanillaTable implements StaticLinkedList {

    private VirtualInteger first, last, current;

    public Deque() {
        super();
        setObject(null, -5);
        setObject(null, 5);
        first   = new VirtualInteger(0, this, "index of the beginning of the deque in array");
        last    = new VirtualInteger(0, this, "index of the end of the deque in array");
        current = new VirtualInteger(0, this, "index of the cursor to the deque in array");
    }

    /**
     * Puts the argument Object to the beginning of the Deque
     * @param o the Object put
     */
    public void put(Object o) {
        Key k;
        if(o instanceof Key)
            k = (Key)o;
        else
            k = new Key(o);
        setObject(k, first.eval());
        first.dec();
    }

    /** Pushes the argument Object to the end of the Deque */
    public void push(Object o) {
        Key k;
        if(o instanceof Key)
            k = (Key)o;
        else
            k = new Key(o);
        //last.inc();
        setObject(k, last.inc());
    }

    /** Retrieves the Object at the beginning of the Deque and removes it from the Deque */
    public Object get() {
        Object tmp = getObject(first.inc());
        setObject(null, first.eval());
        return tmp;
    }

    /** Retrieves the Object at the end of the Deque and removes it from the Deque */
    public Object pop() {
        Object tmp = getObject(last.eval());
        setObject(null, last.eval());
        last.dec();
        return tmp;
    }

    // Interface List

    public void append(Object item) {
        push(item);
    }

    public Object currValue() {
        return getObject(current.eval());
    }

    public void insert(Object item) {
        Note.out(this, "Cannot insert to Deque");
    }

    public boolean isEmpty() {
        return first.eval() == last.eval();
    }

    public boolean isInList() {
        return !(getObject(current.eval()) == null);
    }

    public void next() {
        current.inc();
    }

    public void setFirst() {
        current.assign(first.eval());
        while(getObject(current.eval()) == null && current.eval() < last.eval())
          current.inc();
    }

    public void setPos(int pos) {
        setFirst();
        while(pos > 0) {
            current.inc();
            pos--;
        }
    }


    static final long serialVersionUID = -1072022093050565153L;
}
