package matrix.structures.spatial.FDT.probe;

import matrix.structures.spatial.PaintingStyleDecorator;

/**
 * A data structure representing a circle of radius r centered at point (x,y). This class is essentially
 * a shorthand for an <code>Ellipse</code> where width and height are equal. See javadoc for
 * <code>matrix.structures.spatial.FDT.probe.Ellipse</code> for further information.
 * 
 * @author Jussi Nikander
 *
 */
public class Circle extends Ellipse {

    public Circle() {
		super();
	}
	
    /**
     *  Constructor for a circle.
     * @param x The x coordinate of the center of the circle.
     * @param y The y coordinate of the center of the circle.
     * @param r The radius of the circle.
     * @param key The key (string identifier) of the circle.
     */
	public Circle(double x, double y, double r, String key) {
		super(x, y, 2 * r, 2 * r, key);
	}
	
    public Circle(double x, double y, double r) {
        this(x, y, r, "");
    }

    public Circle(double x, double y, double r, PaintingStyleDecorator psd) {
        this(x, y, r, "");
        this.setPaintingStyleDecorator(psd);
    }
    
    private static final long serialVersionUID = 1151090030094939185L;
}
