package matrix.structures.spatial.FDT.probe;

import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;

import content.exercises.sda.structures.GeometryFunctions;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;

/**
 * A class for storing and visualizing parabolic curves 
 * in the Area visualization.
 * 
 * @author Juha Helminen
 * 
 */
public class Parabola extends SpatialElement {
	
	private final QuadBezier bezier;
	
	public Parabola(Point2D focus, Line2D directrix) {
		this(focus, directrix, directrix.ptLineDist(focus), directrix.ptLineDist(focus), null);
	}

	public Parabola(Point2D focus, Line2D directrix, double w1, double w2, PaintingStyleDecorator psd) {
	    if (psd != null) {
	        this.setPaintingStyleDecorator(psd);
	    }
		double length = Math.sqrt(sqr(directrix.getX2() - directrix.getX1()) + 
								  sqr(directrix.getY2() - directrix.getY1()));
		double h = directrix.ptLineDist(focus);
		double nx, ny;
		if (directrix.relativeCCW(focus) < 0) {
			nx = (directrix.getY2() - directrix.getY1()) / length;  
			ny = (directrix.getX1() - directrix.getX2()) / length;
		} else {
			nx = -(directrix.getY2() - directrix.getY1()) / length;  
			ny = -(directrix.getX1() - directrix.getX2()) / length;
		}
		double v1x = (w2 * (directrix.getX2() - directrix.getX1())) / length;  
		double v1y = (w2 * (directrix.getY2() - directrix.getY1())) / length;
		double[] result = new double[2];
		double[] args = new double[]{sqr(h)*sqr(nx) - 2*h*nx*v1x + sqr(v1x) + sqr(h)*sqr(ny) - 2*h*ny*v1y + sqr(v1y),
									 -2*h*sqr(nx) + 2*nx*v1x - 2*h*sqr(ny) + 2*ny*v1y,
									 sqr(nx) + sqr(ny) - 1};
		QuadCurve2D.solveQuadratic(args, result);
		
		double d1 = result[0];
		// XXX How to properly choose the correct root
		if (Math.abs(d1) > 1E6) {
			d1 = result[1];
		}

		Point2D start = new Point2D.Double(focus.getX() + v1x + (h - d1)*nx,
										   focus.getY() + v1y + (h - d1)*ny);
		

		double v2x = (-w1 * (directrix.getX2() - directrix.getX1())) / length;  
		double v2y = (-w1 * (directrix.getY2() - directrix.getY1())) / length;
		result = new double[2];
		args = new double[]{sqr(h)*sqr(nx) - 2*h*nx*v2x + sqr(v2x) + sqr(h)*sqr(ny) - 2*h*ny*v2y + sqr(v2y),
  						    -2*h*sqr(nx) + 2*nx*v2x - 2*h*sqr(ny) + 2*ny*v2y,
							sqr(nx) + sqr(ny) - 1};
		QuadCurve2D.solveQuadratic(args, result);
		
		double d2 = result[0];
		// XXX How to properly choose the correct root
		if (Math.abs(d2) > 1E6) {
			d2 = result[1];
		}

		Point2D end = new Point2D.Double(focus.getX() + v2x + (h - d2)*nx,
										 focus.getY() + v2y + (h - d2)*ny);
		
		
		double r1x = (-v1x + (d1 - h)*nx)  + d1 * nx;
		double r1y = (-v1y + (d1 - h)*ny) + d1 * ny;
		double r2x = (-v2x + (d2 - h)*nx) + d2 * nx;
		double r2y = (-v2y + (d2 - h)*ny) + d2 * ny;
		
		Line2D tan1 = new Line2D.Double(start.getX(), start.getY(), start.getX() + r1x, start.getY() + r1y);
		Line2D tan2 = new Line2D.Double(end.getX(), end.getY(), end.getX() + r2x, end.getY() + r2y);
		Point2D control = GeometryFunctions.lineIntersection(tan1, tan2);
		
		this.bezier = new QuadBezier(start.getX(), start.getY(), 
									 control.getX(), control.getY(), 
									 end.getX(), end.getY());
	}
	
	private static double sqr(double number) {
		return (number * number);
	}

    public Shape getShape() {
        return this.bezier.getShape();
    }

    public boolean contains(SpatialComparable sc) {
		return this.bezier.contains(sc);
	}
	
	public boolean intersects(SpatialComparable sc) {
		return this.bezier.intersects(sc);
	}

	private static final long serialVersionUID = -8829116686076383341L;
}
