package matrix.structures.spatial.FDT.probe;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.util.MatrixComparable;
import matrix.structures.FDT.Struct;
import matrix.structures.memory.VirtualObject;
import matrix.util.Note;

import java.awt.geom.Point2D;
import java.awt.Shape;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

/**
 * A representation of a point object in two-dimensional space using double accuracy. Separate
 * class from the representation of point in awt. This point is a data element, and not graphical element.
 * The Point object contains an instance of <code>Point2D.Double</code> <strong>and</strong> instance of
 * java.awt.geom.GeneralPath. GeneralPath is used for comparison with other <code>SpatialComparable</code> objects.
 *
 * @author Jussi Nikander
 *
 */

public class Point extends SpatialElement implements SpatialComparable, MatrixComparable, Struct {

	
	private VirtualObject x, y;
	//private double x, y;
	//private Point2D.Double p;
	
	private static String[] fieldNames = {"id", "x", "y"};
	
	public Point() {
		this(0,0,"");
	}
	
	public Point(Point2D point) {
        this(point.getX(), point.getY(), "");
    }

	public Point(Point2D point, PaintingStyleDecorator psd) {
        this(point.getX(), point.getY(), "");
        this.setPaintingStyleDecorator(psd);
    }
    
    public Point(Point2D point, String key) {
        this(point.getX(), point.getY(), key);
    }

    public Point(double x, double y) {
        this(x, y, "");
    }

    public Point(double x, double y, String key) {
		super(key);
		this.x = new VirtualObject(new Double(x), this, "x");
		this.y = new VirtualObject(new Double(y), this, "y");
		//p = new Point2D.Double(x,y);
	}

	public Point2D getPoint2D() {
		return new Point2D.Double(((Double)x.getObject()).doubleValue(),((Double)y.getObject()).doubleValue());
	}

	public java.awt.Point getPoint() {
		return new java.awt.Point(((Double)x.getObject()).intValue(),
		                          ((Double)y.getObject()).intValue());
	}
	
	public double getX() {
		return ((Double)x.getObject()).doubleValue();
	}
	
	public void setX(double x) {
		this.x.setObject(new Double(x));
	}
	
	public double getY() {
		return ((Double)y.getObject()).doubleValue();
	}
	
	public void setY(double y) {
		this.y.setObject(new Double(y));
	}
	
	public boolean equals(SpatialElement o) {
		if(o instanceof Point) {
			Point comp = (Point)o;
			return spatialEquals(comp) && 
				getComparisonString().equals(comp.getComparisonString());
		}
		return false;
	}
	
	public boolean intersects(SpatialComparable sc) {
		if(sc instanceof Point) {
			return spatialEquals(sc);
		} else  {
			return sc.contains(this);
		}
	}

	public boolean contains(SpatialComparable sc) {
		if(sc instanceof Point) {
			return spatialEquals(sc);
		} else {
			return false;
		}
	}

	public Rectangle2D getBoundingBox() {
		return new Rectangle((int)getX(), (int)getY(), 0, 0);
	}

	public boolean spatialEquals(Object o) {
		if(o instanceof Point) {
			Point comp = (Point)o;
			return equals(comp.getX(), getX())  &&
				equals(comp.getY(), getY());
		}
		return false;
	}

    public String toString() {
		return super.toString();
	}

	public Shape getShape() {
		return null;
	}

	public String[] getFieldNames() {
		return fieldNames;
	}

	public String getFieldName(int location) {
		if(location < 3 && location > -1) {
			return fieldNames[location];
		}
		return "";
	}

	public Object[] getFields() {
		Object[] o = {getVisualizationString(), x.getObject(), y.getObject() };
		return o;
	}

	public Object getField(int location) {
		if(location == 0) {
			return getVisualizationString();
		} else if(location == 1) {
			return x.getObject();
		} else if(location == 2) {
			return y.getObject();
		} else {
			return null;
		}
	}

	public void setField(Object value, int location) {
		if(value == null) {
			Note.out(this, "Null value for setField, ignoring");
		}
		if(location == 0) {
			setKey(value);
		} else if (location == 1) {
			try {
				x.setObject(new Double(Double.parseDouble(value.toString())));
			}catch(NumberFormatException e) {
				Note.out(this, "cannot assign to x non-numeral value, ignoring");
			}
		} else if(location == 2) {
			try {
				y.setObject(new Double(Double.parseDouble(value.toString())));
			}catch(NumberFormatException e) {
				Note.out(this, "cannot assign to x non-numeral value, ignoring");
			}
			
		}
	}
	
}
