package matrix.structures.spatial.FDT.probe;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Area;

import matrix.structures.FDT.Struct;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.structures.FDT.probe.Table;

/**
 * 
 * @author Jussi Nikander
 *
 */

public class PolyLine extends SpatialElement implements SpatialComparable, Struct {

	private Table points;
	// transient since GeneralPath cannot be serialized
	
	private transient GeneralPath path;
	
	private static String[] fieldNames = {"id", "points" };

	public PolyLine() {
		this(new Point[0], "");
	}
	
    public PolyLine(Point[] p) {
        this(p, "");
    }
    
    public PolyLine(Line2D line) {
        this(new Point[]{new Point(line.getP1()), new Point(line.getP2())}, "");
    }

    public PolyLine(Point[] p, String key) {
		super(key);
		points = new Table(p);
		GeneralPath ph = new GeneralPath();
		/*
		path = new VirtualObject(ph);
		if(p.length > 0) {
			ph.moveTo((float)p[0].getX(), (float)p[0].getY());
			for(int i = 1; i<p.length;i++) {
				ph.lineTo((float)p[i].getX(), (float)p[i].getY());
			}
		}
		*/
	}

	public PolyLine(double[] x, double[] y, String key) {
		super(key);		
		if(x.length == y.length) {
			Point[] ps = new Point[x.length];
			//GeneralPath ph = new GeneralPath();
			if (ps.length > 0) {
				ps[0] = new Point(x[0], y[0], null);
				//ph.moveTo((float)x[0], (float)y[0]);
			}

			for(int i = 1; i<ps.length;i++) {
				ps[i] = new Point(x[i], y[i], null);
				//ph.lineTo((float)x[i], (float)y[i]);
			}
			points = new Table(ps);
			//path = new VirtualObject(ph);
		}
	}

    public PolyLine(Point2D[] points) {
        this(points, "");
    }

    public PolyLine(Point2D[] points, PaintingStyleDecorator psd) {
        this(points, "");
        this.setPaintingStyleDecorator(psd);
    }

    public PolyLine(java.awt.Point[] points, PaintingStyleDecorator psd) {
        this(points, "");
        this.setPaintingStyleDecorator(psd);
    }

    public PolyLine(Point2D[] points, String key) {
        super(key);     
        Point[] ps = new Point[points.length];
        if (ps.length > 0) {
            ps[0] = new Point(points[0]);
        }

        for(int i = 1; i<ps.length;i++) {
            ps[i] = new Point(points[i]);
        }
        this.points = new Table(ps);
    }

    public GeneralPath getPath() {
		// path is null when getPath is called for the first time, or after this object has been
		// serialized. 
		if(path == null) {
			createPath(getPoints());
		}
		//return (GeneralPath)path.getObject();
		return path;
	}

	private void createPath(Point[] p) {
		//path = new VirtualObject();
		path = new GeneralPath();
		if(p.length > 0) {
			path.moveTo((float)p[0].getX(), (float)p[0].getY());
			for(int i = 1; i<p.length;i++) {
				path.lineTo((float)p[i].getX(), (float)p[i].getY());
			}
		}
		//path.setObject(ph);
	}
	
	public Point[] getPoints() {
		Point[] ps = new Point[points.size()];
		for(int i = 0; i < ps.length; i++) {
			ps[i] = (Point)points.getObject(i);
		}
		return ps;
	}
	
	public java.awt.Point[] getPointsAwt() {
		java.awt.Point[] ps = new java.awt.Point[points.size()];
		for(int i = 0; i < ps.length; i++) {
			ps[i] = ((Point)points.getObject(i)).getPoint();
		}
		return ps;
	}

	// does not work for lines, because the area of line is always
	// empty -> never intersects
	public boolean intersects(SpatialComparable sc) {
		if(sc instanceof Point) {
			Point2D p = ((Point)sc).getPoint2D();
			return getPath().contains(p);
		} else {
			Area a = new Area(getPath());
			Area ca = new Area(sc.getShape());
			a.intersect(ca);
			return !a.isEmpty();
		}
	}

	public boolean contains(SpatialComparable sc) {
		if(sc instanceof Point) {
			Point2D p = ((Point)sc).getPoint2D();
			return getPath().contains(p);
		} else {
			Area a = new Area(getPath());
			Area ca = new Area(sc.getShape());
			ca.subtract(a);
			return ca.isEmpty();		
		}
	}

	public Rectangle2D getBoundingBox() {
	    Rectangle b = getShape().getBounds();
        return new Rectangle(b.x, b.y, b.width, b.height);
	}
	
	public boolean equals(SpatialElement o) {
		if(o instanceof PolyLine) {
			PolyLine p = (PolyLine)o;
			return spatialEquals(p) &&
			getComparisonString().equals(p.getComparisonString());
		}
		return false;
	}

	public Shape getShape() {
		return getPath();
	}

	public String[] getFieldNames() {
		return fieldNames;
	}

	public String getFieldName(int location) {
		if(location < 2 && location > -1) {
			return fieldNames[location];
		}
		return "";
	}

	public Object[] getFields() {
		Object[] o = new Object[2];
		o[0] = getVisualizationString();
		o[1] = points;
		return o;
	}

	public Object getField(int location) {
		if(location == 0) {
			return getVisualizationString();
		} 
		if (location == 1) {
			return points;
		}
		return null;
	}

	public void setField(Object value, int location) {
		if(location == 0) {
			setKey(value);
		}
		if(location == 1 && value instanceof Table) {
			points = (Table)value;
		}
	}

}
