package matrix.structures.spatial.FDT.probe;

import java.awt.Shape;
import java.awt.geom.QuadCurve2D;

import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.util.Note;

/**
 * A class for storing and visualizing quadratic bezier curves 
 * in the Area visualization.
 * 
 * @author Juha Helminen
 * @see matrix.structures.spatial.Area VisualArea 
 */

public class QuadBezier extends SpatialElement {

    private transient QuadCurve2D quadBezier;
    private final double x1, y1;
    private final double ctrlx, ctrly;
    private final double x2, y2;

	public QuadBezier(double x1, double y1, 
					 double ctrlx, double ctrly, 
					 double x2, double y2) {
		super("");
		this.x1 = x1;
		this.y1 = y1;
		this.ctrlx = ctrlx;
		this.ctrly = ctrly;
		this.x2 = x2;
		this.y2 = y2;
	}

	public QuadBezier(double x1, double y1, 
                     double ctrlx, double ctrly, 
                     double x2, double y2, PaintingStyleDecorator psd) {
        this(x1, y1, ctrlx, ctrly, x2, y2);
        this.setPaintingStyleDecorator(psd);
    }

	protected void createShape() {
	    if (this.quadBezier == null) {
	        this.quadBezier = new QuadCurve2D.Double(this.x1, this.y1, this.ctrlx, this.ctrly, this.x2, this.y2);
	    }
	}
	
    public Shape getShape() {
        this.createShape();
        return this.quadBezier;
    }

    public boolean contains(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
	}

	public boolean intersects(SpatialComparable sc) {
        Note.warning(this, "Operation not implemented.");
        return false;
	}

    private static final long serialVersionUID = -3783231279141466376L;
}
