package matrix.structures.spatial.FDT.probe;

import matrix.structures.spatial.SpatialComparable;

public class Rectangle extends Polygon {

	// helper method for constructor
	private static double[] makeXArray(double base, double delta) {
		double[] f = {base, base+delta, base+delta, base};
		return f;
	}

	// helper method for constructor
	private static double[] makeYArray(double base, double delta) {
		double[] f = {base, base, base+delta, base+delta};
		return f;
	}
	
	public Rectangle() {
		super();
	}
	
	public Rectangle(double TopLeftX, double TopLeftY, double w, double h, String key) {
		super(makeXArray(TopLeftX, w), makeYArray(TopLeftY, h), key);
	}
		
	public Rectangle(double x, double y, double w, double h) {
		this(x, y, w, h, "{ x:" + x + ", y:" + y + " [" + w + " x " + h + "]}");
	}

	/**
	 * @return The area of this rectangle.
	 */
	public double calculateArea() {
		Point[] points = this.getPoints();
		return ((points[1].getX() - points[0].getX()) * 
		        (points[2].getY() - points[0].getY())); 
	}
	
	public boolean contains(SpatialComparable sc) {
		if(sc instanceof Point) {
			Point p = (Point)sc;
			Point[] corners = this.getPoints();
			return ((p.getX() >= corners[0].getX()) &&
					(p.getX() <= corners[1].getX()) &&
					(p.getY() >= corners[0].getY()) &&
					(p.getY() <= corners[2].getY()));
		} else {
			return super.contains(sc);
		}
	}

	/**
	 * Creates the smallest rectangle which contains this rectangle
	 * and the rectangle r given as parameter.

	 * @param r A rectangle. 
	 * @return The smallest rectangle which contains this rectangle and 
	 * the rectangle r given as parameter.
	 */
	public Rectangle add(Rectangle r) {
		Point[] pa = this.getPoints();
		if (r == null) {
			return this;
		} else {
			Point[] pb = r.getPoints();
			return new Rectangle(Math.min(pa[0].getX(), pb[0].getX()), 
								 Math.min(pa[0].getY(), pb[0].getY()), 
								 Math.max(pa[1].getX(), pb[1].getX()) - Math.min(pa[0].getX(), pb[0].getX()), 
								 Math.max(pa[2].getY(), pb[2].getY()) - Math.min(pa[0].getY(), pb[0].getY()));
		}
	}

    public double getWidth(){
        Point[] p = this.getPoints();
        return p[1].getX() - p[0].getX();
    }
    
    public double getHeight(){
        Point[] p = this.getPoints();
        return p[2].getY() - p[0].getY();        
    }
    
    public double getX(){
        Point[] p = this.getPoints();
        return p[0].getX();
    }
    
    public double getY(){
        Point[] p = this.getPoints();
        return p[0].getY();
    }
}
