package matrix.structures.spatial.FDT.probe;

import java.awt.Rectangle;
import java.util.Random;
import java.util.Vector;

import matrix.structures.FDT.probe.Table;
import matrix.structures.spatial.Area;
import matrix.structures.spatial.PaintingStyleDecorator;
import matrix.structures.spatial.SpatialComparable;
import matrix.util.Note;
import matrix.util.RandomKey;

public class SpatialElementTable extends Table implements Area {

		public static final int POINT = 0;
		public static final int POLYLINE = 1;
		public static final int POLYGON = 2;
		public static final int RECTANGLE = 3;
		public static final int ELLIPSE = 4;
		public static final int CIRCLE = 5;
		public static final int POLYGONAL = 6;
		public static final int ELLIPSOID = 7;
		public static final int ALL = 8;

		
		
	   protected boolean rnd = false;
	   protected Random r;
	   protected int maxx, maxy, type, polygonSize;
	   private Rectangle bounds;
	   
	   public SpatialElementTable() {
		   this(8);
	   }
	   
	    public SpatialElementTable(int size) {
	    	this(size, 300, 300, POINT);
	        }

	    public SpatialElementTable(int size, int type) {
	    	this(size, 300, 300, type);
	    }
	    
	    public SpatialElementTable(int size, int maxx, int maxy) {
	    	super(size);
	        this.maxx = maxx;
	        this.maxy = maxy;
	        this.type = POINT;
	        this.polygonSize = 1;
			bounds = new Rectangle(maxx, maxy, 0, 0);
	    }

	    public SpatialElementTable(int size, int maxx, int maxy, int type) {
	    	super(size);
	        rnd = true;
	        this.maxx = maxx;
	        this.maxy = maxy;
	        if(type < POINT || type > ALL) {
	        	this.type = POINT;
	        } else {
	        	this.type = type;
	        }
	        polygonSize = 4;
	        r = new Random();
	        randomize();	    	
			bounds = new Rectangle(maxx, maxy, 0, 0);
	    }

	    // create random ellipses with width at least wMin nad heigh hMin
	    public SpatialElementTable(int size, int maxx, int maxy, int wMin, int hMin, long seed) {
	    	super(size);
	        rnd = true;
        	this.r = new Random(seed);
	        this.maxx = maxx;
	        this.maxy = maxy;
        	this.type = SpatialElementTable.ELLIPSE;
	        this.polygonSize = 4;
	        for(int i = this.getFirst(); i <= this.getLast() ; i++) {
	        	setObject(createRandomEllipse("" + i, wMin, hMin), i);
	        }
			bounds = new Rectangle(maxx, maxy, 0, 0);
	    }

	    private Point createRandomPoint(String key) {
        	int j = RandomKey.createIntKey(r, 0, maxx);
        	int k = RandomKey.createIntKey(r, 0, maxy);
            return new Point(j, k, key == null ? "(" + j + "," + k + ")" : key);	    	
	    }

	    private PolyLine createRandomPolyLine(String key) {
	    	return createRandomPolyLine(key, polygonSize);
	    }
	    
	    private PolyLine createRandomPolyLine(String key, int points) {
	    	Point[] p = getRandomPoints(points);
	    	return new PolyLine(p, key);
	    }

	    private Polygon createRandomPolygon(String key) {
	    	return createRandomPolygon(key, polygonSize); 
	    }
	    
	    private Polygon createRandomPolygon(String key, int points) {
	    	Point[] p = getRandomPoints(points);
	    	return new Polygon(p, key);
	    }

	    private matrix.structures.spatial.FDT.probe.Rectangle createRandomRectangle(String key) {
	    	int x, y, w, h;
	    	x = RandomKey.createIntKey(r,0,maxx);
	    	y = RandomKey.createIntKey(r,0,maxy);
	    	w = RandomKey.createIntKey(r,0,maxx-x);
	    	h = RandomKey.createIntKey(r,0,maxy-y);
	    	return new matrix.structures.spatial.FDT.probe.Rectangle(x,y,w,h, key);
	    }

	    private Ellipse createRandomEllipse(String key) {
	    	int x, y, w, h;
	    	x = RandomKey.createIntKey(r, 0, maxx);
	    	y = RandomKey.createIntKey(r, 0, maxy);
	    	w = Math.min(x, maxx - x);
	    	h = Math.min(y, maxy - y);
	    	w = RandomKey.createIntKey(r, 0, 2 * w);
	    	h = RandomKey.createIntKey(r, 0, 2 * h);
	    	return new Ellipse(x, y, w, h, key);
	    }
	    
	    private Ellipse createRandomEllipse(String key, int wMin, int hMin) {
	    	int x, y, w, h;
	    	x = RandomKey.createIntKey(r, (wMin / 2), maxx - (wMin / 2));
	    	y = RandomKey.createIntKey(r, (hMin / 2), maxy - (hMin / 2));
	    	w = Math.min(x, maxx - x);
	    	h = Math.min(y, maxy - y);
	    	w = RandomKey.createIntKey(r, wMin, 2 * w);
	    	h = RandomKey.createIntKey(r, hMin, 2 * h);
	    	return new Ellipse(x, y, w, h, key);
	    }

	    private Circle createRandomCircle(String key) {
	    	int x, y, radi;
	    	x = RandomKey.createIntKey(r,0,maxx);
	    	y = RandomKey.createIntKey(r,0,maxy);
	    	radi = RandomKey.createIntKey(r,0,Math.min(Math.min(x,maxx-x), Math.min(y, maxy-y)));
	    	return new Circle(x,y,radi,key);
	    }
	    
	    private Point[] getRandomPoints(int nro) {
	    	Point[] p = new Point[nro];
	    	for(int i = 0; i < p.length; i++) {
	    		p[i] = createRandomPoint(""+i);
	    	}
	    	return p;
	    }
	    
	    
	    /**
	     * Tells if this table random table or not.
	     * @return true if table is random.
	     */
	    public boolean isRandomTable() {
	        return rnd;
	    }

	    /**
	     * Sets the type for this SpatialElementArray. Type must be one of the types
	     * specified in public static int fields of this class.
	     * @param type the new type.
	     */
	    public void setType(int type) {
	    	if(type < POINT || type > ALL) {
	    		return;
	    	}
	    	this.type = type;
	    	randomize();
	    }
	    
	    /**
	     * Randomizes the contents of this table.
	     */
	    public void randomize() {
//			matrix.util.Note.out(this, "RANDOMIZE");
	        for(int i = this.getFirst(); i <= this.getLast() ; i++) {
	        	setObject(getRandomElement(i), i);
	        }
	    }

	    public SpatialElementTable randomizePoints(int minD, long seed) {
	    	Point p;
	    	int counter;
	    	int maxTries = 50;
	    	this.rnd = true;
    		this.r = new Random(seed);
	        for (int i = this.getFirst(); i <= this.getLast() ; i++) {
	        	counter = 0;
	        	loop:
		        	while (true) {
		        		if (++counter > maxTries) {
		        			// start over
		        			return randomizePoints(minD, seed);
		        		}
		        		p = createRandomPoint(""+i);
		        		if ((p.getX() < minD) ||
        				    (p.getY() < minD) ||
        				    ((this.maxx - p.getX()) < minD) ||
        				    ((this.maxy - p.getY()) < minD)) {
		        			continue loop;
		        		}
		        		for (int j = this.getFirst(); j < i; j++) {
		        			if ((Math.abs(p.getX() - ((Point)this.getObject(j)).getX()) < minD) &&
		        				(Math.abs(p.getY() - ((Point)this.getObject(j)).getY()) < minD)) {
		        				// Pick another random point.
		        				continue loop;
		        			}
		        		}
		        		// Point p satisfies the contraints.
		        		break loop;
		        	}
	        	setObject(p, i);
	        }
	        return this;
	    }

	    private Object getRandomElement(int i) {
        	Object o = null;
        	switch(type) {
        	case POINT:
        		o = createRandomPoint(""+i);
        		break;
        	case POLYLINE:
        		o = createRandomPolyLine(""+i);
        		break;
        	case POLYGON:
        		o = createRandomPolygon(""+i);
        		break;
        	case RECTANGLE:
        		o = createRandomRectangle(""+i);
        		break;
        	case ELLIPSE:
        		o = createRandomEllipse(""+i);
        		break;
        	case CIRCLE:
        		o = createRandomCircle(""+i);
        		break;
        	case POLYGONAL:
        		int j = r.nextInt(3);
        		switch(j) {
        		case 0:
        			o = createRandomPolyLine(""+i);
        			break;
        		case 1:
	        		o = createRandomPolygon(""+i);
	        		break;
        		case 2:
	        		o = createRandomRectangle(""+i);
	        		break;	        			
        		}
        		break;
        	case ELLIPSOID:
        		int k = r.nextInt(2);
        		switch(k) {
        		case 0:
        			o = createRandomEllipse(""+i);
        			break;
        		case 1:
	        		o = createRandomCircle(""+i);
	        		break;	        		
        		}
        		break;
        	case ALL:
        		o = getRandomElement(r.nextInt(CIRCLE+1));
        		break;
        	default:
        		o = null;
        	}
        	return o;
	    }

		public Area[] getSubAreas() {
			return new Area[0];
		}

		public SpatialElement[] getSpatialElements() {
			Vector v = new Vector();
			int size = size();
			for(int i = 0; i < size; i++) {
				if(getObject(i) instanceof SpatialElement) {
					v.add(getObject(i));
				}
			}
            SpatialElement[] ret = new SpatialElement[v.size()];
			for(int i = 0; i < v.size(); i++) {
				ret[i] = (SpatialElement)v.get(i);
			}
			return ret;
		}

		public void insert(SpatialComparable e) {
			int loc = getLast();
			if(getObject(loc) == null) {
				setObject(e,loc);
			}
			setObject(e, loc+1);
		}

		public void delete(SpatialComparable e) {
			for(int i = 0; i < size(); i++) {
				if(getObject(i) == e) {
					setObject(null, i);
					return;
				}
			}
		}

		public void addSubArea(Area a) {
			Note.out(this, "SpatialElementTable cannot have subareas");
		}

		public void removeSubArea(Area a) {
			Note.out(this, "SpatialElementTable cannot have subareas");			
		}

		public Rectangle getBoundingPolygon() {
			return bounds;
		}

        public PaintingStyleDecorator getPaintingStyleDecorator() {
            return null;
        }

        public boolean hasPaintingStyleDecorator() {
            return false;
        }

        public void setPaintingStyleDecorator(PaintingStyleDecorator psd) {
        }

		private static final long serialVersionUID = 6286532907207707266L;
}
