/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: FDTVisitAssistant.java,v 1.3 2006/06/20 12:22:45 oseppala Exp $ */
/* 24/05/2006 moved from package matrix.structures.FDT (vkaravir) */ 
package matrix.structures.util;

import matrix.structures.FDT.Array;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.Graph;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.Tree;
import matrix.structures.FDT.indexed.StaticLinkedList;

/**
 * Created on Apr 28, 2005
 *
 * The FDTVisitAssistant class together with the FDTVisitor class form an implementation
 * for a pseudo-visitor pattern to be used with FDT:s. Currently the classes support visiting
 * Arrays, Trees, LinkedLists and StaticLinkedSLists.
 * 
 * In contrast to the typical Visitor pattern, this class is used not by calling the accept method of
 * the data structure directly, but by giving this object and the visitor as parameters to the
 * FDTVisitAssistant.
 *
 * @author Otto Seppl
 */
public class FDTVisitAssistant {

    public static Object accept(Object o, FDTVisitor visitor){
    	if (o instanceof FDT){            
    		FDT fdt = (FDT)o;
    		if (fdt instanceof Array)
    			return visitor.visit((Array) fdt);
    		if (fdt instanceof Tree)
    			return visitor.visit((Tree) fdt);
    		if (fdt instanceof LinkedList)
    			return visitor.visit((LinkedList) fdt);
    		if (fdt instanceof Graph)
    			return visitor.visit((Graph) fdt);
    		if (fdt instanceof StaticLinkedList)
    			return visitor.visit((StaticLinkedList) fdt);
    		if (fdt instanceof Struct)
    			return visitor.visit((Struct) fdt);
    		return visitor.visit(fdt);
    		
    	}
    	else
    		return visitor.visitNonFDT(o);
    }
    
}
