/**
 * Linearizer class for creating String representations of FDTs
 * Implemented using the visitor pattern.
 * 
 * Typically used by calling the <b>linearize</b> method with
 * the FDT to be linearized.
 * 
 * @author Otto Seppl
 * @version 1.0
 * Created on Apr 28, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package matrix.structures.util;

import matrix.structures.FDT.Array;
import matrix.structures.FDT.LinkedList;
import matrix.structures.FDT.Graph;
import matrix.structures.FDT.Struct;
import matrix.structures.FDT.Tree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.indexed.StaticLinkedList;
import matrix.structures.FDT.substructures.Vertex;


import java.util.*;
import matrix.animation.Animator;

public class Linearizer implements FDTVisitor{
    
    private HashSet alreadyVisited;
    
    public static String[][] linearize(FDT fdt, Animator a){
        a.rewind();
        ArrayList list=new ArrayList();
        
        list.add(linearize(fdt));
        while(a.hasNextOperation()){
            a.redo();
            list.add(linearize(fdt));            
        }
        
        return (String[][])list.toArray(new String[0][0]);
    }
    
    public Linearizer(){
        alreadyVisited = new HashSet(); // to avoid endless recursion    
    } 
    
    public static String[] linearize(FDT fdt){
        Linearizer linearizer = new Linearizer();
        return (String[])((ArrayList)FDTVisitAssistant.accept(fdt, linearizer)).toArray(new String[0]);
    }
    
    public Object visitNonFDT(Object o){
        ArrayList list = new ArrayList();
        if (o != null)
            list.add(o.toString());
        else
            list.add("null");
        return list;
    }
    
    public Object visit(FDT fdt){
        
        if (alreadyVisited.contains(fdt))
            return new ArrayList();
        else
            alreadyVisited.add(fdt);
        
        ArrayList list = new ArrayList();
        list.add(fdt.getElement().toString());
        return list;
    }
    
    public Object visit(Array a) {
        
        if (alreadyVisited.contains(a))
            return new ArrayList();
        else
            alreadyVisited.add(a);
        
        
        ArrayList resultBuffer = new ArrayList();
        
        int aFirst = a.getFirst();
        int aLast  = a.getLast();
        
        Object aObj;
        
        for(int i = aFirst; i <= aLast; i++) {
            aObj = a.getObject(i);
            resultBuffer.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        }
        
        return resultBuffer;
    }
    
    public Object visit(Struct s) {
        
        if (alreadyVisited.contains(s))
            return new ArrayList();
        else
            alreadyVisited.add(s);
        
        
        ArrayList resultBuffer = new ArrayList();
        Object[] fields = s.getFields();
        
        for(int i = 0; i <fields.length; i++) {
            resultBuffer.addAll((ArrayList)FDTVisitAssistant.accept(fields[i], this));
        }
        
        return resultBuffer;
    }
    
    public Object visit(LinkedList a) {
        
        
        if (alreadyVisited.contains(a))
            return new ArrayList();
        else
            alreadyVisited.add(a);
        
        ArrayList returnValue = new ArrayList();
        
        Object aObj = a.getElement();
        if(aObj == null)
            returnValue.add(null);
        else
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        
        while(a.hasNext()){
            a    = a.getNext();
            aObj = a.getElement();
            
            if(aObj == null)
                returnValue.add(null);
            else
                returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        }
        
        return returnValue;
    }
    
    
    public Object visit(StaticLinkedList a) {
        
        
        if (alreadyVisited.contains(a))
            return new ArrayList();
        else
            alreadyVisited.add(a);
        
        a.setFirst();
        Object aObj;
        ArrayList returnValue = new ArrayList();
        
        while(a.isInList()) {
            aObj = a.currValue();
            if(aObj == null)
                returnValue.add(null);
            else
                returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
            a.next();
        }
        return returnValue;
    }
    
    /**
     * Compares two common trees. This is an identical comparison, so the children of
     * the nodes must be in the same order in both trees.
     * @param a the first tree to be compared
     * @param b the second tree to be compared
     * @return true if and only if the contents of the two trees are equal
     */
    public Object visit(Tree a) {
        
        if (alreadyVisited.contains(a))
            return new ArrayList();
        else
            alreadyVisited.add(a);
        
        ArrayList returnValue = new ArrayList();
        
        if (a==null){
            returnValue.add("null");
            return returnValue;
        }
        Object aObj = a.getElement();
        if(aObj == null)
            returnValue.add("null");
        else
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aObj, this));
        
        if (a instanceof matrix.structures.CDT.probe.BinSearchTree)
            return returnValue;
        
        Tree[] aTree = a.getSubTrees();
        for(int i = 0; i < aTree.length; i++){
            if (aTree[i] == null)continue;
            returnValue.addAll((ArrayList)FDTVisitAssistant.accept(aTree[i], this));
        }
        return returnValue;
    }
    
    public Object visit(Graph a) {
        
        if (alreadyVisited.contains(a))
            return new ArrayList();
        else
            alreadyVisited.add(a);
        
        ArrayList returnValue = new ArrayList();
        
        if(a == null)
            return " ";
        
        Vertex[] aVertices = a.getVertices();
        Object ao;
        
        for(int i = 0; i < aVertices.length; i++) {
            ao = aVertices[i].getElement();
            if(ao == null)
                returnValue.add(null);
            else
                returnValue.addAll((ArrayList)FDTVisitAssistant.accept(ao, this));
            
        }
        return returnValue;
    }
    
}
