/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: MenuFrame.java,v 1.14 2007/09/05 16:57:31 oseppala Exp $ */

package matrix.util;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

public class MenuFrame extends JFrame implements ActionListener {
    protected ActionListener listener = null;
    private boolean dblbuf = true;
    /*
    private MenuBar menuBar;
    private Menu myStructures;
    private Menu fontMenu, fontSizeMenu; // for setActiveFont[Size]()
     */

    public MenuFrame() {
        super();
        // set default font here
        setFont(new Font("sansserif", Font.PLAIN, 12));
        // activate window close button        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { close(); }
        });
    }

    protected JMenu createMenu(JMenuBar mb, String menu) {
        JMenu m = new JMenu(menu);
        mb.add(m);
        return m;
    }

    protected JMenu createMenu(JMenu mb, String menu) {
        JMenu m = new JMenu(menu);
        mb.add(m);
        return m;
    }

    protected JMenuItem addMenuItem(JMenu m,
            String itemName,
            int keyEvent,
            ActionListener callback) {
        JMenuItem mi;
        if (keyEvent >= 0) {
            m.add(mi = new JMenuItem(itemName));
            mi.setAccelerator(KeyStroke.getKeyStroke(keyEvent, 
                    Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        }
        else
            m.add(mi = new JMenuItem(itemName));
        mi.addActionListener(callback);
        return mi;
    }

    protected JMenuItem addMenuItem(JMenu m,
            String itemName,
            ActionListener callback) {
        return addMenuItem(m, itemName, -1, callback);
    }

    protected JCheckBoxMenuItem addCheckboxMenuItem(JMenu m,
            String itemName,
            boolean initialState,
            ItemListener callback) {
        JCheckBoxMenuItem mi;
        m.add(mi = new JCheckBoxMenuItem(itemName, initialState));
        mi.addItemListener(callback);
        return mi;
    }

    /*
    private void setActiveFont() {
        CheckboxMenuItem mi;

        for (int i = 0; i < fontMenu.getItemCount(); i++) {
            mi = (CheckboxMenuItem)fontMenu.getItem(i);
            mi.setState((mi.getLabel().equals(getFont().getName())));
        }
    }

    private void setActiveFontSize() {
        CheckboxMenuItem mi;

        for (int i=0; i<fontSizeMenu.getItemCount(); i++) {
            mi = (CheckboxMenuItem)fontSizeMenu.getItem(i);
            String label = Integer.toString(getFont().getSize());
            mi.setState(mi.getLabel().equals(label));
        }
    }
     */

    public void actionPerformed(ActionEvent event) {
        Note.out(this, "action " + event +" Performed for " + this);

        // String command = event.getActionCommand();
        // if (command.equals("print")) { print(); return; }
    }


    public void addActionListener(ActionListener a) {
        listener = AWTEventMulticaster.add(listener, a);
    }

    public void removeActionListener(ActionListener a) {
        listener = AWTEventMulticaster.remove(listener, a);
    }

    public void fireActionEvent(ActionEvent e) {
        if (listener != null) listener.actionPerformed(e);
    }

    public void close() {
        dispose();
    }

    public void quit() {
        System.exit(0);
    }

    /*
    void newWindow() {
        Animator.operation(new Command() {
                public void execute() {
                    FDT[] ss = {};
                    MyFrame cf = new MyFrame(ss);
                    cf.setSize(getMinimumSize());
                }});
    }

    static Dimension paperSize = null;
    void pageSetup() {
        // Toolkit needed to obtain printJob
        Toolkit t = getToolkit();
        // new properties object to be passed to printJob
        // (for possible reuse)
        Properties p = new Properties();
        PrintJob pj = null;
        try {
            // new printJob 
            pj = t.getPrintJob(this, getName(), p);
        } catch(Exception e) {
            // discard error
        } finally {
            paperSize = pj.getPageDimension();
        }                    
    }

    void print() {
        // Toolkit needed to obtain printJob
        Toolkit t = getToolkit();
        // new properties object to be passed to printJob (for possible reuse)
        Properties p = new Properties();
        // new printJob 
        PrintJob pj = t.getPrintJob(this, getName(), p);
        // obtain graphics object (new page) from printJob
        Graphics g = pj.getGraphics();
        // all drawings are printed rather than displayed on-screen
        print(g);
        // force the current page to be printed
        g.dispose();
        // end printing
        pj.end();
    }

    void printSequence() {
        Note.out(this, "printSequence");
        // Toolkit needed to obtain printJob
        Toolkit t = getToolkit();
        // new properties object to be passed to printJob (for possible reuse)
        Properties p = new Properties();
        // new printJob 
        PrintJob pj = t.getPrintJob(this, getName(), p);

        while (Animator.hasNextState()) {
            // obtain graphics object (new page) from printJob
            Graphics g = pj.getGraphics();
            // all drawings are printed rather than displayed on-screen
            print(g);
            // force the current page to be printed
            g.dispose();
            // next state
            Animator.redo();
            validate();
            repaint();
        } 
        Graphics g = pj.getGraphics();
        print(g);
        g.dispose();
        pj.end();
    }
     */

    public Dimension getMinimumSize() {
        return new Dimension(290,210);
    }

    /*
    void click(MouseEvent e) {
        System.out.println("Frame: got mouseClicked");
        if (e.isPopupTrigger()) System.out.println("PopUpTrigger");
    }
     */

    // Double Buffering

    Image offscreen;

    /**
     * null out the offscreen buffer as part of invalidation
     */
    public void invalidate() {
        offscreen = null;
        super.invalidate();
    }


    /**
     * override update to *not* erase the background before painting
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     * paint children into an offscreen buffer, then blast entire image
     * at once.
     */
    public void paintComponent(Graphics g) {
        if (g instanceof PrintGraphics) { // just printing...
            super.paint(g);
            return;
        }
        if (dblbuf) {
            if(offscreen == null) {
                offscreen = createImage(getSize().width, getSize().height);
            }

            Graphics og = offscreen.getGraphics();
            og.setClip(g.getClip());
            Rectangle r = g.getClip().getBounds();
            og.clearRect(r.x,r.y,r.width,r.height);
            super.paint(og);
            g.drawImage(offscreen, 0, 0, this);
            og.dispose();
        } else{ // no double buffering
            Rectangle r = g.getClip().getBounds();
            g.clearRect(r.x,r.y,r.width,r.height);
            super.paint(g);
        }
    }
}
