/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: ColorConverter.java,v 1.7 2004/05/25 13:28:27 kstalnac Exp $ */

package matrix.util.export;

import java.awt.Color;

/**
 * Utility class used in export to convert Java colors.
 */
public class ColorConverter {

    /**
     * Converts the given color to a float gray level. Gray level is a
     * value from 0 (black) through intermediate values (grays) to 
     * 1 (white).
     * @param color The Color that is converted.
     * @return Float value between 0 and 1.
     */
    public static float toGrayLevel(Color color) {
    
	float value = (color.getBlue() + color.getRed() + color.getGreen())/3;
	return value/255;
    }

    /**
     * Returns RGB color component values as int array. Index 0 is
     * red component, index 1 is green and index 2 is blue.
     * @param color The color that is converted.
     * @return Int array of the RGB components.
     */
    public static int[] toRGB(Color color) {
	if (color == null)   // if no color, use black
	    return new int[] { 0, 0, 0 };
	int[] colors = { color.getRed(), color.getGreen(), color.getBlue() };
	return colors;
    }
}
