/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: FileUtil.java,v 1.10 2004/10/07 10:36:42 kstalnac Exp $ */

package matrix.util.io;

import java.io.File;

import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.uitools.*;
import matrix.util.Note;
import matrix.visual.VisualType;

public class FileUtil {

    public static final int FDT_CLASS = 1;
    public static final int GRAPH_DESCRIPTION = 2;
    public static final int SERIALIZED_ANIMATION = 3;
    public static final int TABLE_DESCRIPTION = 4;
    public static final int STRUCTURES_DESCRIPTION = 5;

    public static int sniffType(String fileName) {
        // is it a class file?

        if (fileName.endsWith(".class")) {
            MatrixClassLoader ld = new MatrixClassLoader();

            try {
                Class clazz = ld.loadClassFromFile(fileName);
                if (MatrixClassLoader.implementsInterface(clazz, FDT.class)) {
                    return FDT_CLASS;
                }
            } catch (Exception e) {

                // not a class
            }
        }

        // try to load the file as a graph (graphs have special textual
        // notation)

        try {
            //FDT fdt = new GraphFromFile(fileName);
            //if ( ((GraphFromFile)fdt).isGraph() )
            if (GraphFromFile.isGraph(fileName) == true)
                return GRAPH_DESCRIPTION;

        } catch (Exception e) {

        }

        // try to load the file as a table
        try {
            //TableFromFile tff = new TableFromFile(fileName);
            //if (tff.isTable())
            if (TableFromFile.isTable(fileName) == true)
                return TABLE_DESCRIPTION;
        } catch (Exception e) {

        }

        try {
            //StructuresFromFile sff = new StructuresFromFile(fileName);
            //if(sff.hasStructures())
            if (StructuresFromFile.hasStructures(fileName) == true)
                return STRUCTURES_DESCRIPTION;
        } catch (Exception e) {

        }

        // is it a serialized java object
	try {
	    if (MatrixSerialization.loadFromFile(new File(fileName)) != null)
		return SERIALIZED_ANIMATION;
	} catch (Exception e){

	}

        return -1;

    }

    public static FDT instantiateFDTClass(String fileName) {
        MatrixClassLoader ld = new MatrixClassLoader();

        try {
            Class clazz = ld.loadClassFromFile(fileName);
			if (clazz.isInterface()) {
				Note.show(null, "Can't load " + fileName + ". It is an interface not a class.");
				return null;
			}
            return (FDT) clazz.newInstance();
        } catch (InstantiationException e) { //not an FDT structure
	    Note.show(null, "Can't load class " + fileName + ". It is not an FDT structure.");
	    Note.out(null, "Error in class loading: " + e.getMessage());
	    return null;
	} catch (Exception e) {
	    Note.out(null, "Error in class loading: " + e.getMessage());
	    e.printStackTrace();
	    return null;
	}
    }

    public static FDT loadGraphFromFile(String fileName) {
        try {
            FDT fdt = new GraphFromFile(fileName);
            if (((GraphFromFile) fdt).isGraph())
                return fdt;
        } catch (Exception e) {
            Note.err(
                null,
                "Can't load file: " + fileName + ": " + e.getMessage());
            return null;
        }
        return null;
    }

    public static MatrixSerialization openAnimation(String fileName) {
	try {
	    return MatrixSerialization.loadFromFile(new File(fileName));
	} catch (Exception e) {
	    Note.show(null, "The file " + fileName + " doesn't contain serializated animation.");
	    return null;
	}
    }

    public static FDT loadTableFromFile(String fileName) {
        TableFromFile tff = new TableFromFile(fileName);
        if (tff.isTable) {
            Table table = tff.getTable();
            return table;
        }
        return null;
    }

    public static FDT[] loadStructuresFromFile(String fileName) {
        StructuresFromFile sff = new StructuresFromFile(fileName);
        if (sff.hasStructures()) {
            FDT[] temp = sff.getStructures();
            return temp;
        }
        return null;
    }

    /**
     * Opens and loads structures from ascii-file. Also configures VisualTypes
     * if structures in file contain extra information about visualization.
     * @param fileName file to be opened
     * @param mf current MatrixFrame
     */
    public static void loadVisualStructuresFromFile(
        String fileName,
        MatrixFrame mf) {
        StructuresFromFile sff = new StructuresFromFile(fileName);
        StructurePanel sp = mf.getStructurePanel();
        FDT[] temp = sff.getStructures();
        VisualType[] temp2 = sp.addStructures(temp);
        sff.configureVisualTypes(temp2, sp);
    }

}
