/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2003-2004 See the AUTHORS file and the 
 * author tag below for more information on the copyright owners.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.visual;

import java.awt.Component;

import matrix.structures.FDT.Graph;
import matrix.structures.FDT.substructures.Vertex;

/**
 * @author vkaravir
 */
public class VisualGridGraph extends VisualGraph {
    public VisualGridGraph(Vertex v) {
        super(v);
    }

    public VisualGridGraph(Graph g) {
        super(g);
    }

    /* (non-Javadoc)
     * @see matrix.visual.VisualContainer#doActualLayout()
     */
    protected LayoutSize doActualLayout() {
        Component[] nodeList = getItems();
        int nodes = nodeList.length;
        int gridWidth = (int)Math.ceil(Math.sqrt(nodes));
        if (gridWidth%2 == 0)
            gridWidth++;
        int maxW = 0, maxH = 0;
        for (int i=0; i < nodes; i++) {
            LayoutSize nodeSize = ((VisualNode)nodeList[i]).lGetPreferredSize();
            maxW = Math.max(maxW, nodeSize.width);
            maxH = Math.max(maxH, nodeSize.height);
        }
        int maxX = 0, maxY = 0;
        for (int i = 0; i < nodes; i++) {
            LayoutSize nodeSize = ((VisualNode)nodeList[i]).lGetPreferredSize();
            int w = nodeSize.width;
            int h = nodeSize.height;
            int xC = (int)((i%gridWidth)*3*maxW);
            int yC = (int)((i/gridWidth)*2*maxH);
            yC += ((i%gridWidth)%2)*maxH;
            ((VisualNode)nodeList[i]).lSetBounds(xC, yC, w, h);
            maxX = Math.max(maxX, xC);
            maxY = Math.max(maxY, yC);
        }
        return new LayoutSize(maxX + maxW, maxY + maxH);
    }
        
}
