

/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualHead.java,v 1.19 2006/05/26 11:05:12 vkaravir Exp $ */

package matrix.visual;

import java.awt.*;

import matrix.structures.FDT.*;
import matrix.util.*;

public class VisualHead extends VisualTreeComponent {
    private VisualType root;
    private VisualContainer cont;

    public VisualHead(VisualType t) {
        super(null, null);
        root = t;   
        setVisible(false);
        setEnabled(false);
    }


    // needed for rotations and flipping (lSetBounds etc.)
    public void setVisualContainer(VisualContainer c) { cont = c; }
    public VisualContainer getVisualContainer() {
        return cont;
    }

    public VisualTree getTreeContainer() {
        return ((VisualTreeComponent)root).getTreeContainer();
    }

    /** This method is invoked by VisualHeadReference */
    public void setRoot(VisualType newRoot) {
	root = newRoot;
    }

    public void moveReference(VisualReference r, 
                              VisualType oldTarget, 
                              VisualType newTarget) {
        Container target = newTarget;
        
        //Note.out(this, "move from "+oldTarget+" into "+newTarget);
        
        while ((target instanceof VisualType) && 
               (!(target instanceof VisualTreeComponent)))
            target = target.getParent();
        
        if (target instanceof VisualTreeComponent) {
            VisualTreeComponent visualTarget = (VisualTreeComponent)target;
            Tree targetTree = (Tree)visualTarget.getStructure();
            if (root.getParent() instanceof VisualTree) {
                //Note.out(this, "root.getParent() was " + root.getParent());
                //Note.out(this, "oldTarget = " + oldTarget);
                
                // if new root is null we create a new empty tree
                if (targetTree == null) {
                    targetTree = ((Tree)oldTarget.getStructure()).getNewNode(null); 
                    //newTarget.setStructure(targetTree);
                }
		    
                //Note.out(this, "targetTree = " + targetTree);
                // delay execution until validateAll invoked
                Validator.add(new Command(root.getParent(), targetTree) {
                    public void execute() {
                        ((VisualTree)parameter).
                        setRoot((Tree)parameter2);
                    }
                });
                Validator.add(new Command(r) {
                    public void execute() {
                        ((VisualType)parameter).setVisited(false);
                    }
                });
                //root = newTarget;
                r.setVisited(true);
                
            } else
                Note.show(this, 
                          "Invalid parent ("+
                          root.getParent()+
                          ") of root " + 
                          root);
        } else 
            Note.show(this, 
                      "Cannot move reference: Illegal target " + newTarget);
    
    }
    
    public void paintComponent(Graphics g) {
    }
}
