/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.*;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.util.Note;
import matrix.util.export.ConfigurableGraphics;


/** VisualHotSpot is an abstraction for special active components within containers.
 * @see VisualContainer
 *
 * @author Ari Korhonen
 * @version $Revision: 1.19 $
 */
public class VisualHotSpot extends VisualType {
    protected static final Color opaqueWhite = new Color(255, 255, 255, 100);

    private static final long serialVersionUID = 3031739039455677949L;
    //private int x,y;
    private String label;


    // --- constructors ---

    public VisualHotSpot(FDT fdt) {
        super(fdt);
        label = "hotspot";
    }

    public String getLabel() { return label; }

    public void setLabel(String label) { this.label = label; }

    public void assign() {
        Note.show(this, this+": Action (assing) not defined for this HotSpot");
    }

    /** Returns the preferred size of this component.
     * By default the preferred size of this component
     * consists of the insets and the size of the label string
     */
    public Dimension getPreferredSize() {
        Font f = getFont();
        int w = 1;
        int h = 1;
        if (f != null) {
            w = getFontMetrics(f).stringWidth(label);
            h = getFontMetrics(f).getHeight();
        }
        return new Dimension(w, h);
    }

    /** This method draws the representation of this hot spot
     */
    public void paintComponent(Graphics g) {
        if (g instanceof ConfigurableGraphics) {
            if (((ConfigurableGraphics)g).isDisabled("StructureBorder")) {
                //if exporting and structure borders are not wanted then stop here
                return;
            }
        }
        Rectangle r = getBounds();

        setBorderColor(g);
        g.fillRect(0, 0, r.width, r.height);
        if (g instanceof ConfigurableGraphics) {
	    // If g is ConfigurableGraphics, we have to check whether the hotspots are 
	    // wanted to be painted or not.
	    //
            if (((ConfigurableGraphics)g).isDisabled("VisualHotSpot")) {
                super.paintComponent(g);
                return;            
            }
        }

        setPenColor(g);
        int nWidth = getMetrics().stringWidth(label);
        int nHeight = getMetrics().getHeight();
        int descent = getMetrics().getDescent();
        g.drawString(label,
                     r.width/2 - nWidth/2,
                     nHeight - descent);
        super.paintComponent(g);
    }
    
    public void setBorderColor(Graphics g) {
        if (fdt instanceof CDT)
            g.setColor(getCDTColor());
        else
            super.setBorderColor(g);
    }
}
