/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualLayeredGraphDummyComponent.java,v 1.12 2006/05/24 13:58:53 vkaravir Exp $ */

package matrix.visual;

import java.awt.Dimension;

import matrix.structures.FDT.substructures.Vertex;

/**
 * Dummy component for the VisualLayeredGraph layout.
 *
 * @author Riku Saikkonen
 * @version $Revision: 1.12 $
 * @see VisualLayeredGraph
 * @see VisualLayeredGraphComponent
 */
public class VisualLayeredGraphDummyComponent extends VisualLayeredGraphComponent {
    private VisualContainer cont;

    /** Constructor.
     * @param c which container this dummy component belongs to
     * @param ref where to attach this dummy component
     */
    public VisualLayeredGraphDummyComponent(VisualContainer c,
                                            VisualReference ref) {
        super((Vertex) null);
        cont = c;
        ref.addMidPoint(this);
    }

    /*
     * The size of a dummy node is 1*1 pixels. Overriding
     * getPreferredSize is enough because lGetPreferredSize uses it.
     */
    public Dimension getPreferredSize() {
        return new Dimension(1, 1);
    }

    /*
     * Needed for rotations and flipping (lSetBounds etc.). Needs to
     * be overridden here, because this component doesn't actually
     * belong to an AWT container.
     */
    public VisualContainer getVisualContainer() {
        return cont;
    }

    // paint() is never called, because dummy nodes aren't part of an
    // AWT Container
}
