/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2005 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualListHeadReference.java,v 1.3 2005/12/20 08:56:21 kstalnac Exp $ */

package matrix.visual;

import java.awt.Graphics;
import java.awt.Rectangle;

import matrix.structures.FDT.FDT;
import matrix.util.Note;

/**
 * Reference which points to the first element of the list. 
 * 
 * @author kstalnac
 */
public class VisualListHeadReference extends VisualReference {

    /**
     * Constructor. 
     * Constructs a new Reference which points from VisualListHead
     * to the first actual element of the list. 
     * 
     * @param t first element (VisualComponent) of the list
     */
    public VisualListHeadReference(VisualComponent t) {
        super(new VisualListHead(t), t);
    }

    /**
     * Returns always null. There are no underlying FDT 
     * structure for a reference.
     */
    public FDT getStructure() { return null; }

    public void setTarget(VisualComponent vlc) {
	super.setTarget(vlc);
	((VisualListHead)from).setVisualRoot((VisualListComponent)vlc);
    }

    /**
     * Sets the source coordinate of this reference.
     */
    public void doLayout() {
    	Rectangle r = to.getBounds();
    	if (from != null) {
	    from.setBounds(1, r.y, 2, r.height);
	} else 
            Note.err(this, "Warning: no VisualListHead found during doLayout.");
    	super.doLayout();
    }

    /**
     * Paint method.
     * This calls just super.paint.
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
    }
    
}
