/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

//import matrix.visual.stylesheet.*;
import java.awt.Dimension;

import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Key;

/**
 * This is the container for single visual text line representation.
 *
 * @author Ari Korhonen
 * @version $Revision: 1.11 $
 */
public class VisualTextLine extends VisualNode {
    String text;
    String description;

    public VisualTextLine(FDT fdt) {
        super(fdt);
    }
    
    public VisualTextLine(String text) {
        this(text, "no description");
    }

    public VisualTextLine(String text, String description) {
        this(new Key(text));
        this.text = text != null ? text : "";
        this.description = description != null ? description : "";
        setStructure(new myKey(text, description));
    }

    class myKey extends Key implements matrix.decoration.Styled {
        String description = "";
        public myKey(String s, String description) { super(s); this.description = description; }
        public matrix.decoration.StyleSheet getStyleSheet() {
            return new matrix.decoration.StyleSheetAdapter() {
                    public boolean isInfoEnabled() { return true; }
                    public String getInfo() { return description; } 
                    public int getWaitTime() { return 10; }
                };
        }
    }    

    public void setLine(String text, String description) { 
        if ((!this.text.equals(text)) || (!this.description.equals(description))) {
            this.text = text; 
            setStructure(new myKey(text, description));
        }
    }
    
    
    /** This class overrides this method return false. 
     * @see VisualNode#isRoundRectangle()
     */
    public boolean isRoundRectangle() { return false; }
    
    static int commonWidth = 0;
    public Dimension getPreferredSize() {
        Dimension d = super.getPreferredSize();
        if (d.width > commonWidth)
            commonWidth = d.width;
        else
            d.width = commonWidth;
        return d;
    }
}


