/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualTrash.java,v 1.14 2006/06/29 12:29:13 oseppala Exp $ */

package matrix.visual;

import java.awt.*;

import matrix.structures.FDT.*;
import matrix.structures.util.Trash;

public class VisualTrash extends VisualType {
    Trash trash;
    String label = "Trash";

    public VisualTrash() { 
        super(null); 
        this.trash = new Trash();
    }

    public VisualTrash(Trash t) {
        super(t);
        this.trash = t;
    }

    /** Returns the preferred size of this component. 
     * By default the preferred size of this component 
     * consists of the size of the label string
     */
    public Dimension getPreferredSize() {
        Font f = getFont();
        int w = 1;
        int h = 1;
        if (f != null) {
            w = getFontMetrics(f).stringWidth(label);
            h = getFontMetrics(f).getHeight();
        }
        return new Dimension(w, h);
    }

    public FDT getStructure() { return trash; }

    public void assign() {
        //Note.show(this, "Assign into Trash interpreted as delete.");
        read.delete();
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Rectangle r = getBounds();
        setBorderColor(g);
        g.fillRect(1, 1, r.width-2, r.height-2);
        setPenColor(g);
        int nWidth = getMetrics().stringWidth(label);
        int nHeight = getMetrics().getHeight();
        int descent = getMetrics().getDescent();
        g.drawString(label, r.width/2 -  nWidth/2, nHeight - descent);
    }
    
}
