/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package prototype.ui;

import java.awt.*;
import javax.swing.*;

import content.interfaces.*;

import content.assessment.*;

import matrix.animation.Animator;
import matrix.uitools.*;
import matrix.util.Note;

public class ExerciseFrame extends MenuFrame {

    private SimulationExercise exercise;
    private ModelAnswerFrame maf = null;

    JButton gradeButton;

    protected JTextPane descriptionTextArea ;

    public ExerciseFrame(Animator a, SimulationExercise exercise) {
        super(a, new ExercisePanel(a, exercise));
        setTitle("Exercise");
        this.exercise = exercise;
        descriptionTextArea.setText(exercise.getDescription());
        getVisualAnimator().setResetButtonVisible(false);
        getVisualAnimator().setPlayButtonVisible(false);
        setSize(695, 715);
    }

    public ExerciseFrame(SimulationExercise exercise) {
        this(new Animator(), exercise);
    }


    protected void layoutComponents() {

        descriptionTextArea = new JTextPane();
        descriptionTextArea.setEditable(false);
        descriptionTextArea.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        // See java.awt.GridBagLayout for details.
        GridBagLayout grid = new  GridBagLayout();
        getContentPane().setLayout(grid); 
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(10,10,10,10);
        gbc.gridy = 0;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.BOTH;
        getContentPane().add(descriptionTextArea, gbc);

        gbc.gridy = 1;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.NONE;
        getContentPane().add(getVisualAnimator(), gbc);

        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        JScrollPane sp = getScrollPane();
        //sp.add(getStructurePanel());
        /*Add method shouldn't be called when adding component to JScrollPane.
         * setViewportView -method should be used instead.
         */
        sp.setViewportView(getStructurePanel());
        getContentPane().add(sp, gbc);

    }



    protected JMenuBar createMenuBar() {
        return MenuFactory.getInstance().getMenuBar(this);
    }




    protected ExercisePanel getExercisePanel() {
        return (ExercisePanel)getStructurePanel();
    }



    public void reset() {
        //bugi, bugi exercise was already defined, Ville 8.4.03
        //SimulationExerciseModel exercise = (SimulationExerciseModel)getExercisePanel().getExercise();
        if (maf != null) {
            maf.close();
            maf = null;
        }
        try {
            exercise = (SimulationExercise)exercise.getClass().newInstance();
        } catch (Exception e) {
            Note.show(this, "ERROR: " + e.getMessage());
            Note.err(this, e.getMessage());
            Note.printStackTrace();
        }
        long seed = System.currentTimeMillis();
        ((SimulationExercise)exercise).setSeed(seed);
        getExercisePanel().setExercise(exercise);
        getExercisePanel().reset();

        descriptionTextArea.setText(exercise.getDescription());
        validate();
        getExercisePanel().repaint();
    }


    public void modelAnswer() {
        if(exercise instanceof SimulationExerciseModel) {
            maf = new ModelAnswerFrame((SimulationExerciseModel)exercise);
            maf.setVisible(true);
        } else {
            Note.show(this, "Cannot show model answer, none given.");
        }
    }


    public void compare() {
        if (exercise instanceof ComparableExercise) {
            ComparisonFrame cf = new ComparisonFrame(getAnimator(), (ComparableExercise)exercise);
            cf.setVisible(true);
        } else {
            Note.show(this, "This exercise is not comparable.");
        }
    }

    public void grade() {
        if(exercise instanceof SimulationExerciseModel) {
            Judgement j = Jury.getInstance().judge(((SimulationExercise)exercise).getAnswer(), getAnimator(), (SimulationExerciseModel)exercise);
            DeprecatedJudgement dj = DeprecatedJury.judge(((SimulationExercise)exercise).getAnswer(), getAnimator(), (SimulationExerciseModel)exercise);

            if (dj.equals(j) == false)
                Note.show(this, "Refactored Judgement does not match old Judgement");

            new GradeDialog(j).setVisible(true);
        } else if(exercise instanceof MultipleChoiceExercise) {
            // TODO: Check if the class MultipleCHoiceExercise is really used anywhere and remove the class and dangling references
            // if not.
            
            //MultipleChoiceExercise mce = (MultipleChoiceExercise) exercise;
        } else {
            Note.show(this, "Cannot judge this exercise. No method of judging given.");
        }
        getVisualAnimator().validate();
        getStructurePanel().validate();
    }

    private static final long serialVersionUID = 2025513569765939342L;
}