/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package prototype.ui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import content.assessment.Judgement;

public class GradeDialog extends JDialog {

    public static final int ROWS = 10;
    public static final int COLUMNS = 50;
    
    Judgement judgement;

    protected JTextArea text;
    protected JPanel buttonPanel;
    protected StringBuffer buf;

    public GradeDialog() {
        super(new JFrame(), "Grade");
        getContentPane().setLayout(new GridBagLayout());

	text = makeTextArea();

        GridBagConstraints gbc = new GridBagConstraints();
	gbc.insets = new Insets(10,10,10,10);
    gbc.fill=GridBagConstraints.BOTH;
	gbc.anchor=GridBagConstraints.NORTH;
	gbc.weighty=1.0;
    gbc.weightx=1.0;

        getContentPane().add(new JScrollPane(text), gbc);
    
        gbc.gridx = 0;
        gbc.gridy = 1;
	gbc.weighty=0.0;

        getContentPane().add(buttonPanel=makeButtonPanel(), gbc);

        setModal(true);

        // avctivate window close button
        this.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) { close(); }
            });

        pack();
    }



  protected JTextArea makeTextArea() {

	JTextArea t = new JTextArea("", ROWS, COLUMNS);
	t.enableInputMethods(false);
        t.setEditable(false);
        t.setBorder(BorderFactory.createLineBorder(Color.BLACK));
	return t;
    }

    protected JPanel makeButtonPanel() {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());

        JButton contButton = new JButton("Close");
        contButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    close();
                }
            });
        buttonPanel.add(contButton);
        return buttonPanel;
    }



    public GradeDialog(Judgement j) {
        this();
        judgement = j;
	int multipleQuestions = (j.next == null) ? 0 : 1;
	buf = new StringBuffer();

	do {
	    int[] points = j.getPoints();
	    int max = j.getMax();
	    String exerciseNumber = (multipleQuestions == 0) ? "" : " ("+multipleQuestions+")";
	    String feedback = j.getFeedbackString();
        if(feedback != null) {
            buf.append(feedback);
        } else if (max != 1) {
	        buf.append(sum(points));
	        buf.append(" steps out of "+ max + " correct.");
	        buf.append("\n");
	    } else if (sum(points) == 1)
	        buf.append("Exercise"+exerciseNumber+" correct!\n");
	    else 
	        buf.append("Exercise"+exerciseNumber+" incorrect.\n");
	    
	    /*
	      if (j.isComplete())
	      buf.append("You have reached the final state.\n");
	    */
	    j = j.next;
	    multipleQuestions++;
	} while (j != null);
	    
	//text.setText(buf.toString());
	text();
	pack();
    }


    protected void text() {
	text.setText(buf.toString());
    }
    
    protected void close() {
        dispose();
    }
    
    
    protected Judgement getJudgement() {
        return judgement;
    }
    

    static int sum(int[] a) {
	int val = 0;
	for (int i = 0; i < a.length; i++) {
	    val = val + a[i];
	}
	return val;
    }

}
