/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package prototype.ui;

import javax.swing.*;
import java.awt.event.*;

//import applications.trakla2.datalogging.*;

import content.interfaces.SimulationExercise;

import matrix.uitools.*;
import matrix.uitools.MenuFrame;
import matrix.util.*;
import matrix.util.configuration.MatrixConfiguration;

public class StartFrame extends MenuFrame {
	
	public StartFrame() {
		super();
		setTitle("Animation window");
	}
	
	public StartFrame(StructurePanel p) {
		super(p);
	}
	
	public JFrame getNewFrame(StructurePanel p) {
		return new StartFrame(p);
	}
	
	public JFrame getNewFrame() {
		return new StartFrame();
	}
	
	protected JMenuBar createMenuBar() {
		JMenuBar mb = super.createMenuBar();
		//mb.add(new ContentMenu(this));
		
		// this adds a dependency from prototype to Trakla2 application...
		//addTrakla2FileMenu(mb);
		
		return mb;
	}
	
	public static void main(String[] args) {
		if(args.length > 0 && !(args[0].equals(""))) {
			//matrix.util.Note.out(null, "Setting conf path");
			//matrix.visual.RepresentationFactory.setConfPath(args[0]);
			MatrixConfiguration.setConfPath(args[0]);
		}
		
		// find the configuration file as a resource
		// it doesn't matter what class is used below
		MatrixConfiguration.setInputStream(
				new Object().getClass().getResourceAsStream("/matrixconf.xml"));
		
		if(args.length > 1) {
			String arg = args[1].toLowerCase();
			if(arg.equals("t") || arg.equals("true"))
				MatrixConfiguration.setValidatingParser(true);
		}
		
		//if(args.length > 1 && args[1].equals("true"))
		//m.parse(true);
		//else
		//m.parse(false);
		//m.process();
		
		MatrixConfiguration.getInstance(); // Loads the configuration
		StartFrame mf = new StartFrame();
		ApplicationAdapter.setApplication(mf.getStructurePanel());
		
		//  the following lines would add a dependency from prototype to Trakla2 application...
		/*    
		 Menu traklaMenu = new Menu("Trakla");
		 traklaMenu.add(applications.trakla2.util.SubmissionViewer.addTrakla2FileMenu(mf));
		 mf.getMenuBar().add(traklaMenu);
		 */
	}
	
	private JMenu getMenu(JMenuBar mb, String name) {
		
		for (int i = 0; i < mb.getMenuCount(); i++) {
			JMenu m = mb.getMenu(i);
			if (m.getText().equals(name)) {
				return m;
			}
		}
		return null;
	}
		
	public void addMenuItem(JMenu m, String itemName, ActionListener callback) {
		JMenuItem mi;
		
		m.add(mi = new JMenuItem(itemName));
		mi.addActionListener(callback);
	}
	
	public void newExercise(SimulationExercise e) {
		e.setSeed(System.currentTimeMillis());
		ExerciseFrame ef = new ExerciseFrame(e);
		ef.setVisible(true);
	}
	
	
}


