/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2005 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: PriorityQueueTest.java,v 1.4 2006/09/08 23:20:46 oseppala Exp $ */

package tests.matrix.structures.ADT.probe;

import junit.framework.TestCase;
import matrix.structures.ADT.probe.PriorityQueue;
import matrix.structures.FDT.probe.Key;

/**
 * This class tests the methods of the matrix.structures.ADT.probe.PriorityQueue class
 * 
 * @author archie
 */
public class PriorityQueueTest extends TestCase {

	private PriorityQueue priorityQueue;
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(PriorityQueueTest.class);
	}

	/**
	 * Sets up the objects for the test method.
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		priorityQueue = new PriorityQueue();
	}

	/**
	 * Removes objects after every test method.
	 * 
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		priorityQueue = null;
	}

	/**
	 * Constructor for TestPriorityQueue.
	 * @param name
	 */
	public PriorityQueueTest() {
		super();
	}


	/**
	 * Tests isEmpty method.
	 *
	 */
	public final void testADTIsEmpty() {
		assertEquals("PriorityQueue should be empty in the beginning!",true, priorityQueue.ADTIsEmpty());
	}

	/**
	 * Tests insert method.
	 *
	 */
	public final void testADTinsert() {
		priorityQueue.ADTinsert(new Key(5));
		assertEquals("PriorityQueue should not be empty",false,priorityQueue.ADTIsEmpty());
		priorityQueue.ADTinsert(new Key(17));
		assertEquals("PriorityQueue should not be empty",false,priorityQueue.ADTIsEmpty());
		priorityQueue.ADTinsert(new Key(5));
		assertEquals("PriorityQueue should not be empty",false,priorityQueue.ADTIsEmpty());
		priorityQueue.ADTinsert(new Key(17));
		assertEquals("PriorityQueue should not be empty",false,priorityQueue.ADTIsEmpty());
	}
	
	/**
	 * Tests pop method.
	 *
	 */
	public final void testADTdeleteMin() {
		assertEquals("PriorityQueue should be empty in the beginning!",true, priorityQueue.ADTIsEmpty());
		Key k5;
		priorityQueue.ADTinsert(k5 = new Key(5));
		assertEquals("PriorityQueue should not be empty after insertion",false, priorityQueue.ADTIsEmpty());

		Object p5 = priorityQueue.ADTdeleteMin();
		assertEquals("Deleted object should be Key", "class matrix.structures.FDT.probe.Key", p5.getClass().toString());
		assertEquals("PriorityQueue should be empty", true, priorityQueue.ADTIsEmpty());
		assertEquals("Deleted Key is not the same that was inserted!", k5, p5);
		
		priorityQueue.ADTinsert(new Key(1));
		priorityQueue.ADTinsert(new Key(2));
		priorityQueue.ADTinsert(new Key(3));
		priorityQueue.ADTdeleteMin();
		priorityQueue.ADTdeleteMin();
		priorityQueue.ADTdeleteMin();
		assertEquals("PriorityQueue should be empty", true, priorityQueue.ADTIsEmpty());
		priorityQueue.ADTinsert(new Key(1));
		priorityQueue.ADTinsert(new Key(1));
		priorityQueue.ADTdeleteMin();
		assertEquals("PriorityQueue should not be empty after two insertions and one deleteMin",false, priorityQueue.ADTIsEmpty());
		priorityQueue.ADTinsert(new Key(2));
		priorityQueue.ADTdeleteMin();
		priorityQueue.ADTdeleteMin();
		assertEquals("PriorityQueue should be empty", true, priorityQueue.ADTIsEmpty());
	}

}
