/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2005 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: StackTest.java,v 1.4 2005/12/20 08:57:14 kstalnac Exp $ */

package tests.matrix.structures.ADT.probe;

import junit.framework.TestCase;
import matrix.structures.ADT.probe.Stack;

/**
 * This class tests the methods of the matrix.structures.ADT.probe.Stack class
 * 
 * @author kstalnac
 */
public class StackTest extends TestCase {

	private Stack stack1;
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(StackTest.class);
	}

	/**
	 * Sets up the objects for the test method.
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		stack1 = new Stack();
	}

	/**
	 * Removes objects after every test method.
	 * 
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		stack1 = null;
	}

	/**
	 * Constructor for TestStack.
	 * @param name
	 */
	public StackTest(String name) {
		super(name);
	}


	/**
	 * Tests isEmpty method.
	 *
	 */
	public final void testIsEmpty() {
		assertEquals("Stack should be empty in the beginning!",true, stack1.ADTIsEmpty());
	}

	/**
	 * Tests push method.
	 *
	 */
	public final void testPush() {
		stack1.ADTPush(new Integer(5));
		assertEquals("Stack should not be empty",false,stack1.ADTIsEmpty());
		stack1.ADTPush(new Integer(17));
	}
	
	/**
	 * Tests pop method.
	 *
	 */
	public final void testPop() {
		stack1.ADTPop();

		stack1.ADTPush(new Integer(5));
		assertEquals("Stack should not be empty",false, stack1.ADTIsEmpty());
		Object o5 = stack1.ADTPop();
		assertEquals("Popped object should be Integer", "class java.lang.Integer", o5.getClass().toString());
		assertEquals("Stack should be empty", true, stack1.ADTIsEmpty());
		int i5 = ((Integer)o5).intValue();
		assertEquals("Popped integer is not 5!", i5, 5);
		
		stack1.ADTPush(new Integer(1));
		stack1.ADTPush(new Integer(2));
		stack1.ADTPush(new Integer(3));
		stack1.ADTPop();
		stack1.ADTPop();
		stack1.ADTPop();
		assertEquals("Stack should be empty", true, stack1.ADTIsEmpty());
		
	}

	/**
	 * Tests top method.
	 *
	 */
	public final void testTop() {
		stack1.ADTPeek();
		
		stack1.ADTPush(new Integer(5));
		stack1.ADTPush(new Integer(6));
		stack1.ADTPush(new Integer(7));
		stack1.ADTPush(new Integer(8));
		stack1.ADTPush(new Integer(9));
		stack1.ADTPush(new Integer(10));

		assertEquals("Object on the top of the stack should be 10.",
					10, ((Integer)stack1.ADTPeek()).intValue());
		
		stack1.ADTPop();
		stack1.ADTPop();
		stack1.ADTPop();
		
		assertEquals("7 should be on the top of the stack.", 
					7, ((Integer)stack1.ADTPeek()).intValue());

		stack1.ADTPeek();
		stack1.ADTPop();
		stack1.ADTPeek();
		stack1.ADTPop();
		stack1.ADTIsEmpty();
		stack1.ADTPop();
		assertEquals("Stack should be empty",true,stack1.ADTIsEmpty());
		assertEquals("top() for empty stack should return null", null, stack1.ADTPeek());

	}

}
