/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2005 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VirtualIntegerTest.java,v 1.2 2007/08/20 13:26:11 oseppala Exp $ */

package tests.matrix.structures.memory;

import junit.framework.TestCase;
import matrix.structures.memory.VirtualInteger;

/**
 * This class is a Junit TestCase to test methods of 
 * matrix.structures.memory.VirtualInteger.
 * 
 * @author kstalnac
 */
public class VirtualIntegerTest extends TestCase {

    VirtualInteger vi;
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(VirtualIntegerTest.class);
    }

    /**
     * Sets up the objects for the test methods.
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        vi = new VirtualInteger(5, null, "VirtualInteger test - object being tested");    
    }

    /**
     * Removes the objects after every test method is finished.
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        vi = null;
    }

    /**
     * Constructor for VirtualIntegerTest.
     * @param name name of the test
     */
    public VirtualIntegerTest(String name) {
        super(name);
    }

    /**
     * Tests assign method by assigning a new value to the virtualInteger object and 
     * then testing that the value is correct by eval method.
     *
     */
    public final void testAssign() {
        vi.assign(101);
        assertEquals("VirtualInteger should be 101.", 101, vi.eval());       
    }

    /**
     * Tests equals method that compares the equality of two VirtualIntegers.
     *
     */
    public final void testEqualsVirtualInteger() {
        VirtualInteger tempInteger = new VirtualInteger(5, null, "VirtualInteger test - temp 1 in testEqualsVirtualInteger");
        assertTrue("Both VirtualInteger should be evaluated as 5", vi.equals(tempInteger));
        tempInteger = new VirtualInteger(-1011, null, "VirtualInteger test - temp 2 in testEqualsVirtualInteger");
        assertFalse("VirtualIntegers should not be equal.", vi.equals(tempInteger));
    }

    
    /**
     * Tests "greater than" method.
     *
     */
    public final void testGt() {
        VirtualInteger tempInteger = new VirtualInteger(10, null, "VirtualInteger test - temp 1 in testGt");
        assertFalse("This VirtualInteger should not be greater than the VirtualInteger " +
                "given as a parameter of the method", vi.gt(tempInteger));
        tempInteger = new VirtualInteger(-12, null, "VirtualInteger test - temp 2 in testGt");
        assertTrue("This virtualInteger 5 should be greater than virtualInteger -12", vi.gt(tempInteger));
    }

    /**
     * Tests "less than" method.
     *
     */
    public final void testLt() {
        VirtualInteger tempInteger = new VirtualInteger(1000, null, "VirtualInteger test - temp 1 in testLt");
        assertTrue("This VirtualInteger (5) should be less than the VirtualInger 1000", vi.lt(tempInteger));
        tempInteger = new VirtualInteger(-1234, null, "VirtualInteger test - temp 2 in testLt");
        assertFalse("This VirtualInteger (5) should not be less than the VirtualInger -1234", vi.lt(tempInteger));
    }

    /**
     * Tests increment method.
     *
     */
    public final void testInc() {
        for (int i = 0; i < 8; i++) {
            vi.inc();
        }
        assertEquals("VirtualInteger should be 13", 5 + 8, vi.eval());
    }

    /**
     * Tests decrement method.
     *
     */
    public final void testDec() {
        for (int i = 0; i < 101; i++) {
            vi.dec();
        }
        assertEquals("VirtualInteger should be -96", 5 - 101, vi.eval());
    }

    /**
     * Tests eval method by creating new VirtualInteger objects with different values and
     * after that testing that eval method returns the correct value.
     *
     */
    public final void testEval() {
        assertEquals("VirtualInteger should be 5.", 5, vi.eval());
        VirtualInteger tempInteger = new VirtualInteger(null, "VirtualInteger test - temp in testEval");
        assertEquals("VirtualInteger should be 0 by default.", 0, tempInteger.eval());
        tempInteger = new VirtualInteger(Integer.MAX_VALUE, null, "VirtualInteger test - temp in testEval");
        assertEquals("VirtualInteger should be 2^31 - 1", 2147483647, tempInteger.eval());
        tempInteger = new VirtualInteger(Integer.MIN_VALUE, null, "VirtualInteger test - temp in testEval");
        assertEquals("VirtualInteger should be -2^31", -2147483648, tempInteger.eval());
    }

}
