// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar;

import java.util.Enumeration;

import matrix.visual.VisualType;

import matrixpro.ui.MainFrame;

/**
 * Interface for Toolbar used with the application. Every toolbar must provide
 * access to the selected <code>VisualType</code> and the
 * <code>MatrixFrame</code> this Toolbar is in plus a way to add components to
 * this toolbar.
 * 
 * @version $Id: Toolbar.java,v 1.6 2007/09/10 08:32:31 vkaravir Exp $
 */
public interface Toolbar {
    /**
     * Adds a <code>ToolbarComponent</code> to this toolbar.
     * 
     * @param c
     *            The ToolbarComponent to be added.
     */
    public void addComponent(ToolbarComponent c);

    /**
     * Adds a <code>ToolbarComponent</code> to this toolbar with the given
     * name.
     * 
     * @param c
     *            The ToolbarComponent to be added.
     * @param name
     *            Name of the component.
     */
    public void addComponent(ToolbarComponent c, String name);

    /**
     * Adds a <code>ToolbarComponent</code> to this toolbar with the given
     * name and position.
     * 
     * @param c
     *            The ToolbarComponent to be added.
     * @param name
     *            Name of the component.
     * @param position
     *            Position in the toolbar that this component should be at.
     */
    public void addComponent(ToolbarComponent c, String name, int position);

    /**
     * Returns the <code>MainFrame</code> that this Toolbar is in.
     * 
     * @return The MainFrame that this Toolbar is in.
     */
    public MainFrame getFrame();

    /**
     * Return the <code>VisualType</code> that is currently selected in the
     * MatrixFrame that this Toolbar is in.
     * 
     * @return The <code>VisualType</code> that is currently selected.
     */
    public VisualType getSelectedVisualType();

    /**
     * Returns an <code>Enumeration</code> of the
     * <code>ToolbarComponents</code>.
     * 
     * @return An Enumeration of the ToolbarComponents.
     */
    public Enumeration getToolbarComponents();

    /** Resets the selected VisualType. */
    public void resetSelectedVisualType();

    /**
     * Sets the selected <code>VisualType</code> to the given value.
     * 
     * @param newVt
     *            The new selected VisualType.
     */
    public void setSelectedVisualType(VisualType newVt);

    /** Updates the application. */
    public void updateApplication();

    /** Validates the VisualAnimator. */
    public void validateVisualAnimator();

    /**
     * Returns true, if a VisualType is currently selected, false otherwise.
     * 
     * @return Boolean representing whether or not a VisualType is selected.
     */
    public boolean visualTypeSelected();
}
