// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;

import matrix.util.Note;
import matrix.visual.VisualKKGraph;
import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that is used to set the optimal edge-length for
 * the Kamada-Kawai and Fruchterman-Reingold graph layouts.
 * 
 * @version $Id: ChangeEdgeLengthComponent.java,v 1.6 2007/09/03 13:25:53
 *          vkaravir Exp $
 */
public class ChangeEdgeLengthComponent extends ToolbarComponent implements
        ActionListener {
    private Toolbar tb;

    private JPanel panel;

    private JButton change;

    private JTextField length;

    /**
     * Constructs a new instance of this class.
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public ChangeEdgeLengthComponent(Toolbar tb) {
        panel = new JPanel();
        this.tb = tb;
        panel.setLayout(new GridBagLayout());
        length = new JTextField("", 8);
        length.setEditable(true);
        length.setEnabled(false);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = 2;
        panel.add(length, gbc);
        change = new JButton("Set edge length");
        change.setEnabled(false);
        gbc.gridwidth = 1;
        gbc.gridx = 2;
        panel.add(change, gbc);
        change.addActionListener(this);
        length.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        changeEdgeLength();
    }

    private void changeEdgeLength() {
        VisualType vt = tb.getSelectedVisualType();
        if (vt instanceof VisualKKGraph) {
            double d = 0;
            try {
                d = Double.parseDouble(length.getText());
            } catch (NumberFormatException ex) {
                Note.show("Sorry", "Invalid edge length");
                return;
            }
            ((VisualKKGraph) vt).setEdgeLength(d);
        }
    }

    public void update() {
        VisualType vt = tb.getSelectedVisualType();
        if (vt instanceof VisualKKGraph) {
            length.setText(((VisualKKGraph) vt).getEdgeLength() + "");
            change.setEnabled(true);
            length.setEnabled(true);
        } else {
            length.setText("");
            change.setEnabled(false);
            length.setEnabled(false);
        }
    }

    public String getDescription() {
        return "Changes the length of the edge";
    }

    public JComponent getComponent() {
        return panel;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new ChangeEdgeLengthComponent(tb);
    }
}
