// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.GridLayout;

import javax.swing.JComponent;
import javax.swing.JPanel;

import matrix.structures.simulationextensions.CallableMethods;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that shows buttons for the operations implemented
 * in different structures. The operations are the ones indicated in the
 * structure by implementing the CallableMethods interface.
 * 
 * @version $Id: ContextualPanel.java,v 1.8 2007/09/10 08:39:11 vkaravir Exp $
 * @see matrix.structures.simulationextensions.CallableMethods
 */
public class ContextualPanel extends ToolbarComponent {
    /** The Toolbar that this component is added to. */
    private Toolbar tb;

    /** The Panel that contains the buttons for the operations. */
    private JPanel panel;

    /** Constructor. Creates a new instance of this class. */
    public ContextualPanel(Toolbar tb) {
        this.tb = tb;
        panel = new JPanel();
    }

    public void update() {
        panel.removeAll();
        if (tb.getSelectedVisualType().getStructure() instanceof CallableMethods)
            handleCallableMethods();
    }

    /**
     * Adds a new button for each CallableMethod that the selected structure
     * has. The components that are added are MethodComponents.
     * 
     * @see matrtixpro.toolbar.components.MethodComponent
     */
    private void handleCallableMethods() {
        String[] met = ((CallableMethods) tb.getSelectedVisualType()
                .getStructure()).getCallableMethodNames();
        panel.setLayout(new GridLayout(met.length, 1));
        for (int i = 0; i < met.length; i++)
            panel.add(new MethodComponent(tb, met[i]).getComponent());
        tb.getFrame().validate();
    }

    public JComponent getComponent() {
        return panel;
    }

    public String getDescription() {
        return "Component for context-dependent buttons";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new ContextualPanel(tb);
    }
}
