// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import matrix.util.Note;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;
import matrixpro.ui.MainFrame;

/**
 * Component for <code>Toolbar</code> to turn on and off Matrix frameworks
 * debug output. When the state of this <code>Checkbox</code> is changed a
 * <code>PropertyChangeEvent</code> named "debug" is fired for all
 * <code>MatrixFrames</code>.
 * 
 * @version $Id: DebugComponent.java,v 1.7 2007/09/10 08:39:10 vkaravir Exp $
 */
public class DebugComponent extends ToolbarComponent implements
        PropertyChangeListener {
    /** Toolbar that this component is added to. */
    private Toolbar tb;

    private JCheckBox checkbox;

    private static ItemListener itemListener = null;

    /**
     * Constructor. Creates a new instance of this class with the default text
     * Debug.
     */
    public DebugComponent(Toolbar tb) {
        this(tb, "Debug");
    }

    /**
     * Constructor. Creates a new instance of this class with the given text.
     */
    public DebugComponent(Toolbar tb, String text) {
        checkbox = new JCheckBox(text, Note.getOut());
        this.tb = tb;
        this.tb.getFrame().addPropertyChangeListener("debug", this);
        if (itemListener == null)
            itemListener = new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    // change the value of debug output and fire the proper
                    // event for all MatrixFrames
                    boolean oldValue = Note.getOut();
                    Note.setOut(!oldValue);
                    MainFrame.firePropertyChangeForAllFrames("debug",
                            new Boolean(oldValue), new Boolean(!oldValue));
                }
            };
        checkbox.addItemListener(itemListener);
    }

    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equals("debug")) {
            checkbox.setSelected(Note.getOut());
        }
    }

    public void update() {
    }

    public JComponent getComponent() {
        return checkbox;
    }

    public String getDescription() {
        return "Toggles the debug output";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new DebugComponent(tb);
    }
}
