// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that dumps debug information about either the
 * selected visualization or, if none selected, about all the visualizations in
 * the frame.
 * 
 * @version $Id: DumpComponent.java,v 1.8 2007/09/10 08:39:11 vkaravir Exp $
 */
public class DumpComponent extends ToolbarComponent {
    private Toolbar tb;

    private JButton button;

    /**
     * Constructs a new DumpComponent with the default text "Dump".
     * 
     * @param tb
     *            The Toolbar that this component will be added to.
     */
    public DumpComponent(Toolbar tb) {
        this(tb, "Dump");
    }

    /**
     * Constructs a new DumpComponent with the given text.
     * 
     * @param tb
     *            The Toolbar that this component will be added to.
     * @param name
     *            The text that will be on the button of this component
     */
    public DumpComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dump();
            }
        });
    }

    /** Shows a popup window containing the debug information. */
    private void dump() {
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        PrintStream out = new PrintStream(buffer);
        if (tb.visualTypeSelected())
            tb.getSelectedVisualType().dump(out);
        else
            tb.getFrame().getStructurePanel().dump(out);
        tb.getFrame().getStructurePanel()
                .showMessage("Dump", buffer.toString());
    }

    public void update() {
    }

    public JComponent getComponent() {
        return button;
    }

    public String getDescription() {
        return "Dumps either the selected visual component or all components";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new DumpComponent(tb);
    }
}
