// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;
import matrixpro.util.FileHelper;

/**
 * 
 * @version $Id: EditComponent.java,v 1.11 2007/09/10 08:39:10 vkaravir Exp $
 */
public class EditComponent extends ToolbarComponent implements ActionListener,
        PropertyChangeListener {
    /** The toolbar that this component is added. */
    private Toolbar tb;

    /** The panel that contains the buttons. */
    private JPanel panel;

    /** The buttons. */
    private JButton copy, cut, paste, delete, undo, redo;

    /**
     * Constructor.
     * 
     * @param tb
     *            The Toolbar that this component will be added to.
     */
    public EditComponent(Toolbar tb) {
        panel = new JPanel();
        this.tb = tb;
        panel.setLayout(new GridLayout(1, 3));
        addComponents();
        addListeners();
    }

    /** Add the components to the panel. */
    private void addComponents() {
        copy = FileHelper.getJButton("/pics/Copy24.gif", "Copy");
        copy.setToolTipText("Copy");
        copy.setMargin(new Insets(1, 1, 1, 1));
        panel.add(copy);
        cut = FileHelper.getJButton("/pics/Cut24.gif", "Cut");
        cut.setToolTipText("Cut");
        cut.setMargin(new Insets(1, 1, 1, 1));
        panel.add(cut);
        paste = FileHelper.getJButton("/pics/Paste24.gif", "Paste");
        paste.setToolTipText("Paste");
        paste.setMargin(new Insets(1, 1, 1, 1));
        panel.add(paste);

        delete = FileHelper.getJButton("/pics/Delete24.gif", "Delete");
        delete.setToolTipText("Delete");
        delete.setMargin(new Insets(1, 1, 1, 1));
        panel.add(delete);

        undo = FileHelper.getJButton("/pics/Undo24.gif", "Undo");
        undo.setToolTipText("Undo");
        undo.setMargin(new Insets(1, 1, 1, 1));
        panel.add(undo);
        redo = FileHelper.getJButton("/pics/Redo24.gif", "Redo");
        redo.setToolTipText("Redo");
        redo.setMargin(new Insets(1, 1, 1, 1));
        panel.add(redo);
    }

    /** Add listeners to buttons. */
    private void addListeners() {
        copy.setActionCommand("copy");
        copy.addActionListener(this);
        cut.setActionCommand("cut");
        cut.addActionListener(this);
        paste.setActionCommand("paste");
        paste.addActionListener(this);
        delete.setActionCommand("delete");
        delete.addActionListener(this);
        undo.setActionCommand("undo");
        undo.addActionListener(this);
        redo.setActionCommand("redo");
        redo.addActionListener(this);
        tb.getFrame().addPropertyChangeListener(this);
    }

    /**
     * For the interface ActionListener. Called when an action is performed on
     * the buttons.
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if (command.equals("copy"))
            tb.getFrame().copy();
        else if (command.equals("cut"))
            tb.getFrame().cut();
        else if (command.equals("paste"))
            tb.getFrame().paste();
        else if (command.equals("delete"))
            tb.getFrame().delete();
        else if (command.equals("redo"))
            tb.getFrame().redo();
        else if (command.equals("undo"))
            tb.getFrame().undo();
    }

    public void update() {
        boolean selected = tb.visualTypeSelected();
        cut.setEnabled(selected);
        copy.setEnabled(selected);
        undo.setEnabled(tb.getFrame().isUndoEnabled());
        redo.setEnabled(tb.getFrame().isRedoEnabled());
        paste.setEnabled(tb.getFrame().isPasteEnabled());
        delete.setEnabled(selected);
    }

    public JComponent getComponent() {
        return panel;
    }

    public String getDescription() {
        return "Copy, Cut, Paste, Undo and Redo buttons";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new EditComponent(tb);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String property = evt.getPropertyName();
        if (property.equals("clipboard")) {
            update();
        }
    }
}
