// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;
import matrixpro.util.FileHelper;
import matrixpro.util.export.MPExporter;

/**
 * Component that can be used to quickly access the most useful commands in the
 * file-menu.
 * 
 * @version $Id: FileComponent.java,v 1.11 2007/09/10 08:39:09 vkaravir Exp $
 */
public class FileComponent extends ToolbarComponent implements ActionListener {
    /** The toolbar that this component is added. */
    private Toolbar tb;

    /** The panel that contains the buttons. */
    private JPanel panel;

    /** The buttons. */
    private JButton neww, open, save, export, page, print;

    /**
     * Constructor.
     * 
     * @param tb
     *            The Toolbar that this component will be added to.
     */
    public FileComponent(Toolbar tb) {
        panel = new JPanel();
        this.tb = tb;
        panel.setLayout(new GridLayout(1, 3));
        addComponents();
        addListeners();
    }

    /** Add the components to the panel. */
    private void addComponents() {
        neww = FileHelper.getJButton("/pics/New24.gif", "New");
        neww.setToolTipText("New window");
        neww.setMargin(new Insets(1, 1, 1, 1));
        panel.add(neww);
        open = FileHelper.getJButton("/pics/Open24.gif", "Open");
        open.setToolTipText("Open");
        open.setMargin(new Insets(1, 1, 1, 1));
        panel.add(open);
        save = FileHelper.getJButton("/pics/Save24.gif", "Save");
        save.setToolTipText("Save");
        save.setMargin(new Insets(1, 1, 1, 1));
        panel.add(save);
        export = FileHelper.getJButton("/pics/Export24.gif", "Export");
        export.setToolTipText("Export");
        export.setMargin(new Insets(1, 1, 1, 1));
        panel.add(export);
        page = FileHelper.getJButton("/pics/PageSetup24.gif", "Page Setup");
        page.setToolTipText("Page Setup");
        page.setMargin(new Insets(1, 1, 1, 1));
        panel.add(page);
        print = FileHelper.getJButton("/pics/Print24.gif", "Print");
        print.setToolTipText("Print");
        print.setMargin(new Insets(1, 1, 1, 1));
        panel.add(print);
    }

    /** Add the listeners to the buttons. */
    private void addListeners() {
        neww.setActionCommand("new");
        neww.addActionListener(this);
        open.setActionCommand("open");
        open.addActionListener(this);
        save.setActionCommand("save");
        save.addActionListener(this);
        page.setActionCommand("page");
        page.addActionListener(this);
        print.setActionCommand("print");
        print.addActionListener(this);
        export.setActionCommand("export");
        export.addActionListener(this);
    }

    /**
     * For the interface ActionListener. Called when an action is performed on
     * the buttons.
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if (command.equals("new"))
            tb.getFrame().getNewFrame();
        else if (command.equals("open"))
            FileHelper.open(tb.getFrame());
        else if (command.equals("save"))
            FileHelper.save(tb.getFrame());
        else if (command.equals("page"))
            tb.getFrame().pageSetup();
        else if (command.equals("print"))
            tb.getFrame().print(false);
        else if (command.equals("export"))
            MPExporter.getInstance().export(tb.getFrame());
    }

    public void update() {
    }

    public JComponent getComponent() {
        return panel;
    }

    public String getDescription() {
        return "New, Open, Save, Export, Page setup, and Print buttons";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new FileComponent(tb);
    }
}
