// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.visual.VisualGraphComponent;
import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that is used when inserting edges to a graph.
 * 
 * @version $Id: InsertEdgeComponent.java,v 1.8 2007/09/10 08:39:11 vkaravir Exp $
 */
public class InsertEdgeComponent extends ToolbarComponent {
    private Toolbar tb;

    private JButton button;

    private static final String DEFAULT_NAME = "Insert edge";

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "Insert edge".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public InsertEdgeComponent(Toolbar tb) {
        this(tb, DEFAULT_NAME);
    }

    /**
     * Constructs a new instance of this class. The text on the button will be
     * the given text.
     * 
     * @param tb
     *            The Toolbar this component is added.
     * @param text
     *            The text to appear on the button.
     */
    public InsertEdgeComponent(Toolbar tb, String text) {
        button = new JButton(text);
        this.tb = tb;
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                insertEdge();
            }
        });
    }

    public void update() {
        VisualType vt = tb.getSelectedVisualType();
        button.setEnabled(vt instanceof VisualGraphComponent);
    }

    /**
     * Calls the method insertEdge of VisualGraphComponent.
     */
    private void insertEdge() {
        VisualType vt = tb.getSelectedVisualType();
        if (vt instanceof VisualGraphComponent)
            ((VisualGraphComponent) vt).insertEdge();
    }

    public String getDescription() {
        return "Insert edge -button";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new InsertEdgeComponent(tb);
    }
}
