// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.util.ApplicationAdapter;
import matrix.visual.VisualKey;
import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that adds a new representation of the selected
 * visualization to the current frame.
 * 
 * @version $Id: NewRepresentationComponent.java,v 1.9 2007/09/03 13:25:55
 *          vkaravir Exp $
 */
public class NewRepresentationComponent extends ToolbarComponent implements
        ActionListener {
    private JButton button;

    private Toolbar tb;

    /**
     * Constructs a new instance of this class. The text on the button will be
     * "New representation".
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public NewRepresentationComponent(Toolbar tb) {
        this(tb, "New visualization");
    }

    /**
     * Constructs a new instance of this class. The text on the button will be
     * the given text.
     * 
     * @param tb
     *            The Toolbar this component is added.
     * @param name
     *            The text to appear on the button.
     */
    public NewRepresentationComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(this);
    }

    public void update() {
        boolean sel = tb.visualTypeSelected();
        VisualType vt = tb.getSelectedVisualType();
        if (sel && !(vt instanceof VisualKey))
            button.setEnabled(true);
        else
            button.setEnabled(false);
    }

    public void actionPerformed(ActionEvent e) {
        VisualType vt = tb.getSelectedVisualType();
        ApplicationAdapter.getApplication().addStructure(vt.getStructure());
        vt.setInvalid();
    }

    public JComponent getComponent() {
        return button;
    }

    public String getDescription() {
        return "Add new visualization of the selected visualization";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new NewRepresentationComponent(tb);
    }
}
