// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.structures.FDT.FDT;
import matrix.uitools.StructurePanel;
import matrix.util.Application;
import matrix.visual.VisualKey;
import matrix.visual.VisualType;

import matrixpro.animation.CountingAnimator;
import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component for the <code>Toolbar</code> that opens the selected
 * <code>VisualType</code> in a new window. Note, that the new visualization
 * is a representation of the same underlying structure and changes to one of
 * the visualizations affects both of them.
 * 
 * @version $Id: NewWindowComponent.java,v 1.10 2007/09/10 08:39:09 vkaravir Exp $
 */
public class NewWindowComponent extends ToolbarComponent implements
        ActionListener {
    /** Toolbar that this component is added to. */
    private Toolbar tb;

    /** The button that will be added to the Toolbar. */
    private JButton button;

    /**
     * Creates a new NewWindowComponent with the default text "Open in new
     * window".
     */
    public NewWindowComponent(Toolbar tb) {
        this(tb, "Open in new window");
    }

    /**
     * Creates a new NewWindowComponent with the given String as text of the
     * Button.
     */
    public NewWindowComponent(Toolbar tb, String name) {
        button = new JButton(name);
        this.tb = tb;
        button.addActionListener(this);
    }

    public void update() {
        boolean sel = tb.visualTypeSelected();
        VisualType vt = tb.getSelectedVisualType();
        if (sel && !(vt instanceof VisualKey))
            button.setEnabled(true);
        else
            button.setEnabled(false);
    }

    public void actionPerformed(ActionEvent e) {
        VisualType vt = tb.getSelectedVisualType();
        FDT[] ss = { vt.getStructure() };

        Application oldApp = vt.getApplication();
        // Application newApp = oldApp.getNewInstance(ss,new Animator());
        StructurePanel sp = new StructurePanel(ss, new CountingAnimator());
        oldApp.engage((Application) sp);
        /*
         * if (newApp instanceof Frame) { ((Frame)newApp).show(); } else {
         */
        ((Application) sp).engage(oldApp);
        tb.getFrame().getNewFrame(sp);
        tb.resetSelectedVisualType();
    }

    public String getDescription() {
        return "Open the selected visualization in new window";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new NewWindowComponent(tb);
    }
}
