// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;

import matrix.visual.VisualType;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component for <code>Toolbar</code> to rename the selected
 * <code>VisualType</code>.
 * 
 * @version $Id: RenameComponent.java,v 1.8 2007/09/10 08:39:09 vkaravir Exp $
 */
public class RenameComponent extends ToolbarComponent implements ActionListener {
    private JPanel panel;

    private Toolbar tb;

    private JButton renameButton;

    private JTextField nameField;

    /**
     * Constructs a new instance of this class.
     * 
     * @param tb
     *            The Toolbar this component is added.
     */
    public RenameComponent(Toolbar tb) {
        panel = new JPanel();
        this.tb = tb;
        panel.setLayout(new GridBagLayout());
        nameField = new JTextField("", 8);
        nameField.setEditable(true);
        nameField.setEnabled(false);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = 2;
        panel.add(nameField, gbc);
        renameButton = new JButton("Rename");
        renameButton.setEnabled(false);
        gbc.gridwidth = 1;
        gbc.gridx = 2;
        panel.add(renameButton, gbc);
        renameButton.addActionListener(this);
        nameField.addActionListener(this);
    }

    /** Handles ActionEvents from the textfield and button. */
    public void actionPerformed(ActionEvent e) {
        VisualType vt = tb.getSelectedVisualType();
        if (vt == null)
            return;
        vt.rename(nameField.getText());
        tb.updateApplication();
    }

    public void update() {
        String str = tb.getSelectedVisualType().getName();
        nameField.setText(str);
        nameField.setEnabled(tb.visualTypeSelected());
        if (nameField.isEnabled()) {
            nameField.requestFocus();
        }
        renameButton.setEnabled(tb.visualTypeSelected());
    }

    public JComponent getComponent() {
        return panel;
    }

    public String getDescription() {
        return "Renames the selected visualization";
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new RenameComponent(tb);
    }
}
