// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import matrix.util.ApplicationAdapter;

import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * Component for <code>Toolbar</code> to set the end of the animation to the
 * current state. All steps after the current step in animation are lost after
 * pressing this button.
 * 
 * @version $Id: SetEndComponent.java,v 1.8 2007/09/10 08:39:11 vkaravir Exp $
 */
public class SetEndComponent extends ToolbarComponent implements ActionListener {
    /** Toolbar that this component is added to. */
    private Toolbar tb;

    private JButton button;

    /** Default label used for this button. */
    private static final String DEFAULT_TEXT = "Set end";

    /**
     * Constructor. Creates a new instance of this class with the default text
     * "Set end".
     */
    public SetEndComponent(Toolbar tb) {
        this(tb, DEFAULT_TEXT);
    }

    /**
     * Constructor. Creates a new instance of this class with the given text.
     */
    public SetEndComponent(Toolbar tb, String text) {
        button = new JButton(text);
        this.tb = tb;
        button.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
        setEnd();
    }

    /**
     * Sets the end of the animation to the current state. All steps after the
     * current step in animation are lost after pressing this button.
     */
    private void setEnd() {
        tb.getFrame().setUndoPoint("set end");
        ApplicationAdapter.getApplication().getAnimator().disposeRedo();
        tb.updateApplication();
    }

    public String getDescription() {
        return "Set end of animation to the current step";
    }

    public JComponent getComponent() {
        return button;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new SetEndComponent(tb);
    }
}
