// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.toolbar.components;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import matrix.animation.Animator;
import matrix.animation.VisualAnimator;
import matrix.util.ApplicationAdapter;
import matrix.util.Note;

import matrixpro.animation.CountingAnimator;
import matrixpro.toolbar.Toolbar;
import matrixpro.toolbar.ToolbarComponent;

/**
 * A component for the Toolbar that shows the current step that the Animator is
 * in. The component can also be used to quickly jump to a step in the
 * animation.
 * 
 * @version $Id: StepLabelComponent.java,v 1.9 2007/09/10 08:39:10 vkaravir Exp $
 */
public class StepLabelComponent extends ToolbarComponent implements
        PropertyChangeListener {
    /** Toolbar that this component is added to. */
    private Toolbar tb;

    /** Label used in the panel. */
    private JLabel label;

    /** TextField that shows the current step. */
    private JTextField stepText;

    private JButton go;

    private Animator animator;

    /** The panel that all the components are added. */
    private JPanel panel;

    /** Constructs a new instance of this class. */
    public StepLabelComponent(Toolbar tb) {
        panel = new JPanel();
        this.tb = tb;
        animator = ApplicationAdapter.getApplication().getAnimator();
        label = new JLabel("Step");
        panel.setLayout(new FlowLayout());// GridLayout(1, 3));
        stepText = new JTextField("1 / 1");
        go = new JButton("Go");
        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                go();
            }
        };
        stepText.addActionListener(listener);
        go.addActionListener(listener);
        panel.add(label);
        panel.add(stepText);
        panel.add(go);
        VisualAnimator va = this.tb.getFrame().getVisualAnimator();
        if (va != null)
            va.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    update();
                }
            });
        tb.getFrame().addPropertyChangeListener(this);
    }

    /** Jumps to the Animator step given in the TextField. */
    private void go() {
        try {
            CountingAnimator a = (CountingAnimator) animator;
            String newVal = stepText.getText().trim();
            String sub = "";
            for (int i = 1; i <= newVal.length(); i++) {
                try {
                    new Integer(newVal.substring(0, i));
                    sub = newVal.substring(0, i);
                } catch (Exception e) {
                    break;
                }
            }
            if (!sub.equals("")) {
                int newStep = Integer.parseInt(sub);
                if (newStep < 1 || newStep > a.getMaxStep()) {
                    invalidStep(newVal);
                    return;
                }
                a.rewind();
                for (int i = 1; i < newStep; i++)
                    a.redo();
            } else {
                invalidStep(newVal);
                return;
            }
            tb.updateApplication();
            panel.validate();
        } catch (ClassCastException e) {
            stepText.setEnabled(false);
            go.setEnabled(false);
        }
    }

    /**
     * Method called when trying to jump to a unexisting step. Action depends on
     * the application.
     */
    private void invalidStep(String newStep) {
        Note.show("Invalid step", "Step " + newStep + " not found");
    }

    public void update() {
        try {
            CountingAnimator a = (CountingAnimator) animator;
            stepText.setText(a.getStep() + " / " + a.getMaxStep());
            stepText.invalidate();
        } catch (ClassCastException e) {
            stepText.setText("? / ?");
            stepText.setEnabled(false);
            go.setEnabled(false);
        }
    }

    public String getDescription() {
        return "Animator step component";
    }

    public JComponent getComponent() {
        return panel;
    }

    public static ToolbarComponent getNewInstance(Toolbar tb) {
        return new StepLabelComponent(tb);
    }

    public void propertyChange(PropertyChangeEvent e) {
        String property = e.getPropertyName();
        if (property.equals("step"))
            update();
    }
}
