// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import matrix.util.Note;

import matrixpro.util.ClassFileFilter;

/** Dialog used to add new components to the toolbar in class ToolbarTab. */
public class AddComponentDialog extends JDialog {
    /**
     * 
     */
    private static final long serialVersionUID = -4548685175838966319L;

    private JTextField className, componentName;

    private JButton ok, cancel, browse;

    public AddComponentDialog(JFrame f, boolean modal) {
        super(f, "Add component", modal);
        getContentPane().setLayout(new GridBagLayout());
        setSize(450, 100);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        getContentPane().add(new JLabel("Class name"), gbc);
        gbc.gridx = 1;
        getContentPane().add(className = new JTextField(20), gbc);
        className.setText("");
        gbc.gridx = 2;
        getContentPane().add(browse = new JButton("Browse"));
        gbc.gridx = 0;
        gbc.gridy = 1;
        getContentPane().add(new JLabel("Component name"), gbc);
        gbc.gridx = 1;
        getContentPane().add(componentName = new JTextField(20), gbc);
        gbc.gridx = 1;
        gbc.gridy = 2;
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout());
        buttonPanel.add(ok = new JButton("OK"));
        ok.setEnabled(false);
        buttonPanel.add(cancel = new JButton("Cancel"));
        getContentPane().add(buttonPanel, gbc);
        setResizable(false);
        addListeners();
    }

    private boolean cancelled = false;

    private void addListeners() {
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelled = true;
                dispose();
            }
        });
        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        browse.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                browse();
            }
        });
        KeyListener keyListener = new KeyListener() {
            public void keyPressed(KeyEvent e) {
                setOkButtonEnabled();
            }

            public void keyReleased(KeyEvent e) {
                setOkButtonEnabled();
            }

            public void keyTyped(KeyEvent e) {
                setOkButtonEnabled();
            }
        };
        className.addKeyListener(keyListener);
        componentName.addKeyListener(keyListener);
    }

    /**
     * Sets the ok button enabled if both of the text-fields are not empty.
     */
    private void setOkButtonEnabled() {
        boolean enable = !className.getText().equals("")
                && !componentName.getText().equals("");
        ok.setEnabled(enable);
    }

    /**
     * Opens a FileChooser to choose the class of the new component. Sets the
     * text in the Class name textbox if the class is found from the classpath.
     */
    private void browse() {
        JFileChooser fileChooser = new JFileChooser(System.getProperties()
                .getProperty("user.dir"));
        fileChooser.removeChoosableFileFilter(fileChooser.getFileFilter());
        ClassFileFilter ff = new ClassFileFilter();
        fileChooser.addChoosableFileFilter(ff);
        fileChooser.setFileFilter(ff);
        fileChooser.setDialogTitle("Select class");
        int state = fileChooser.showDialog(this, "OK");
        if (state == JFileChooser.CANCEL_OPTION)
            return;
        String fileName = fileChooser.getSelectedFile().getAbsolutePath();
        className.setText(fileName);
        fileName = fileName.replace('/', '.');
        fileName = fileName.replace('\\', '.');
        int startPoint = fileName.lastIndexOf(".",
                fileName.lastIndexOf(".") - 1);
        while (fileName.lastIndexOf(".", startPoint) != -1) {
            String cName = fileName.substring(startPoint + 1, fileName
                    .lastIndexOf("."));
            try {
                Class.forName(cName);
                className.setText(cName);
                return;
            } catch (ClassNotFoundException e) {
                startPoint = fileName.lastIndexOf(".", startPoint - 1);
            }
        }
        Note.show("Error", "Class not found from classpath!");
    }

    /** Returns the name of the components added with this dialog. */
    public String getComponentName() {
        return componentName.getText();
    }

    /**
     * Returns the Class name of the components added with this dialog.
     */
    public String getClassName() {
        return className.getText();
    }

    /** Shows this dialog. */
    public boolean showDialog() {
        setVisible(true);
        return !cancelled;
    }
}
