// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import matrix.visual.VisualType;

/**
 * This is a panel that allows the change of the layout settings in MatrixPro.
 * 
 * @author <a href="mailto:vkaravir@cs.hut.fi">Ville Karavirta</a>
 */
public class LayoutPanel extends JPanel {
    /**
     * 
     */
    private static final long serialVersionUID = -8042524877084744984L;

    private MainFrame mf;

    private VisualType type;

    private JTextField gridx;

    private JTextField gridy;

    private JTextField gridheight;

    private JTextField gridwidth;

    private JTextField ipadx;

    private JTextField ipady;

    private JTextField weightx;

    private JTextField weighty;

    private JTextField bottom;

    private JTextField top;

    private JTextField left;

    private JTextField right;

    // private JComboBox anchor;
    // private JComboBox fill;

    public static final String[] anchor_options = { "CENTER", "NORTH",
            "NORTHEAST", "EAST", "SOUTHEAST", "SOUTH", "SOUTHWEST", "WEST",
            "NORTHWEST" };

    public static final String[] fill_options = { "NONE", "HORIZONTAL",
            "VERTICAL", "BOTH" };

    /**
     * @param type
     *            The VisualType this Panel is used for
     * @param string
     *            Name of the visualization
     */
    public LayoutPanel(VisualType type, MainFrame mf) {
        super();
        setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createEmptyBorder(5, 0, 5, 0), type.getName()));
        this.type = type;
        this.mf = mf;
        addComponents();
    }

    private void addComponents() {
        setLayout(new GridLayout(6, 4));
        GridBagLayout layout = (GridBagLayout) mf.getStructurePanel()
                .getLayout();
        GridBagConstraints gbc = layout.getConstraints(type);
        gridx = new JTextField("" + gbc.gridx, 5);
        gridy = new JTextField("" + gbc.gridy, 5);
        add(new JLabel("gridx"));
        add(gridx);
        add(new JLabel("gridy"));
        add(gridy);
        gridheight = new JTextField("" + gbc.gridheight, 5);
        add(new JLabel("gridheight"));
        add(gridheight);
        gridwidth = new JTextField("" + gbc.gridwidth, 5);
        add(new JLabel("gridwidth"));
        add(gridwidth);
        ipadx = new JTextField("" + gbc.ipadx, 5);
        add(new JLabel("ipadx"));
        add(ipadx);
        ipady = new JTextField("" + gbc.ipady, 5);
        add(new JLabel("ipady"));
        add(ipady);
        weightx = new JTextField("" + gbc.weightx, 5);
        add(new JLabel("weightx"));
        add(weightx);
        weighty = new JTextField("" + gbc.weighty, 5);
        add(new JLabel("weighty"));
        add(weighty);
        // add(new JLabel("anchor"));
        // anchor = new JComboBox(anchor_options);
        // add(anchor);
        // add(new JLabel("fill"));
        // fill = new JComboBox(fill_options);
        // add(fill);
        add(new JLabel("Top inset"));
        add(top = new JTextField("" + gbc.insets.top, 5));
        add(new JLabel("Bottom inset"));
        add(bottom = new JTextField("" + gbc.insets.bottom, 5));
        add(new JLabel("Left inset"));
        add(left = new JTextField("" + gbc.insets.left, 5));
        add(new JLabel("Right inset"));
        add(right = new JTextField("" + gbc.insets.right, 5));
        // setFill(gbc.fill, gbc);
        // setAnchor(gbc.anchor, gbc);
    }

    /*
     * private void setFill(int visFill, GridBagConstraints gbc) { for (int i =
     * 0; i < fill_options.length; i++) { try { int value =
     * gbc.getClass().getField(fill_options[i]).getInt(gbc); if (value ==
     * visFill) { fill.setSelectedIndex(i); return; } } catch (Exception e) {
     * e.printStackTrace(); } } }
     * 
     * private void setAnchor(int visAnchor, GridBagConstraints gbc) { for (int
     * i = 0; i < anchor_options.length; i++) { try { int value =
     * gbc.getClass().getField(anchor_options[i]).getInt(gbc); if (value ==
     * visAnchor) { anchor.setSelectedIndex(i); return; } } catch (Exception e) {
     * e.printStackTrace(); } } }
     * 
     * private void updateFill(GridBagConstraints gbc) { try { int value =
     * gbc.getClass().getField(fill.getSelectedItem().toString()).getInt(gbc);
     * gbc.fill = value; } catch (Exception e) { e.printStackTrace(); } }
     * 
     * private void updateAnchor(GridBagConstraints gbc) { try { int value =
     * gbc.getClass().getField(anchor.getSelectedItem().toString()).getInt(gbc);
     * gbc.anchor = value; } catch (Exception e) { e.printStackTrace(); } }
     */

    public void setValues() {
        GridBagLayout layout = (GridBagLayout) mf.getStructurePanel()
                .getLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Insets i = new Insets(0, 0, 0, 0);
        i.top = Integer.parseInt(top.getText());
        i.bottom = Integer.parseInt(bottom.getText());
        i.left = Integer.parseInt(left.getText());
        i.right = Integer.parseInt(right.getText());
        gbc.insets = i;
        gbc.gridx = Integer.parseInt(gridx.getText());
        gbc.gridy = Integer.parseInt(gridy.getText());
        gbc.ipadx = Integer.parseInt(ipadx.getText());
        gbc.ipady = Integer.parseInt(ipady.getText());
        gbc.weightx = Double.parseDouble(weightx.getText());
        gbc.weighty = Double.parseDouble(weighty.getText());
        gbc.gridwidth = Integer.parseInt(gridwidth.getText());
        gbc.gridheight = Integer.parseInt(gridheight.getText());
        // updateFill(gbc);
        // updateAnchor(gbc);
        layout.setConstraints(type, gbc);
    }
}
