// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.BufferedReader;
import java.io.StringReader;

import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;

import prototype.ui.ExerciseFrame;
import content.interfaces.SimulationExercise;

import matrix.animation.Animator;

import matrixpro.util.ExerciseDescriptionParser;
import matrixpro.util.FileHelper;

/**
 * ExerciseFrame for MatrixPro that shows the exercise descriptions for the
 * exercises.
 * 
 * @author vkaravir
 * 
 */
public class MPExerciseFrame extends ExerciseFrame {
    private static final long serialVersionUID = -6492492607385462275L;

    private JTabbedPane description;

    private JEditorPane task;

    private String taskText;

    /**
     * @param a
     *            The animator.
     * @param exercise
     *            The exercise.
     */
    public MPExerciseFrame(Animator a, SimulationExercise exercise) {
        super(a, exercise);
        setSize(900, 700);
        ImageIcon icon = FileHelper.getImageIcon("pics/matrixpro-icon.png");
        if (icon != null)
            setIconImage(icon.getImage());
    }

    /**
     * @param exercise
     *            The exercise.
     */
    public MPExerciseFrame(SimulationExercise exercise) {
        super(exercise);
        setSize(900, 700);
        ImageIcon icon = FileHelper.getImageIcon("pics/matrixpro-icon.png");
        if (icon != null)
            setIconImage(icon.getImage());
    }

    protected void layoutComponents() {
        description = new JTabbedPane();
        ExerciseDescriptionParser parser = new ExerciseDescriptionParser(
                new BufferedReader(new StringReader(FileHelper
                        .getDocumentFile("/docs/exercises/"
                                + getExercisePanel().getExercise().getClass()
                                        .getName() + ".html"))));
        task = new JEditorPane();
        task.setEditable(false);
        task.setContentType("text/html");
        taskText = parser.getTask();
        setText();
        JScrollPane taskScroll = new JScrollPane(task);
        description.add(taskScroll, "Task");

        JEditorPane instructions = new JEditorPane();
        instructions.setEditable(false);
        instructions.setContentType("text/html");
        instructions.setText(parser.getInstructions());
        JScrollPane instrScroll = new JScrollPane(instructions);
        description.add(instrScroll, "Instructions");

        for (int i = 0; i < parser.getNumberOfCodeBlocks(); i++) {
            JEditorPane codeBlock = new JEditorPane();
            codeBlock.setEditable(false);
            codeBlock.setContentType("text/html");
            String name = parser.getCodeBlockName(i);
            codeBlock.setText(parser.getCodeBlock(name));
            JScrollPane codeScroll = new JScrollPane(codeBlock);
            description.add(codeScroll, name);
        }

        // this must be created for the superclass to work properly
        descriptionTextArea = new JTextPane();

        // See java.awt.GridBagLayout for details.
        GridBagLayout grid = new GridBagLayout();
        getContentPane().setLayout(grid);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(10, 10, 10, 10);
        gbc.gridheight = GridBagConstraints.REMAINDER;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;

        gbc.gridheight = 1;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.NONE;
        getContentPane().add(getVisualAnimator(), gbc);

        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        JScrollPane sp = getScrollPane();
        sp.setViewportView(getStructurePanel());
        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                description, sp);
        split.setOneTouchExpandable(true);
        split.setDividerLocation(300);
        getContentPane().add(split, gbc);
    }

    private void setText() {
        String exer = getExercisePanel().getExercise().getDescription();
        String text = taskText;
        if (exer != null && !exer.trim().equals(""))
            text += "<hr><pre style=\"font-size:small;font-family:serif;\">"
                    + exer + "</pre>";
        task.setText(text);

    }

    public void reset() {
        super.reset();
        setText();
    }
}
