// MatrixPro - algorithm simulation system
// Copyright (C) 2000-2007 See the AUTHORS file and the author tag
// below for more information on the authors.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package matrixpro.util.export;

import java.awt.GridLayout;
import java.awt.Rectangle;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import matrix.util.export.LatexWriter;
import matrix.util.export.MatrixGraphics;

import matrixpro.ui.MainFrame;

/**
 * TeXdraw picture export format for MatrixPro.
 * 
 * @version $Id: TexExportFormat.java,v 1.8 2007/09/10 08:35:56 vkaravir Exp $
 */
public class TexExportFormat extends MPExportFormat {
    /** The <code>JPanel</code> used when tex is selected. */
    private JPanel tex;

    /** <code>JCheckBox</code> for the tex document selection. */
    private JCheckBox document;

    /** <code>JCheckBox</code> for the structure border selection. */
    private JCheckBox border;

    public TexExportFormat() {
        super();
    }

    public FileFilter getFileFilter() {
        return new TexFileFilter();
    }

    public String getFileExtension() {
        return "tex";
    }

    public JComponent getAccessory() {
        if (tex == null)
            initTexPanel();
        return tex;
    }

    /**
     * Method that adds components to the panel used as a file chooser
     * accessory.
     */
    private void initTexPanel() {
        tex = new JPanel();
        tex.setLayout(new GridLayout(2, 1));
        tex.setBorder(BorderFactory.createTitledBorder(BorderFactory
                .createBevelBorder(2), "LaTeX"));
        document = new JCheckBox("Complete document", true);
        document.setToolTipText("Check to export a compelete LaTeX document,"
                + " uncheck to export only the picture.");
        tex.add(document);
        border = new JCheckBox("Structures with borders", true);
        border.setToolTipText("Check to export structures with border, "
                + "uncheck to export structures without borders");
        tex.add(border);
    }

    public boolean export(MainFrame mf, File saveFile) {
        boolean doc = document.isSelected();
        boolean brdr = border.isSelected();
        JPanel p = mf.getStructurePanel();
        MatrixGraphics matrix = new MatrixGraphics(new Rectangle(0, 0, 10000,
                10000), p.getGraphics());
        matrix.addDisable("VisualHotSpot");
        if (!brdr) {
            matrix.addDisable("StructureBorder");
        }
        p.paint(matrix);
        LatexWriter tolatex = new LatexWriter(matrix.getContents());
        tolatex.write(saveFile, doc, false);
        return true;
    }

}
